namespace Godot
{
    public static partial class GD
    {
        /// <summary>
        /// Scancodes with this bit applied are non printable.
        /// </summary>
        public const int Spkey = 16777216;
    }

    public enum Margin
    {
        /// <summary>
        /// Left margin, used usually for [Control] or [StyleBox] derived classes.
        /// </summary>
        Left = 0,
        /// <summary>
        /// Top margin, used usually for [Control] or [StyleBox] derived classes.
        /// </summary>
        Top = 1,
        /// <summary>
        /// Right margin, used usually for [Control] or [StyleBox] derived classes.
        /// </summary>
        Right = 2,
        /// <summary>
        /// Bottom margin, used usually for [Control] or [StyleBox] derived classes.
        /// </summary>
        Bottom = 3
    }

    public enum Corner
    {
        /// <summary>
        /// </summary>
        TopLeft = 0,
        /// <summary>
        /// </summary>
        TopRight = 1,
        /// <summary>
        /// </summary>
        BottomRight = 2,
        /// <summary>
        /// </summary>
        BottomLeft = 3
    }

    public enum Orientation
    {
        /// <summary>
        /// General vertical alignment, used usually for [Separator], [ScrollBar], [Slider], etc.
        /// </summary>
        Vertical = 1,
        /// <summary>
        /// General horizontal alignment, used usually for [Separator], [ScrollBar], [Slider], etc.
        /// </summary>
        Horizontal = 0
    }

    public enum HAlign
    {
        /// <summary>
        /// Horizontal left alignment, usually for text-derived classes.
        /// </summary>
        Left = 0,
        /// <summary>
        /// Horizontal center alignment, usually for text-derived classes.
        /// </summary>
        Center = 1,
        /// <summary>
        /// Horizontal right alignment, usually for text-derived classes.
        /// </summary>
        Right = 2
    }

    public enum VAlign
    {
        /// <summary>
        /// Vertical top alignment, usually for text-derived classes.
        /// </summary>
        Top = 0,
        /// <summary>
        /// Vertical center alignment, usually for text-derived classes.
        /// </summary>
        Center = 1,
        /// <summary>
        /// Vertical bottom alignment, usually for text-derived classes.
        /// </summary>
        Bottom = 2
    }

    public enum KeyList
    {
        /// <summary>
        /// Escape Key
        /// </summary>
        Escape = 16777217,
        /// <summary>
        /// Tab Key
        /// </summary>
        Tab = 16777218,
        /// <summary>
        /// Shift-Tab Key
        /// </summary>
        Backtab = 16777219,
        /// <summary>
        /// Backspace Key
        /// </summary>
        Backspace = 16777220,
        /// <summary>
        /// Return Key (On Main Keyboard)
        /// </summary>
        Enter = 16777221,
        /// <summary>
        /// Enter Key (On Numpad)
        /// </summary>
        KpEnter = 16777222,
        /// <summary>
        /// Insert Key
        /// </summary>
        Insert = 16777223,
        /// <summary>
        /// Delete Key
        /// </summary>
        Delete = 16777224,
        /// <summary>
        /// Pause Key
        /// </summary>
        Pause = 16777225,
        /// <summary>
        /// Printscreen Key
        /// </summary>
        Print = 16777226,
        /// <summary>
        /// System Request Key
        /// </summary>
        Sysreq = 16777227,
        /// <summary>
        /// Clear Key
        /// </summary>
        Clear = 16777228,
        /// <summary>
        /// Home Key
        /// </summary>
        Home = 16777229,
        /// <summary>
        /// End Key
        /// </summary>
        End = 16777230,
        /// <summary>
        /// Left Arrow Key
        /// </summary>
        Left = 16777231,
        /// <summary>
        /// Up Arrow Key
        /// </summary>
        Up = 16777232,
        /// <summary>
        /// Right Arrow Key
        /// </summary>
        Right = 16777233,
        /// <summary>
        /// Down Arrow Key
        /// </summary>
        Down = 16777234,
        /// <summary>
        /// Pageup Key
        /// </summary>
        Pageup = 16777235,
        /// <summary>
        /// Pagedown Key
        /// </summary>
        Pagedown = 16777236,
        /// <summary>
        /// Shift Key
        /// </summary>
        Shift = 16777237,
        /// <summary>
        /// Control Key
        /// </summary>
        Control = 16777238,
        /// <summary>
        /// Meta Key
        /// </summary>
        Meta = 16777239,
        /// <summary>
        /// Alt Key
        /// </summary>
        Alt = 16777240,
        /// <summary>
        /// Capslock Key
        /// </summary>
        Capslock = 16777241,
        /// <summary>
        /// Numlock Key
        /// </summary>
        Numlock = 16777242,
        /// <summary>
        /// Scrolllock Key
        /// </summary>
        Scrolllock = 16777243,
        /// <summary>
        /// F1 Key
        /// </summary>
        F1 = 16777244,
        /// <summary>
        /// F2 Key
        /// </summary>
        F2 = 16777245,
        /// <summary>
        /// F3 Key
        /// </summary>
        F3 = 16777246,
        /// <summary>
        /// F4 Key
        /// </summary>
        F4 = 16777247,
        /// <summary>
        /// F5 Key
        /// </summary>
        F5 = 16777248,
        /// <summary>
        /// F6 Key
        /// </summary>
        F6 = 16777249,
        /// <summary>
        /// F7 Key
        /// </summary>
        F7 = 16777250,
        /// <summary>
        /// F8 Key
        /// </summary>
        F8 = 16777251,
        /// <summary>
        /// F9 Key
        /// </summary>
        F9 = 16777252,
        /// <summary>
        /// F10 Key
        /// </summary>
        F10 = 16777253,
        /// <summary>
        /// F11 Key
        /// </summary>
        F11 = 16777254,
        /// <summary>
        /// F12 Key
        /// </summary>
        F12 = 16777255,
        /// <summary>
        /// F13 Key
        /// </summary>
        F13 = 16777256,
        /// <summary>
        /// F14 Key
        /// </summary>
        F14 = 16777257,
        /// <summary>
        /// F15 Key
        /// </summary>
        F15 = 16777258,
        /// <summary>
        /// F16 Key
        /// </summary>
        F16 = 16777259,
        /// <summary>
        /// Multiply Key on Numpad
        /// </summary>
        KpMultiply = 16777345,
        /// <summary>
        /// Divide Key on Numpad
        /// </summary>
        KpDivide = 16777346,
        /// <summary>
        /// Subtract Key on Numpad
        /// </summary>
        KpSubtract = 16777347,
        /// <summary>
        /// Period Key on Numpad
        /// </summary>
        KpPeriod = 16777348,
        /// <summary>
        /// Add Key on Numpad
        /// </summary>
        KpAdd = 16777349,
        /// <summary>
        /// Number 0 on Numpad
        /// </summary>
        Kp0 = 16777350,
        /// <summary>
        /// Number 1 on Numpad
        /// </summary>
        Kp1 = 16777351,
        /// <summary>
        /// Number 2 on Numpad
        /// </summary>
        Kp2 = 16777352,
        /// <summary>
        /// Number 3 on Numpad
        /// </summary>
        Kp3 = 16777353,
        /// <summary>
        /// Number 4 on Numpad
        /// </summary>
        Kp4 = 16777354,
        /// <summary>
        /// Number 5 on Numpad
        /// </summary>
        Kp5 = 16777355,
        /// <summary>
        /// Number 6 on Numpad
        /// </summary>
        Kp6 = 16777356,
        /// <summary>
        /// Number 7 on Numpad
        /// </summary>
        Kp7 = 16777357,
        /// <summary>
        /// Number 8 on Numpad
        /// </summary>
        Kp8 = 16777358,
        /// <summary>
        /// Number 9 on Numpad
        /// </summary>
        Kp9 = 16777359,
        /// <summary>
        /// Left Super Key (Windows Key)
        /// </summary>
        SuperL = 16777260,
        /// <summary>
        /// Right Super Key (Windows Key)
        /// </summary>
        SuperR = 16777261,
        /// <summary>
        /// Context menu key
        /// </summary>
        Menu = 16777262,
        /// <summary>
        /// Left Hyper Key
        /// </summary>
        HyperL = 16777263,
        /// <summary>
        /// Right Hyper Key
        /// </summary>
        HyperR = 16777264,
        /// <summary>
        /// Help key
        /// </summary>
        Help = 16777265,
        /// <summary>
        /// Left Direction Key
        /// </summary>
        DirectionL = 16777266,
        /// <summary>
        /// Right Direction Key
        /// </summary>
        DirectionR = 16777267,
        /// <summary>
        /// Back key
        /// </summary>
        Back = 16777280,
        /// <summary>
        /// Forward key
        /// </summary>
        Forward = 16777281,
        /// <summary>
        /// Stop key
        /// </summary>
        Stop = 16777282,
        /// <summary>
        /// Refresh key
        /// </summary>
        Refresh = 16777283,
        /// <summary>
        /// Volume down key
        /// </summary>
        Volumedown = 16777284,
        /// <summary>
        /// Mute volume key
        /// </summary>
        Volumemute = 16777285,
        /// <summary>
        /// Volume up key
        /// </summary>
        Volumeup = 16777286,
        /// <summary>
        /// Bass Boost Key
        /// </summary>
        Bassboost = 16777287,
        /// <summary>
        /// Bass Up Key
        /// </summary>
        Bassup = 16777288,
        /// <summary>
        /// Bass Down Key
        /// </summary>
        Bassdown = 16777289,
        /// <summary>
        /// Treble Up Key
        /// </summary>
        Trebleup = 16777290,
        /// <summary>
        /// Treble Down Key
        /// </summary>
        Trebledown = 16777291,
        /// <summary>
        /// Media play key
        /// </summary>
        Mediaplay = 16777292,
        /// <summary>
        /// Media stop key
        /// </summary>
        Mediastop = 16777293,
        /// <summary>
        /// Previous song key
        /// </summary>
        Mediaprevious = 16777294,
        /// <summary>
        /// Next song key
        /// </summary>
        Medianext = 16777295,
        /// <summary>
        /// Media record key
        /// </summary>
        Mediarecord = 16777296,
        /// <summary>
        /// Home page key
        /// </summary>
        Homepage = 16777297,
        /// <summary>
        /// Favorites key
        /// </summary>
        Favorites = 16777298,
        /// <summary>
        /// Search key
        /// </summary>
        Search = 16777299,
        /// <summary>
        /// Standby Key
        /// </summary>
        Standby = 16777300,
        /// <summary>
        /// Open URL / Launch Browser Key
        /// </summary>
        Openurl = 16777301,
        /// <summary>
        /// Launch Mail Key
        /// </summary>
        Launchmail = 16777302,
        /// <summary>
        /// Launch Media Key
        /// </summary>
        Launchmedia = 16777303,
        /// <summary>
        /// Launch Shortcut 0 Key
        /// </summary>
        Launch0 = 16777304,
        /// <summary>
        /// Launch Shortcut 1 Key
        /// </summary>
        Launch1 = 16777305,
        /// <summary>
        /// Launch Shortcut 2 Key
        /// </summary>
        Launch2 = 16777306,
        /// <summary>
        /// Launch Shortcut 3 Key
        /// </summary>
        Launch3 = 16777307,
        /// <summary>
        /// Launch Shortcut 4 Key
        /// </summary>
        Launch4 = 16777308,
        /// <summary>
        /// Launch Shortcut 5 Key
        /// </summary>
        Launch5 = 16777309,
        /// <summary>
        /// Launch Shortcut 6 Key
        /// </summary>
        Launch6 = 16777310,
        /// <summary>
        /// Launch Shortcut 7 Key
        /// </summary>
        Launch7 = 16777311,
        /// <summary>
        /// Launch Shortcut 8 Key
        /// </summary>
        Launch8 = 16777312,
        /// <summary>
        /// Launch Shortcut 9 Key
        /// </summary>
        Launch9 = 16777313,
        /// <summary>
        /// Launch Shortcut A Key
        /// </summary>
        Launcha = 16777314,
        /// <summary>
        /// Launch Shortcut B Key
        /// </summary>
        Launchb = 16777315,
        /// <summary>
        /// Launch Shortcut C Key
        /// </summary>
        Launchc = 16777316,
        /// <summary>
        /// Launch Shortcut D Key
        /// </summary>
        Launchd = 16777317,
        /// <summary>
        /// Launch Shortcut E Key
        /// </summary>
        Launche = 16777318,
        /// <summary>
        /// Launch Shortcut F Key
        /// </summary>
        Launchf = 16777319,
        /// <summary>
        /// Unknown Key
        /// </summary>
        Unknown = 33554431,
        /// <summary>
        /// Space Key
        /// </summary>
        Space = 32,
        /// <summary>
        /// ! key
        /// </summary>
        Exclam = 33,
        /// <summary>
        /// " key
        /// </summary>
        Quotedbl = 34,
        /// <summary>
        /// # key
        /// </summary>
        Numbersign = 35,
        /// <summary>
        /// $ key
        /// </summary>
        Dollar = 36,
        /// <summary>
        /// % key
        /// </summary>
        Percent = 37,
        /// <summary>
        /// &amp; key
        /// </summary>
        Ampersand = 38,
        /// <summary>
        /// ' key
        /// </summary>
        Apostrophe = 39,
        /// <summary>
        /// ( key
        /// </summary>
        Parenleft = 40,
        /// <summary>
        /// ) key
        /// </summary>
        Parenright = 41,
        /// <summary>
        /// * key
        /// </summary>
        Asterisk = 42,
        /// <summary>
        /// + key
        /// </summary>
        Plus = 43,
        /// <summary>
        /// , key
        /// </summary>
        Comma = 44,
        /// <summary>
        /// - key
        /// </summary>
        Minus = 45,
        /// <summary>
        /// . key
        /// </summary>
        Period = 46,
        /// <summary>
        /// / key
        /// </summary>
        Slash = 47,
        /// <summary>
        /// Number 0
        /// </summary>
        Key0 = 48,
        /// <summary>
        /// Number 1
        /// </summary>
        Key1 = 49,
        /// <summary>
        /// Number 2
        /// </summary>
        Key2 = 50,
        /// <summary>
        /// Number 3
        /// </summary>
        Key3 = 51,
        /// <summary>
        /// Number 4
        /// </summary>
        Key4 = 52,
        /// <summary>
        /// Number 5
        /// </summary>
        Key5 = 53,
        /// <summary>
        /// Number 6
        /// </summary>
        Key6 = 54,
        /// <summary>
        /// Number 7
        /// </summary>
        Key7 = 55,
        /// <summary>
        /// Number 8
        /// </summary>
        Key8 = 56,
        /// <summary>
        /// Number 9
        /// </summary>
        Key9 = 57,
        /// <summary>
        /// : key
        /// </summary>
        Colon = 58,
        /// <summary>
        /// ; key
        /// </summary>
        Semicolon = 59,
        /// <summary>
        /// Lower than key
        /// </summary>
        Less = 60,
        /// <summary>
        /// = key
        /// </summary>
        Equal = 61,
        /// <summary>
        /// Greater than key
        /// </summary>
        Greater = 62,
        /// <summary>
        /// ? key
        /// </summary>
        Question = 63,
        /// <summary>
        /// @ key
        /// </summary>
        At = 64,
        /// <summary>
        /// A Key
        /// </summary>
        A = 65,
        /// <summary>
        /// B Key
        /// </summary>
        B = 66,
        /// <summary>
        /// C Key
        /// </summary>
        C = 67,
        /// <summary>
        /// D Key
        /// </summary>
        D = 68,
        /// <summary>
        /// E Key
        /// </summary>
        E = 69,
        /// <summary>
        /// F Key
        /// </summary>
        F = 70,
        /// <summary>
        /// G Key
        /// </summary>
        G = 71,
        /// <summary>
        /// H Key
        /// </summary>
        H = 72,
        /// <summary>
        /// I Key
        /// </summary>
        I = 73,
        /// <summary>
        /// J Key
        /// </summary>
        J = 74,
        /// <summary>
        /// K Key
        /// </summary>
        K = 75,
        /// <summary>
        /// L Key
        /// </summary>
        L = 76,
        /// <summary>
        /// M Key
        /// </summary>
        M = 77,
        /// <summary>
        /// N Key
        /// </summary>
        N = 78,
        /// <summary>
        /// O Key
        /// </summary>
        O = 79,
        /// <summary>
        /// P Key
        /// </summary>
        P = 80,
        /// <summary>
        /// Q Key
        /// </summary>
        Q = 81,
        /// <summary>
        /// R Key
        /// </summary>
        R = 82,
        /// <summary>
        /// S Key
        /// </summary>
        S = 83,
        /// <summary>
        /// T Key
        /// </summary>
        T = 84,
        /// <summary>
        /// U Key
        /// </summary>
        U = 85,
        /// <summary>
        /// V Key
        /// </summary>
        V = 86,
        /// <summary>
        /// W Key
        /// </summary>
        W = 87,
        /// <summary>
        /// X Key
        /// </summary>
        X = 88,
        /// <summary>
        /// Y Key
        /// </summary>
        Y = 89,
        /// <summary>
        /// Z Key
        /// </summary>
        Z = 90,
        /// <summary>
        /// [ key
        /// </summary>
        Bracketleft = 91,
        /// <summary>
        /// \ key
        /// </summary>
        Backslash = 92,
        /// <summary>
        /// ] key
        /// </summary>
        Bracketright = 93,
        /// <summary>
        /// ^ key
        /// </summary>
        Asciicircum = 94,
        /// <summary>
        /// _ key
        /// </summary>
        Underscore = 95,
        /// <summary>
        /// Left Quote Key
        /// </summary>
        Quoteleft = 96,
        /// <summary>
        /// { key
        /// </summary>
        Braceleft = 123,
        /// <summary>
        /// | key
        /// </summary>
        Bar = 124,
        /// <summary>
        /// } key
        /// </summary>
        Braceright = 125,
        /// <summary>
        /// ~ key
        /// </summary>
        Asciitilde = 126,
        /// <summary>
        /// </summary>
        Nobreakspace = 160,
        /// <summary>
        /// </summary>
        Exclamdown = 161,
        /// <summary>
        /// ¢ key
        /// </summary>
        Cent = 162,
        /// <summary>
        /// </summary>
        Sterling = 163,
        /// <summary>
        /// </summary>
        Currency = 164,
        /// <summary>
        /// Yen Key
        /// </summary>
        Yen = 165,
        /// <summary>
        /// ¦ key
        /// </summary>
        Brokenbar = 166,
        /// <summary>
        /// § key
        /// </summary>
        Section = 167,
        /// <summary>
        /// ¨ key
        /// </summary>
        Diaeresis = 168,
        /// <summary>
        /// © key
        /// </summary>
        Copyright = 169,
        /// <summary>
        /// </summary>
        Ordfeminine = 170,
        /// <summary>
        /// « key
        /// </summary>
        Guillemotleft = 171,
        /// <summary>
        /// » key
        /// </summary>
        Notsign = 172,
        /// <summary>
        /// ‐ key
        /// </summary>
        Hyphen = 173,
        /// <summary>
        /// ® key
        /// </summary>
        Registered = 174,
        /// <summary>
        /// Macron Key
        /// </summary>
        Macron = 175,
        /// <summary>
        /// ° key
        /// </summary>
        Degree = 176,
        /// <summary>
        /// ± key
        /// </summary>
        Plusminus = 177,
        /// <summary>
        /// ² key
        /// </summary>
        Twosuperior = 178,
        /// <summary>
        /// ³ key
        /// </summary>
        Threesuperior = 179,
        /// <summary>
        /// ´ key
        /// </summary>
        Acute = 180,
        /// <summary>
        /// µ key
        /// </summary>
        Mu = 181,
        /// <summary>
        /// Paragraph Key
        /// </summary>
        Paragraph = 182,
        /// <summary>
        /// · key
        /// </summary>
        Periodcentered = 183,
        /// <summary>
        /// ¬ key
        /// </summary>
        Cedilla = 184,
        /// <summary>
        /// ¹ key
        /// </summary>
        Onesuperior = 185,
        /// <summary>
        /// ♂ key
        /// </summary>
        Masculine = 186,
        /// <summary>
        /// » key
        /// </summary>
        Guillemotright = 187,
        /// <summary>
        /// ¼ key
        /// </summary>
        Onequarter = 188,
        /// <summary>
        /// ½ key
        /// </summary>
        Onehalf = 189,
        /// <summary>
        /// ¾ key
        /// </summary>
        Threequarters = 190,
        /// <summary>
        /// ¿ key
        /// </summary>
        Questiondown = 191,
        /// <summary>
        /// à key
        /// </summary>
        Agrave = 192,
        /// <summary>
        /// á key
        /// </summary>
        Aacute = 193,
        /// <summary>
        /// â key
        /// </summary>
        Acircumflex = 194,
        /// <summary>
        /// ã key
        /// </summary>
        Atilde = 195,
        /// <summary>
        /// ä key
        /// </summary>
        Adiaeresis = 196,
        /// <summary>
        /// å key
        /// </summary>
        Aring = 197,
        /// <summary>
        /// æ key
        /// </summary>
        Ae = 198,
        /// <summary>
        /// ç key
        /// </summary>
        Ccedilla = 199,
        /// <summary>
        /// è key
        /// </summary>
        Egrave = 200,
        /// <summary>
        /// é key
        /// </summary>
        Eacute = 201,
        /// <summary>
        /// ê key
        /// </summary>
        Ecircumflex = 202,
        /// <summary>
        /// ë key
        /// </summary>
        Ediaeresis = 203,
        /// <summary>
        /// ì key
        /// </summary>
        Igrave = 204,
        /// <summary>
        /// í key
        /// </summary>
        Iacute = 205,
        /// <summary>
        /// î key
        /// </summary>
        Icircumflex = 206,
        /// <summary>
        /// ë key
        /// </summary>
        Idiaeresis = 207,
        /// <summary>
        /// ð key
        /// </summary>
        Eth = 208,
        /// <summary>
        /// ñ key
        /// </summary>
        Ntilde = 209,
        /// <summary>
        /// ò key
        /// </summary>
        Ograve = 210,
        /// <summary>
        /// ó key
        /// </summary>
        Oacute = 211,
        /// <summary>
        /// ô key
        /// </summary>
        Ocircumflex = 212,
        /// <summary>
        /// õ key
        /// </summary>
        Otilde = 213,
        /// <summary>
        /// ö key
        /// </summary>
        Odiaeresis = 214,
        /// <summary>
        /// × key
        /// </summary>
        Multiply = 215,
        /// <summary>
        /// ø key
        /// </summary>
        Ooblique = 216,
        /// <summary>
        /// ù key
        /// </summary>
        Ugrave = 217,
        /// <summary>
        /// ú key
        /// </summary>
        Uacute = 218,
        /// <summary>
        /// û key
        /// </summary>
        Ucircumflex = 219,
        /// <summary>
        /// ü key
        /// </summary>
        Udiaeresis = 220,
        /// <summary>
        /// ý key
        /// </summary>
        Yacute = 221,
        /// <summary>
        /// þ key
        /// </summary>
        Thorn = 222,
        /// <summary>
        /// ß key
        /// </summary>
        Ssharp = 223,
        /// <summary>
        /// ÷ key
        /// </summary>
        Division = 247,
        /// <summary>
        /// ÿ key
        /// </summary>
        Ydiaeresis = 255
    }

    public enum KeyModifierMask
    {
        /// <summary>
        /// Key Code Mask
        /// </summary>
        CodeMask = 33554431,
        /// <summary>
        /// Modifier Key Mask
        /// </summary>
        ModifierMask = -16777216,
        /// <summary>
        /// Shift Key Mask
        /// </summary>
        MaskShift = 33554432,
        /// <summary>
        /// Alt Key Mask
        /// </summary>
        MaskAlt = 67108864,
        /// <summary>
        /// Meta Key Mask
        /// </summary>
        MaskMeta = 134217728,
        /// <summary>
        /// CTRL Key Mask
        /// </summary>
        MaskCtrl = 268435456,
        /// <summary>
        /// CMD Key Mask
        /// </summary>
        MaskCmd = 268435456,
        /// <summary>
        /// Keypad Key Mask
        /// </summary>
        MaskKpad = 536870912,
        /// <summary>
        /// Group Switch Key Mask
        /// </summary>
        MaskGroupSwitch = 1073741824
    }

    public enum ButtonList
    {
        /// <summary>
        /// Left Mouse Button
        /// </summary>
        Left = 1,
        /// <summary>
        /// Right Mouse Button
        /// </summary>
        Right = 2,
        /// <summary>
        /// Middle Mouse Button
        /// </summary>
        Middle = 3,
        /// <summary>
        /// Mouse wheel up
        /// </summary>
        WheelUp = 4,
        /// <summary>
        /// Mouse wheel down
        /// </summary>
        WheelDown = 5,
        /// <summary>
        /// Mouse wheel left button
        /// </summary>
        WheelLeft = 6,
        /// <summary>
        /// Mouse wheel right button
        /// </summary>
        WheelRight = 7,
        /// <summary>
        /// Left Mouse Button Mask
        /// </summary>
        MaskLeft = 1,
        /// <summary>
        /// Right Mouse Button Mask
        /// </summary>
        MaskRight = 2,
        /// <summary>
        /// Middle Mouse Button Mask
        /// </summary>
        MaskMiddle = 4
    }

    public enum JoystickList
    {
        /// <summary>
        /// Joypad Button 0
        /// </summary>
        Button0 = 0,
        /// <summary>
        /// Joypad Button 1
        /// </summary>
        Button1 = 1,
        /// <summary>
        /// Joypad Button 2
        /// </summary>
        Button2 = 2,
        /// <summary>
        /// Joypad Button 3
        /// </summary>
        Button3 = 3,
        /// <summary>
        /// Joypad Button 4
        /// </summary>
        Button4 = 4,
        /// <summary>
        /// Joypad Button 5
        /// </summary>
        Button5 = 5,
        /// <summary>
        /// Joypad Button 6
        /// </summary>
        Button6 = 6,
        /// <summary>
        /// Joypad Button 7
        /// </summary>
        Button7 = 7,
        /// <summary>
        /// Joypad Button 8
        /// </summary>
        Button8 = 8,
        /// <summary>
        /// Joypad Button 9
        /// </summary>
        Button9 = 9,
        /// <summary>
        /// Joypad Button 10
        /// </summary>
        Button10 = 10,
        /// <summary>
        /// Joypad Button 11
        /// </summary>
        Button11 = 11,
        /// <summary>
        /// Joypad Button 12
        /// </summary>
        Button12 = 12,
        /// <summary>
        /// Joypad Button 13
        /// </summary>
        Button13 = 13,
        /// <summary>
        /// Joypad Button 14
        /// </summary>
        Button14 = 14,
        /// <summary>
        /// Joypad Button 15
        /// </summary>
        Button15 = 15,
        /// <summary>
        /// Joypad Button 16
        /// </summary>
        ButtonMax = 16,
        /// <summary>
        /// DUALSHOCK circle button
        /// </summary>
        SonyCircle = 1,
        /// <summary>
        /// DUALSHOCK X button
        /// </summary>
        SonyX = 0,
        /// <summary>
        /// DUALSHOCK square button
        /// </summary>
        SonySquare = 2,
        /// <summary>
        /// DUALSHOCK triangle button
        /// </summary>
        SonyTriangle = 3,
        /// <summary>
        /// XBOX controller B button
        /// </summary>
        XboxB = 1,
        /// <summary>
        /// XBOX controller A button
        /// </summary>
        XboxA = 0,
        /// <summary>
        /// XBOX controller X button
        /// </summary>
        XboxX = 2,
        /// <summary>
        /// XBOX controller Y button
        /// </summary>
        XboxY = 3,
        /// <summary>
        /// DualShock controller A button
        /// </summary>
        DsA = 1,
        /// <summary>
        /// DualShock controller B button
        /// </summary>
        DsB = 0,
        /// <summary>
        /// DualShock controller X button
        /// </summary>
        DsX = 3,
        /// <summary>
        /// DualShock controller Y button
        /// </summary>
        DsY = 2,
        /// <summary>
        /// Joypad Button Select
        /// </summary>
        Select = 10,
        /// <summary>
        /// Joypad Button Start
        /// </summary>
        Start = 11,
        /// <summary>
        /// Joypad DPad Up
        /// </summary>
        DpadUp = 12,
        /// <summary>
        /// Joypad DPad Down
        /// </summary>
        DpadDown = 13,
        /// <summary>
        /// Joypad DPad Left
        /// </summary>
        DpadLeft = 14,
        /// <summary>
        /// Joypad DPad Right
        /// </summary>
        DpadRight = 15,
        /// <summary>
        /// Joypad Left Shoulder Button
        /// </summary>
        L = 4,
        /// <summary>
        /// Joypad Left Trigger
        /// </summary>
        L2 = 6,
        /// <summary>
        /// Joypad Left Stick Click
        /// </summary>
        L3 = 8,
        /// <summary>
        /// Joypad Right Shoulder Button
        /// </summary>
        R = 5,
        /// <summary>
        /// Joypad Right Trigger
        /// </summary>
        R2 = 7,
        /// <summary>
        /// Joypad Right Stick Click
        /// </summary>
        R3 = 9,
        /// <summary>
        /// Joypad Left Stick Horizontal Axis
        /// </summary>
        Axis0 = 0,
        /// <summary>
        /// Joypad Left Stick Vertical Axis
        /// </summary>
        Axis1 = 1,
        /// <summary>
        /// Joypad Right Stick Horizontal Axis
        /// </summary>
        Axis2 = 2,
        /// <summary>
        /// Joypad Right Stick Vertical Axis
        /// </summary>
        Axis3 = 3,
        /// <summary>
        /// </summary>
        Axis4 = 4,
        /// <summary>
        /// </summary>
        Axis5 = 5,
        /// <summary>
        /// Joypad Left Trigger Analog Axis
        /// </summary>
        Axis6 = 6,
        /// <summary>
        /// Joypad Right Trigger Analog Axis
        /// </summary>
        Axis7 = 7,
        /// <summary>
        /// </summary>
        Axis8 = 8,
        /// <summary>
        /// </summary>
        Axis9 = 9,
        /// <summary>
        /// </summary>
        AxisMax = 10,
        /// <summary>
        /// Joypad Left Stick Horizontal Axis
        /// </summary>
        AnalogLx = 0,
        /// <summary>
        /// Joypad Left Stick Vertical Axis
        /// </summary>
        AnalogLy = 1,
        /// <summary>
        /// Joypad Right Stick Horizontal Axis
        /// </summary>
        AnalogRx = 2,
        /// <summary>
        /// Joypad Right Stick Vertical Axis
        /// </summary>
        AnalogRy = 3,
        /// <summary>
        /// Joypad Left Analog Trigger
        /// </summary>
        AnalogL2 = 6,
        /// <summary>
        /// Joypad Right Analog Trigger
        /// </summary>
        AnalogR2 = 7
    }

    public enum Error
    {
        /// <summary>
        /// Functions that return Error return OK when no error occured. Most functions don't return errors and/or just print errors to STDOUT.
        /// </summary>
        Ok = 0,
        /// <summary>
        /// Generic error.
        /// </summary>
        Failed = 1,
        /// <summary>
        /// Unavailable error
        /// </summary>
        Unavailable = 2,
        /// <summary>
        /// Unconfigured error
        /// </summary>
        Unconfigured = 3,
        /// <summary>
        /// Unauthorized error
        /// </summary>
        Unauthorized = 4,
        /// <summary>
        /// Parameter range error
        /// </summary>
        ParameterRangeError = 5,
        /// <summary>
        /// Out of memory (OOM) error
        /// </summary>
        OutOfMemory = 6,
        /// <summary>
        /// File: Not found error
        /// </summary>
        FileNotFound = 7,
        /// <summary>
        /// File: Bad drive error
        /// </summary>
        FileBadDrive = 8,
        /// <summary>
        /// File: Bad path error
        /// </summary>
        FileBadPath = 9,
        /// <summary>
        /// File: No permission error
        /// </summary>
        FileNoPermission = 10,
        /// <summary>
        /// File: Already in use error
        /// </summary>
        FileAlreadyInUse = 11,
        /// <summary>
        /// File: Can't open error
        /// </summary>
        FileCantOpen = 12,
        /// <summary>
        /// File: Can't write error
        /// </summary>
        FileCantWrite = 13,
        /// <summary>
        /// File: Can't read error
        /// </summary>
        FileCantRead = 14,
        /// <summary>
        /// File: Unrecognized error
        /// </summary>
        FileUnrecognized = 15,
        /// <summary>
        /// File: Corrupt error
        /// </summary>
        FileCorrupt = 16,
        /// <summary>
        /// File: Missing dependencies error
        /// </summary>
        FileMissingDependencies = 17,
        /// <summary>
        /// File: End of file (EOF) error
        /// </summary>
        FileEof = 18,
        /// <summary>
        /// Can't open error
        /// </summary>
        CantOpen = 19,
        /// <summary>
        /// Can't create error
        /// </summary>
        CantCreate = 20,
        /// <summary>
        /// Parse error
        /// </summary>
        ParseError = 43,
        /// <summary>
        /// Query failed error
        /// </summary>
        QueryFailed = 21,
        /// <summary>
        /// Already in use error
        /// </summary>
        AlreadyInUse = 22,
        /// <summary>
        /// Locked error
        /// </summary>
        Locked = 23,
        /// <summary>
        /// Timeout error
        /// </summary>
        Timeout = 24,
        /// <summary>
        /// Can't acquire resource error
        /// </summary>
        CantAcquireResource = 28,
        /// <summary>
        /// Invalid data error
        /// </summary>
        InvalidData = 30,
        /// <summary>
        /// Invalid parameter error
        /// </summary>
        InvalidParameter = 31,
        /// <summary>
        /// Already exists error
        /// </summary>
        AlreadyExists = 32,
        /// <summary>
        /// Does not exist error
        /// </summary>
        DoesNotExist = 33,
        /// <summary>
        /// Database: Read error
        /// </summary>
        DatabaseCantRead = 34,
        /// <summary>
        /// Database: Write error
        /// </summary>
        DatabaseCantWrite = 35,
        /// <summary>
        /// Compilation failed error
        /// </summary>
        CompilationFailed = 36,
        /// <summary>
        /// Method not found error
        /// </summary>
        MethodNotFound = 37,
        /// <summary>
        /// Linking failed error
        /// </summary>
        LinkFailed = 38,
        /// <summary>
        /// Script failed error
        /// </summary>
        ScriptFailed = 39,
        /// <summary>
        /// Cycling link (import cycle) error
        /// </summary>
        CyclicLink = 40,
        /// <summary>
        /// Busy error
        /// </summary>
        Busy = 44,
        /// <summary>
        /// Help error
        /// </summary>
        Help = 46,
        /// <summary>
        /// Bug error
        /// </summary>
        Bug = 47
    }

    public enum PropertyHint
    {
        /// <summary>
        /// No hint for edited property.
        /// </summary>
        None = 0,
        /// <summary>
        /// Hints that the string is a range, defined as "min,max" or "min,max,step". This is valid for integers and floats.
        /// </summary>
        Range = 1,
        /// <summary>
        /// Hints that the string is an exponential range, defined as "min,max" or "min,max,step". This is valid for integers and floats.
        /// </summary>
        ExpRange = 2,
        /// <summary>
        /// Property hint for an enumerated value, like "Hello,Something,Else". This is valid for integer, float and string properties.
        /// </summary>
        Enum = 3,
        /// <summary>
        /// </summary>
        ExpEasing = 4,
        /// <summary>
        /// </summary>
        Length = 5,
        /// <summary>
        /// </summary>
        KeyAccel = 7,
        /// <summary>
        /// Property hint for a bitmask description, for bits 0,1,2,3 and 5 the hint would be like "Bit0,Bit1,Bit2,Bit3,,Bit5". Valid only for integers.
        /// </summary>
        Flags = 8,
        /// <summary>
        /// </summary>
        Layers2dRender = 9,
        /// <summary>
        /// </summary>
        Layers2dPhysics = 10,
        /// <summary>
        /// </summary>
        Layers3dRender = 11,
        /// <summary>
        /// </summary>
        Layers3dPhysics = 12,
        /// <summary>
        /// String property is a file (so pop up a file dialog when edited). Hint string can be a set of wildcards like "*.doc".
        /// </summary>
        File = 13,
        /// <summary>
        /// String property is a directory (so pop up a file dialog when edited).
        /// </summary>
        Dir = 14,
        /// <summary>
        /// </summary>
        GlobalFile = 15,
        /// <summary>
        /// </summary>
        GlobalDir = 16,
        /// <summary>
        /// String property is a resource, so open the resource popup menu when edited.
        /// </summary>
        ResourceType = 17,
        /// <summary>
        /// </summary>
        MultilineText = 18,
        /// <summary>
        /// </summary>
        ColorNoAlpha = 19,
        /// <summary>
        /// Hints that the image is compressed using lossy compression.
        /// </summary>
        ImageCompressLossy = 20,
        /// <summary>
        /// Hints that the image is compressed using lossless compression.
        /// </summary>
        ImageCompressLossless = 21
    }

    public enum PropertyUsageFlags
    {
        /// <summary>
        /// Property will be used as storage (default).
        /// </summary>
        Storage = 1,
        /// <summary>
        /// Property will be visible in editor (default).
        /// </summary>
        Editor = 2,
        /// <summary>
        /// </summary>
        Network = 4,
        /// <summary>
        /// </summary>
        EditorHelper = 8,
        /// <summary>
        /// </summary>
        Checkable = 16,
        /// <summary>
        /// </summary>
        Checked = 32,
        /// <summary>
        /// </summary>
        Internationalized = 64,
        /// <summary>
        /// </summary>
        Group = 128,
        /// <summary>
        /// </summary>
        Category = 256,
        /// <summary>
        /// </summary>
        StoreIfNonzero = 512,
        /// <summary>
        /// </summary>
        StoreIfNonone = 1024,
        /// <summary>
        /// </summary>
        NoInstanceState = 2048,
        /// <summary>
        /// </summary>
        RestartIfChanged = 4096,
        /// <summary>
        /// </summary>
        ScriptVariable = 8192,
        /// <summary>
        /// Default usage (storage and editor).
        /// </summary>
        Default = 7,
        /// <summary>
        /// </summary>
        DefaultIntl = 71,
        /// <summary>
        /// </summary>
        Noeditor = 5
    }

    public enum MethodFlags
    {
        /// <summary>
        /// Flag for normal method
        /// </summary>
        Normal = 1,
        /// <summary>
        /// Flag for editor method
        /// </summary>
        Editor = 2,
        /// <summary>
        /// </summary>
        Noscript = 4,
        /// <summary>
        /// Flag for constant method
        /// </summary>
        Const = 8,
        /// <summary>
        /// </summary>
        Reverse = 16,
        /// <summary>
        /// Flag for virtual method
        /// </summary>
        Virtual = 32,
        /// <summary>
        /// Flag for method from script
        /// </summary>
        FromScript = 64,
        /// <summary>
        /// Default method flags
        /// </summary>
        sDefault = 1
    }

    public static partial class Variant
    {

    public enum Type
    {
        /// <summary>
        /// Variable is of type nil (only applied for null).
        /// </summary>
        Nil = 0,
        /// <summary>
        /// Variable is of type [bool].
        /// </summary>
        Bool = 1,
        /// <summary>
        /// Variable is of type [int].
        /// </summary>
        Int = 2,
        /// <summary>
        /// Variable is of type [float]/real.
        /// </summary>
        Real = 3,
        /// <summary>
        /// Variable is of type [String].
        /// </summary>
        String = 4,
        /// <summary>
        /// Variable is of type [Vector2].
        /// </summary>
        Vector2 = 5,
        /// <summary>
        /// Variable is of type [Rect2].
        /// </summary>
        Rect2 = 6,
        /// <summary>
        /// Variable is of type [Vector3].
        /// </summary>
        Vector3 = 7,
        /// <summary>
        /// Variable is of type [Transform2D].
        /// </summary>
        Transform2d = 8,
        /// <summary>
        /// Variable is of type [Plane].
        /// </summary>
        Plane = 9,
        /// <summary>
        /// Variable is of type [Quat].
        /// </summary>
        Quat = 10,
        /// <summary>
        /// Variable is of type [AABB].
        /// </summary>
        Aabb = 11,
        /// <summary>
        /// Variable is of type [Basis].
        /// </summary>
        Basis = 12,
        /// <summary>
        /// Variable is of type [Transform].
        /// </summary>
        Transform = 13,
        /// <summary>
        /// Variable is of type [Color].
        /// </summary>
        Color = 14,
        /// <summary>
        /// Variable is of type [NodePath].
        /// </summary>
        NodePath = 15,
        /// <summary>
        /// Variable is of type [RID].
        /// </summary>
        Rid = 16,
        /// <summary>
        /// Variable is of type [Object].
        /// </summary>
        Object = 17,
        /// <summary>
        /// Variable is of type [Dictionary].
        /// </summary>
        Dictionary = 18,
        /// <summary>
        /// Variable is of type [Array].
        /// </summary>
        Array = 19,
        /// <summary>
        /// Variable is of type [PoolByteArray].
        /// </summary>
        RawArray = 20,
        /// <summary>
        /// Variable is of type [PoolIntArray].
        /// </summary>
        IntArray = 21,
        /// <summary>
        /// Variable is of type [PoolRealArray].
        /// </summary>
        RealArray = 22,
        /// <summary>
        /// Variable is of type [PoolStringArray].
        /// </summary>
        StringArray = 23,
        /// <summary>
        /// Variable is of type [PoolVector2Array].
        /// </summary>
        Vector2Array = 24,
        /// <summary>
        /// Variable is of type [PoolVector3Array].
        /// </summary>
        Vector3Array = 25,
        /// <summary>
        /// Variable is of type [PoolColorArray].
        /// </summary>
        ColorArray = 26,
        /// <summary>
        /// Marker for end of type constants.
        /// </summary>
        Max = 27
    }
    }

    public static partial class Variant
    {

    public enum Operator
    {
        /// <summary>
        /// </summary>
        Equal = 0,
        /// <summary>
        /// </summary>
        NotEqual = 1,
        /// <summary>
        /// </summary>
        Less = 2,
        /// <summary>
        /// </summary>
        LessEqual = 3,
        /// <summary>
        /// </summary>
        Greater = 4,
        /// <summary>
        /// </summary>
        GreaterEqual = 5,
        /// <summary>
        /// </summary>
        Add = 6,
        /// <summary>
        /// </summary>
        Subtract = 7,
        /// <summary>
        /// </summary>
        Multiply = 8,
        /// <summary>
        /// </summary>
        Divide = 9,
        /// <summary>
        /// </summary>
        Negate = 10,
        /// <summary>
        /// </summary>
        Positive = 11,
        /// <summary>
        /// </summary>
        Module = 12,
        /// <summary>
        /// </summary>
        StringConcat = 13,
        /// <summary>
        /// </summary>
        ShiftLeft = 14,
        /// <summary>
        /// </summary>
        ShiftRight = 15,
        /// <summary>
        /// </summary>
        BitAnd = 16,
        /// <summary>
        /// </summary>
        BitOr = 17,
        /// <summary>
        /// </summary>
        BitXor = 18,
        /// <summary>
        /// </summary>
        BitNegate = 19,
        /// <summary>
        /// </summary>
        And = 20,
        /// <summary>
        /// </summary>
        Or = 21,
        /// <summary>
        /// </summary>
        Xor = 22,
        /// <summary>
        /// </summary>
        Not = 23,
        /// <summary>
        /// </summary>
        In = 24,
        /// <summary>
        /// </summary>
        Max = 25
    }
    }
}
