using System;

namespace Godot
{
    /// <summary>
    /// This dialog is useful for small notifications to the user about an event. It can only be accepted or closed, with the same result.
    /// </summary>
    public class AcceptDialog : WindowDialog
    {
        /// <summary>
        /// The text displayed by this dialog.
        /// </summary>
        public string DialogText
        {
            get
            {
                return GetText();
            }
            set
            {
                SetText(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the dialog is hidden when accepted. Default value: [code]true[/code].
        /// </summary>
        public bool DialogHideOnOk
        {
            get
            {
                return GetHideOnOk();
            }
            set
            {
                SetHideOnOk(value);
            }
        }


        private const string nativeName = "AcceptDialog";

        public AcceptDialog() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_AcceptDialog_Ctor(this);
        }

        internal AcceptDialog(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_ok");

        /// <summary>
        /// Return the OK Button.
        /// </summary>
        [GodotMethod("get_ok")]
        public Button GetOk()
        {
            return NativeCalls.godot_icall_0_38(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_label");

        /// <summary>
        /// Return the label used for built-in text.
        /// </summary>
        [GodotMethod("get_label")]
        public Label GetLabel()
        {
            return NativeCalls.godot_icall_0_39(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_hide_on_ok");

        [GodotMethod("set_hide_on_ok")]
        public void SetHideOnOk(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_2, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_hide_on_ok");

        [GodotMethod("get_hide_on_ok")]
        public bool GetHideOnOk()
        {
            return NativeCalls.godot_icall_0_3(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_button");

        /// <summary>
        /// Adds a button with label [i]text[/i] and a custom [i]action[/i] to the dialog and returns the created button. [i]action[/i] will be passed to the [custom_action] signal when pressed.
        /// If [code]true[/code], [i]right[/i] will place the button to the right of any sibling buttons. Default value: [code]false[/code].
        /// </summary>
        [GodotMethod("add_button")]
        public Button AddButton(string text, bool right = false, string action = "")
        {
            return NativeCalls.godot_icall_3_40(method_bind_4, Object.GetPtr(this), text, right, action);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_cancel");

        /// <summary>
        /// Adds a button with label [i]name[/i] and a cancel action to the dialog and returns the created button.
        /// </summary>
        [GodotMethod("add_cancel")]
        public Button AddCancel(string name)
        {
            return NativeCalls.godot_icall_1_41(method_bind_5, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "register_text_enter");

        /// <summary>
        /// Registers a [LineEdit] in the dialog. When the enter key is pressed, the dialog will be accepted.
        /// </summary>
        [GodotMethod("register_text_enter")]
        public void RegisterTextEnter(Node lineEdit)
        {
            NativeCalls.godot_icall_1_42(method_bind_6, Object.GetPtr(this), Object.GetPtr(lineEdit));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_text");

        [GodotMethod("set_text")]
        public void SetText(string text)
        {
            NativeCalls.godot_icall_1_43(method_bind_7, Object.GetPtr(this), text);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_text");

        [GodotMethod("get_text")]
        public string GetText()
        {
            return NativeCalls.godot_icall_0_2(method_bind_8, Object.GetPtr(this));
        }
    }
}
