using System;

namespace Godot
{
    public class ArrayMesh : Mesh
    {
        public const int NoIndexArray = -1;
        public const int ArrayWeightsSize = 4;

        public enum ArrayFormat
        {
            Vertex = 1,
            Normal = 2,
            Tangent = 4,
            Color = 8,
            TexUv = 16,
            TexUv2 = 32,
            Bones = 64,
            Weights = 128,
            Index = 256
        }

        public enum ArrayType
        {
            Vertex = 0,
            Normal = 1,
            Tangent = 2,
            Color = 3,
            TexUv = 4,
            TexUv2 = 5,
            Bones = 6,
            Weights = 7,
            Index = 8,
            Max = 9
        }


        /// <summary>
        /// </summary>
        public Mesh.BlendShapeMode BlendShapeMode
        {
            get
            {
                return GetBlendShapeMode();
            }
            set
            {
                SetBlendShapeMode(value);
            }
        }

        /// <summary>
        /// </summary>
        public AABB CustomAabb
        {
            get
            {
                return GetCustomAabb();
            }
            set
            {
                SetCustomAabb(value);
            }
        }


        private const string nativeName = "ArrayMesh";

        public ArrayMesh() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_ArrayMesh_Ctor(this);
        }

        internal ArrayMesh(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_blend_shape");

        /// <summary>
        /// </summary>
        [GodotMethod("add_blend_shape")]
        public void AddBlendShape(string name)
        {
            NativeCalls.godot_icall_1_43(method_bind_0, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_blend_shape_count");

        /// <summary>
        /// </summary>
        [GodotMethod("get_blend_shape_count")]
        public int GetBlendShapeCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_blend_shape_name");

        /// <summary>
        /// </summary>
        [GodotMethod("get_blend_shape_name")]
        public string GetBlendShapeName(int index)
        {
            return NativeCalls.godot_icall_1_108(method_bind_2, Object.GetPtr(this), index);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear_blend_shapes");

        /// <summary>
        /// </summary>
        [GodotMethod("clear_blend_shapes")]
        public void ClearBlendShapes()
        {
            NativeCalls.godot_icall_0_12(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_blend_shape_mode");

        [GodotMethod("set_blend_shape_mode")]
        public void SetBlendShapeMode(Mesh.BlendShapeMode mode)
        {
            NativeCalls.godot_icall_1_109(method_bind_4, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_blend_shape_mode");

        [GodotMethod("get_blend_shape_mode")]
        public Mesh.BlendShapeMode GetBlendShapeMode()
        {
            return NativeCalls.godot_icall_0_110(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_surface_from_arrays");

        /// <summary>
        /// Create a new surface ([method get_surface_count] that will become surf_idx for this.
        /// Surfaces are created to be rendered using a "primitive", which may be PRIMITIVE_POINTS, PRIMITIVE_LINES, PRIMITIVE_LINE_STRIP, PRIMITIVE_LINE_LOOP, PRIMITIVE_TRIANGLES, PRIMITIVE_TRIANGLE_STRIP, PRIMITIVE_TRIANGLE_FAN. (As a note, when using indices, it is recommended to only use just points, lines or triangles).
        /// &lt;param name="blendShapes"&gt;If the param is null, then the default value is new object[] {}&lt;/param&gt;
        /// </summary>
        [GodotMethod("add_surface_from_arrays")]
        public void AddSurfaceFromArrays(Mesh.PrimitiveType primitive, object[] arrays, object[] blendShapes = null, int compressFlags = 97792)
        {
            object[] blendShapes_in = blendShapes != null ? blendShapes : new object[] {};
            NativeCalls.godot_icall_4_111(method_bind_6, Object.GetPtr(this), primitive, arrays, blendShapes_in, compressFlags);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_surface_count");

        /// <summary>
        /// Return the amount of surfaces that the [code]ArrayMesh[/code] holds.
        /// </summary>
        [GodotMethod("get_surface_count")]
        public int GetSurfaceCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "surface_remove");

        /// <summary>
        /// Remove a surface at position surf_idx, shifting greater surfaces one surf_idx slot down.
        /// </summary>
        [GodotMethod("surface_remove")]
        public void SurfaceRemove(int surfIdx)
        {
            NativeCalls.godot_icall_1_0(method_bind_8, Object.GetPtr(this), surfIdx);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "surface_update_region");

        /// <summary>
        /// </summary>
        [GodotMethod("surface_update_region")]
        public void SurfaceUpdateRegion(int surfIdx, int offset, byte[] data)
        {
            NativeCalls.godot_icall_3_112(method_bind_9, Object.GetPtr(this), surfIdx, offset, data);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "surface_get_array_len");

        /// <summary>
        /// Return the length in vertices of the vertex array in the requested surface (see [method add_surface]).
        /// </summary>
        [GodotMethod("surface_get_array_len")]
        public int SurfaceGetArrayLen(int surfIdx)
        {
            return NativeCalls.godot_icall_1_6(method_bind_10, Object.GetPtr(this), surfIdx);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "surface_get_array_index_len");

        /// <summary>
        /// Return the length in indices of the index array in the requested surface (see [method add_surface]).
        /// </summary>
        [GodotMethod("surface_get_array_index_len")]
        public int SurfaceGetArrayIndexLen(int surfIdx)
        {
            return NativeCalls.godot_icall_1_6(method_bind_11, Object.GetPtr(this), surfIdx);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "surface_get_format");

        /// <summary>
        /// Return the format mask of the requested surface (see [method add_surface]).
        /// </summary>
        [GodotMethod("surface_get_format")]
        public int SurfaceGetFormat(int surfIdx)
        {
            return NativeCalls.godot_icall_1_6(method_bind_12, Object.GetPtr(this), surfIdx);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "surface_get_primitive_type");

        /// <summary>
        /// Return the primitive type of the requested surface (see [method add_surface]).
        /// </summary>
        [GodotMethod("surface_get_primitive_type")]
        public Mesh.PrimitiveType SurfaceGetPrimitiveType(int surfIdx)
        {
            return NativeCalls.godot_icall_1_113(method_bind_13, Object.GetPtr(this), surfIdx);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "surface_set_material");

        /// <summary>
        /// </summary>
        [GodotMethod("surface_set_material")]
        public void SurfaceSetMaterial(int surfIdx, Material material)
        {
            NativeCalls.godot_icall_2_67(method_bind_14, Object.GetPtr(this), surfIdx, Object.GetPtr(material));
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "surface_get_material");

        /// <summary>
        /// Return a [Material] in a given surface. Surface is rendered using this material.
        /// </summary>
        [GodotMethod("surface_get_material")]
        public Material SurfaceGetMaterial(int surfIdx)
        {
            return NativeCalls.godot_icall_1_114(method_bind_15, Object.GetPtr(this), surfIdx);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "surface_set_name");

        /// <summary>
        /// Set a [Material] for a given surface. Surface will be rendered using this material.
        /// </summary>
        [GodotMethod("surface_set_name")]
        public void SurfaceSetName(int surfIdx, string name)
        {
            NativeCalls.godot_icall_2_115(method_bind_16, Object.GetPtr(this), surfIdx, name);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "surface_get_name");

        /// <summary>
        /// </summary>
        [GodotMethod("surface_get_name")]
        public string SurfaceGetName(int surfIdx)
        {
            return NativeCalls.godot_icall_1_108(method_bind_17, Object.GetPtr(this), surfIdx);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "surface_get_arrays");

        /// <summary>
        /// </summary>
        [GodotMethod("surface_get_arrays")]
        public object[] SurfaceGetArrays(int surfIdx)
        {
            return NativeCalls.godot_icall_1_116(method_bind_18, Object.GetPtr(this), surfIdx);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "surface_get_blend_shape_arrays");

        /// <summary>
        /// </summary>
        [GodotMethod("surface_get_blend_shape_arrays")]
        public object[] SurfaceGetBlendShapeArrays(int surfIdx)
        {
            return NativeCalls.godot_icall_1_116(method_bind_19, Object.GetPtr(this), surfIdx);
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "center_geometry");

        /// <summary>
        /// </summary>
        [GodotMethod("center_geometry")]
        public void CenterGeometry()
        {
            NativeCalls.godot_icall_0_12(method_bind_20, Object.GetPtr(this));
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "regen_normalmaps");

        /// <summary>
        /// </summary>
        [GodotMethod("regen_normalmaps")]
        public void RegenNormalmaps()
        {
            NativeCalls.godot_icall_0_12(method_bind_21, Object.GetPtr(this));
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_custom_aabb");

        [GodotMethod("set_custom_aabb")]
        public void SetCustomAabb(AABB aabb)
        {
            NativeCalls.godot_icall_1_117(method_bind_22, Object.GetPtr(this), ref aabb);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_custom_aabb");

        [GodotMethod("get_custom_aabb")]
        public AABB GetCustomAabb()
        {
            object ret = NativeCalls.godot_icall_0_118(method_bind_23, Object.GetPtr(this));
            return (AABB)ret;
        }
    }
}
