using System;

namespace Godot
{
    /// <summary>
    /// Increases or decreases the volume being routed through the audio bus.
    /// </summary>
    public class AudioEffectAmplify : AudioEffect
    {
        /// <summary>
        /// Amount of amplification. Positive values make the sound louder, negative values make it quieter. Value can range from -80 to 24. Default value: [code]0[/code].
        /// </summary>
        public float VolumeDb
        {
            get
            {
                return GetVolumeDb();
            }
            set
            {
                SetVolumeDb(value);
            }
        }


        private const string nativeName = "AudioEffectAmplify";

        public AudioEffectAmplify() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_AudioEffectAmplify_Ctor(this);
        }

        internal AudioEffectAmplify(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_volume_db");

        [GodotMethod("set_volume_db")]
        public void SetVolumeDb(float volume)
        {
            NativeCalls.godot_icall_1_10(method_bind_0, Object.GetPtr(this), volume);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_volume_db");

        [GodotMethod("get_volume_db")]
        public float GetVolumeDb()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }
    }
}
