using System;

namespace Godot
{
    /// <summary>
    /// Adds a chorus audio effect. The effect applies a filter with voices to duplicate the audio source and manipulate it through the filter.
    /// </summary>
    public class AudioEffectChorus : AudioEffect
    {
        /// <summary>
        /// The amount of voices in the effect.
        /// </summary>
        public int VoiceCount
        {
            get
            {
                return GetVoiceCount();
            }
            set
            {
                SetVoiceCount(value);
            }
        }

        /// <summary>
        /// The effect's raw signal.
        /// </summary>
        public float Dry
        {
            get
            {
                return GetDry();
            }
            set
            {
                SetDry(value);
            }
        }

        /// <summary>
        /// The effect's processed signal.
        /// </summary>
        public float Wet
        {
            get
            {
                return GetWet();
            }
            set
            {
                SetWet(value);
            }
        }

        /// <summary>
        /// The voice's signal delay.
        /// </summary>
        public float Voice__1__delayMs
        {
            get
            {
                return GetVoiceDelayMs(0);
            }
            set
            {
                SetVoiceDelayMs(0, value);
            }
        }

        /// <summary>
        /// The voice's filter rate.
        /// </summary>
        public float Voice__1__rateHz
        {
            get
            {
                return GetVoiceRateHz(0);
            }
            set
            {
                SetVoiceRateHz(0, value);
            }
        }

        /// <summary>
        /// The voice filter's depth.
        /// </summary>
        public float Voice__1__depthMs
        {
            get
            {
                return GetVoiceDepthMs(0);
            }
            set
            {
                SetVoiceDepthMs(0, value);
            }
        }

        /// <summary>
        /// The voice's volume.
        /// </summary>
        public float Voice__1__levelDb
        {
            get
            {
                return GetVoiceLevelDb(0);
            }
            set
            {
                SetVoiceLevelDb(0, value);
            }
        }

        /// <summary>
        /// The voice's cutoff frequency.
        /// </summary>
        public float Voice__1__cutoffHz
        {
            get
            {
                return GetVoiceCutoffHz(0);
            }
            set
            {
                SetVoiceCutoffHz(0, value);
            }
        }

        /// <summary>
        /// The voice's pan level.
        /// </summary>
        public float Voice__1__pan
        {
            get
            {
                return GetVoicePan(0);
            }
            set
            {
                SetVoicePan(0, value);
            }
        }

        /// <summary>
        /// The voice's signal delay.
        /// </summary>
        public float Voice__2__delayMs
        {
            get
            {
                return GetVoiceDelayMs(1);
            }
            set
            {
                SetVoiceDelayMs(1, value);
            }
        }

        /// <summary>
        /// The voice's filter rate.
        /// </summary>
        public float Voice__2__rateHz
        {
            get
            {
                return GetVoiceRateHz(1);
            }
            set
            {
                SetVoiceRateHz(1, value);
            }
        }

        /// <summary>
        /// The voice filter's depth.
        /// </summary>
        public float Voice__2__depthMs
        {
            get
            {
                return GetVoiceDepthMs(1);
            }
            set
            {
                SetVoiceDepthMs(1, value);
            }
        }

        /// <summary>
        /// The voice's volume.
        /// </summary>
        public float Voice__2__levelDb
        {
            get
            {
                return GetVoiceLevelDb(1);
            }
            set
            {
                SetVoiceLevelDb(1, value);
            }
        }

        /// <summary>
        /// The voice's cutoff frequency.
        /// </summary>
        public float Voice__2__cutoffHz
        {
            get
            {
                return GetVoiceCutoffHz(1);
            }
            set
            {
                SetVoiceCutoffHz(1, value);
            }
        }

        /// <summary>
        /// The voice's pan level.
        /// </summary>
        public float Voice__2__pan
        {
            get
            {
                return GetVoicePan(1);
            }
            set
            {
                SetVoicePan(1, value);
            }
        }

        /// <summary>
        /// The voice's signal delay.
        /// </summary>
        public float Voice__3__delayMs
        {
            get
            {
                return GetVoiceDelayMs(2);
            }
            set
            {
                SetVoiceDelayMs(2, value);
            }
        }

        /// <summary>
        /// The voice's filter rate.
        /// </summary>
        public float Voice__3__rateHz
        {
            get
            {
                return GetVoiceRateHz(2);
            }
            set
            {
                SetVoiceRateHz(2, value);
            }
        }

        /// <summary>
        /// The voice filter's depth.
        /// </summary>
        public float Voice__3__depthMs
        {
            get
            {
                return GetVoiceDepthMs(2);
            }
            set
            {
                SetVoiceDepthMs(2, value);
            }
        }

        /// <summary>
        /// The voice's volume.
        /// </summary>
        public float Voice__3__levelDb
        {
            get
            {
                return GetVoiceLevelDb(2);
            }
            set
            {
                SetVoiceLevelDb(2, value);
            }
        }

        /// <summary>
        /// The voice's cutoff frequency.
        /// </summary>
        public float Voice__3__cutoffHz
        {
            get
            {
                return GetVoiceCutoffHz(2);
            }
            set
            {
                SetVoiceCutoffHz(2, value);
            }
        }

        /// <summary>
        /// The voice's pan level.
        /// </summary>
        public float Voice__3__pan
        {
            get
            {
                return GetVoicePan(2);
            }
            set
            {
                SetVoicePan(2, value);
            }
        }

        /// <summary>
        /// The voice's signal delay.
        /// </summary>
        public float Voice__4__delayMs
        {
            get
            {
                return GetVoiceDelayMs(3);
            }
            set
            {
                SetVoiceDelayMs(3, value);
            }
        }

        /// <summary>
        /// The voice's filter rate.
        /// </summary>
        public float Voice__4__rateHz
        {
            get
            {
                return GetVoiceRateHz(3);
            }
            set
            {
                SetVoiceRateHz(3, value);
            }
        }

        /// <summary>
        /// The voice filter's depth.
        /// </summary>
        public float Voice__4__depthMs
        {
            get
            {
                return GetVoiceDepthMs(3);
            }
            set
            {
                SetVoiceDepthMs(3, value);
            }
        }

        /// <summary>
        /// The voice's volume.
        /// </summary>
        public float Voice__4__levelDb
        {
            get
            {
                return GetVoiceLevelDb(3);
            }
            set
            {
                SetVoiceLevelDb(3, value);
            }
        }

        /// <summary>
        /// The voice's cutoff frequency.
        /// </summary>
        public float Voice__4__cutoffHz
        {
            get
            {
                return GetVoiceCutoffHz(3);
            }
            set
            {
                SetVoiceCutoffHz(3, value);
            }
        }

        /// <summary>
        /// The voice's pan level.
        /// </summary>
        public float Voice__4__pan
        {
            get
            {
                return GetVoicePan(3);
            }
            set
            {
                SetVoicePan(3, value);
            }
        }


        private const string nativeName = "AudioEffectChorus";

        public AudioEffectChorus() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_AudioEffectChorus_Ctor(this);
        }

        internal AudioEffectChorus(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_voice_count");

        [GodotMethod("set_voice_count")]
        public void SetVoiceCount(int voices)
        {
            NativeCalls.godot_icall_1_0(method_bind_0, Object.GetPtr(this), voices);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_voice_count");

        [GodotMethod("get_voice_count")]
        public int GetVoiceCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_voice_delay_ms");

        [GodotMethod("set_voice_delay_ms")]
        public void SetVoiceDelayMs(int voiceIdx, float delayMs)
        {
            NativeCalls.godot_icall_2_28(method_bind_2, Object.GetPtr(this), voiceIdx, delayMs);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_voice_delay_ms");

        [GodotMethod("get_voice_delay_ms")]
        public float GetVoiceDelayMs(int voiceIdx)
        {
            return NativeCalls.godot_icall_1_7(method_bind_3, Object.GetPtr(this), voiceIdx);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_voice_rate_hz");

        [GodotMethod("set_voice_rate_hz")]
        public void SetVoiceRateHz(int voiceIdx, float rateHz)
        {
            NativeCalls.godot_icall_2_28(method_bind_4, Object.GetPtr(this), voiceIdx, rateHz);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_voice_rate_hz");

        [GodotMethod("get_voice_rate_hz")]
        public float GetVoiceRateHz(int voiceIdx)
        {
            return NativeCalls.godot_icall_1_7(method_bind_5, Object.GetPtr(this), voiceIdx);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_voice_depth_ms");

        [GodotMethod("set_voice_depth_ms")]
        public void SetVoiceDepthMs(int voiceIdx, float depthMs)
        {
            NativeCalls.godot_icall_2_28(method_bind_6, Object.GetPtr(this), voiceIdx, depthMs);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_voice_depth_ms");

        [GodotMethod("get_voice_depth_ms")]
        public float GetVoiceDepthMs(int voiceIdx)
        {
            return NativeCalls.godot_icall_1_7(method_bind_7, Object.GetPtr(this), voiceIdx);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_voice_level_db");

        [GodotMethod("set_voice_level_db")]
        public void SetVoiceLevelDb(int voiceIdx, float levelDb)
        {
            NativeCalls.godot_icall_2_28(method_bind_8, Object.GetPtr(this), voiceIdx, levelDb);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_voice_level_db");

        [GodotMethod("get_voice_level_db")]
        public float GetVoiceLevelDb(int voiceIdx)
        {
            return NativeCalls.godot_icall_1_7(method_bind_9, Object.GetPtr(this), voiceIdx);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_voice_cutoff_hz");

        [GodotMethod("set_voice_cutoff_hz")]
        public void SetVoiceCutoffHz(int voiceIdx, float cutoffHz)
        {
            NativeCalls.godot_icall_2_28(method_bind_10, Object.GetPtr(this), voiceIdx, cutoffHz);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_voice_cutoff_hz");

        [GodotMethod("get_voice_cutoff_hz")]
        public float GetVoiceCutoffHz(int voiceIdx)
        {
            return NativeCalls.godot_icall_1_7(method_bind_11, Object.GetPtr(this), voiceIdx);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_voice_pan");

        [GodotMethod("set_voice_pan")]
        public void SetVoicePan(int voiceIdx, float pan)
        {
            NativeCalls.godot_icall_2_28(method_bind_12, Object.GetPtr(this), voiceIdx, pan);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_voice_pan");

        [GodotMethod("get_voice_pan")]
        public float GetVoicePan(int voiceIdx)
        {
            return NativeCalls.godot_icall_1_7(method_bind_13, Object.GetPtr(this), voiceIdx);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_wet");

        [GodotMethod("set_wet")]
        public void SetWet(float amount)
        {
            NativeCalls.godot_icall_1_10(method_bind_14, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_wet");

        [GodotMethod("get_wet")]
        public float GetWet()
        {
            return NativeCalls.godot_icall_0_9(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_dry");

        [GodotMethod("set_dry")]
        public void SetDry(float amount)
        {
            NativeCalls.godot_icall_1_10(method_bind_16, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_dry");

        [GodotMethod("get_dry")]
        public float GetDry()
        {
            return NativeCalls.godot_icall_0_9(method_bind_17, Object.GetPtr(this));
        }
    }
}
