using System;

namespace Godot
{
    /// <summary>
    /// Plays input signal back after a period of time. The delayed signal may be played back multiple times to create the sound of a repeating, decaying echo. Delay effects range from a subtle echo effect to a pronounced blending of previous sounds with new sounds.
    /// </summary>
    public class AudioEffectDelay : AudioEffect
    {
        /// <summary>
        /// Output percent of original sound. At 0, only delayed sounds are output. Value can range from 0 to 1. Default value: [code]1[/code].
        /// </summary>
        public float Dry
        {
            get
            {
                return GetDry();
            }
            set
            {
                SetDry(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], [code]tap1[/code] will be enabled. Default value: [code]true[/code].
        /// </summary>
        public bool Tap1__active
        {
            get
            {
                return IsTap1Active();
            }
            set
            {
                SetTap1Active(value);
            }
        }

        /// <summary>
        /// [b]Tap1[/b] delay time in milliseconds. Default value: [code]250ms[/code].
        /// </summary>
        public float Tap1__delayMs
        {
            get
            {
                return GetTap1DelayMs();
            }
            set
            {
                SetTap1DelayMs(value);
            }
        }

        /// <summary>
        /// Sound level for [code]tap1[/code]. Default value: [code]-6 dB[/code].
        /// </summary>
        public float Tap1__levelDb
        {
            get
            {
                return GetTap1LevelDb();
            }
            set
            {
                SetTap1LevelDb(value);
            }
        }

        /// <summary>
        /// Pan position for [code]tap1[/code]. Value can range from -1 (fully left) to 1 (fully right). Default value: [code]0.2[/code].
        /// </summary>
        public float Tap1__pan
        {
            get
            {
                return GetTap1Pan();
            }
            set
            {
                SetTap1Pan(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], [code]tap2[/code] will be enabled. Default value: [code]true[/code].
        /// </summary>
        public bool Tap2__active
        {
            get
            {
                return IsTap2Active();
            }
            set
            {
                SetTap2Active(value);
            }
        }

        /// <summary>
        /// [b]Tap2[/b] delay time in milliseconds. Default value: [code]500ms[/code].
        /// </summary>
        public float Tap2__delayMs
        {
            get
            {
                return GetTap2DelayMs();
            }
            set
            {
                SetTap2DelayMs(value);
            }
        }

        /// <summary>
        /// Sound level for [code]tap2[/code]. Default value: [code]-12 dB[/code].
        /// </summary>
        public float Tap2__levelDb
        {
            get
            {
                return GetTap2LevelDb();
            }
            set
            {
                SetTap2LevelDb(value);
            }
        }

        /// <summary>
        /// Pan position for [code]tap2[/code]. Value can range from -1 (fully left) to 1 (fully right). Default value: [code]-0.4[/code].
        /// </summary>
        public float Tap2__pan
        {
            get
            {
                return GetTap2Pan();
            }
            set
            {
                SetTap2Pan(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] feedback is enabled. Default value: [code]false[/code].
        /// </summary>
        public bool Feedback__active
        {
            get
            {
                return IsFeedbackActive();
            }
            set
            {
                SetFeedbackActive(value);
            }
        }

        /// <summary>
        /// Feedback delay time in milliseconds. Default value: [code]340[/code].
        /// </summary>
        public float Feedback__delayMs
        {
            get
            {
                return GetFeedbackDelayMs();
            }
            set
            {
                SetFeedbackDelayMs(value);
            }
        }

        /// <summary>
        /// Sound level for [code]tap1[/code]. Default value: [code]-6 dB[/code].
        /// </summary>
        public float Feedback__levelDb
        {
            get
            {
                return GetFeedbackLevelDb();
            }
            set
            {
                SetFeedbackLevelDb(value);
            }
        }

        /// <summary>
        /// Low-pass filter for feedback. Frequencies below the Low Cut value are filtered out of the source signal. Default value: [code]16000[/code].
        /// </summary>
        public float Feedback__lowpass
        {
            get
            {
                return GetFeedbackLowpass();
            }
            set
            {
                SetFeedbackLowpass(value);
            }
        }


        private const string nativeName = "AudioEffectDelay";

        public AudioEffectDelay() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_AudioEffectDelay_Ctor(this);
        }

        internal AudioEffectDelay(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_dry");

        [GodotMethod("set_dry")]
        public void SetDry(float amount)
        {
            NativeCalls.godot_icall_1_10(method_bind_0, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_dry");

        [GodotMethod("get_dry")]
        public float GetDry()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tap1_active");

        [GodotMethod("set_tap1_active")]
        public void SetTap1Active(bool amount)
        {
            NativeCalls.godot_icall_1_11(method_bind_2, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_tap1_active");

        [GodotMethod("is_tap1_active")]
        public bool IsTap1Active()
        {
            return NativeCalls.godot_icall_0_3(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tap1_delay_ms");

        [GodotMethod("set_tap1_delay_ms")]
        public void SetTap1DelayMs(float amount)
        {
            NativeCalls.godot_icall_1_10(method_bind_4, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tap1_delay_ms");

        [GodotMethod("get_tap1_delay_ms")]
        public float GetTap1DelayMs()
        {
            return NativeCalls.godot_icall_0_9(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tap1_level_db");

        [GodotMethod("set_tap1_level_db")]
        public void SetTap1LevelDb(float amount)
        {
            NativeCalls.godot_icall_1_10(method_bind_6, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tap1_level_db");

        [GodotMethod("get_tap1_level_db")]
        public float GetTap1LevelDb()
        {
            return NativeCalls.godot_icall_0_9(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tap1_pan");

        [GodotMethod("set_tap1_pan")]
        public void SetTap1Pan(float amount)
        {
            NativeCalls.godot_icall_1_10(method_bind_8, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tap1_pan");

        [GodotMethod("get_tap1_pan")]
        public float GetTap1Pan()
        {
            return NativeCalls.godot_icall_0_9(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tap2_active");

        [GodotMethod("set_tap2_active")]
        public void SetTap2Active(bool amount)
        {
            NativeCalls.godot_icall_1_11(method_bind_10, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_tap2_active");

        [GodotMethod("is_tap2_active")]
        public bool IsTap2Active()
        {
            return NativeCalls.godot_icall_0_3(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tap2_delay_ms");

        [GodotMethod("set_tap2_delay_ms")]
        public void SetTap2DelayMs(float amount)
        {
            NativeCalls.godot_icall_1_10(method_bind_12, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tap2_delay_ms");

        [GodotMethod("get_tap2_delay_ms")]
        public float GetTap2DelayMs()
        {
            return NativeCalls.godot_icall_0_9(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tap2_level_db");

        [GodotMethod("set_tap2_level_db")]
        public void SetTap2LevelDb(float amount)
        {
            NativeCalls.godot_icall_1_10(method_bind_14, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tap2_level_db");

        [GodotMethod("get_tap2_level_db")]
        public float GetTap2LevelDb()
        {
            return NativeCalls.godot_icall_0_9(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tap2_pan");

        [GodotMethod("set_tap2_pan")]
        public void SetTap2Pan(float amount)
        {
            NativeCalls.godot_icall_1_10(method_bind_16, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tap2_pan");

        [GodotMethod("get_tap2_pan")]
        public float GetTap2Pan()
        {
            return NativeCalls.godot_icall_0_9(method_bind_17, Object.GetPtr(this));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_feedback_active");

        [GodotMethod("set_feedback_active")]
        public void SetFeedbackActive(bool amount)
        {
            NativeCalls.godot_icall_1_11(method_bind_18, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_feedback_active");

        [GodotMethod("is_feedback_active")]
        public bool IsFeedbackActive()
        {
            return NativeCalls.godot_icall_0_3(method_bind_19, Object.GetPtr(this));
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_feedback_delay_ms");

        [GodotMethod("set_feedback_delay_ms")]
        public void SetFeedbackDelayMs(float amount)
        {
            NativeCalls.godot_icall_1_10(method_bind_20, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_feedback_delay_ms");

        [GodotMethod("get_feedback_delay_ms")]
        public float GetFeedbackDelayMs()
        {
            return NativeCalls.godot_icall_0_9(method_bind_21, Object.GetPtr(this));
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_feedback_level_db");

        [GodotMethod("set_feedback_level_db")]
        public void SetFeedbackLevelDb(float amount)
        {
            NativeCalls.godot_icall_1_10(method_bind_22, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_feedback_level_db");

        [GodotMethod("get_feedback_level_db")]
        public float GetFeedbackLevelDb()
        {
            return NativeCalls.godot_icall_0_9(method_bind_23, Object.GetPtr(this));
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_feedback_lowpass");

        [GodotMethod("set_feedback_lowpass")]
        public void SetFeedbackLowpass(float amount)
        {
            NativeCalls.godot_icall_1_10(method_bind_24, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_feedback_lowpass");

        [GodotMethod("get_feedback_lowpass")]
        public float GetFeedbackLowpass()
        {
            return NativeCalls.godot_icall_0_9(method_bind_25, Object.GetPtr(this));
        }
    }
}
