using System;

namespace Godot
{
    /// <summary>
    /// AudioEffectEQ gives you control over frequencies. Use it to compensate for existing deficiencies in audio. AudioEffectEQ are very useful on the Master Bus to completely master a mix and give it character. They are also very useful when a game is run on a mobile device, to adjust the mix to that kind of speakers (it can be added but disabled when headphones are plugged).
    /// </summary>
    public class AudioEffectEQ : AudioEffect
    {
        private const string nativeName = "AudioEffectEQ";

        public AudioEffectEQ() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_AudioEffectEQ_Ctor(this);
        }

        internal AudioEffectEQ(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_band_gain_db");

        /// <summary>
        /// Sets band's gain at the specified index, in dB.
        /// </summary>
        [GodotMethod("set_band_gain_db")]
        public void SetBandGainDb(int bandIdx, float volumeDb)
        {
            NativeCalls.godot_icall_2_28(method_bind_0, Object.GetPtr(this), bandIdx, volumeDb);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_band_gain_db");

        /// <summary>
        /// Returns the band's gain at the specified index, in dB.
        /// </summary>
        [GodotMethod("get_band_gain_db")]
        public float GetBandGainDb(int bandIdx)
        {
            return NativeCalls.godot_icall_1_7(method_bind_1, Object.GetPtr(this), bandIdx);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_band_count");

        /// <summary>
        /// Returns the number of bands of the equalizer.
        /// </summary>
        [GodotMethod("get_band_count")]
        public int GetBandCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_2, Object.GetPtr(this));
        }
    }
}
