using System;

namespace Godot
{
    /// <summary>
    /// Allows frequencies other than the [member cutoff_hz] to pass.
    /// </summary>
    public class AudioEffectFilter : AudioEffect
    {
        public enum FilterDB
        {
            Filter6db = 0,
            Filter12db = 1,
            Filter18db = 2,
            Filter24db = 3
        }


        /// <summary>
        /// Threshold frequency for the filter.
        /// </summary>
        public float CutoffHz
        {
            get
            {
                return GetCutoff();
            }
            set
            {
                SetCutoff(value);
            }
        }

        /// <summary>
        /// Amount of boost in the overtones near the cutoff frequency.
        /// </summary>
        public float Resonance
        {
            get
            {
                return GetResonance();
            }
            set
            {
                SetResonance(value);
            }
        }

        /// <summary>
        /// Gain amount of the frequencies after the filter.
        /// </summary>
        public float Gain
        {
            get
            {
                return GetGain();
            }
            set
            {
                SetGain(value);
            }
        }

        /// <summary>
        /// </summary>
        public AudioEffectFilter.FilterDB Db
        {
            get
            {
                return GetDb();
            }
            set
            {
                SetDb(value);
            }
        }


        private const string nativeName = "AudioEffectFilter";

        public AudioEffectFilter() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_AudioEffectFilter_Ctor(this);
        }

        internal AudioEffectFilter(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_cutoff");

        [GodotMethod("set_cutoff")]
        public void SetCutoff(float freq)
        {
            NativeCalls.godot_icall_1_10(method_bind_0, Object.GetPtr(this), freq);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_cutoff");

        [GodotMethod("get_cutoff")]
        public float GetCutoff()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_resonance");

        [GodotMethod("set_resonance")]
        public void SetResonance(float amount)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_resonance");

        [GodotMethod("get_resonance")]
        public float GetResonance()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_gain");

        [GodotMethod("set_gain")]
        public void SetGain(float amount)
        {
            NativeCalls.godot_icall_1_10(method_bind_4, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_gain");

        [GodotMethod("get_gain")]
        public float GetGain()
        {
            return NativeCalls.godot_icall_0_9(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_db");

        [GodotMethod("set_db")]
        public void SetDb(AudioEffectFilter.FilterDB amount)
        {
            NativeCalls.godot_icall_1_124(method_bind_6, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_db");

        [GodotMethod("get_db")]
        public AudioEffectFilter.FilterDB GetDb()
        {
            return NativeCalls.godot_icall_0_125(method_bind_7, Object.GetPtr(this));
        }
    }
}
