using System;

namespace Godot
{
    /// <summary>
    /// Determines how much of an audio signal is sent to the left and right buses.
    /// </summary>
    public class AudioEffectPanner : AudioEffect
    {
        /// <summary>
        /// Pan position. Value can range from -1 (fully left) to 1 (fully right).
        /// </summary>
        public float Pan
        {
            get
            {
                return GetPan();
            }
            set
            {
                SetPan(value);
            }
        }


        private const string nativeName = "AudioEffectPanner";

        public AudioEffectPanner() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_AudioEffectPanner_Ctor(this);
        }

        internal AudioEffectPanner(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_pan");

        [GodotMethod("set_pan")]
        public void SetPan(float cpanume)
        {
            NativeCalls.godot_icall_1_10(method_bind_0, Object.GetPtr(this), cpanume);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_pan");

        [GodotMethod("get_pan")]
        public float GetPan()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }
    }
}
