using System;

namespace Godot
{
    /// <summary>
    /// Allows modulation of pitch independently of tempo. All frequencies can be increased/decreased with minimal effect on transients.
    /// </summary>
    public class AudioEffectPitchShift : AudioEffect
    {
        /// <summary>
        /// Pitch value. Can range from 0 (-1 octave) to 16 (+16 octaves).
        /// </summary>
        public float PitchScale
        {
            get
            {
                return GetPitchScale();
            }
            set
            {
                SetPitchScale(value);
            }
        }


        private const string nativeName = "AudioEffectPitchShift";

        public AudioEffectPitchShift() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_AudioEffectPitchShift_Ctor(this);
        }

        internal AudioEffectPitchShift(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_pitch_scale");

        [GodotMethod("set_pitch_scale")]
        public void SetPitchScale(float rate)
        {
            NativeCalls.godot_icall_1_10(method_bind_0, Object.GetPtr(this), rate);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_pitch_scale");

        [GodotMethod("get_pitch_scale")]
        public float GetPitchScale()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }
    }
}
