using System;

namespace Godot
{
    /// <summary>
    /// Simulates rooms of different sizes. Its parameters can be adjusted to simulate the sound of a specific room.
    /// </summary>
    public class AudioEffectReverb : AudioEffect
    {
        /// <summary>
        /// Time between the original signal and the early reflections of the reverb signal. Default value: [code]150ms[/code].
        /// </summary>
        public float PredelayMsec
        {
            get
            {
                return GetPredelayMsec();
            }
            set
            {
                SetPredelayMsec(value);
            }
        }

        /// <summary>
        /// Output percent of predelay. Value can range from 0 to 1. Default value: [code]1[/code].
        /// </summary>
        public float PredelayFeedback
        {
            get
            {
                return GetPredelayFeedback();
            }
            set
            {
                SetPredelayFeedback(value);
            }
        }

        /// <summary>
        /// Dimensions of simulated room. Bigger means more echoes. Value can range from 0 to 1. Default value: [code]0.8[/code].
        /// </summary>
        public float RoomSize
        {
            get
            {
                return GetRoomSize();
            }
            set
            {
                SetRoomSize(value);
            }
        }

        /// <summary>
        /// Widens or narrows the stereo image of the reverb tail. 1 means fully widens. Value can range from 0 to 1. Default value: [code]1[/code].
        /// </summary>
        public float Damping
        {
            get
            {
                return GetDamping();
            }
            set
            {
                SetDamping(value);
            }
        }

        /// <summary>
        /// Defines how reflective the imaginary room's walls are. Value can range from 0 to 1. Default value: [code]1[/code].
        /// </summary>
        public float Spread
        {
            get
            {
                return GetSpread();
            }
            set
            {
                SetSpread(value);
            }
        }

        /// <summary>
        /// High-pass filter passes signals with a frequency higher than a certain cutoff frequency and attenuates signals with frequencies lower than the cutoff frequency. Value can range from 0 to 1. Default value: [code]0[/code].
        /// </summary>
        public float Hipass
        {
            get
            {
                return GetHpf();
            }
            set
            {
                SetHpf(value);
            }
        }

        /// <summary>
        /// Output percent of original sound. At 0, only modified sound is outputted. Value can range from 0 to 1. Default value: [code]1[/code].
        /// </summary>
        public float Dry
        {
            get
            {
                return GetDry();
            }
            set
            {
                SetDry(value);
            }
        }

        /// <summary>
        /// Output percent of modified sound. At 0, only original sound is outputted. Value can range from 0 to 1. Default value: [code]0.5[/code].
        /// </summary>
        public float Wet
        {
            get
            {
                return GetWet();
            }
            set
            {
                SetWet(value);
            }
        }


        private const string nativeName = "AudioEffectReverb";

        public AudioEffectReverb() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_AudioEffectReverb_Ctor(this);
        }

        internal AudioEffectReverb(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_predelay_msec");

        [GodotMethod("set_predelay_msec")]
        public void SetPredelayMsec(float msec)
        {
            NativeCalls.godot_icall_1_10(method_bind_0, Object.GetPtr(this), msec);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_predelay_msec");

        [GodotMethod("get_predelay_msec")]
        public float GetPredelayMsec()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_predelay_feedback");

        [GodotMethod("set_predelay_feedback")]
        public void SetPredelayFeedback(float feedback)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), feedback);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_predelay_feedback");

        [GodotMethod("get_predelay_feedback")]
        public float GetPredelayFeedback()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_room_size");

        [GodotMethod("set_room_size")]
        public void SetRoomSize(float size)
        {
            NativeCalls.godot_icall_1_10(method_bind_4, Object.GetPtr(this), size);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_room_size");

        [GodotMethod("get_room_size")]
        public float GetRoomSize()
        {
            return NativeCalls.godot_icall_0_9(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_damping");

        [GodotMethod("set_damping")]
        public void SetDamping(float amount)
        {
            NativeCalls.godot_icall_1_10(method_bind_6, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_damping");

        [GodotMethod("get_damping")]
        public float GetDamping()
        {
            return NativeCalls.godot_icall_0_9(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_spread");

        [GodotMethod("set_spread")]
        public void SetSpread(float amount)
        {
            NativeCalls.godot_icall_1_10(method_bind_8, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_spread");

        [GodotMethod("get_spread")]
        public float GetSpread()
        {
            return NativeCalls.godot_icall_0_9(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_dry");

        [GodotMethod("set_dry")]
        public void SetDry(float amount)
        {
            NativeCalls.godot_icall_1_10(method_bind_10, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_dry");

        [GodotMethod("get_dry")]
        public float GetDry()
        {
            return NativeCalls.godot_icall_0_9(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_wet");

        [GodotMethod("set_wet")]
        public void SetWet(float amount)
        {
            NativeCalls.godot_icall_1_10(method_bind_12, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_wet");

        [GodotMethod("get_wet")]
        public float GetWet()
        {
            return NativeCalls.godot_icall_0_9(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_hpf");

        [GodotMethod("set_hpf")]
        public void SetHpf(float amount)
        {
            NativeCalls.godot_icall_1_10(method_bind_14, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_hpf");

        [GodotMethod("get_hpf")]
        public float GetHpf()
        {
            return NativeCalls.godot_icall_0_9(method_bind_15, Object.GetPtr(this));
        }
    }
}
