using System;

namespace Godot
{
    /// <summary>
    /// AudioServer is a low level server interface for audio access. It is in charge of creating sample data (playable audio) as well as its playback via a voice interface.
    /// </summary>
    public static class AudioServer
    {
        public enum SpeakerMode
        {
            ModeStereo = 0,
            Surround51 = 2,
            Surround71 = 3
        }


        private const string nativeName = "AudioServer";
        internal static IntPtr ptr = NativeCalls.godot_icall_AudioServer_get_singleton();

        private static IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bus_count");

        /// <summary>
        /// Adds and removes busses to make the number of busses match [code]amount[/code].
        /// </summary>
        [GodotMethod("set_bus_count")]
        public static void SetBusCount(int amount)
        {
            NativeCalls.godot_icall_1_0(method_bind_0, ptr, amount);
        }

        private static IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bus_count");

        /// <summary>
        /// Returns the number of available busses.
        /// </summary>
        [GodotMethod("get_bus_count")]
        public static int GetBusCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_1, ptr);
        }

        private static IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_bus");

        /// <summary>
        /// Removes the bus at index [code]index[/code].
        /// </summary>
        [GodotMethod("remove_bus")]
        public static void RemoveBus(int index)
        {
            NativeCalls.godot_icall_1_0(method_bind_2, ptr, index);
        }

        private static IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_bus");

        /// <summary>
        /// Adds a bus at [code]at_position[/code].
        /// </summary>
        [GodotMethod("add_bus")]
        public static void AddBus(int atPosition = -1)
        {
            NativeCalls.godot_icall_1_0(method_bind_3, ptr, atPosition);
        }

        private static IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "move_bus");

        /// <summary>
        /// Moves the bus from index [code]index[/code] to index [code]to_index[/code].
        /// </summary>
        [GodotMethod("move_bus")]
        public static void MoveBus(int index, int toIndex)
        {
            NativeCalls.godot_icall_2_32(method_bind_4, ptr, index, toIndex);
        }

        private static IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bus_name");

        /// <summary>
        /// Sets the name of the bus at index [code]bus_idx[/code] to [code]name[/code].
        /// </summary>
        [GodotMethod("set_bus_name")]
        public static void SetBusName(int busIdx, string name)
        {
            NativeCalls.godot_icall_2_115(method_bind_5, ptr, busIdx, name);
        }

        private static IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bus_name");

        /// <summary>
        /// Returns the name of the bus with the index [code]bus_idx[/code].
        /// </summary>
        [GodotMethod("get_bus_name")]
        public static string GetBusName(int busIdx)
        {
            return NativeCalls.godot_icall_1_108(method_bind_6, ptr, busIdx);
        }

        private static IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bus_index");

        /// <summary>
        /// Returns the index of the bus with the name [code]bus_name[/code].
        /// </summary>
        [GodotMethod("get_bus_index")]
        public static int GetBusIndex(string busName)
        {
            return NativeCalls.godot_icall_1_87(method_bind_7, ptr, busName);
        }

        private static IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bus_volume_db");

        /// <summary>
        /// Sets the volume of the bus at index [code]bus_idx[/code] to [code]volume_db[/code].
        /// </summary>
        [GodotMethod("set_bus_volume_db")]
        public static void SetBusVolumeDb(int busIdx, float volumeDb)
        {
            NativeCalls.godot_icall_2_28(method_bind_8, ptr, busIdx, volumeDb);
        }

        private static IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bus_volume_db");

        /// <summary>
        /// Returns the volume of the bus at index [code]bus_idx[/code] in dB.
        /// </summary>
        [GodotMethod("get_bus_volume_db")]
        public static float GetBusVolumeDb(int busIdx)
        {
            return NativeCalls.godot_icall_1_7(method_bind_9, ptr, busIdx);
        }

        private static IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bus_send");

        /// <summary>
        /// Connects the output of the bus at [code]bus_idx[/code] to the bus named [code]send[/code].
        /// </summary>
        [GodotMethod("set_bus_send")]
        public static void SetBusSend(int busIdx, string send)
        {
            NativeCalls.godot_icall_2_115(method_bind_10, ptr, busIdx, send);
        }

        private static IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bus_send");

        /// <summary>
        /// Returns the name of the bus that the bus at index [code]bus_idx[/code] sends to.
        /// </summary>
        [GodotMethod("get_bus_send")]
        public static string GetBusSend(int busIdx)
        {
            return NativeCalls.godot_icall_1_108(method_bind_11, ptr, busIdx);
        }

        private static IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bus_solo");

        /// <summary>
        /// If [code]true[/code] the bus at index [code]bus_idx[/code] is in solo mode.
        /// </summary>
        [GodotMethod("set_bus_solo")]
        public static void SetBusSolo(int busIdx, bool enable)
        {
            NativeCalls.godot_icall_2_51(method_bind_12, ptr, busIdx, enable);
        }

        private static IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_bus_solo");

        /// <summary>
        /// If [code]true[/code] the bus at index [code]bus_idx[/code] is in solo mode.
        /// </summary>
        [GodotMethod("is_bus_solo")]
        public static bool IsBusSolo(int busIdx)
        {
            return NativeCalls.godot_icall_1_29(method_bind_13, ptr, busIdx);
        }

        private static IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bus_mute");

        /// <summary>
        /// If [code]true[/code] the bus at index [code]bus_idx[/code] is muted.
        /// </summary>
        [GodotMethod("set_bus_mute")]
        public static void SetBusMute(int busIdx, bool enable)
        {
            NativeCalls.godot_icall_2_51(method_bind_14, ptr, busIdx, enable);
        }

        private static IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_bus_mute");

        /// <summary>
        /// If [code]true[/code] the bus at index [code]bus_idx[/code] is muted.
        /// </summary>
        [GodotMethod("is_bus_mute")]
        public static bool IsBusMute(int busIdx)
        {
            return NativeCalls.godot_icall_1_29(method_bind_15, ptr, busIdx);
        }

        private static IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bus_bypass_effects");

        /// <summary>
        /// If [code]true[/code] the bus at index [code]bus_idx[/code] is bypassing effects.
        /// </summary>
        [GodotMethod("set_bus_bypass_effects")]
        public static void SetBusBypassEffects(int busIdx, bool enable)
        {
            NativeCalls.godot_icall_2_51(method_bind_16, ptr, busIdx, enable);
        }

        private static IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_bus_bypassing_effects");

        /// <summary>
        /// If [code]true[/code] the bus at index [code]bus_idx[/code] is bypassing effects.
        /// </summary>
        [GodotMethod("is_bus_bypassing_effects")]
        public static bool IsBusBypassingEffects(int busIdx)
        {
            return NativeCalls.godot_icall_1_29(method_bind_17, ptr, busIdx);
        }

        private static IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_bus_effect");

        /// <summary>
        /// Adds an [AudioEffect] effect to the bus [code]bus_idx[/code] at [code]at_position[/code].
        /// </summary>
        [GodotMethod("add_bus_effect")]
        public static void AddBusEffect(int busIdx, AudioEffect effect, int atPosition = -1)
        {
            NativeCalls.godot_icall_3_126(method_bind_18, ptr, busIdx, Object.GetPtr(effect), atPosition);
        }

        private static IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_bus_effect");

        /// <summary>
        /// Removes the effect at index [code]effect_idx[/code] from the bus at index [code]bus_idx[/code].
        /// </summary>
        [GodotMethod("remove_bus_effect")]
        public static void RemoveBusEffect(int busIdx, int effectIdx)
        {
            NativeCalls.godot_icall_2_32(method_bind_19, ptr, busIdx, effectIdx);
        }

        private static IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bus_effect_count");

        /// <summary>
        /// Returns the number of effects on the bus at [code]bus_idx[/code].
        /// </summary>
        [GodotMethod("get_bus_effect_count")]
        public static int GetBusEffectCount(int busIdx)
        {
            return NativeCalls.godot_icall_1_6(method_bind_20, ptr, busIdx);
        }

        private static IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bus_effect");

        /// <summary>
        /// Returns the [AudioEffect] at position [code]effect_idx[/code] in bus [code]bus_idx[/code].
        /// </summary>
        [GodotMethod("get_bus_effect")]
        public static AudioEffect GetBusEffect(int busIdx, int effectIdx)
        {
            return NativeCalls.godot_icall_2_127(method_bind_21, ptr, busIdx, effectIdx);
        }

        private static IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "swap_bus_effects");

        /// <summary>
        /// Swaps the position of two effects in bus [code]bus_idx[/code].
        /// </summary>
        [GodotMethod("swap_bus_effects")]
        public static void SwapBusEffects(int busIdx, int effectIdx, int byEffectIdx)
        {
            NativeCalls.godot_icall_3_128(method_bind_22, ptr, busIdx, effectIdx, byEffectIdx);
        }

        private static IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bus_effect_enabled");

        /// <summary>
        /// If [code]true[/code] the effect at index [code]effect_idx[/code] on the bus at index [code]bus_idx[/code] is enabled.
        /// </summary>
        [GodotMethod("set_bus_effect_enabled")]
        public static void SetBusEffectEnabled(int busIdx, int effectIdx, bool enabled)
        {
            NativeCalls.godot_icall_3_31(method_bind_23, ptr, busIdx, effectIdx, enabled);
        }

        private static IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_bus_effect_enabled");

        /// <summary>
        /// If [code]true[/code] the effect at index [code]effect_idx[/code] on the bus at index [code]bus_idx[/code] is enabled.
        /// </summary>
        [GodotMethod("is_bus_effect_enabled")]
        public static bool IsBusEffectEnabled(int busIdx, int effectIdx)
        {
            return NativeCalls.godot_icall_2_33(method_bind_24, ptr, busIdx, effectIdx);
        }

        private static IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bus_peak_volume_left_db");

        /// <summary>
        /// Returns the peak volume of the left speaker at bus index [code]bus_idx[/code] and channel index [code]channel[/code].
        /// </summary>
        [GodotMethod("get_bus_peak_volume_left_db")]
        public static float GetBusPeakVolumeLeftDb(int busIdx, int channel)
        {
            return NativeCalls.godot_icall_2_56(method_bind_25, ptr, busIdx, channel);
        }

        private static IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bus_peak_volume_right_db");

        /// <summary>
        /// Returns the peak volume of the right speaker at bus index [code]bus_idx[/code] and channel index [code]channel[/code].
        /// </summary>
        [GodotMethod("get_bus_peak_volume_right_db")]
        public static float GetBusPeakVolumeRightDb(int busIdx, int channel)
        {
            return NativeCalls.godot_icall_2_56(method_bind_26, ptr, busIdx, channel);
        }

        private static IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "lock");

        /// <summary>
        /// Locks the audio drivers mainloop. Remember to unlock it afterwards.
        /// </summary>
        [GodotMethod("lock")]
        public static void Lock()
        {
            NativeCalls.godot_icall_0_12(method_bind_27, ptr);
        }

        private static IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "unlock");

        /// <summary>
        /// Unlocks the audiodriver's main loop. After locking it always unlock it.
        /// </summary>
        [GodotMethod("unlock")]
        public static void Unlock()
        {
            NativeCalls.godot_icall_0_12(method_bind_28, ptr);
        }

        private static IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_speaker_mode");

        /// <summary>
        /// Returns the speaker configuration.
        /// </summary>
        [GodotMethod("get_speaker_mode")]
        public static AudioServer.SpeakerMode GetSpeakerMode()
        {
            return NativeCalls.godot_icall_0_129(method_bind_29, ptr);
        }

        private static IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_mix_rate");

        /// <summary>
        /// Returns the sample rate at the output of the audioserver.
        /// </summary>
        [GodotMethod("get_mix_rate")]
        public static float GetMixRate()
        {
            return NativeCalls.godot_icall_0_9(method_bind_30, ptr);
        }

        private static IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bus_layout");

        /// <summary>
        /// Overwrites the currently used [AudioBusLayout].
        /// </summary>
        [GodotMethod("set_bus_layout")]
        public static void SetBusLayout(AudioBusLayout busLayout)
        {
            NativeCalls.godot_icall_1_24(method_bind_31, ptr, Object.GetPtr(busLayout));
        }

        private static IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "generate_bus_layout");

        /// <summary>
        /// Generates an [AudioBusLayout] using the available busses and effects.
        /// </summary>
        [GodotMethod("generate_bus_layout")]
        public static AudioBusLayout GenerateBusLayout()
        {
            return NativeCalls.godot_icall_0_130(method_bind_32, ptr);
        }
    }
}
