using System;

namespace Godot
{
    /// <summary>
    /// Randomly varies pitch on each start.
    /// </summary>
    public class AudioStreamRandomPitch : AudioStream
    {
        /// <summary>
        /// The current [AudioStream].
        /// </summary>
        public AudioStream AudioStream
        {
            get
            {
                return GetAudioStream();
            }
            set
            {
                SetAudioStream(value);
            }
        }

        /// <summary>
        /// The intensity of random pitch variation.
        /// </summary>
        public float RandomPitch
        {
            get
            {
                return GetRandomPitch();
            }
            set
            {
                SetRandomPitch(value);
            }
        }


        private const string nativeName = "AudioStreamRandomPitch";

        public AudioStreamRandomPitch() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_AudioStreamRandomPitch_Ctor(this);
        }

        internal AudioStreamRandomPitch(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_audio_stream");

        [GodotMethod("set_audio_stream")]
        public void SetAudioStream(AudioStream stream)
        {
            NativeCalls.godot_icall_1_24(method_bind_0, Object.GetPtr(this), Object.GetPtr(stream));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_audio_stream");

        [GodotMethod("get_audio_stream")]
        public AudioStream GetAudioStream()
        {
            return NativeCalls.godot_icall_0_133(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_random_pitch");

        [GodotMethod("set_random_pitch")]
        public void SetRandomPitch(float scale)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), scale);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_random_pitch");

        [GodotMethod("get_random_pitch")]
        public float GetRandomPitch()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }
    }
}
