using System;

namespace Godot
{
    /// <summary>
    /// Plays audio, can loop.
    /// </summary>
    public class AudioStreamSample : AudioStream
    {
        public enum LoopModeEnum
        {
            Disabled = 0,
            Forward = 1,
            PingPong = 2
        }

        public enum FormatEnum
        {
            Format8Bits = 0,
            Format16Bits = 1,
            ImaAdpcm = 2
        }


        /// <summary>
        /// Audio format. See FORMAT_* constants for values.
        /// </summary>
        public AudioStreamSample.FormatEnum Format
        {
            get
            {
                return GetFormat();
            }
            set
            {
                SetFormat(value);
            }
        }

        /// <summary>
        /// Loop mode. See LOOP_* constants for values.
        /// </summary>
        public AudioStreamSample.LoopModeEnum LoopMode
        {
            get
            {
                return GetLoopMode();
            }
            set
            {
                SetLoopMode(value);
            }
        }

        /// <summary>
        /// Loop start in bytes.
        /// </summary>
        public int LoopBegin
        {
            get
            {
                return GetLoopBegin();
            }
            set
            {
                SetLoopBegin(value);
            }
        }

        /// <summary>
        /// Loop end in bytes.
        /// </summary>
        public int LoopEnd
        {
            get
            {
                return GetLoopEnd();
            }
            set
            {
                SetLoopEnd(value);
            }
        }

        /// <summary>
        /// The sample rate for mixing this audio.
        /// </summary>
        public int MixRate
        {
            get
            {
                return GetMixRate();
            }
            set
            {
                SetMixRate(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], audio is stereo. Default value: [code]false[/code].
        /// </summary>
        public bool Stereo
        {
            get
            {
                return IsStereo();
            }
            set
            {
                SetStereo(value);
            }
        }

        public byte[] Data
        {
            get
            {
                return _GetData();
            }
            set
            {
                _SetData(value);
            }
        }


        private const string nativeName = "AudioStreamSample";

        public AudioStreamSample() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_AudioStreamSample_Ctor(this);
        }

        internal AudioStreamSample(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_format");

        [GodotMethod("set_format")]
        public void SetFormat(AudioStreamSample.FormatEnum format)
        {
            NativeCalls.godot_icall_1_142(method_bind_0, Object.GetPtr(this), format);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_format");

        [GodotMethod("get_format")]
        public AudioStreamSample.FormatEnum GetFormat()
        {
            return NativeCalls.godot_icall_0_143(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_loop_mode");

        [GodotMethod("set_loop_mode")]
        public void SetLoopMode(AudioStreamSample.LoopModeEnum loopMode)
        {
            NativeCalls.godot_icall_1_144(method_bind_2, Object.GetPtr(this), loopMode);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_loop_mode");

        [GodotMethod("get_loop_mode")]
        public AudioStreamSample.LoopModeEnum GetLoopMode()
        {
            return NativeCalls.godot_icall_0_145(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_loop_begin");

        [GodotMethod("set_loop_begin")]
        public void SetLoopBegin(int loopBegin)
        {
            NativeCalls.godot_icall_1_0(method_bind_4, Object.GetPtr(this), loopBegin);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_loop_begin");

        [GodotMethod("get_loop_begin")]
        public int GetLoopBegin()
        {
            return NativeCalls.godot_icall_0_1(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_loop_end");

        [GodotMethod("set_loop_end")]
        public void SetLoopEnd(int loopEnd)
        {
            NativeCalls.godot_icall_1_0(method_bind_6, Object.GetPtr(this), loopEnd);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_loop_end");

        [GodotMethod("get_loop_end")]
        public int GetLoopEnd()
        {
            return NativeCalls.godot_icall_0_1(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_mix_rate");

        [GodotMethod("set_mix_rate")]
        public void SetMixRate(int mixRate)
        {
            NativeCalls.godot_icall_1_0(method_bind_8, Object.GetPtr(this), mixRate);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_mix_rate");

        [GodotMethod("get_mix_rate")]
        public int GetMixRate()
        {
            return NativeCalls.godot_icall_0_1(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_stereo");

        [GodotMethod("set_stereo")]
        public void SetStereo(bool stereo)
        {
            NativeCalls.godot_icall_1_11(method_bind_10, Object.GetPtr(this), stereo);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_stereo");

        [GodotMethod("is_stereo")]
        public bool IsStereo()
        {
            return NativeCalls.godot_icall_0_3(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_data");

        internal void _SetData(byte[] data)
        {
            NativeCalls.godot_icall_1_131(method_bind_12, Object.GetPtr(this), data);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_data");

        internal byte[] _GetData()
        {
            return NativeCalls.godot_icall_0_132(method_bind_13, Object.GetPtr(this));
        }
    }
}
