using System;

namespace Godot
{
    /// <summary>
    /// Node for back-buffering the currently displayed screen. The region defined in the BackBufferCopy node is bufferized with the content of the screen it covers, or the entire screen according to the copy mode set. Accessing this buffer is done with the texscreen() shader instruction.
    /// </summary>
    public class BackBufferCopy : Node2D
    {
        public enum CopyModeEnum
        {
            Disabled = 0,
            Rect = 1,
            Viewport = 2
        }


        /// <summary>
        /// Buffer mode. See [code]COPY_MODE_*[/code] constants.
        /// </summary>
        public BackBufferCopy.CopyModeEnum CopyMode
        {
            get
            {
                return GetCopyMode();
            }
            set
            {
                SetCopyMode(value);
            }
        }

        /// <summary>
        /// The area covered by the BackBufferCopy. Only used if [code]copy_mode[/code] is [code]COPY_MODE_RECT[/code].
        /// </summary>
        public Rect2 Rect
        {
            get
            {
                return GetRect();
            }
            set
            {
                SetRect(value);
            }
        }


        private const string nativeName = "BackBufferCopy";

        public BackBufferCopy() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_BackBufferCopy_Ctor(this);
        }

        internal BackBufferCopy(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_rect");

        [GodotMethod("set_rect")]
        public void SetRect(Rect2 rect)
        {
            NativeCalls.godot_icall_1_120(method_bind_0, Object.GetPtr(this), ref rect);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_rect");

        [GodotMethod("get_rect")]
        public Rect2 GetRect()
        {
            object ret = NativeCalls.godot_icall_0_121(method_bind_1, Object.GetPtr(this));
            return (Rect2)ret;
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_copy_mode");

        [GodotMethod("set_copy_mode")]
        public void SetCopyMode(BackBufferCopy.CopyModeEnum copyMode)
        {
            NativeCalls.godot_icall_1_146(method_bind_2, Object.GetPtr(this), copyMode);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_copy_mode");

        [GodotMethod("get_copy_mode")]
        public BackBufferCopy.CopyModeEnum GetCopyMode()
        {
            return NativeCalls.godot_icall_0_147(method_bind_3, Object.GetPtr(this));
        }
    }
}
