using System;
using System.Collections.Generic;

namespace Godot
{
    /// <summary>
    /// A two-dimensional array of boolean values, can be used to efficiently store a binary matrix (every matrix element takes only one bit) and query the values using natural cartesian coordinates.
    /// </summary>
    public class BitMap : Resource
    {
        public Dictionary<object, object> Data
        {
            get
            {
                return _GetData();
            }
            set
            {
                _SetData(value);
            }
        }

        private const string nativeName = "BitMap";

        public BitMap() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_BitMap_Ctor(this);
        }

        internal BitMap(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "create");

        /// <summary>
        /// Creates a bitmap with the specified size, filled with false.
        /// </summary>
        [GodotMethod("create")]
        public void Create(Vector2 size)
        {
            NativeCalls.godot_icall_1_45(method_bind_0, Object.GetPtr(this), ref size);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "create_from_image_alpha");

        /// <summary>
        /// Creates a bitmap that matches the given image dimensions, every element of the bitmap is set to false if the alpha value of the image at that position is 0, and true in other case.
        /// </summary>
        [GodotMethod("create_from_image_alpha")]
        public void CreateFromImageAlpha(Image image)
        {
            NativeCalls.godot_icall_1_24(method_bind_1, Object.GetPtr(this), Object.GetPtr(image));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bit");

        /// <summary>
        /// Sets the bitmap's element at the specified position, to the specified value.
        /// </summary>
        [GodotMethod("set_bit")]
        public void SetBit(Vector2 position, bool bit)
        {
            NativeCalls.godot_icall_2_165(method_bind_2, Object.GetPtr(this), ref position, bit);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bit");

        /// <summary>
        /// Returns bitmap's value at the specified position.
        /// </summary>
        [GodotMethod("get_bit")]
        public bool GetBit(Vector2 position)
        {
            return NativeCalls.godot_icall_1_166(method_bind_3, Object.GetPtr(this), ref position);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bit_rect");

        /// <summary>
        /// Sets a rectangular portion of the bitmap to the specified value.
        /// </summary>
        [GodotMethod("set_bit_rect")]
        public void SetBitRect(Rect2 pRect, bool bit)
        {
            NativeCalls.godot_icall_2_167(method_bind_4, Object.GetPtr(this), ref pRect, bit);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_true_bit_count");

        /// <summary>
        /// Returns the amount of bitmap elements that are set to true.
        /// </summary>
        [GodotMethod("get_true_bit_count")]
        public int GetTrueBitCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_size");

        /// <summary>
        /// Returns bitmap's dimensions.
        /// </summary>
        [GodotMethod("get_size")]
        public Vector2 GetSize()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_6, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_data");

        internal void _SetData(Dictionary<object, object> arg0)
        {
            NativeCalls.godot_icall_1_168(method_bind_7, Object.GetPtr(this), arg0);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_data");

        internal Dictionary<object, object> _GetData()
        {
            return NativeCalls.godot_icall_0_169(method_bind_8, Object.GetPtr(this));
        }
    }
}
