using System;

namespace Godot
{
    /// <summary>
    /// Arranges child controls vertically or horizontally, and rearranges the controls automatically when their minimum size changes.
    /// </summary>
    public class BoxContainer : Container
    {
        public enum AlignMode
        {
            Begin = 0,
            Center = 1,
            End = 2
        }


        /// <summary>
        /// The alignment of the container's children (must be one of ALIGN_BEGIN, ALIGN_CENTER, or ALIGN_END).
        /// </summary>
        public BoxContainer.AlignMode Alignment
        {
            get
            {
                return GetAlignment();
            }
            set
            {
                SetAlignment(value);
            }
        }


        private const string nativeName = "BoxContainer";

        internal BoxContainer() {}

        internal BoxContainer(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_spacer");

        /// <summary>
        /// Adds a control to the box as a spacer. If [code]true[/code], [i]begin[/i] will insert the spacer control in front of other children.
        /// </summary>
        [GodotMethod("add_spacer")]
        public void AddSpacer(bool begin)
        {
            NativeCalls.godot_icall_1_11(method_bind_0, Object.GetPtr(this), begin);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_alignment");

        [GodotMethod("get_alignment")]
        public BoxContainer.AlignMode GetAlignment()
        {
            return NativeCalls.godot_icall_0_177(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_alignment");

        [GodotMethod("set_alignment")]
        public void SetAlignment(BoxContainer.AlignMode alignment)
        {
            NativeCalls.godot_icall_1_178(method_bind_2, Object.GetPtr(this), alignment);
        }
    }
}
