using System;

namespace Godot
{
    /// <summary>
    /// [code]CanvasItemMaterial[/code]s provide a means of modifying the textures associated with a CanvasItem. They specialize in describing blend and lighting behaviors for textures. Use a [ShaderMaterial] to more fully customize a material's interactions with a [CanvasItem].
    /// </summary>
    public class CanvasItemMaterial : Material
    {
        public enum LightModeEnum
        {
            Normal = 0,
            Unshaded = 1,
            LightOnly = 2
        }

        public enum BlendModeEnum
        {
            Mix = 0,
            Add = 1,
            Sub = 2,
            Mul = 3,
            PremultAlpha = 4
        }


        /// <summary>
        /// The manner in which a material's rendering is applied to underlying textures.
        /// </summary>
        public CanvasItemMaterial.BlendModeEnum BlendMode
        {
            get
            {
                return GetBlendMode();
            }
            set
            {
                SetBlendMode(value);
            }
        }

        /// <summary>
        /// The manner in which material reacts to lighting.
        /// </summary>
        public CanvasItemMaterial.LightModeEnum LightMode
        {
            get
            {
                return GetLightMode();
            }
            set
            {
                SetLightMode(value);
            }
        }


        private const string nativeName = "CanvasItemMaterial";

        public CanvasItemMaterial() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_CanvasItemMaterial_Ctor(this);
        }

        internal CanvasItemMaterial(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_blend_mode");

        [GodotMethod("set_blend_mode")]
        public void SetBlendMode(CanvasItemMaterial.BlendModeEnum blendMode)
        {
            NativeCalls.godot_icall_1_225(method_bind_0, Object.GetPtr(this), blendMode);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_blend_mode");

        [GodotMethod("get_blend_mode")]
        public CanvasItemMaterial.BlendModeEnum GetBlendMode()
        {
            return NativeCalls.godot_icall_0_226(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_light_mode");

        [GodotMethod("set_light_mode")]
        public void SetLightMode(CanvasItemMaterial.LightModeEnum lightMode)
        {
            NativeCalls.godot_icall_1_227(method_bind_2, Object.GetPtr(this), lightMode);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_light_mode");

        [GodotMethod("get_light_mode")]
        public CanvasItemMaterial.LightModeEnum GetLightMode()
        {
            return NativeCalls.godot_icall_0_228(method_bind_3, Object.GetPtr(this));
        }
    }
}
