using System;

namespace Godot
{
    /// <summary>
    /// Class representing a capsule-shaped [PrimitiveMesh].
    /// </summary>
    public class CapsuleMesh : PrimitiveMesh
    {
        /// <summary>
        /// Radius of the capsule mesh. Defaults to 1.0.
        /// </summary>
        public float Radius
        {
            get
            {
                return GetRadius();
            }
            set
            {
                SetRadius(value);
            }
        }

        /// <summary>
        /// Height of the capsule mesh from the center point. Defaults to 1.0.
        /// </summary>
        public float MidHeight
        {
            get
            {
                return GetMidHeight();
            }
            set
            {
                SetMidHeight(value);
            }
        }

        /// <summary>
        /// Number of radial segments on the capsule mesh. Defaults to 64.
        /// </summary>
        public int RadialSegments
        {
            get
            {
                return GetRadialSegments();
            }
            set
            {
                SetRadialSegments(value);
            }
        }

        /// <summary>
        /// Number of rings along the height of the capsule. Defaults to 8.
        /// </summary>
        public int Rings
        {
            get
            {
                return GetRings();
            }
            set
            {
                SetRings(value);
            }
        }


        private const string nativeName = "CapsuleMesh";

        public CapsuleMesh() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_CapsuleMesh_Ctor(this);
        }

        internal CapsuleMesh(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_radius");

        [GodotMethod("set_radius")]
        public void SetRadius(float radius)
        {
            NativeCalls.godot_icall_1_10(method_bind_0, Object.GetPtr(this), radius);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_radius");

        [GodotMethod("get_radius")]
        public float GetRadius()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_mid_height");

        [GodotMethod("set_mid_height")]
        public void SetMidHeight(float midHeight)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), midHeight);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_mid_height");

        [GodotMethod("get_mid_height")]
        public float GetMidHeight()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_radial_segments");

        [GodotMethod("set_radial_segments")]
        public void SetRadialSegments(int segments)
        {
            NativeCalls.godot_icall_1_0(method_bind_4, Object.GetPtr(this), segments);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_radial_segments");

        [GodotMethod("get_radial_segments")]
        public int GetRadialSegments()
        {
            return NativeCalls.godot_icall_0_1(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_rings");

        [GodotMethod("set_rings")]
        public void SetRings(int rings)
        {
            NativeCalls.godot_icall_1_0(method_bind_6, Object.GetPtr(this), rings);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_rings");

        [GodotMethod("get_rings")]
        public int GetRings()
        {
            return NativeCalls.godot_icall_0_1(method_bind_7, Object.GetPtr(this));
        }
    }
}
