using System;

namespace Godot
{
    /// <summary>
    /// Capsule shape for 2D collisions.
    /// </summary>
    public class CapsuleShape2D : Shape2D
    {
        /// <summary>
        /// The capsules's radius.
        /// </summary>
        public float Radius
        {
            get
            {
                return GetRadius();
            }
            set
            {
                SetRadius(value);
            }
        }

        /// <summary>
        /// The capsule's height.
        /// </summary>
        public float Height
        {
            get
            {
                return GetHeight();
            }
            set
            {
                SetHeight(value);
            }
        }


        private const string nativeName = "CapsuleShape2D";

        public CapsuleShape2D() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_CapsuleShape2D_Ctor(this);
        }

        internal CapsuleShape2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_radius");

        [GodotMethod("set_radius")]
        public void SetRadius(float radius)
        {
            NativeCalls.godot_icall_1_10(method_bind_0, Object.GetPtr(this), radius);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_radius");

        [GodotMethod("get_radius")]
        public float GetRadius()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_height");

        [GodotMethod("set_height")]
        public void SetHeight(float height)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), height);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_height");

        [GodotMethod("get_height")]
        public float GetHeight()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }
    }
}
