using System;

namespace Godot
{
    /// <summary>
    /// CenterContainer Keeps children controls centered. This container keeps all children to their minimum size, in the center.
    /// </summary>
    public class CenterContainer : Container
    {
        /// <summary>
        /// If [code]true[/code] centers children relative to the [code]CenterContainer[/code]'s top left corner. Default value: [code]false[/code].
        /// </summary>
        public bool UseTopLeft
        {
            get
            {
                return IsUsingTopLeft();
            }
            set
            {
                SetUseTopLeft(value);
            }
        }


        private const string nativeName = "CenterContainer";

        public CenterContainer() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_CenterContainer_Ctor(this);
        }

        internal CenterContainer(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_use_top_left");

        [GodotMethod("set_use_top_left")]
        public void SetUseTopLeft(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_0, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_using_top_left");

        [GodotMethod("is_using_top_left")]
        public bool IsUsingTopLeft()
        {
            return NativeCalls.godot_icall_0_3(method_bind_1, Object.GetPtr(this));
        }
    }
}
