using System;

namespace Godot
{
    /// <summary>
    /// Circular shape for 2D collisions. This shape is useful for modeling balls or small characters and its collision detection with everything else is very fast.
    /// </summary>
    public class CircleShape2D : Shape2D
    {
        /// <summary>
        /// The circle's radius.
        /// </summary>
        public float Radius
        {
            get
            {
                return GetRadius();
            }
            set
            {
                SetRadius(value);
            }
        }


        private const string nativeName = "CircleShape2D";

        public CircleShape2D() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_CircleShape2D_Ctor(this);
        }

        internal CircleShape2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_radius");

        [GodotMethod("set_radius")]
        public void SetRadius(float radius)
        {
            NativeCalls.godot_icall_1_10(method_bind_0, Object.GetPtr(this), radius);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_radius");

        [GodotMethod("get_radius")]
        public float GetRadius()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }
    }
}
