using System;
using System.Collections.Generic;

namespace Godot
{
    /// <summary>
    /// Provides access to metadata stored for every available class.
    /// </summary>
    public static class ClassDB
    {
        private const string nativeName = "_ClassDB";
        internal static IntPtr ptr = NativeCalls.godot_icall__ClassDB_get_singleton();

        private static IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_class_list");

        /// <summary>
        /// Returns the names of all the classes available.
        /// </summary>
        [GodotMethod("get_class_list")]
        public static string[] GetClassList()
        {
            return NativeCalls.godot_icall_0_72(method_bind_0, ptr);
        }

        private static IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_inheriters_from_class");

        /// <summary>
        /// Returns the names of all the classes that directly or indirectly inherit from 'class'.
        /// </summary>
        [GodotMethod("get_inheriters_from_class")]
        public static string[] GetInheritersFromClass(string @class)
        {
            return NativeCalls.godot_icall_1_252(method_bind_1, ptr, @class);
        }

        private static IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_parent_class");

        /// <summary>
        /// Returns the parent class of 'class'.
        /// </summary>
        [GodotMethod("get_parent_class")]
        public static string GetParentClass(string @class)
        {
            return NativeCalls.godot_icall_1_73(method_bind_2, ptr, @class);
        }

        private static IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "class_exists");

        /// <summary>
        /// Returns whether the specified 'class' is available or not.
        /// </summary>
        [GodotMethod("class_exists")]
        public static bool ClassExists(string @class)
        {
            return NativeCalls.godot_icall_1_70(method_bind_3, ptr, @class);
        }

        private static IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_parent_class");

        /// <summary>
        /// Returns whether 'inherits' is an ancestor of 'class' or not.
        /// </summary>
        [GodotMethod("is_parent_class")]
        public static bool IsParentClass(string @class, string inherits)
        {
            return NativeCalls.godot_icall_2_251(method_bind_4, ptr, @class, inherits);
        }

        private static IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "can_instance");

        /// <summary>
        /// Returns true if you can instance objects from the specified 'class', false in other case.
        /// </summary>
        [GodotMethod("can_instance")]
        public static bool CanInstance(string @class)
        {
            return NativeCalls.godot_icall_1_70(method_bind_5, ptr, @class);
        }

        private static IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "instance");

        /// <summary>
        /// Creates an instance of 'class'.
        /// </summary>
        [GodotMethod("instance")]
        public static object Instance(string @class)
        {
            return NativeCalls.godot_icall_1_338(method_bind_6, ptr, @class);
        }

        private static IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "class_has_signal");

        /// <summary>
        /// Return whether 'class' or its ancestry has a signal called 'signal' or not.
        /// </summary>
        [GodotMethod("class_has_signal")]
        public static bool ClassHasSignal(string @class, string signal)
        {
            return NativeCalls.godot_icall_2_251(method_bind_7, ptr, @class, signal);
        }

        private static IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "class_get_signal");

        /// <summary>
        /// Returns the 'signal' data of 'class' or its ancestry. The returned value is a [Dictionary] with the following keys: args, default_args, flags, id, name, return: (class_name, hint, hint_string, name, type, usage).
        /// </summary>
        [GodotMethod("class_get_signal")]
        public static Dictionary<object, object> ClassGetSignal(string @class, string signal)
        {
            return NativeCalls.godot_icall_2_1029(method_bind_8, ptr, @class, signal);
        }

        private static IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "class_get_signal_list");

        /// <summary>
        /// Returns an array with all the signals of 'class' or its ancestry if 'no_inheritance' is false. Every element of the array is a [Dictionary] as described in [class_get_signal].
        /// </summary>
        [GodotMethod("class_get_signal_list")]
        public static object[] ClassGetSignalList(string @class, bool noInheritance = false)
        {
            return NativeCalls.godot_icall_2_1030(method_bind_9, ptr, @class, noInheritance);
        }

        private static IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "class_get_property_list");

        /// <summary>
        /// Returns an array with all the properties of 'class' or its ancestry if 'no_inheritance' is false.
        /// </summary>
        [GodotMethod("class_get_property_list")]
        public static object[] ClassGetPropertyList(string @class, bool noInheritance = false)
        {
            return NativeCalls.godot_icall_2_1030(method_bind_10, ptr, @class, noInheritance);
        }

        private static IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "class_get_property");

        /// <summary>
        /// Returns the value of 'property' of 'class' or its ancestry.
        /// </summary>
        [GodotMethod("class_get_property")]
        public static object ClassGetProperty(Object @object, string property)
        {
            return NativeCalls.godot_icall_2_1031(method_bind_11, ptr, Object.GetPtr(@object), property);
        }

        private static IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "class_set_property");

        /// <summary>
        /// Sets 'property' value of 'class' to 'value'.
        /// </summary>
        [GodotMethod("class_set_property")]
        public static Error ClassSetProperty(Object @object, string property, object value)
        {
            return NativeCalls.godot_icall_3_1032(method_bind_12, ptr, Object.GetPtr(@object), property, value);
        }

        private static IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "class_has_method");

        /// <summary>
        /// Return whether 'class' (or its ancestry if 'no_inheritance' is false) has a method called 'method' or not.
        /// </summary>
        [GodotMethod("class_has_method")]
        public static bool ClassHasMethod(string @class, string method, bool noInheritance = false)
        {
            return NativeCalls.godot_icall_3_1033(method_bind_13, ptr, @class, method, noInheritance);
        }

        private static IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "class_get_method_list");

        /// <summary>
        /// Returns an array with all the methods of 'class' or its ancestry if 'no_inheritance' is false. Every element of the array is a [Dictionary] with the following keys: args, default_args, flags, id, name, return: (class_name, hint, hint_string, name, type, usage).
        /// </summary>
        [GodotMethod("class_get_method_list")]
        public static object[] ClassGetMethodList(string @class, bool noInheritance = false)
        {
            return NativeCalls.godot_icall_2_1030(method_bind_14, ptr, @class, noInheritance);
        }

        private static IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "class_get_integer_constant_list");

        /// <summary>
        /// Returns an array with the names all the integer constants of 'class' or its ancestry.
        /// </summary>
        [GodotMethod("class_get_integer_constant_list")]
        public static string[] ClassGetIntegerConstantList(string @class, bool noInheritance = false)
        {
            return NativeCalls.godot_icall_2_1034(method_bind_15, ptr, @class, noInheritance);
        }

        private static IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "class_has_integer_constant");

        /// <summary>
        /// Return whether 'class' or its ancestry has an integer constant called 'name' or not.
        /// </summary>
        [GodotMethod("class_has_integer_constant")]
        public static bool ClassHasIntegerConstant(string @class, string name)
        {
            return NativeCalls.godot_icall_2_251(method_bind_16, ptr, @class, name);
        }

        private static IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "class_get_integer_constant");

        /// <summary>
        /// Returns the value of the integer constant 'name' of 'class' or its ancestry. Always returns 0 when the constant could not be found.
        /// </summary>
        [GodotMethod("class_get_integer_constant")]
        public static int ClassGetIntegerConstant(string @class, string name)
        {
            return NativeCalls.godot_icall_2_265(method_bind_17, ptr, @class, name);
        }

        private static IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "class_get_category");

        /// <summary>
        /// Returns a category associated with the class for use in documentation and the Asset Library. Debug mode required.
        /// </summary>
        [GodotMethod("class_get_category")]
        public static string ClassGetCategory(string @class)
        {
            return NativeCalls.godot_icall_1_73(method_bind_18, ptr, @class);
        }

        private static IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_class_enabled");

        /// <summary>
        /// Returns whether this class is enabled or not.
        /// </summary>
        [GodotMethod("is_class_enabled")]
        public static bool IsClassEnabled(string @class)
        {
            return NativeCalls.godot_icall_1_70(method_bind_19, ptr, @class);
        }
    }
}
