using System;

namespace Godot
{
    /// <summary>
    /// Provides a 2D collision polygon to a [CollisionObject2D] parent. Polygon can be drawn in the editor or specified by a list of vertices.
    /// </summary>
    public class CollisionPolygon2D : Node2D
    {
        public enum BuildModeEnum
        {
            olids = 0,
            egments = 1
        }


        /// <summary>
        /// Collision build mode. Use one of the [code]BUILD_*[/code] constants. Default value: [code]BUILD_SOLIDS[/code].
        /// </summary>
        public CollisionPolygon2D.BuildModeEnum BuildMode
        {
            get
            {
                return GetBuildMode();
            }
            set
            {
                SetBuildMode(value);
            }
        }

        /// <summary>
        /// The polygon's list of vertices. The final point will be connected to the first.
        /// </summary>
        public Vector2[] Polygon
        {
            get
            {
                return GetPolygon();
            }
            set
            {
                SetPolygon(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] no collisions will be detected.
        /// </summary>
        public bool Disabled
        {
            get
            {
                return IsDisabled();
            }
            set
            {
                SetDisabled(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] only edges that face up, relative to CollisionPolygon2D's rotation, will collide with other objects.
        /// </summary>
        public bool OneWayCollision
        {
            get
            {
                return IsOneWayCollisionEnabled();
            }
            set
            {
                SetOneWayCollision(value);
            }
        }


        private const string nativeName = "CollisionPolygon2D";

        public CollisionPolygon2D() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_CollisionPolygon2D_Ctor(this);
        }

        internal CollisionPolygon2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_polygon");

        [GodotMethod("set_polygon")]
        public void SetPolygon(Vector2[] polygon)
        {
            NativeCalls.godot_icall_1_237(method_bind_0, Object.GetPtr(this), polygon);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_polygon");

        [GodotMethod("get_polygon")]
        public Vector2[] GetPolygon()
        {
            return NativeCalls.godot_icall_0_238(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_build_mode");

        [GodotMethod("set_build_mode")]
        public void SetBuildMode(CollisionPolygon2D.BuildModeEnum buildMode)
        {
            NativeCalls.godot_icall_1_239(method_bind_2, Object.GetPtr(this), buildMode);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_build_mode");

        [GodotMethod("get_build_mode")]
        public CollisionPolygon2D.BuildModeEnum GetBuildMode()
        {
            return NativeCalls.godot_icall_0_240(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_disabled");

        [GodotMethod("set_disabled")]
        public void SetDisabled(bool disabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_4, Object.GetPtr(this), disabled);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_disabled");

        [GodotMethod("is_disabled")]
        public bool IsDisabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_one_way_collision");

        [GodotMethod("set_one_way_collision")]
        public void SetOneWayCollision(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_6, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_one_way_collision_enabled");

        [GodotMethod("is_one_way_collision_enabled")]
        public bool IsOneWayCollisionEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_7, Object.GetPtr(this));
        }
    }
}
