using System;

namespace Godot
{
    /// <summary>
    /// This is a simple color picker [Control]. It's useful for selecting a color from an RGB/RGBA colorspace.
    /// </summary>
    public class ColorPicker : BoxContainer
    {
        /// <summary>
        /// The currently selected color.
        /// </summary>
        public Color Color
        {
            get
            {
                return GetPickColor();
            }
            set
            {
                SetPickColor(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], shows an alpha channel slider (transparency).
        /// </summary>
        public bool EditAlpha
        {
            get
            {
                return IsEditingAlpha();
            }
            set
            {
                SetEditAlpha(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], allows the color R, G, B component values to go beyond 1.0, which can be used for certain special operations that require it (like tinting without darkening or rendering sprites in HDR).
        /// </summary>
        public bool RawMode
        {
            get
            {
                return IsRawMode();
            }
            set
            {
                SetRawMode(value);
            }
        }


        private const string nativeName = "ColorPicker";

        public ColorPicker() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_ColorPicker_Ctor(this);
        }

        internal ColorPicker(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_pick_color");

        [GodotMethod("set_pick_color")]
        public void SetPickColor(Color color)
        {
            NativeCalls.godot_icall_1_202(method_bind_0, Object.GetPtr(this), ref color);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_pick_color");

        [GodotMethod("get_pick_color")]
        public Color GetPickColor()
        {
            object ret = NativeCalls.godot_icall_0_203(method_bind_1, Object.GetPtr(this));
            return (Color)ret;
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_raw_mode");

        [GodotMethod("set_raw_mode")]
        public void SetRawMode(bool mode)
        {
            NativeCalls.godot_icall_1_11(method_bind_2, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_raw_mode");

        [GodotMethod("is_raw_mode")]
        public bool IsRawMode()
        {
            return NativeCalls.godot_icall_0_3(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_edit_alpha");

        [GodotMethod("set_edit_alpha")]
        public void SetEditAlpha(bool show)
        {
            NativeCalls.godot_icall_1_11(method_bind_4, Object.GetPtr(this), show);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_editing_alpha");

        [GodotMethod("is_editing_alpha")]
        public bool IsEditingAlpha()
        {
            return NativeCalls.godot_icall_0_3(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_preset");

        /// <summary>
        /// Adds the current selected to color to a list of colors (presets), the presets will be displayed in the color picker and the user will be able to select them, notice that the presets list is only for this color picker.
        /// </summary>
        [GodotMethod("add_preset")]
        public void AddPreset(Color color)
        {
            NativeCalls.godot_icall_1_202(method_bind_6, Object.GetPtr(this), ref color);
        }
    }
}
