using System;

namespace Godot
{
    /// <summary>
    /// Encapsulates a [ColorPicker] making it accesible by pressing a button, pressing the button will toggle the [ColorPicker] visibility
    /// </summary>
    public class ColorPickerButton : Button
    {
        /// <summary>
        /// The currently selected color.
        /// </summary>
        public Color Color
        {
            get
            {
                return GetPickColor();
            }
            set
            {
                SetPickColor(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the alpha channel in the displayed [ColorPicker] will be visible. Default value: [code]true[/code].
        /// </summary>
        public bool EditAlpha
        {
            get
            {
                return IsEditingAlpha();
            }
            set
            {
                SetEditAlpha(value);
            }
        }


        private const string nativeName = "ColorPickerButton";

        public ColorPickerButton() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_ColorPickerButton_Ctor(this);
        }

        internal ColorPickerButton(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_pick_color");

        [GodotMethod("set_pick_color")]
        public void SetPickColor(Color color)
        {
            NativeCalls.godot_icall_1_202(method_bind_0, Object.GetPtr(this), ref color);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_pick_color");

        [GodotMethod("get_pick_color")]
        public Color GetPickColor()
        {
            object ret = NativeCalls.godot_icall_0_203(method_bind_1, Object.GetPtr(this));
            return (Color)ret;
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_picker");

        /// <summary>
        /// Returns the [code]ColorPicker[/code] that this [code]ColorPickerButton[/code] toggles.
        /// </summary>
        [GodotMethod("get_picker")]
        public ColorPicker GetPicker()
        {
            return NativeCalls.godot_icall_0_243(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_popup");

        /// <summary>
        /// Returns the control's [PopupPanel] which allows you to connect to Popup Signals. This allows you to handle events when the ColorPicker is shown or hidden.
        /// </summary>
        [GodotMethod("get_popup")]
        public PopupPanel GetPopup()
        {
            return NativeCalls.godot_icall_0_244(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_edit_alpha");

        [GodotMethod("set_edit_alpha")]
        public void SetEditAlpha(bool show)
        {
            NativeCalls.godot_icall_1_11(method_bind_4, Object.GetPtr(this), show);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_editing_alpha");

        [GodotMethod("is_editing_alpha")]
        public bool IsEditingAlpha()
        {
            return NativeCalls.godot_icall_0_3(method_bind_5, Object.GetPtr(this));
        }
    }
}
