using System;

namespace Godot
{
    /// <summary>
    /// An object that is represented on the canvas as a rect with color. [Color] is used to set or get color info for the rect.
    /// </summary>
    public class ColorRect : Control
    {
        /// <summary>
        /// The color to fill the [code]ColorRect[/code].
        /// [codeblock]
        /// $ColorRect.color = Color(1, 0, 0, 1) # Set ColorRect node's color to red
        /// [/codeblock]
        /// </summary>
        public Color Color
        {
            get
            {
                return GetFrameColor();
            }
            set
            {
                SetFrameColor(value);
            }
        }


        private const string nativeName = "ColorRect";

        public ColorRect() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_ColorRect_Ctor(this);
        }

        internal ColorRect(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_frame_color");

        [GodotMethod("set_frame_color")]
        public void SetFrameColor(Color color)
        {
            NativeCalls.godot_icall_1_202(method_bind_0, Object.GetPtr(this), ref color);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_frame_color");

        [GodotMethod("get_frame_color")]
        public Color GetFrameColor()
        {
            object ret = NativeCalls.godot_icall_0_203(method_bind_1, Object.GetPtr(this));
            return (Color)ret;
        }
    }
}
