using System;

namespace Godot
{
    /// <summary>
    /// Concave polygon shape resource, which can be set into a [PhysicsBody] or area. This shape is created by feeding a list of triangles.
    /// </summary>
    public class ConcavePolygonShape : Shape
    {
        public Vector3[] Data
        {
            get
            {
                return GetFaces();
            }
            set
            {
                SetFaces(value);
            }
        }

        private const string nativeName = "ConcavePolygonShape";

        public ConcavePolygonShape() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_ConcavePolygonShape_Ctor(this);
        }

        internal ConcavePolygonShape(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_faces");

        /// <summary>
        /// Set the faces (an array of triangles).
        /// </summary>
        [GodotMethod("set_faces")]
        public void SetFaces(Vector3[] faces)
        {
            NativeCalls.godot_icall_1_245(method_bind_0, Object.GetPtr(this), faces);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_faces");

        /// <summary>
        /// Return the faces (an array of triangles).
        /// </summary>
        [GodotMethod("get_faces")]
        public Vector3[] GetFaces()
        {
            return NativeCalls.godot_icall_0_246(method_bind_1, Object.GetPtr(this));
        }
    }
}
