using System;

namespace Godot
{
    /// <summary>
    /// Concave polygon 2D shape resource for physics. It is made out of segments and is very optimal for complex polygonal concave collisions. It is really not advised to use for [RigidBody2D] nodes. A CollisionPolygon2D in convex decomposition mode (solids) or several convex objects are advised for that instead. Otherwise, a concave polygon 2D shape is better for static collisions.
    /// The main difference between a [ConvexPolygonShape2D] and a [code]ConcavePolygonShape2D[/code] is that a concave polygon assumes it is concave and uses a more complex method of collision detection, and a convex one forces itself to be convex in order to speed up collision detection.
    /// </summary>
    public class ConcavePolygonShape2D : Shape2D
    {
        /// <summary>
        /// The array of points that make up the [code]ConcavePolygonShape2D[/code]'s line segments.
        /// </summary>
        public Vector2[] Segments
        {
            get
            {
                return GetSegments();
            }
            set
            {
                SetSegments(value);
            }
        }


        private const string nativeName = "ConcavePolygonShape2D";

        public ConcavePolygonShape2D() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_ConcavePolygonShape2D_Ctor(this);
        }

        internal ConcavePolygonShape2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_segments");

        [GodotMethod("set_segments")]
        public void SetSegments(Vector2[] segments)
        {
            NativeCalls.godot_icall_1_237(method_bind_0, Object.GetPtr(this), segments);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_segments");

        [GodotMethod("get_segments")]
        public Vector2[] GetSegments()
        {
            return NativeCalls.godot_icall_0_238(method_bind_1, Object.GetPtr(this));
        }
    }
}
