using System;

namespace Godot
{
    /// <summary>
    /// Base node for containers. A [code]Container[/code] contains other controls and automatically arranges them in a certain way.
    /// A Control can inherit this to create custom container classes.
    /// </summary>
    public class Container : Control
    {
        public const int NotificationSortChildren = 50;

        private const string nativeName = "Container";

        public Container() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Container_Ctor(this);
        }

        internal Container(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "queue_sort");

        /// <summary>
        /// Queue resort of the contained children. This is called automatically anyway, but can be called upon request.
        /// </summary>
        [GodotMethod("queue_sort")]
        public void QueueSort()
        {
            NativeCalls.godot_icall_0_12(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "fit_child_in_rect");

        /// <summary>
        /// Fit a child control in a given rect. This is mainly a helper for creating custom container classes.
        /// </summary>
        [GodotMethod("fit_child_in_rect")]
        public void FitChildInRect(Control child, Rect2 rect)
        {
            NativeCalls.godot_icall_2_253(method_bind_1, Object.GetPtr(this), Object.GetPtr(child), ref rect);
        }
    }
}
