using System;

namespace Godot
{
    /// <summary>
    /// Convex polygon shape resource, which can be added to a [PhysicsBody] or area.
    /// </summary>
    public class ConvexPolygonShape : Shape
    {
        /// <summary>
        /// The list of 3D points forming the convex polygon shape.
        /// </summary>
        public Vector3[] Points
        {
            get
            {
                return GetPoints();
            }
            set
            {
                SetPoints(value);
            }
        }


        private const string nativeName = "ConvexPolygonShape";

        public ConvexPolygonShape() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_ConvexPolygonShape_Ctor(this);
        }

        internal ConvexPolygonShape(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_points");

        [GodotMethod("set_points")]
        public void SetPoints(Vector3[] points)
        {
            NativeCalls.godot_icall_1_245(method_bind_0, Object.GetPtr(this), points);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_points");

        [GodotMethod("get_points")]
        public Vector3[] GetPoints()
        {
            return NativeCalls.godot_icall_0_246(method_bind_1, Object.GetPtr(this));
        }
    }
}
