using System;

namespace Godot
{
    /// <summary>
    /// Convex Polygon Shape for 2D physics. A convex polygon, whatever its shape, is internally decomposed into as many convex polygons as needed to ensure all collision checks against it are always done on convex polygons (which are faster to check).
    /// The main difference between a [code]ConvexPolygonShape2D[/code] and a [ConcavePolygonShape2D] is that a concave polygon assumes it is concave and uses a more complex method of collision detection, and a convex one forces itself to be convex in order to speed up collision detection.
    /// </summary>
    public class ConvexPolygonShape2D : Shape2D
    {
        /// <summary>
        /// The polygon's list of vertices. Can be in either clockwise or counterclockwise order.
        /// </summary>
        public Vector2[] Points
        {
            get
            {
                return GetPoints();
            }
            set
            {
                SetPoints(value);
            }
        }


        private const string nativeName = "ConvexPolygonShape2D";

        public ConvexPolygonShape2D() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_ConvexPolygonShape2D_Ctor(this);
        }

        internal ConvexPolygonShape2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_point_cloud");

        /// <summary>
        /// Currently, this method does nothing.
        /// </summary>
        [GodotMethod("set_point_cloud")]
        public void SetPointCloud(Vector2[] pointCloud)
        {
            NativeCalls.godot_icall_1_237(method_bind_0, Object.GetPtr(this), pointCloud);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_points");

        [GodotMethod("set_points")]
        public void SetPoints(Vector2[] points)
        {
            NativeCalls.godot_icall_1_237(method_bind_1, Object.GetPtr(this), points);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_points");

        [GodotMethod("get_points")]
        public Vector2[] GetPoints()
        {
            return NativeCalls.godot_icall_0_238(method_bind_2, Object.GetPtr(this));
        }
    }
}
