using System;

namespace Godot
{
    /// <summary>
    /// Generate an axis-aligned cuboid [PrimitiveMesh].
    /// </summary>
    public class CubeMesh : PrimitiveMesh
    {
        /// <summary>
        /// Size of the cuboid mesh. Defaults to (2, 2, 2).
        /// </summary>
        public Vector3 Size
        {
            get
            {
                return GetSize();
            }
            set
            {
                SetSize(value);
            }
        }

        /// <summary>
        /// Number of extra edge loops inserted along the x-axis. Defaults to 0.
        /// </summary>
        public int SubdivideWidth
        {
            get
            {
                return GetSubdivideWidth();
            }
            set
            {
                SetSubdivideWidth(value);
            }
        }

        /// <summary>
        /// Number of extra edge loops inserted along the y-axis. Defaults to 0.
        /// </summary>
        public int SubdivideHeight
        {
            get
            {
                return GetSubdivideHeight();
            }
            set
            {
                SetSubdivideHeight(value);
            }
        }

        /// <summary>
        /// Number of extra edge loops inserted along the z-axis. Defaults to 0.
        /// </summary>
        public int SubdivideDepth
        {
            get
            {
                return GetSubdivideDepth();
            }
            set
            {
                SetSubdivideDepth(value);
            }
        }


        private const string nativeName = "CubeMesh";

        public CubeMesh() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_CubeMesh_Ctor(this);
        }

        internal CubeMesh(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_size");

        [GodotMethod("set_size")]
        public void SetSize(Vector3 size)
        {
            NativeCalls.godot_icall_1_104(method_bind_0, Object.GetPtr(this), ref size);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_size");

        [GodotMethod("get_size")]
        public Vector3 GetSize()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_1, Object.GetPtr(this));
            return (Vector3)ret;
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_subdivide_width");

        [GodotMethod("set_subdivide_width")]
        public void SetSubdivideWidth(int subdivide)
        {
            NativeCalls.godot_icall_1_0(method_bind_2, Object.GetPtr(this), subdivide);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_subdivide_width");

        [GodotMethod("get_subdivide_width")]
        public int GetSubdivideWidth()
        {
            return NativeCalls.godot_icall_0_1(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_subdivide_height");

        [GodotMethod("set_subdivide_height")]
        public void SetSubdivideHeight(int divisions)
        {
            NativeCalls.godot_icall_1_0(method_bind_4, Object.GetPtr(this), divisions);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_subdivide_height");

        [GodotMethod("get_subdivide_height")]
        public int GetSubdivideHeight()
        {
            return NativeCalls.godot_icall_0_1(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_subdivide_depth");

        [GodotMethod("set_subdivide_depth")]
        public void SetSubdivideDepth(int divisions)
        {
            NativeCalls.godot_icall_1_0(method_bind_6, Object.GetPtr(this), divisions);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_subdivide_depth");

        [GodotMethod("get_subdivide_depth")]
        public int GetSubdivideDepth()
        {
            return NativeCalls.godot_icall_0_1(method_bind_7, Object.GetPtr(this));
        }
    }
}
