using System;

namespace Godot
{
    /// <summary>
    /// Renders a given [Curve] provided to it. Simplifies the task of drawing curves and/or saving them as image files.
    /// </summary>
    public class CurveTexture : Texture
    {
        /// <summary>
        /// The width of the texture.
        /// </summary>
        public int Width
        {
            get
            {
                return GetWidth();
            }
            set
            {
                SetWidth(value);
            }
        }

        /// <summary>
        /// The [code]curve[/code] rendered onto the texture.
        /// </summary>
        public Curve Curve
        {
            get
            {
                return GetCurve();
            }
            set
            {
                SetCurve(value);
            }
        }


        private const string nativeName = "CurveTexture";

        public CurveTexture() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_CurveTexture_Ctor(this);
        }

        internal CurveTexture(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_width");

        [GodotMethod("set_width")]
        public void SetWidth(int width)
        {
            NativeCalls.godot_icall_1_0(method_bind_0, Object.GetPtr(this), width);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_curve");

        [GodotMethod("set_curve")]
        public void SetCurve(Curve curve)
        {
            NativeCalls.godot_icall_1_24(method_bind_1, Object.GetPtr(this), Object.GetPtr(curve));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_curve");

        [GodotMethod("get_curve")]
        public Curve GetCurve()
        {
            return NativeCalls.godot_icall_0_299(method_bind_2, Object.GetPtr(this));
        }
    }
}
