using System;

namespace Godot
{
    /// <summary>
    /// Class representing a cylindrical [PrimitiveMesh].
    /// </summary>
    public class CylinderMesh : PrimitiveMesh
    {
        /// <summary>
        /// Top radius of the cylinder. Defaults to 1.0.
        /// </summary>
        public float TopRadius
        {
            get
            {
                return GetTopRadius();
            }
            set
            {
                SetTopRadius(value);
            }
        }

        /// <summary>
        /// Bottom radius of the cylinder. Defaults to 1.0.
        /// </summary>
        public float BottomRadius
        {
            get
            {
                return GetBottomRadius();
            }
            set
            {
                SetBottomRadius(value);
            }
        }

        /// <summary>
        /// Full height of the cylinder. Defaults to 2.0.
        /// </summary>
        public float Height
        {
            get
            {
                return GetHeight();
            }
            set
            {
                SetHeight(value);
            }
        }

        /// <summary>
        /// Number of radial segments on the cylinder. Defaults to 64.
        /// </summary>
        public int RadialSegments
        {
            get
            {
                return GetRadialSegments();
            }
            set
            {
                SetRadialSegments(value);
            }
        }

        /// <summary>
        /// Number of edge rings along the height of the cylinder. Defaults to 4.
        /// </summary>
        public int Rings
        {
            get
            {
                return GetRings();
            }
            set
            {
                SetRings(value);
            }
        }


        private const string nativeName = "CylinderMesh";

        public CylinderMesh() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_CylinderMesh_Ctor(this);
        }

        internal CylinderMesh(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_top_radius");

        [GodotMethod("set_top_radius")]
        public void SetTopRadius(float radius)
        {
            NativeCalls.godot_icall_1_10(method_bind_0, Object.GetPtr(this), radius);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_top_radius");

        [GodotMethod("get_top_radius")]
        public float GetTopRadius()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bottom_radius");

        [GodotMethod("set_bottom_radius")]
        public void SetBottomRadius(float radius)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), radius);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bottom_radius");

        [GodotMethod("get_bottom_radius")]
        public float GetBottomRadius()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_height");

        [GodotMethod("set_height")]
        public void SetHeight(float height)
        {
            NativeCalls.godot_icall_1_10(method_bind_4, Object.GetPtr(this), height);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_height");

        [GodotMethod("get_height")]
        public float GetHeight()
        {
            return NativeCalls.godot_icall_0_9(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_radial_segments");

        [GodotMethod("set_radial_segments")]
        public void SetRadialSegments(int segments)
        {
            NativeCalls.godot_icall_1_0(method_bind_6, Object.GetPtr(this), segments);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_radial_segments");

        [GodotMethod("get_radial_segments")]
        public int GetRadialSegments()
        {
            return NativeCalls.godot_icall_0_1(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_rings");

        [GodotMethod("set_rings")]
        public void SetRings(int rings)
        {
            NativeCalls.godot_icall_1_0(method_bind_8, Object.GetPtr(this), rings);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_rings");

        [GodotMethod("get_rings")]
        public int GetRings()
        {
            return NativeCalls.godot_icall_0_1(method_bind_9, Object.GetPtr(this));
        }
    }
}
