using System;

namespace Godot
{
    /// <summary>
    /// A DirectionalLight is a type of [Light] node that emits light constantly in one direction (the negative z axis of the node). It is used lights with strong intensity that are located far away from the scene to model sunlight or moonlight. The worldspace location of the DirectionalLight transform (origin) is ignored, only the basis is used do determine light direction.
    /// </summary>
    public class DirectionalLight : Light
    {
        public enum ShadowMode
        {
            Orthogonal = 0,
            Parallel2Splits = 1,
            Parallel4Splits = 2
        }

        public enum ShadowDepthRange
        {
            Stable = 0,
            Optimized = 1
        }


        /// <summary>
        /// </summary>
        public DirectionalLight.ShadowMode DirectionalShadowMode
        {
            get
            {
                return GetShadowMode();
            }
            set
            {
                SetShadowMode(value);
            }
        }

        /// <summary>
        /// </summary>
        public float DirectionalShadowSplit1
        {
            get
            {
                return GetParam((Light.Param)9);
            }
            set
            {
                SetParam((Light.Param)9, value);
            }
        }

        /// <summary>
        /// </summary>
        public float DirectionalShadowSplit2
        {
            get
            {
                return GetParam((Light.Param)10);
            }
            set
            {
                SetParam((Light.Param)10, value);
            }
        }

        /// <summary>
        /// </summary>
        public float DirectionalShadowSplit3
        {
            get
            {
                return GetParam((Light.Param)11);
            }
            set
            {
                SetParam((Light.Param)11, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool DirectionalShadowBlendSplits
        {
            get
            {
                return IsBlendSplitsEnabled();
            }
            set
            {
                SetBlendSplits(value);
            }
        }

        /// <summary>
        /// </summary>
        public float DirectionalShadowNormalBias
        {
            get
            {
                return GetParam((Light.Param)12);
            }
            set
            {
                SetParam((Light.Param)12, value);
            }
        }

        /// <summary>
        /// </summary>
        public float DirectionalShadowBiasSplitScale
        {
            get
            {
                return GetParam((Light.Param)14);
            }
            set
            {
                SetParam((Light.Param)14, value);
            }
        }

        /// <summary>
        /// </summary>
        public DirectionalLight.ShadowDepthRange DirectionalShadowDepthRange
        {
            get
            {
                return GetShadowDepthRange();
            }
            set
            {
                SetShadowDepthRange(value);
            }
        }

        /// <summary>
        /// </summary>
        public float DirectionalShadowMaxDistance
        {
            get
            {
                return GetParam((Light.Param)8);
            }
            set
            {
                SetParam((Light.Param)8, value);
            }
        }


        private const string nativeName = "DirectionalLight";

        public DirectionalLight() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_DirectionalLight_Ctor(this);
        }

        internal DirectionalLight(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_shadow_mode");

        [GodotMethod("set_shadow_mode")]
        public void SetShadowMode(DirectionalLight.ShadowMode mode)
        {
            NativeCalls.godot_icall_1_300(method_bind_0, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_shadow_mode");

        [GodotMethod("get_shadow_mode")]
        public DirectionalLight.ShadowMode GetShadowMode()
        {
            return NativeCalls.godot_icall_0_301(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_shadow_depth_range");

        [GodotMethod("set_shadow_depth_range")]
        public void SetShadowDepthRange(DirectionalLight.ShadowDepthRange mode)
        {
            NativeCalls.godot_icall_1_302(method_bind_2, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_shadow_depth_range");

        [GodotMethod("get_shadow_depth_range")]
        public DirectionalLight.ShadowDepthRange GetShadowDepthRange()
        {
            return NativeCalls.godot_icall_0_303(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_blend_splits");

        [GodotMethod("set_blend_splits")]
        public void SetBlendSplits(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_4, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_blend_splits_enabled");

        [GodotMethod("is_blend_splits_enabled")]
        public bool IsBlendSplitsEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_5, Object.GetPtr(this));
        }
    }
}
