using System;

namespace Godot
{
    /// <summary>
    /// Used with [DynamicFont] to describe the location of a vector font file for dynamic rendering at runtime.
    /// </summary>
    public class DynamicFontData : Resource
    {
        /// <summary>
        /// The path to the vector font file.
        /// </summary>
        public string FontPath
        {
            get
            {
                return GetFontPath();
            }
            set
            {
                SetFontPath(value);
            }
        }


        private const string nativeName = "DynamicFontData";

        public DynamicFontData() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_DynamicFontData_Ctor(this);
        }

        internal DynamicFontData(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_font_path");

        [GodotMethod("set_font_path")]
        public void SetFontPath(string path)
        {
            NativeCalls.godot_icall_1_43(method_bind_0, Object.GetPtr(this), path);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_font_path");

        [GodotMethod("get_font_path")]
        public string GetFontPath()
        {
            return NativeCalls.godot_icall_0_2(method_bind_1, Object.GetPtr(this));
        }
    }
}
