using System;
using System.Collections.Generic;

namespace Godot
{
    /// <summary>
    /// The [code]Engine[/code] class allows you to query and modify the game's run-time parameters, such as frames per second, time scale, and others.
    /// </summary>
    public static class Engine
    {
        /// <summary>
        /// If [code]true[/code], it is running inside the editor. Useful for tool scripts.
        /// </summary>
        public static bool EditorHint
        {
            get
            {
                return IsEditorHint();
            }
            set
            {
                SetEditorHint(value);
            }
        }

        /// <summary>
        /// The number of fixed iterations per second (for fixed process and physics).
        /// </summary>
        public static int IterationsPerSecond
        {
            get
            {
                return GetIterationsPerSecond();
            }
            set
            {
                SetIterationsPerSecond(value);
            }
        }

        /// <summary>
        /// The desired frames per second. If the hardware cannot keep up, this setting may not be respected. Defaults to 0, which indicates no limit.
        /// </summary>
        public static int TargetFps
        {
            get
            {
                return GetTargetFps();
            }
            set
            {
                SetTargetFps(value);
            }
        }

        /// <summary>
        /// Controls how fast or slow the in-game clock ticks versus the real life one. It defaults to 1.0. A value of 2.0 means the game moves twice as fast as real life, whilst a value of 0.5 means the game moves at half the regular speed.
        /// </summary>
        public static float TimeScale
        {
            get
            {
                return GetTimeScale();
            }
            set
            {
                SetTimeScale(value);
            }
        }


        private const string nativeName = "_Engine";
        internal static IntPtr ptr = NativeCalls.godot_icall__Engine_get_singleton();

        private static IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_iterations_per_second");

        [GodotMethod("set_iterations_per_second")]
        public static void SetIterationsPerSecond(int iterationsPerSecond)
        {
            NativeCalls.godot_icall_1_0(method_bind_0, ptr, iterationsPerSecond);
        }

        private static IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_iterations_per_second");

        [GodotMethod("get_iterations_per_second")]
        public static int GetIterationsPerSecond()
        {
            return NativeCalls.godot_icall_0_1(method_bind_1, ptr);
        }

        private static IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_target_fps");

        [GodotMethod("set_target_fps")]
        public static void SetTargetFps(int targetFps)
        {
            NativeCalls.godot_icall_1_0(method_bind_2, ptr, targetFps);
        }

        private static IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_target_fps");

        [GodotMethod("get_target_fps")]
        public static int GetTargetFps()
        {
            return NativeCalls.godot_icall_0_1(method_bind_3, ptr);
        }

        private static IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_time_scale");

        [GodotMethod("set_time_scale")]
        public static void SetTimeScale(float timeScale)
        {
            NativeCalls.godot_icall_1_10(method_bind_4, ptr, timeScale);
        }

        private static IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_time_scale");

        [GodotMethod("get_time_scale")]
        public static float GetTimeScale()
        {
            return NativeCalls.godot_icall_0_9(method_bind_5, ptr);
        }

        private static IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_frames_drawn");

        /// <summary>
        /// Returns the total number of frames drawn.
        /// </summary>
        [GodotMethod("get_frames_drawn")]
        public static int GetFramesDrawn()
        {
            return NativeCalls.godot_icall_0_1(method_bind_6, ptr);
        }

        private static IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_frames_per_second");

        /// <summary>
        /// Returns the frames per second of the running game.
        /// </summary>
        [GodotMethod("get_frames_per_second")]
        public static float GetFramesPerSecond()
        {
            return NativeCalls.godot_icall_0_9(method_bind_7, ptr);
        }

        private static IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_main_loop");

        /// <summary>
        /// Returns the main loop object (see [MainLoop] and [SceneTree]).
        /// </summary>
        [GodotMethod("get_main_loop")]
        public static MainLoop GetMainLoop()
        {
            return NativeCalls.godot_icall_0_1036(method_bind_8, ptr);
        }

        private static IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_version_info");

        /// <summary>
        /// Returns the current engine version information in a Dictionary.
        /// "major"    - Holds the major version number as a String
        /// "minor"    - Holds the minor version number as a String
        /// "patch"    - Holds the patch version number as a String
        /// "status"   - Holds the status (e.g. "beta", "rc1", "rc2", ... "stable") as a String
        /// "build"    - Holds the build name (e.g. "custom-build") as a String
        /// "string"   - major + minor + patch + status + build in a single String
        /// </summary>
        [GodotMethod("get_version_info")]
        public static Dictionary<object, object> GetVersionInfo()
        {
            return NativeCalls.godot_icall_0_169(method_bind_9, ptr);
        }

        private static IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_in_physics_frame");

        /// <summary>
        /// Returns [code]true[/code] if the game is inside the fixed process and physics phase of the game loop.
        /// </summary>
        [GodotMethod("is_in_physics_frame")]
        public static bool IsInPhysicsFrame()
        {
            return NativeCalls.godot_icall_0_3(method_bind_10, ptr);
        }

        private static IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_singleton");

        /// <summary>
        /// </summary>
        [GodotMethod("has_singleton")]
        public static bool HasSingleton(string name)
        {
            return NativeCalls.godot_icall_1_70(method_bind_11, ptr, name);
        }

        private static IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_singleton");

        /// <summary>
        /// </summary>
        [GodotMethod("get_singleton")]
        public static Object GetSingleton(string name)
        {
            return NativeCalls.godot_icall_1_1037(method_bind_12, ptr, name);
        }

        private static IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_editor_hint");

        [GodotMethod("set_editor_hint")]
        public static void SetEditorHint(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_13, ptr, enabled);
        }

        private static IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_editor_hint");

        [GodotMethod("is_editor_hint")]
        public static bool IsEditorHint()
        {
            return NativeCalls.godot_icall_0_3(method_bind_14, ptr);
        }
    }
}
