using System;

namespace Godot
{
    /// <summary>
    /// Font contains a unicode compatible character set, as well as the ability to draw it with variable width, ascent, descent and kerning. For creating fonts from TTF files (or other font formats), see the editor support for fonts. TODO check wikipedia for graph of ascent/baseline/descent/height/etc.
    /// </summary>
    public class Font : Resource
    {
        private const string nativeName = "Font";

        internal Font() {}

        internal Font(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "draw");

        /// <summary>
        /// Draw "string" into a canvas item using the font at a given position, with "modulate" color, and optionally clipping the width. "position" specifies the baseline, not the top. To draw from the top, [i]ascent[/i] must be added to the Y axis.
        /// &lt;param name="modulate"&gt;If the param is null, then the default value is new Color(1, 1, 1, 1)&lt;/param&gt;
        /// </summary>
        [GodotMethod("draw")]
        public void Draw(RID canvasItem, Vector2 position, string @string, Nullable<Color> modulate = null, int clipW = -1)
        {
            Color modulate_in = modulate.HasValue ? modulate.Value : new Color(1, 1, 1, 1);
            NativeCalls.godot_icall_5_363(method_bind_0, Object.GetPtr(this), RID.GetPtr(canvasItem), ref position, @string, ref modulate_in, clipW);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_ascent");

        /// <summary>
        /// Return the font ascent (number of pixels above the baseline).
        /// </summary>
        [GodotMethod("get_ascent")]
        public float GetAscent()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_descent");

        /// <summary>
        /// Return the font descent (number of pixels below the baseline).
        /// </summary>
        [GodotMethod("get_descent")]
        public float GetDescent()
        {
            return NativeCalls.godot_icall_0_9(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_height");

        /// <summary>
        /// Return the total font height (ascent plus descent) in pixels.
        /// </summary>
        [GodotMethod("get_height")]
        public float GetHeight()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_distance_field_hint");

        /// <summary>
        /// </summary>
        [GodotMethod("is_distance_field_hint")]
        public bool IsDistanceFieldHint()
        {
            return NativeCalls.godot_icall_0_3(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_string_size");

        /// <summary>
        /// Return the size of a string, taking kerning and advance into account.
        /// </summary>
        [GodotMethod("get_string_size")]
        public Vector2 GetStringSize(string @string)
        {
            object ret = NativeCalls.godot_icall_1_94(method_bind_5, Object.GetPtr(this), @string);
            return (Vector2)ret;
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "draw_char");

        /// <summary>
        /// Draw character "char" into a canvas item using the font at a given position, with "modulate" color, and optionally kerning if "next" is passed. clipping the width. "position" specifies the baseline, not the top. To draw from the top, [i]ascent[/i] must be added to the Y axis. The width used by the character is returned, making this function useful for drawing strings character by character.
        /// &lt;param name="modulate"&gt;If the param is null, then the default value is new Color(1, 1, 1, 1)&lt;/param&gt;
        /// </summary>
        [GodotMethod("draw_char")]
        public float DrawChar(RID canvasItem, Vector2 position, int @char, int next = -1, Nullable<Color> modulate = null)
        {
            Color modulate_in = modulate.HasValue ? modulate.Value : new Color(1, 1, 1, 1);
            return NativeCalls.godot_icall_5_364(method_bind_6, Object.GetPtr(this), RID.GetPtr(canvasItem), ref position, @char, next, ref modulate_in);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "update_changes");

        /// <summary>
        /// After editing a font (changing size, ascent, char rects, etc.). Call this function to propagate changes to controls that might use it.
        /// </summary>
        [GodotMethod("update_changes")]
        public void UpdateChanges()
        {
            NativeCalls.godot_icall_0_12(method_bind_7, Object.GetPtr(this));
        }
    }
}
