using System;

namespace Godot
{
    /// <summary>
    /// A script implemented in the GDScript programming language. The script exends the functionality of all objects that instance it.
    /// [method new] creates a new instance of the script. [method Object.set_script] extends an existing object, if that object's class matches one of the script's base classes.
    /// </summary>
    public class GDScript : Script
    {
        private const string nativeName = "GDScript";

        public GDScript() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_GDScript_Ctor(this);
        }

        internal GDScript(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "new");

        /// <summary>
        /// Returns a new instance of the script.
        /// For example:
        /// [codeblock]
        /// var MyClass = load("myclass.gd")
        /// var instance = MyClass.new()
        /// assert(instance.get_script() == MyClass)
        /// [/codeblock]
        /// </summary>
        [GodotMethod("new")]
        public Object New(params object[] @args)
        {
            return NativeCalls.godot_icall_1_182(method_bind_0, Object.GetPtr(this), @args);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_as_byte_code");

        /// <summary>
        /// Returns byte code for the script source code.
        /// </summary>
        [GodotMethod("get_as_byte_code")]
        public byte[] GetAsByteCode()
        {
            return NativeCalls.godot_icall_0_132(method_bind_1, Object.GetPtr(this));
        }
    }
}
