using System;

namespace Godot
{
    /// <summary>
    /// The first 3 dof axes are linear axes, which represent translation of Bodies, and the latter 3 dof axes represent the angular motion. Each axis can be either locked, or limited.
    /// </summary>
    public class Generic6DOFJoint : Joint
    {
        public enum Param
        {
            LinearLowerLimit = 0,
            LinearUpperLimit = 1,
            LinearLimitSoftness = 2,
            LinearRestitution = 3,
            LinearDamping = 4,
            AngularLowerLimit = 5,
            AngularUpperLimit = 6,
            AngularLimitSoftness = 7,
            AngularDamping = 8,
            AngularRestitution = 9,
            AngularForceLimit = 10,
            AngularErp = 11,
            AngularMotorTargetVelocity = 12,
            AngularMotorForceLimit = 13,
            Max = 14
        }

        public enum Flag
        {
            EnableLinearLimit = 0,
            EnableAngularLimit = 1,
            EnableMotor = 2,
            Max = 3
        }


        /// <summary>
        /// If [code]true[/code] the linear motion across the x-axis is enabled.
        /// </summary>
        public bool LinearLimitX__enabled
        {
            get
            {
                return GetFlagX((Generic6DOFJoint.Flag)0);
            }
            set
            {
                SetFlagX((Generic6DOFJoint.Flag)0, value);
            }
        }

        /// <summary>
        /// The maximum difference between the pivot points' x-axis.
        /// </summary>
        public float LinearLimitX__upperDistance
        {
            get
            {
                return GetParamX((Generic6DOFJoint.Param)1);
            }
            set
            {
                SetParamX((Generic6DOFJoint.Param)1, value);
            }
        }

        /// <summary>
        /// The minimum difference between the pivot points' x-axis.
        /// </summary>
        public float LinearLimitX__lowerDistance
        {
            get
            {
                return GetParamX((Generic6DOFJoint.Param)0);
            }
            set
            {
                SetParamX((Generic6DOFJoint.Param)0, value);
            }
        }

        /// <summary>
        /// A factor applied to the movement across the x-axis The lower, the slower the movement.
        /// </summary>
        public float LinearLimitX__softness
        {
            get
            {
                return GetParamX((Generic6DOFJoint.Param)2);
            }
            set
            {
                SetParamX((Generic6DOFJoint.Param)2, value);
            }
        }

        /// <summary>
        /// The amount of restitution on the x-axis movement The lower, the more momentum gets lost.
        /// </summary>
        public float LinearLimitX__restitution
        {
            get
            {
                return GetParamX((Generic6DOFJoint.Param)3);
            }
            set
            {
                SetParamX((Generic6DOFJoint.Param)3, value);
            }
        }

        /// <summary>
        /// The amount of damping that happens at the x-motion.
        /// </summary>
        public float LinearLimitX__damping
        {
            get
            {
                return GetParamX((Generic6DOFJoint.Param)4);
            }
            set
            {
                SetParamX((Generic6DOFJoint.Param)4, value);
            }
        }

        /// <summary>
        /// If [code]true[/code] rotation across the x-axis is enabled.
        /// </summary>
        public bool AngularLimitX__enabled
        {
            get
            {
                return GetFlagX((Generic6DOFJoint.Flag)1);
            }
            set
            {
                SetFlagX((Generic6DOFJoint.Flag)1, value);
            }
        }

        /// <summary>
        /// The minimum rotation in positive direction to break loose and rotate arround the x-axis.
        /// </summary>
        public float AngularLimitX__upperAngle
        {
            get
            {
                return _GetAngularHiLimitX();
            }
            set
            {
                _SetAngularHiLimitX(value);
            }
        }

        /// <summary>
        /// The minimum rotation in negative direction to break loose and rotate arround the x-axis.
        /// </summary>
        public float AngularLimitX__lowerAngle
        {
            get
            {
                return _GetAngularLoLimitX();
            }
            set
            {
                _SetAngularLoLimitX(value);
            }
        }

        /// <summary>
        /// The speed of all rotations across the x-axis.
        /// </summary>
        public float AngularLimitX__softness
        {
            get
            {
                return GetParamX((Generic6DOFJoint.Param)7);
            }
            set
            {
                SetParamX((Generic6DOFJoint.Param)7, value);
            }
        }

        /// <summary>
        /// The amount of rotational restitution across the x-axis. The lower, the more restitution occurs.
        /// </summary>
        public float AngularLimitX__restitution
        {
            get
            {
                return GetParamX((Generic6DOFJoint.Param)9);
            }
            set
            {
                SetParamX((Generic6DOFJoint.Param)9, value);
            }
        }

        /// <summary>
        /// The amount of rotational damping across the x-axis.
        /// The lower, the longer an impulse from one side takes to travel to the other side.
        /// </summary>
        public float AngularLimitX__damping
        {
            get
            {
                return GetParamX((Generic6DOFJoint.Param)8);
            }
            set
            {
                SetParamX((Generic6DOFJoint.Param)8, value);
            }
        }

        /// <summary>
        /// The maximum amount of force that can occur, when rotating arround x-axis.
        /// </summary>
        public float AngularLimitX__forceLimit
        {
            get
            {
                return GetParamX((Generic6DOFJoint.Param)10);
            }
            set
            {
                SetParamX((Generic6DOFJoint.Param)10, value);
            }
        }

        /// <summary>
        /// When rotating across x-axis, this error tolerance factor defines how much the correction gets slowed down. The lower, the slower.
        /// </summary>
        public float AngularLimitX__erp
        {
            get
            {
                return GetParamX((Generic6DOFJoint.Param)11);
            }
            set
            {
                SetParamX((Generic6DOFJoint.Param)11, value);
            }
        }

        /// <summary>
        /// If [code]true[/code] a rotating motor at the x-axis is enabled.
        /// </summary>
        public bool AngularMotorX__enabled
        {
            get
            {
                return GetFlagX((Generic6DOFJoint.Flag)2);
            }
            set
            {
                SetFlagX((Generic6DOFJoint.Flag)2, value);
            }
        }

        /// <summary>
        /// Target speed for the motor at the x-axis.
        /// </summary>
        public float AngularMotorX__targetVelocity
        {
            get
            {
                return GetParamX((Generic6DOFJoint.Param)12);
            }
            set
            {
                SetParamX((Generic6DOFJoint.Param)12, value);
            }
        }

        /// <summary>
        /// Maximum acceleration for the motor at the x-axis.
        /// </summary>
        public float AngularMotorX__forceLimit
        {
            get
            {
                return GetParamX((Generic6DOFJoint.Param)13);
            }
            set
            {
                SetParamX((Generic6DOFJoint.Param)13, value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the linear motion across the y-axis is enabled.
        /// </summary>
        public bool LinearLimitY__enabled
        {
            get
            {
                return GetFlagY((Generic6DOFJoint.Flag)0);
            }
            set
            {
                SetFlagY((Generic6DOFJoint.Flag)0, value);
            }
        }

        /// <summary>
        /// The maximum difference between the pivot points' y-axis.
        /// </summary>
        public float LinearLimitY__upperDistance
        {
            get
            {
                return GetParamY((Generic6DOFJoint.Param)1);
            }
            set
            {
                SetParamY((Generic6DOFJoint.Param)1, value);
            }
        }

        /// <summary>
        /// The minimum difference between the pivot points' y-axis.
        /// </summary>
        public float LinearLimitY__lowerDistance
        {
            get
            {
                return GetParamY((Generic6DOFJoint.Param)0);
            }
            set
            {
                SetParamY((Generic6DOFJoint.Param)0, value);
            }
        }

        /// <summary>
        /// A factor applied to the movement across the y-axis The lower, the slower the movement.
        /// </summary>
        public float LinearLimitY__softness
        {
            get
            {
                return GetParamY((Generic6DOFJoint.Param)2);
            }
            set
            {
                SetParamY((Generic6DOFJoint.Param)2, value);
            }
        }

        /// <summary>
        /// The amount of restitution on the y-axis movement The lower, the more momentum gets lost.
        /// </summary>
        public float LinearLimitY__restitution
        {
            get
            {
                return GetParamY((Generic6DOFJoint.Param)3);
            }
            set
            {
                SetParamY((Generic6DOFJoint.Param)3, value);
            }
        }

        /// <summary>
        /// The amount of damping that happens at the y-motion.
        /// </summary>
        public float LinearLimitY__damping
        {
            get
            {
                return GetParamY((Generic6DOFJoint.Param)4);
            }
            set
            {
                SetParamY((Generic6DOFJoint.Param)4, value);
            }
        }

        /// <summary>
        /// If [code]true[/code] rotation across the y-axis is enabled.
        /// </summary>
        public bool AngularLimitY__enabled
        {
            get
            {
                return GetFlagY((Generic6DOFJoint.Flag)1);
            }
            set
            {
                SetFlagY((Generic6DOFJoint.Flag)1, value);
            }
        }

        /// <summary>
        /// The minimum rotation in positive direction to break loose and rotate arround the y-axis.
        /// </summary>
        public float AngularLimitY__upperAngle
        {
            get
            {
                return _GetAngularHiLimitY();
            }
            set
            {
                _SetAngularHiLimitY(value);
            }
        }

        /// <summary>
        /// The minimum rotation in negative direction to break loose and rotate arround the y-axis.
        /// </summary>
        public float AngularLimitY__lowerAngle
        {
            get
            {
                return _GetAngularLoLimitY();
            }
            set
            {
                _SetAngularLoLimitY(value);
            }
        }

        /// <summary>
        /// The speed of all rotations across the y-axis.
        /// </summary>
        public float AngularLimitY__softness
        {
            get
            {
                return GetParamY((Generic6DOFJoint.Param)7);
            }
            set
            {
                SetParamY((Generic6DOFJoint.Param)7, value);
            }
        }

        /// <summary>
        /// The amount of rotational restitution across the y-axis. The lower, the more restitution occurs.
        /// </summary>
        public float AngularLimitY__restitution
        {
            get
            {
                return GetParamY((Generic6DOFJoint.Param)9);
            }
            set
            {
                SetParamY((Generic6DOFJoint.Param)9, value);
            }
        }

        /// <summary>
        /// The amount of rotational damping across the y-axis. The lower, the more dampening occurs.
        /// </summary>
        public float AngularLimitY__damping
        {
            get
            {
                return GetParamY((Generic6DOFJoint.Param)8);
            }
            set
            {
                SetParamY((Generic6DOFJoint.Param)8, value);
            }
        }

        /// <summary>
        /// The maximum amount of force that can occur, when rotating arround y-axis.
        /// </summary>
        public float AngularLimitY__forceLimit
        {
            get
            {
                return GetParamY((Generic6DOFJoint.Param)10);
            }
            set
            {
                SetParamY((Generic6DOFJoint.Param)10, value);
            }
        }

        /// <summary>
        /// When rotating across y-axis, this error tolerance factor defines how much the correction gets slowed down. The lower, the slower.
        /// </summary>
        public float AngularLimitY__erp
        {
            get
            {
                return GetParamY((Generic6DOFJoint.Param)11);
            }
            set
            {
                SetParamY((Generic6DOFJoint.Param)11, value);
            }
        }

        /// <summary>
        /// If [code]true[/code] a rotating motor at the y-axis is enabled.
        /// </summary>
        public bool AngularMotorY__enabled
        {
            get
            {
                return GetFlagY((Generic6DOFJoint.Flag)2);
            }
            set
            {
                SetFlagY((Generic6DOFJoint.Flag)2, value);
            }
        }

        /// <summary>
        /// Target speed for the motor at the y-axis.
        /// </summary>
        public float AngularMotorY__targetVelocity
        {
            get
            {
                return GetParamY((Generic6DOFJoint.Param)12);
            }
            set
            {
                SetParamY((Generic6DOFJoint.Param)12, value);
            }
        }

        /// <summary>
        /// Maximum acceleration for the motor at the y-axis.
        /// </summary>
        public float AngularMotorY__forceLimit
        {
            get
            {
                return GetParamY((Generic6DOFJoint.Param)13);
            }
            set
            {
                SetParamY((Generic6DOFJoint.Param)13, value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the linear motion across the z-axis is enabled.
        /// </summary>
        public bool LinearLimitZ__enabled
        {
            get
            {
                return GetFlagZ((Generic6DOFJoint.Flag)0);
            }
            set
            {
                SetFlagZ((Generic6DOFJoint.Flag)0, value);
            }
        }

        /// <summary>
        /// The maximum difference between the pivot points' z-axis.
        /// </summary>
        public float LinearLimitZ__upperDistance
        {
            get
            {
                return GetParamZ((Generic6DOFJoint.Param)1);
            }
            set
            {
                SetParamZ((Generic6DOFJoint.Param)1, value);
            }
        }

        /// <summary>
        /// The minimum difference between the pivot points' z-axis.
        /// </summary>
        public float LinearLimitZ__lowerDistance
        {
            get
            {
                return GetParamZ((Generic6DOFJoint.Param)0);
            }
            set
            {
                SetParamZ((Generic6DOFJoint.Param)0, value);
            }
        }

        /// <summary>
        /// A factor applied to the movement across the z-axis The lower, the slower the movement.
        /// </summary>
        public float LinearLimitZ__softness
        {
            get
            {
                return GetParamZ((Generic6DOFJoint.Param)2);
            }
            set
            {
                SetParamZ((Generic6DOFJoint.Param)2, value);
            }
        }

        /// <summary>
        /// The amount of restitution on the z-axis movement The lower, the more momentum gets lost.
        /// </summary>
        public float LinearLimitZ__restitution
        {
            get
            {
                return GetParamZ((Generic6DOFJoint.Param)3);
            }
            set
            {
                SetParamZ((Generic6DOFJoint.Param)3, value);
            }
        }

        /// <summary>
        /// The amount of damping that happens at the z-motion.
        /// </summary>
        public float LinearLimitZ__damping
        {
            get
            {
                return GetParamZ((Generic6DOFJoint.Param)4);
            }
            set
            {
                SetParamZ((Generic6DOFJoint.Param)4, value);
            }
        }

        /// <summary>
        /// If [code]true[/code] rotation across the z-axis is enabled.
        /// </summary>
        public bool AngularLimitZ__enabled
        {
            get
            {
                return GetFlagZ((Generic6DOFJoint.Flag)1);
            }
            set
            {
                SetFlagZ((Generic6DOFJoint.Flag)1, value);
            }
        }

        /// <summary>
        /// The minimum rotation in positive direction to break loose and rotate arround the z-axis.
        /// </summary>
        public float AngularLimitZ__upperAngle
        {
            get
            {
                return _GetAngularHiLimitZ();
            }
            set
            {
                _SetAngularHiLimitZ(value);
            }
        }

        /// <summary>
        /// The minimum rotation in negative direction to break loose and rotate arround the z-axis.
        /// </summary>
        public float AngularLimitZ__lowerAngle
        {
            get
            {
                return _GetAngularLoLimitZ();
            }
            set
            {
                _SetAngularLoLimitZ(value);
            }
        }

        /// <summary>
        /// The speed of all rotations across the z-axis.
        /// </summary>
        public float AngularLimitZ__softness
        {
            get
            {
                return GetParamZ((Generic6DOFJoint.Param)7);
            }
            set
            {
                SetParamZ((Generic6DOFJoint.Param)7, value);
            }
        }

        /// <summary>
        /// The amount of rotational restitution across the z-axis. The lower, the more restitution occurs.
        /// </summary>
        public float AngularLimitZ__restitution
        {
            get
            {
                return GetParamZ((Generic6DOFJoint.Param)9);
            }
            set
            {
                SetParamZ((Generic6DOFJoint.Param)9, value);
            }
        }

        /// <summary>
        /// The amount of rotational damping across the z-axis. The lower, the more dampening occurs.
        /// </summary>
        public float AngularLimitZ__damping
        {
            get
            {
                return GetParamZ((Generic6DOFJoint.Param)8);
            }
            set
            {
                SetParamZ((Generic6DOFJoint.Param)8, value);
            }
        }

        /// <summary>
        /// The maximum amount of force that can occur, when rotating arround z-axis.
        /// </summary>
        public float AngularLimitZ__forceLimit
        {
            get
            {
                return GetParamZ((Generic6DOFJoint.Param)10);
            }
            set
            {
                SetParamZ((Generic6DOFJoint.Param)10, value);
            }
        }

        /// <summary>
        /// When rotating across z-axis, this error tolerance factor defines how much the correction gets slowed down. The lower, the slower.
        /// </summary>
        public float AngularLimitZ__erp
        {
            get
            {
                return GetParamZ((Generic6DOFJoint.Param)11);
            }
            set
            {
                SetParamZ((Generic6DOFJoint.Param)11, value);
            }
        }

        /// <summary>
        /// If [code]true[/code] a rotating motor at the z-axis is enabled.
        /// </summary>
        public bool AngularMotorZ__enabled
        {
            get
            {
                return GetFlagZ((Generic6DOFJoint.Flag)2);
            }
            set
            {
                SetFlagZ((Generic6DOFJoint.Flag)2, value);
            }
        }

        /// <summary>
        /// Target speed for the motor at the z-axis.
        /// </summary>
        public float AngularMotorZ__targetVelocity
        {
            get
            {
                return GetParamZ((Generic6DOFJoint.Param)12);
            }
            set
            {
                SetParamZ((Generic6DOFJoint.Param)12, value);
            }
        }

        /// <summary>
        /// Maximum acceleration for the motor at the z-axis.
        /// </summary>
        public float AngularMotorZ__forceLimit
        {
            get
            {
                return GetParamZ((Generic6DOFJoint.Param)13);
            }
            set
            {
                SetParamZ((Generic6DOFJoint.Param)13, value);
            }
        }


        private const string nativeName = "Generic6DOFJoint";

        public Generic6DOFJoint() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Generic6DOFJoint_Ctor(this);
        }

        internal Generic6DOFJoint(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_angular_hi_limit_x");

        internal void _SetAngularHiLimitX(float angle)
        {
            NativeCalls.godot_icall_1_10(method_bind_0, Object.GetPtr(this), angle);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_angular_hi_limit_x");

        internal float _GetAngularHiLimitX()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_angular_lo_limit_x");

        internal void _SetAngularLoLimitX(float angle)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), angle);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_angular_lo_limit_x");

        internal float _GetAngularLoLimitX()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_angular_hi_limit_y");

        internal void _SetAngularHiLimitY(float angle)
        {
            NativeCalls.godot_icall_1_10(method_bind_4, Object.GetPtr(this), angle);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_angular_hi_limit_y");

        internal float _GetAngularHiLimitY()
        {
            return NativeCalls.godot_icall_0_9(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_angular_lo_limit_y");

        internal void _SetAngularLoLimitY(float angle)
        {
            NativeCalls.godot_icall_1_10(method_bind_6, Object.GetPtr(this), angle);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_angular_lo_limit_y");

        internal float _GetAngularLoLimitY()
        {
            return NativeCalls.godot_icall_0_9(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_angular_hi_limit_z");

        internal void _SetAngularHiLimitZ(float angle)
        {
            NativeCalls.godot_icall_1_10(method_bind_8, Object.GetPtr(this), angle);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_angular_hi_limit_z");

        internal float _GetAngularHiLimitZ()
        {
            return NativeCalls.godot_icall_0_9(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_angular_lo_limit_z");

        internal void _SetAngularLoLimitZ(float angle)
        {
            NativeCalls.godot_icall_1_10(method_bind_10, Object.GetPtr(this), angle);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_angular_lo_limit_z");

        internal float _GetAngularLoLimitZ()
        {
            return NativeCalls.godot_icall_0_9(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_param_x");

        [GodotMethod("set_param_x")]
        public void SetParamX(Generic6DOFJoint.Param param, float value)
        {
            NativeCalls.godot_icall_2_375(method_bind_12, Object.GetPtr(this), param, value);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_param_x");

        [GodotMethod("get_param_x")]
        public float GetParamX(Generic6DOFJoint.Param param)
        {
            return NativeCalls.godot_icall_1_376(method_bind_13, Object.GetPtr(this), param);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_param_y");

        [GodotMethod("set_param_y")]
        public void SetParamY(Generic6DOFJoint.Param param, float value)
        {
            NativeCalls.godot_icall_2_375(method_bind_14, Object.GetPtr(this), param, value);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_param_y");

        [GodotMethod("get_param_y")]
        public float GetParamY(Generic6DOFJoint.Param param)
        {
            return NativeCalls.godot_icall_1_376(method_bind_15, Object.GetPtr(this), param);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_param_z");

        [GodotMethod("set_param_z")]
        public void SetParamZ(Generic6DOFJoint.Param param, float value)
        {
            NativeCalls.godot_icall_2_375(method_bind_16, Object.GetPtr(this), param, value);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_param_z");

        [GodotMethod("get_param_z")]
        public float GetParamZ(Generic6DOFJoint.Param param)
        {
            return NativeCalls.godot_icall_1_376(method_bind_17, Object.GetPtr(this), param);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_flag_x");

        [GodotMethod("set_flag_x")]
        public void SetFlagX(Generic6DOFJoint.Flag flag, bool value)
        {
            NativeCalls.godot_icall_2_377(method_bind_18, Object.GetPtr(this), flag, value);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_flag_x");

        [GodotMethod("get_flag_x")]
        public bool GetFlagX(Generic6DOFJoint.Flag flag)
        {
            return NativeCalls.godot_icall_1_378(method_bind_19, Object.GetPtr(this), flag);
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_flag_y");

        [GodotMethod("set_flag_y")]
        public void SetFlagY(Generic6DOFJoint.Flag flag, bool value)
        {
            NativeCalls.godot_icall_2_377(method_bind_20, Object.GetPtr(this), flag, value);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_flag_y");

        [GodotMethod("get_flag_y")]
        public bool GetFlagY(Generic6DOFJoint.Flag flag)
        {
            return NativeCalls.godot_icall_1_378(method_bind_21, Object.GetPtr(this), flag);
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_flag_z");

        [GodotMethod("set_flag_z")]
        public void SetFlagZ(Generic6DOFJoint.Flag flag, bool value)
        {
            NativeCalls.godot_icall_2_377(method_bind_22, Object.GetPtr(this), flag, value);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_flag_z");

        [GodotMethod("get_flag_z")]
        public bool GetFlagZ(Generic6DOFJoint.Flag flag)
        {
            return NativeCalls.godot_icall_1_378(method_bind_23, Object.GetPtr(this), flag);
        }
    }
}
