using System;
using System.Collections.Generic;

namespace Godot
{
    public static class Geometry
    {
        private const string nativeName = "_Geometry";
        internal static IntPtr ptr = NativeCalls.godot_icall__Geometry_get_singleton();

        private static IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "build_box_planes");

        /// <summary>
        /// Returns an array with 6 [Plane]s that describe the sides of a box centered at the origin. The box size is defined by [code]extents[/code], which represents one (positive) corner of the box (i.e. half its actual size).
        /// </summary>
        [GodotMethod("build_box_planes")]
        public static object[] BuildBoxPlanes(Vector3 extents)
        {
            return NativeCalls.godot_icall_1_1042(method_bind_0, ptr, ref extents);
        }

        private static IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "build_cylinder_planes");

        /// <summary>
        /// Returns an array of [Plane]s closely bounding a faceted cylinder centered at the origin with radius [code]radius[/code] and height [code]height[/code]. The parameter [code]sides[/code] defines how many planes will be generated for the round part of the cylinder. The parameter [code]axis[/code] describes the axis along which the cylinder is oriented (0 for X, 1 for Y, 2 for Z).
        /// </summary>
        [GodotMethod("build_cylinder_planes")]
        public static object[] BuildCylinderPlanes(float radius, float height, int sides, Vector3.Axis axis = (Vector3.Axis)2)
        {
            return NativeCalls.godot_icall_4_1043(method_bind_1, ptr, radius, height, sides, axis);
        }

        private static IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "build_capsule_planes");

        /// <summary>
        /// Returns an array of [Plane]s closely bounding a faceted capsule centered at the origin with radius [code]radius[/code] and height [code]height[/code]. The parameter [code]sides[/code] defines how many planes will be generated for the side part of the capsule, whereas [code]lats[/code] gives the number of latitudinal steps at the bottom and top of the capsule. The parameter [code]axis[/code] describes the axis along which the capsule is oriented (0 for X, 1 for Y, 2 for Z).
        /// </summary>
        [GodotMethod("build_capsule_planes")]
        public static object[] BuildCapsulePlanes(float radius, float height, int sides, int lats, Vector3.Axis axis = (Vector3.Axis)2)
        {
            return NativeCalls.godot_icall_5_1044(method_bind_2, ptr, radius, height, sides, lats, axis);
        }

        private static IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "segment_intersects_circle");

        /// <summary>
        /// Given the 2d segment ([code]segment_from[/code], [code]segment_to[/code]), returns the position on the segment (as a number between 0 and 1) at which the segment hits the circle that is located at position [code]circle_position[/code] and has radius [code]circle_radius[/code]. If the segment does not intersect the circle, -1 is returned (this is also the case if the line extending the segment would intersect the circle, but the segment does not).
        /// </summary>
        [GodotMethod("segment_intersects_circle")]
        public static float SegmentIntersectsCircle(Vector2 segmentFrom, Vector2 segmentTo, Vector2 circlePosition, float circleRadius)
        {
            return NativeCalls.godot_icall_4_1045(method_bind_3, ptr, ref segmentFrom, ref segmentTo, ref circlePosition, circleRadius);
        }

        private static IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "segment_intersects_segment_2d");

        /// <summary>
        /// Checks if the two segments ([code]from_a[/code], [code]to_a[/code]) and ([code]from_b[/code], [code]to_b[/code]) intersect. If yes, return the point of intersection as [Vector2]. If no intersection takes place, returns an empty [Variant].
        /// </summary>
        [GodotMethod("segment_intersects_segment_2d")]
        public static object SegmentIntersectsSegment2d(Vector2 fromA, Vector2 toA, Vector2 fromB, Vector2 toB)
        {
            return NativeCalls.godot_icall_4_1046(method_bind_4, ptr, ref fromA, ref toA, ref fromB, ref toB);
        }

        private static IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_closest_points_between_segments_2d");

        /// <summary>
        /// Given the two 2d segments ([code]p1[/code], [code]p2[/code]) and ([code]q1[/code], [code]q2[/code]), finds those two points on the two segments that are closest to each other. Returns a [PoolVector2Array] that contains this point on ([code]p1[/code], [code]p2[/code]) as well the accompanying point on ([code]q1[/code], [code]q2[/code]).
        /// </summary>
        [GodotMethod("get_closest_points_between_segments_2d")]
        public static Vector2[] GetClosestPointsBetweenSegments2d(Vector2 p1, Vector2 q1, Vector2 p2, Vector2 q2)
        {
            return NativeCalls.godot_icall_4_1047(method_bind_5, ptr, ref p1, ref q1, ref p2, ref q2);
        }

        private static IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_closest_points_between_segments");

        /// <summary>
        /// Given the two 3d segments ([code]p1[/code], [code]p2[/code]) and ([code]q1[/code], [code]q2[/code]), finds those two points on the two segments that are closest to each other. Returns a [PoolVector3Array] that contains this point on ([code]p1[/code], [code]p2[/code]) as well the accompanying point on ([code]q1[/code], [code]q2[/code]).
        /// </summary>
        [GodotMethod("get_closest_points_between_segments")]
        public static Vector3[] GetClosestPointsBetweenSegments(Vector3 p1, Vector3 p2, Vector3 q1, Vector3 q2)
        {
            return NativeCalls.godot_icall_4_1048(method_bind_6, ptr, ref p1, ref p2, ref q1, ref q2);
        }

        private static IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_closest_point_to_segment_2d");

        /// <summary>
        /// Returns the 2d point on the 2d segment ([code]s1[/code], [code]s2[/code]) that is closest to [code]point[/code]. The returned point will always be inside the specified segment.
        /// </summary>
        [GodotMethod("get_closest_point_to_segment_2d")]
        public static Vector2 GetClosestPointToSegment2d(Vector2 point, Vector2 s1, Vector2 s2)
        {
            object ret = NativeCalls.godot_icall_3_1049(method_bind_7, ptr, ref point, ref s1, ref s2);
            return (Vector2)ret;
        }

        private static IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_closest_point_to_segment");

        /// <summary>
        /// Returns the 3d point on the 3d segment ([code]s1[/code], [code]s2[/code]) that is closest to [code]point[/code]. The returned point will always be inside the specified segment.
        /// </summary>
        [GodotMethod("get_closest_point_to_segment")]
        public static Vector3 GetClosestPointToSegment(Vector3 point, Vector3 s1, Vector3 s2)
        {
            object ret = NativeCalls.godot_icall_3_1050(method_bind_8, ptr, ref point, ref s1, ref s2);
            return (Vector3)ret;
        }

        private static IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_closest_point_to_segment_uncapped_2d");

        /// <summary>
        /// Returns the 2d point on the 2d line defined by ([code]s1[/code], [code]s2[/code]) that is closest to [code]point[/code]. The returned point can be inside the segment ([code]s1[/code], [code]s2[/code]) or outside of it, i.e. somewhere on the line extending from the segment.
        /// </summary>
        [GodotMethod("get_closest_point_to_segment_uncapped_2d")]
        public static Vector2 GetClosestPointToSegmentUncapped2d(Vector2 point, Vector2 s1, Vector2 s2)
        {
            object ret = NativeCalls.godot_icall_3_1049(method_bind_9, ptr, ref point, ref s1, ref s2);
            return (Vector2)ret;
        }

        private static IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_closest_point_to_segment_uncapped");

        /// <summary>
        /// Returns the 3d point on the 3d line defined by ([code]s1[/code], [code]s2[/code]) that is closest to [code]point[/code]. The returned point can be inside the segment ([code]s1[/code], [code]s2[/code]) or outside of it, i.e. somewhere on the line extending from the segment.
        /// </summary>
        [GodotMethod("get_closest_point_to_segment_uncapped")]
        public static Vector3 GetClosestPointToSegmentUncapped(Vector3 point, Vector3 s1, Vector3 s2)
        {
            object ret = NativeCalls.godot_icall_3_1050(method_bind_10, ptr, ref point, ref s1, ref s2);
            return (Vector3)ret;
        }

        private static IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_uv84_normal_bit");

        /// <summary>
        /// </summary>
        [GodotMethod("get_uv84_normal_bit")]
        public static int GetUv84NormalBit(Vector3 normal)
        {
            return NativeCalls.godot_icall_1_34(method_bind_11, ptr, ref normal);
        }

        private static IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "ray_intersects_triangle");

        /// <summary>
        /// Tests if the 3d ray starting at [code]from[/code] with the direction of [code]dir[/code] intersects the triangle specified by [code]a[/code], [code]b[/code] and [code]c[/code]. If yes, returns the point of intersection as [Vector3]. If no intersection takes place, an empty [Variant] is returned.
        /// </summary>
        [GodotMethod("ray_intersects_triangle")]
        public static object RayIntersectsTriangle(Vector3 from, Vector3 dir, Vector3 a, Vector3 b, Vector3 c)
        {
            return NativeCalls.godot_icall_5_1051(method_bind_12, ptr, ref from, ref dir, ref a, ref b, ref c);
        }

        private static IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "segment_intersects_triangle");

        /// <summary>
        /// Tests if the segment ([code]from[/code], [code]to[/code]) intersects the triangle [code]a[/code], [code]b[/code], [code]c[/code]. If yes, returns the point of intersection as [Vector3]. If no intersection takes place, an empty [Variant] is returned.
        /// </summary>
        [GodotMethod("segment_intersects_triangle")]
        public static object SegmentIntersectsTriangle(Vector3 from, Vector3 to, Vector3 a, Vector3 b, Vector3 c)
        {
            return NativeCalls.godot_icall_5_1051(method_bind_13, ptr, ref from, ref to, ref a, ref b, ref c);
        }

        private static IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "segment_intersects_sphere");

        /// <summary>
        /// Checks if the segment ([code]from[/code], [code]to[/code]) intersects the sphere that is located at [code]sphere_position[/code] and has radius [code]sphere_radius[/code]. If no, returns an empty [PoolVector3Array]. If yes, returns a [PoolVector3Array] containing the point of intersection and the sphere's normal at the point of intersection.
        /// </summary>
        [GodotMethod("segment_intersects_sphere")]
        public static Vector3[] SegmentIntersectsSphere(Vector3 from, Vector3 to, Vector3 spherePosition, float sphereRadius)
        {
            return NativeCalls.godot_icall_4_1052(method_bind_14, ptr, ref from, ref to, ref spherePosition, sphereRadius);
        }

        private static IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "segment_intersects_cylinder");

        /// <summary>
        /// Checks if the segment ([code]from[/code], [code]to[/code]) intersects the cylinder with height [code]height[/code] that is centered at the origin and has radius [code]radius[/code]. If no, returns an empty [PoolVector3Array]. If an intersection takes place, the returned array contains the point of intersection and the cylinder's normal at the point of intersection.
        /// </summary>
        [GodotMethod("segment_intersects_cylinder")]
        public static Vector3[] SegmentIntersectsCylinder(Vector3 from, Vector3 to, float height, float radius)
        {
            return NativeCalls.godot_icall_4_1053(method_bind_15, ptr, ref from, ref to, height, radius);
        }

        private static IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "segment_intersects_convex");

        /// <summary>
        /// Given a convex hull defined though the [Plane]s in the array [code]planes[/code], tests if the segment ([code]from[/code], [code]to[/code]) intersects with that hull. If an intersection is found, returns a [PoolVector3Array] containing the point the intersection and the hull's normal. If no intersecion is found, an the returned array is empty.
        /// </summary>
        [GodotMethod("segment_intersects_convex")]
        public static Vector3[] SegmentIntersectsConvex(Vector3 from, Vector3 to, object[] planes)
        {
            return NativeCalls.godot_icall_3_1054(method_bind_16, ptr, ref from, ref to, planes);
        }

        private static IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "point_is_inside_triangle");

        /// <summary>
        /// Returns if [code]point[/code] is inside the triangle specified by [code]a[/code], [code]b[/code] and [code]c[/code].
        /// </summary>
        [GodotMethod("point_is_inside_triangle")]
        public static bool PointIsInsideTriangle(Vector2 point, Vector2 a, Vector2 b, Vector2 c)
        {
            return NativeCalls.godot_icall_4_1055(method_bind_17, ptr, ref point, ref a, ref b, ref c);
        }

        private static IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "triangulate_polygon");

        /// <summary>
        /// Triangulates the polygon specified by the points in [code]polygon[/code]. Returns a [PoolIntArray] where each triangle consists of three consecutive point indices into [code]polygon[/code] (i.e. the returned array will have [code]n * 3[/code] elements, with [code]n[/code] being the number of found triangles). If the triangulation did not succeed, an empty [PoolIntArray] is returned.
        /// </summary>
        [GodotMethod("triangulate_polygon")]
        public static int[] TriangulatePolygon(Vector2[] polygon)
        {
            return NativeCalls.godot_icall_1_1056(method_bind_18, ptr, polygon);
        }

        private static IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "convex_hull_2d");

        /// <summary>
        /// Given an array of [Vector2]s, returns the convex hull as a list of points in counter-clockwise order. The last point is the same as the first one.
        /// </summary>
        [GodotMethod("convex_hull_2d")]
        public static Vector2[] ConvexHull2d(Vector2[] points)
        {
            return NativeCalls.godot_icall_1_1057(method_bind_19, ptr, points);
        }

        private static IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clip_polygon");

        /// <summary>
        /// Clips the polygon defined by the points in [code]points[/code] against the [code]plane[/code] and returns the points of the clipped polygon.
        /// </summary>
        [GodotMethod("clip_polygon")]
        public static Vector3[] ClipPolygon(Vector3[] points, Plane plane)
        {
            return NativeCalls.godot_icall_2_1058(method_bind_20, ptr, points, ref plane);
        }

        private static IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "make_atlas");

        /// <summary>
        /// Given an array of [Vector2]s representing tiles, builds an atlas. The returned dictionary has two keys: [code]points[/code] is a vector of [Vector2] that specifies the positions of each tile, [code]size[/code] contains the overall size of the whole atlas as [Vector2].
        /// </summary>
        [GodotMethod("make_atlas")]
        public static Dictionary<object, object> MakeAtlas(Vector2[] sizes)
        {
            return NativeCalls.godot_icall_1_1059(method_bind_21, ptr, sizes);
        }
    }
}
