using System;

namespace Godot
{
    /// <summary>
    /// Given a set of colors, this node will interpolate them in order, meaning, that if you have color 1, color 2 and color3, the ramp will interpolate (generate the colors between two colors) from color 1 to color 2 and from color 2 to color 3. Initially the ramp will have 2 colors (black and white), one (black) at ramp lower offset offset 0 and the other (white) at the ramp higher offset 1.
    /// </summary>
    public class Gradient : Resource
    {
        /// <summary>
        /// Gradient's offsets returned as a [PoolRealArray].
        /// </summary>
        public float[] Offsets
        {
            get
            {
                return GetOffsets();
            }
            set
            {
                SetOffsets(value);
            }
        }

        /// <summary>
        /// Gradient's colors returned as a [PoolColorArray].
        /// </summary>
        public Color[] Colors
        {
            get
            {
                return GetColors();
            }
            set
            {
                SetColors(value);
            }
        }


        private const string nativeName = "Gradient";

        public Gradient() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Gradient_Ctor(this);
        }

        internal Gradient(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_point");

        /// <summary>
        /// Adds the specified color to the end of the ramp, with the specified offset
        /// </summary>
        [GodotMethod("add_point")]
        public void AddPoint(float offset, Color color)
        {
            NativeCalls.godot_icall_2_383(method_bind_0, Object.GetPtr(this), offset, ref color);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_point");

        /// <summary>
        /// Removes the color at the index [i]offset[/i]
        /// </summary>
        [GodotMethod("remove_point")]
        public void RemovePoint(int offset)
        {
            NativeCalls.godot_icall_1_0(method_bind_1, Object.GetPtr(this), offset);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_offset");

        /// <summary>
        /// Sets the offset for the ramp color at index [i]point[/i]
        /// </summary>
        [GodotMethod("set_offset")]
        public void SetOffset(int point, float offset)
        {
            NativeCalls.godot_icall_2_28(method_bind_2, Object.GetPtr(this), point, offset);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_offset");

        /// <summary>
        /// Returns the offset of the ramp color at index [i]point[/i]
        /// </summary>
        [GodotMethod("get_offset")]
        public float GetOffset(int point)
        {
            return NativeCalls.godot_icall_1_7(method_bind_3, Object.GetPtr(this), point);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_color");

        /// <summary>
        /// Sets the color of the ramp color at index [i]point[/i]
        /// </summary>
        [GodotMethod("set_color")]
        public void SetColor(int point, Color color)
        {
            NativeCalls.godot_icall_2_384(method_bind_4, Object.GetPtr(this), point, ref color);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_color");

        /// <summary>
        /// Returns the color of the ramp color at index [i]point[/i]
        /// </summary>
        [GodotMethod("get_color")]
        public Color GetColor(int point)
        {
            object ret = NativeCalls.godot_icall_1_385(method_bind_5, Object.GetPtr(this), point);
            return (Color)ret;
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "interpolate");

        /// <summary>
        /// Returns the interpolated color specified by [i]offset[/i]
        /// </summary>
        [GodotMethod("interpolate")]
        public Color Interpolate(float offset)
        {
            object ret = NativeCalls.godot_icall_1_386(method_bind_6, Object.GetPtr(this), offset);
            return (Color)ret;
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_point_count");

        /// <summary>
        /// Returns the number of colors in the ramp
        /// </summary>
        [GodotMethod("get_point_count")]
        public int GetPointCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_offsets");

        [GodotMethod("set_offsets")]
        public void SetOffsets(float[] offsets)
        {
            NativeCalls.godot_icall_1_387(method_bind_8, Object.GetPtr(this), offsets);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_offsets");

        [GodotMethod("get_offsets")]
        public float[] GetOffsets()
        {
            return NativeCalls.godot_icall_0_297(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_colors");

        [GodotMethod("set_colors")]
        public void SetColors(Color[] colors)
        {
            NativeCalls.godot_icall_1_388(method_bind_10, Object.GetPtr(this), colors);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_colors");

        [GodotMethod("get_colors")]
        public Color[] GetColors()
        {
            return NativeCalls.godot_icall_0_389(method_bind_11, Object.GetPtr(this));
        }
    }
}
