using System;

namespace Godot
{
    /// <summary>
    /// Uses a [Gradient] to fill the texture data, the gradient will be filled from left to right using colors obtained from the gradient, this means that the texture does not necessarily represent an exact copy of the gradient, but instead an interpolation of samples obtained from the gradient at fixed steps (see [method set_width]).
    /// </summary>
    public class GradientTexture : Texture
    {
        /// <summary>
        /// The [Gradient] that will be used to fill the texture.
        /// </summary>
        public Gradient Gradient
        {
            get
            {
                return GetGradient();
            }
            set
            {
                SetGradient(value);
            }
        }

        /// <summary>
        /// The number of color samples that will be obtained from the [Gradient].
        /// </summary>
        public int Width
        {
            get
            {
                return GetWidth();
            }
            set
            {
                SetWidth(value);
            }
        }


        private const string nativeName = "GradientTexture";

        public GradientTexture() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_GradientTexture_Ctor(this);
        }

        internal GradientTexture(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_gradient");

        [GodotMethod("set_gradient")]
        public void SetGradient(Gradient gradient)
        {
            NativeCalls.godot_icall_1_24(method_bind_0, Object.GetPtr(this), Object.GetPtr(gradient));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_gradient");

        [GodotMethod("get_gradient")]
        public Gradient GetGradient()
        {
            return NativeCalls.godot_icall_0_390(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_width");

        [GodotMethod("set_width")]
        public void SetWidth(int width)
        {
            NativeCalls.godot_icall_1_0(method_bind_2, Object.GetPtr(this), width);
        }
    }
}
