using System;

namespace Godot
{
    /// <summary>
    /// A GraphNode is a container defined by a title. It can have 1 or more input and output slots, which can be enabled (shown) or disabled (not shown) and have different (incompatible) types. Colors can also be assigned to slots. A tuple of input and output slots is defined for each GUI element included in the GraphNode. Input and output connections are left and right slots, but only enabled slots are counted as connections.
    /// </summary>
    public class GraphNode : Container
    {
        public enum OverlayEnum
        {
            Disabled = 0,
            Breakpoint = 1,
            Position = 2
        }


        /// <summary>
        /// </summary>
        public string Title
        {
            get
            {
                return GetTitle();
            }
            set
            {
                SetTitle(value);
            }
        }

        /// <summary>
        /// The offset of the GraphNode, relative to the scroll offset of the [GraphEdit]. Note that you cannot use position directly, as [GraphEdit] is a [Container].
        /// </summary>
        public Vector2 Offset
        {
            get
            {
                return GetOffset();
            }
            set
            {
                SetOffset(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool ShowClose
        {
            get
            {
                return IsCloseButtonVisible();
            }
            set
            {
                SetShowCloseButton(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool Resizable
        {
            get
            {
                return IsResizable();
            }
            set
            {
                SetResizable(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool Selected
        {
            get
            {
                return IsSelected();
            }
            set
            {
                SetSelected(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool Comment
        {
            get
            {
                return IsComment();
            }
            set
            {
                SetComment(value);
            }
        }

        /// <summary>
        /// </summary>
        public GraphNode.OverlayEnum Overlay
        {
            get
            {
                return GetOverlay();
            }
            set
            {
                SetOverlay(value);
            }
        }


        private const string nativeName = "GraphNode";

        public GraphNode() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_GraphNode_Ctor(this);
        }

        internal GraphNode(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_title");

        [GodotMethod("set_title")]
        public void SetTitle(string title)
        {
            NativeCalls.godot_icall_1_43(method_bind_0, Object.GetPtr(this), title);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_title");

        [GodotMethod("get_title")]
        public string GetTitle()
        {
            return NativeCalls.godot_icall_0_2(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_slot");

        /// <summary>
        /// </summary>
        [GodotMethod("set_slot")]
        public void SetSlot(int idx, bool enableLeft, int typeLeft, Color colorLeft, bool enableRight, int typeRight, Color colorRight, Texture customLeft = null, Texture customRight = null)
        {
            NativeCalls.godot_icall_9_394(method_bind_2, Object.GetPtr(this), idx, enableLeft, typeLeft, ref colorLeft, enableRight, typeRight, ref colorRight, Object.GetPtr(customLeft), Object.GetPtr(customRight));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear_slot");

        /// <summary>
        /// Disable input and output slot whose index is 'idx'.
        /// </summary>
        [GodotMethod("clear_slot")]
        public void ClearSlot(int idx)
        {
            NativeCalls.godot_icall_1_0(method_bind_3, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear_all_slots");

        /// <summary>
        /// Disable all input and output slots of the GraphNode.
        /// </summary>
        [GodotMethod("clear_all_slots")]
        public void ClearAllSlots()
        {
            NativeCalls.godot_icall_0_12(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_slot_enabled_left");

        /// <summary>
        /// Return true if left (input) slot 'idx' is enabled. False otherwise.
        /// </summary>
        [GodotMethod("is_slot_enabled_left")]
        public bool IsSlotEnabledLeft(int idx)
        {
            return NativeCalls.godot_icall_1_29(method_bind_5, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_slot_type_left");

        /// <summary>
        /// Return the (integer) type of left (input) 'idx' slot.
        /// </summary>
        [GodotMethod("get_slot_type_left")]
        public int GetSlotTypeLeft(int idx)
        {
            return NativeCalls.godot_icall_1_6(method_bind_6, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_slot_color_left");

        /// <summary>
        /// Return the color set to 'idx' left (input) slot.
        /// </summary>
        [GodotMethod("get_slot_color_left")]
        public Color GetSlotColorLeft(int idx)
        {
            object ret = NativeCalls.godot_icall_1_385(method_bind_7, Object.GetPtr(this), idx);
            return (Color)ret;
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_slot_enabled_right");

        /// <summary>
        /// Return true if right (output) slot 'idx' is enabled. False otherwise.
        /// </summary>
        [GodotMethod("is_slot_enabled_right")]
        public bool IsSlotEnabledRight(int idx)
        {
            return NativeCalls.godot_icall_1_29(method_bind_8, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_slot_type_right");

        /// <summary>
        /// Return the (integer) type of right (output) 'idx' slot.
        /// </summary>
        [GodotMethod("get_slot_type_right")]
        public int GetSlotTypeRight(int idx)
        {
            return NativeCalls.godot_icall_1_6(method_bind_9, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_slot_color_right");

        /// <summary>
        /// Return the color set to 'idx' right (output) slot.
        /// </summary>
        [GodotMethod("get_slot_color_right")]
        public Color GetSlotColorRight(int idx)
        {
            object ret = NativeCalls.godot_icall_1_385(method_bind_10, Object.GetPtr(this), idx);
            return (Color)ret;
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_offset");

        [GodotMethod("set_offset")]
        public void SetOffset(Vector2 offset)
        {
            NativeCalls.godot_icall_1_45(method_bind_11, Object.GetPtr(this), ref offset);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_offset");

        [GodotMethod("get_offset")]
        public Vector2 GetOffset()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_12, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_comment");

        [GodotMethod("set_comment")]
        public void SetComment(bool comment)
        {
            NativeCalls.godot_icall_1_11(method_bind_13, Object.GetPtr(this), comment);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_comment");

        [GodotMethod("is_comment")]
        public bool IsComment()
        {
            return NativeCalls.godot_icall_0_3(method_bind_14, Object.GetPtr(this));
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_resizable");

        [GodotMethod("set_resizable")]
        public void SetResizable(bool resizable)
        {
            NativeCalls.godot_icall_1_11(method_bind_15, Object.GetPtr(this), resizable);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_resizable");

        [GodotMethod("is_resizable")]
        public bool IsResizable()
        {
            return NativeCalls.godot_icall_0_3(method_bind_16, Object.GetPtr(this));
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_selected");

        [GodotMethod("set_selected")]
        public void SetSelected(bool selected)
        {
            NativeCalls.godot_icall_1_11(method_bind_17, Object.GetPtr(this), selected);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_selected");

        [GodotMethod("is_selected")]
        public bool IsSelected()
        {
            return NativeCalls.godot_icall_0_3(method_bind_18, Object.GetPtr(this));
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_connection_output_count");

        /// <summary>
        /// Return the number of enabled output slots (connections) of the GraphNode.
        /// </summary>
        [GodotMethod("get_connection_output_count")]
        public int GetConnectionOutputCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_19, Object.GetPtr(this));
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_connection_input_count");

        /// <summary>
        /// Return the number of enabled input slots (connections) to the GraphNode.
        /// </summary>
        [GodotMethod("get_connection_input_count")]
        public int GetConnectionInputCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_20, Object.GetPtr(this));
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_connection_output_position");

        /// <summary>
        /// Return the position of the output connection 'idx'.
        /// </summary>
        [GodotMethod("get_connection_output_position")]
        public Vector2 GetConnectionOutputPosition(int idx)
        {
            object ret = NativeCalls.godot_icall_1_282(method_bind_21, Object.GetPtr(this), idx);
            return (Vector2)ret;
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_connection_output_type");

        /// <summary>
        /// Return the type of the output connection 'idx'.
        /// </summary>
        [GodotMethod("get_connection_output_type")]
        public int GetConnectionOutputType(int idx)
        {
            return NativeCalls.godot_icall_1_6(method_bind_22, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_connection_output_color");

        /// <summary>
        /// Return the color of the output connection 'idx'.
        /// </summary>
        [GodotMethod("get_connection_output_color")]
        public Color GetConnectionOutputColor(int idx)
        {
            object ret = NativeCalls.godot_icall_1_385(method_bind_23, Object.GetPtr(this), idx);
            return (Color)ret;
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_connection_input_position");

        /// <summary>
        /// Return the position of the input connection 'idx'.
        /// </summary>
        [GodotMethod("get_connection_input_position")]
        public Vector2 GetConnectionInputPosition(int idx)
        {
            object ret = NativeCalls.godot_icall_1_282(method_bind_24, Object.GetPtr(this), idx);
            return (Vector2)ret;
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_connection_input_type");

        /// <summary>
        /// Return the type of the input connection 'idx'.
        /// </summary>
        [GodotMethod("get_connection_input_type")]
        public int GetConnectionInputType(int idx)
        {
            return NativeCalls.godot_icall_1_6(method_bind_25, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_connection_input_color");

        /// <summary>
        /// Return the color of the input connection 'idx'.
        /// </summary>
        [GodotMethod("get_connection_input_color")]
        public Color GetConnectionInputColor(int idx)
        {
            object ret = NativeCalls.godot_icall_1_385(method_bind_26, Object.GetPtr(this), idx);
            return (Color)ret;
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_show_close_button");

        [GodotMethod("set_show_close_button")]
        public void SetShowCloseButton(bool show)
        {
            NativeCalls.godot_icall_1_11(method_bind_27, Object.GetPtr(this), show);
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_close_button_visible");

        [GodotMethod("is_close_button_visible")]
        public bool IsCloseButtonVisible()
        {
            return NativeCalls.godot_icall_0_3(method_bind_28, Object.GetPtr(this));
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_overlay");

        [GodotMethod("set_overlay")]
        public void SetOverlay(GraphNode.OverlayEnum overlay)
        {
            NativeCalls.godot_icall_1_395(method_bind_29, Object.GetPtr(this), overlay);
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_overlay");

        [GodotMethod("get_overlay")]
        public GraphNode.OverlayEnum GetOverlay()
        {
            return NativeCalls.godot_icall_0_396(method_bind_30, Object.GetPtr(this));
        }
    }
}
