using System;

namespace Godot
{
    /// <summary>
    /// Grid container will arrange its children in a grid like structure, the grid columns are specified using the [method set_columns] method and the number of rows will be equal to the number of children in the container divided by the number of columns, for example: if the container has 5 children, and 2 columns, there will be 3 rows in the container. Notice that grid layout will preserve the columns and rows for every size of the container.
    /// </summary>
    public class GridContainer : Container
    {
        /// <summary>
        /// The number of columns in the [code]GridContainer[/code]. If modified, [code]GridContainer[/code] reorders its children to accommodate the new layout.
        /// </summary>
        public int Columns
        {
            get
            {
                return GetColumns();
            }
            set
            {
                SetColumns(value);
            }
        }


        private const string nativeName = "GridContainer";

        public GridContainer() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_GridContainer_Ctor(this);
        }

        internal GridContainer(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_columns");

        [GodotMethod("set_columns")]
        public void SetColumns(int columns)
        {
            NativeCalls.godot_icall_1_0(method_bind_0, Object.GetPtr(this), columns);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_columns");

        [GodotMethod("get_columns")]
        public int GetColumns()
        {
            return NativeCalls.godot_icall_0_1(method_bind_1, Object.GetPtr(this));
        }
    }
}
