using System;

namespace Godot
{
    /// <summary>
    /// Groove constraint for 2D physics. This is useful for making a body "slide" through a segment placed in another.
    /// </summary>
    public class GrooveJoint2D : Joint2D
    {
        /// <summary>
        /// The groove's length. The groove is from the joint's origin towards [member length] along the joint's local y axis. Default value: [code]50[/code]
        /// </summary>
        public float Length
        {
            get
            {
                return GetLength();
            }
            set
            {
                SetLength(value);
            }
        }

        /// <summary>
        /// The body B's initial anchor position defined by the joint's origin and a local offset [member initial_offset] along the joint's y axis (along the groove). Default value: [code]25[/code]
        /// </summary>
        public float InitialOffset
        {
            get
            {
                return GetInitialOffset();
            }
            set
            {
                SetInitialOffset(value);
            }
        }


        private const string nativeName = "GrooveJoint2D";

        public GrooveJoint2D() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_GrooveJoint2D_Ctor(this);
        }

        internal GrooveJoint2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_length");

        [GodotMethod("set_length")]
        public void SetLength(float length)
        {
            NativeCalls.godot_icall_1_10(method_bind_0, Object.GetPtr(this), length);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_length");

        [GodotMethod("get_length")]
        public float GetLength()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_initial_offset");

        [GodotMethod("set_initial_offset")]
        public void SetInitialOffset(float offset)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), offset);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_initial_offset");

        [GodotMethod("get_initial_offset")]
        public float GetInitialOffset()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }
    }
}
