using System;

namespace Godot
{
    /// <summary>
    /// Normaly uses the z-axis of body A as the hinge axis, another axis can be specified when adding it manually though.
    /// </summary>
    public class HingeJoint : Joint
    {
        public enum Param
        {
            Bias = 0,
            LimitUpper = 1,
            LimitLower = 2,
            LimitBias = 3,
            LimitSoftness = 4,
            LimitRelaxation = 5,
            MotorTargetVelocity = 6,
            MotorMaxImpulse = 7,
            Max = 8
        }

        public enum Flag
        {
            UseLimit = 0,
            EnableMotor = 1,
            Max = 2
        }


        /// <summary>
        /// The speed with wich the two bodies get pulled together when they move in different directions.
        /// </summary>
        public float Params__bias
        {
            get
            {
                return GetParam((HingeJoint.Param)0);
            }
            set
            {
                SetParam((HingeJoint.Param)0, value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the hinges maximum and minimum rotation, defined by [member angular_limit/lower] and [member angular_limit/upper] has effects.
        /// </summary>
        public bool AngularLimit__enable
        {
            get
            {
                return GetFlag((HingeJoint.Flag)0);
            }
            set
            {
                SetFlag((HingeJoint.Flag)0, value);
            }
        }

        /// <summary>
        /// The maximum rotation. only active if [member angular_limit/enable] is [code]true[/code].
        /// </summary>
        public float AngularLimit__upper
        {
            get
            {
                return _GetUpperLimit();
            }
            set
            {
                _SetUpperLimit(value);
            }
        }

        /// <summary>
        /// The minimum rotation. only active if [member angular_limit/enable] is [code]true[/code].
        /// </summary>
        public float AngularLimit__lower
        {
            get
            {
                return _GetLowerLimit();
            }
            set
            {
                _SetLowerLimit(value);
            }
        }

        /// <summary>
        /// The speed with which the rotation across the axis perpendicular to the hinge gets corrected.
        /// </summary>
        public float AngularLimit__bias
        {
            get
            {
                return GetParam((HingeJoint.Param)3);
            }
            set
            {
                SetParam((HingeJoint.Param)3, value);
            }
        }

        /// <summary>
        /// </summary>
        public float AngularLimit__softness
        {
            get
            {
                return GetParam((HingeJoint.Param)4);
            }
            set
            {
                SetParam((HingeJoint.Param)4, value);
            }
        }

        /// <summary>
        /// The lower this value, the more the rotation gets slowed down.
        /// </summary>
        public float AngularLimit__relaxation
        {
            get
            {
                return GetParam((HingeJoint.Param)5);
            }
            set
            {
                SetParam((HingeJoint.Param)5, value);
            }
        }

        /// <summary>
        /// When activated, a motor turns the hinge.
        /// </summary>
        public bool Motor__enable
        {
            get
            {
                return GetFlag((HingeJoint.Flag)1);
            }
            set
            {
                SetFlag((HingeJoint.Flag)1, value);
            }
        }

        /// <summary>
        /// Target speed for the motor.
        /// </summary>
        public float Motor__targetVelocity
        {
            get
            {
                return GetParam((HingeJoint.Param)6);
            }
            set
            {
                SetParam((HingeJoint.Param)6, value);
            }
        }

        /// <summary>
        /// Maximum acceleration for the motor.
        /// </summary>
        public float Motor__maxImpulse
        {
            get
            {
                return GetParam((HingeJoint.Param)7);
            }
            set
            {
                SetParam((HingeJoint.Param)7, value);
            }
        }


        private const string nativeName = "HingeJoint";

        public HingeJoint() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_HingeJoint_Ctor(this);
        }

        internal HingeJoint(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_param");

        [GodotMethod("set_param")]
        public void SetParam(HingeJoint.Param param, float value)
        {
            NativeCalls.godot_icall_2_411(method_bind_0, Object.GetPtr(this), param, value);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_param");

        [GodotMethod("get_param")]
        public float GetParam(HingeJoint.Param param)
        {
            return NativeCalls.godot_icall_1_412(method_bind_1, Object.GetPtr(this), param);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_flag");

        [GodotMethod("set_flag")]
        public void SetFlag(HingeJoint.Flag flag, bool enabled)
        {
            NativeCalls.godot_icall_2_413(method_bind_2, Object.GetPtr(this), flag, enabled);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_flag");

        [GodotMethod("get_flag")]
        public bool GetFlag(HingeJoint.Flag flag)
        {
            return NativeCalls.godot_icall_1_414(method_bind_3, Object.GetPtr(this), flag);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_upper_limit");

        internal void _SetUpperLimit(float upperLimit)
        {
            NativeCalls.godot_icall_1_10(method_bind_4, Object.GetPtr(this), upperLimit);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_upper_limit");

        internal float _GetUpperLimit()
        {
            return NativeCalls.godot_icall_0_9(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_lower_limit");

        internal void _SetLowerLimit(float lowerLimit)
        {
            NativeCalls.godot_icall_1_10(method_bind_6, Object.GetPtr(this), lowerLimit);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_lower_limit");

        internal float _GetLowerLimit()
        {
            return NativeCalls.godot_icall_0_9(method_bind_7, Object.GetPtr(this));
        }
    }
}
