using System;

namespace Godot
{
    /// <summary>
    /// IP contains support functions for the Internet Protocol (IP). TCP/IP support is in different classes (see [StreamPeerTCP] and [TCP_Server]). IP provides DNS hostname resolution support, both blocking and threaded.
    /// </summary>
    public static class IP
    {
        public const int ResolverMaxQueries = 32;
        public const int ResolverInvalidId = -1;

        public enum ResolverStatus
        {
            None = 0,
            Waiting = 1,
            Done = 2,
            Error = 3
        }

        public enum Type
        {
            None = 0,
            Ipv4 = 1,
            Ipv6 = 2,
            Any = 3
        }


        private const string nativeName = "IP";
        internal static IntPtr ptr = NativeCalls.godot_icall_IP_get_singleton();

        private static IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "resolve_hostname");

        /// <summary>
        /// Returns a given hostname's IPv4 or IPv6 address when resolved (blocking-type method). The address type returned depends on the TYPE_* constant given as "ip_type".
        /// </summary>
        [GodotMethod("resolve_hostname")]
        public static string ResolveHostname(string host, IP.Type ipType = (IP.Type)3)
        {
            return NativeCalls.godot_icall_2_415(method_bind_0, ptr, host, ipType);
        }

        private static IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "resolve_hostname_queue_item");

        /// <summary>
        /// Creates a queue item to resolve a hostname to an IPv4 or IPv6 address depending on the TYPE_* constant given as "ip_type". Returns the queue ID if successful, or RESOLVER_INVALID_ID on error.
        /// </summary>
        [GodotMethod("resolve_hostname_queue_item")]
        public static int ResolveHostnameQueueItem(string host, IP.Type ipType = (IP.Type)3)
        {
            return NativeCalls.godot_icall_2_416(method_bind_1, ptr, host, ipType);
        }

        private static IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_resolve_item_status");

        /// <summary>
        /// Returns a queued hostname's status as a RESOLVER_STATUS_* constant, given its queue "id".
        /// </summary>
        [GodotMethod("get_resolve_item_status")]
        public static IP.ResolverStatus GetResolveItemStatus(int id)
        {
            return NativeCalls.godot_icall_1_417(method_bind_2, ptr, id);
        }

        private static IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_resolve_item_address");

        /// <summary>
        /// Returns a queued hostname's IP address, given its queue "id". Returns an empty string on error or if resolution hasn't happened yet (see [method get_resolve_item_status]).
        /// </summary>
        [GodotMethod("get_resolve_item_address")]
        public static string GetResolveItemAddress(int id)
        {
            return NativeCalls.godot_icall_1_108(method_bind_3, ptr, id);
        }

        private static IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "erase_resolve_item");

        /// <summary>
        /// Removes a given item "id" from the queue. This should be used to free a queue after it has completed to enable more queries to happen.
        /// </summary>
        [GodotMethod("erase_resolve_item")]
        public static void EraseResolveItem(int id)
        {
            NativeCalls.godot_icall_1_0(method_bind_4, ptr, id);
        }

        private static IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_local_addresses");

        /// <summary>
        /// Returns all of the user's current IPv4 and IPv6 addresses as an array.
        /// </summary>
        [GodotMethod("get_local_addresses")]
        public static object[] GetLocalAddresses()
        {
            return NativeCalls.godot_icall_0_21(method_bind_5, ptr);
        }

        private static IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear_cache");

        /// <summary>
        /// Removes all of a "hostname"'s cached references. If no "hostname" is given then all cached IP addresses are removed.
        /// </summary>
        [GodotMethod("clear_cache")]
        public static void ClearCache(string hostname = "")
        {
            NativeCalls.godot_icall_1_43(method_bind_6, ptr, hostname);
        }
    }
}
