using System;

namespace Godot
{
    /// <summary>
    /// Draws simple geometry from code. Uses a drawing mode similar to OpenGL 1.x.
    /// </summary>
    public class ImmediateGeometry : GeometryInstance
    {
        private const string nativeName = "ImmediateGeometry";

        public ImmediateGeometry() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_ImmediateGeometry_Ctor(this);
        }

        internal ImmediateGeometry(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "begin");

        /// <summary>
        /// Begin drawing (And optionally pass a texture override). When done call end(). For more information on how this works, search for glBegin() glEnd() references.
        /// For the type of primitive, use the [Mesh].PRIMITIVE_* enumerations.
        /// </summary>
        [GodotMethod("begin")]
        public void Begin(Mesh.PrimitiveType primitive, Texture texture = null)
        {
            NativeCalls.godot_icall_2_435(method_bind_0, Object.GetPtr(this), primitive, Object.GetPtr(texture));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_normal");

        /// <summary>
        /// The next vertex's normal.
        /// </summary>
        [GodotMethod("set_normal")]
        public void SetNormal(Vector3 normal)
        {
            NativeCalls.godot_icall_1_104(method_bind_1, Object.GetPtr(this), ref normal);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tangent");

        /// <summary>
        /// The next vertex's tangent (and binormal facing).
        /// </summary>
        [GodotMethod("set_tangent")]
        public void SetTangent(Plane tangent)
        {
            NativeCalls.godot_icall_1_436(method_bind_2, Object.GetPtr(this), ref tangent);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_color");

        /// <summary>
        /// The current drawing color.
        /// </summary>
        [GodotMethod("set_color")]
        public void SetColor(Color color)
        {
            NativeCalls.godot_icall_1_202(method_bind_3, Object.GetPtr(this), ref color);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_uv");

        /// <summary>
        /// The next vertex's UV.
        /// </summary>
        [GodotMethod("set_uv")]
        public void SetUv(Vector2 uv)
        {
            NativeCalls.godot_icall_1_45(method_bind_4, Object.GetPtr(this), ref uv);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_uv2");

        /// <summary>
        /// The next vertex's second layer UV.
        /// </summary>
        [GodotMethod("set_uv2")]
        public void SetUv2(Vector2 uv)
        {
            NativeCalls.godot_icall_1_45(method_bind_5, Object.GetPtr(this), ref uv);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_vertex");

        /// <summary>
        /// Adds a vertex with the currently set color/uv/etc.
        /// </summary>
        [GodotMethod("add_vertex")]
        public void AddVertex(Vector3 position)
        {
            NativeCalls.godot_icall_1_104(method_bind_6, Object.GetPtr(this), ref position);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_sphere");

        /// <summary>
        /// Simple helper to draw a uvsphere, with given latitudes, longitude and radius.
        /// </summary>
        [GodotMethod("add_sphere")]
        public void AddSphere(int lats, int lons, float radius, bool addUv = true)
        {
            NativeCalls.godot_icall_4_437(method_bind_7, Object.GetPtr(this), lats, lons, radius, addUv);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "end");

        /// <summary>
        /// Ends a drawing context and displays the results.
        /// </summary>
        [GodotMethod("end")]
        public void End()
        {
            NativeCalls.godot_icall_0_12(method_bind_8, Object.GetPtr(this));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear");

        /// <summary>
        /// Clears everything that was drawn using begin/end.
        /// </summary>
        [GodotMethod("clear")]
        public void Clear()
        {
            NativeCalls.godot_icall_0_12(method_bind_9, Object.GetPtr(this));
        }
    }
}
