using System;

namespace Godot
{
    /// <summary>
    /// Contains a generic action which can be targeted from several type of inputs. Actions can be created from the project settings menu [code]Project &gt; Project Settings &gt; Input Map[/code]. See [method Node._input].
    /// </summary>
    public class InputEventAction : InputEvent
    {
        /// <summary>
        /// The action's name. Actions are accessed via this [String].
        /// </summary>
        public string Action
        {
            get
            {
                return GetAction();
            }
            set
            {
                SetAction(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the action's state is pressed. If [code]false[/code] the action's state is released.
        /// </summary>
        public bool Pressed
        {
            get
            {
                return IsPressed();
            }
            set
            {
                SetPressed(value);
            }
        }


        private const string nativeName = "InputEventAction";

        public InputEventAction() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_InputEventAction_Ctor(this);
        }

        internal InputEventAction(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_action");

        [GodotMethod("set_action")]
        public void SetAction(string action)
        {
            NativeCalls.godot_icall_1_43(method_bind_0, Object.GetPtr(this), action);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_action");

        [GodotMethod("get_action")]
        public string GetAction()
        {
            return NativeCalls.godot_icall_0_2(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_pressed");

        [GodotMethod("set_pressed")]
        public void SetPressed(bool pressed)
        {
            NativeCalls.godot_icall_1_11(method_bind_2, Object.GetPtr(this), pressed);
        }
    }
}
