using System;

namespace Godot
{
    /// <summary>
    /// Input event type for gamepad buttons. For joysticks see [InputEventJoypadMotion].
    /// </summary>
    public class InputEventJoypadButton : InputEvent
    {
        /// <summary>
        /// Button identifier. One of the [code]JOY_BUTTON_*[/code] constants from [@global Scope].
        /// </summary>
        public int ButtonIndex
        {
            get
            {
                return GetButtonIndex();
            }
            set
            {
                SetButtonIndex(value);
            }
        }

        /// <summary>
        /// Represents the pressure the user puts on the button with his finger, if the controller supports it. Ranges from [code]0[/code] to [code]1[/code].
        /// </summary>
        public float Pressure
        {
            get
            {
                return GetPressure();
            }
            set
            {
                SetPressure(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the button's state is pressed. If [code]false[/code] the button's state is released.
        /// </summary>
        public bool Pressed
        {
            get
            {
                return IsPressed();
            }
            set
            {
                SetPressed(value);
            }
        }


        private const string nativeName = "InputEventJoypadButton";

        public InputEventJoypadButton() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_InputEventJoypadButton_Ctor(this);
        }

        internal InputEventJoypadButton(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_button_index");

        [GodotMethod("set_button_index")]
        public void SetButtonIndex(int buttonIndex)
        {
            NativeCalls.godot_icall_1_0(method_bind_0, Object.GetPtr(this), buttonIndex);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_button_index");

        [GodotMethod("get_button_index")]
        public int GetButtonIndex()
        {
            return NativeCalls.godot_icall_0_1(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_pressure");

        [GodotMethod("set_pressure")]
        public void SetPressure(float pressure)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), pressure);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_pressure");

        [GodotMethod("get_pressure")]
        public float GetPressure()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_pressed");

        [GodotMethod("set_pressed")]
        public void SetPressed(bool pressed)
        {
            NativeCalls.godot_icall_1_11(method_bind_4, Object.GetPtr(this), pressed);
        }
    }
}
