using System;

namespace Godot
{
    /// <summary>
    /// Stores information about joystick motions. One [code]InputEventJoypadMotion[/code] represents one axis at a time.
    /// </summary>
    public class InputEventJoypadMotion : InputEvent
    {
        /// <summary>
        /// Axis identifier. Use one of the [code]JOY_AXIS_*[/code] constants in [@global Scope].
        /// </summary>
        public int Axis
        {
            get
            {
                return GetAxis();
            }
            set
            {
                SetAxis(value);
            }
        }

        /// <summary>
        /// Current position of the joystick on the given axis. The value ranges from [code]-1.0[/code] to [code]1.0[/code]. A value of [code]0[/code] means the axis is in its resting position.
        /// </summary>
        public float AxisValue
        {
            get
            {
                return GetAxisValue();
            }
            set
            {
                SetAxisValue(value);
            }
        }


        private const string nativeName = "InputEventJoypadMotion";

        public InputEventJoypadMotion() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_InputEventJoypadMotion_Ctor(this);
        }

        internal InputEventJoypadMotion(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_axis");

        [GodotMethod("set_axis")]
        public void SetAxis(int axis)
        {
            NativeCalls.godot_icall_1_0(method_bind_0, Object.GetPtr(this), axis);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_axis");

        [GodotMethod("get_axis")]
        public int GetAxis()
        {
            return NativeCalls.godot_icall_0_1(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_axis_value");

        [GodotMethod("set_axis_value")]
        public void SetAxisValue(float axisValue)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), axisValue);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_axis_value");

        [GodotMethod("get_axis_value")]
        public float GetAxisValue()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }
    }
}
