using System;

namespace Godot
{
    /// <summary>
    /// Stores general mouse events informations.
    /// </summary>
    public class InputEventMouse : InputEventWithModifiers
    {
        /// <summary>
        /// Mouse button mask identifier, one of or a bitwise combination of the BUTTON_MASK_* constants in [@GlobalScope].
        /// </summary>
        public int ButtonMask
        {
            get
            {
                return GetButtonMask();
            }
            set
            {
                SetButtonMask(value);
            }
        }

        /// <summary>
        /// Mouse local position relative to the [Viewport]. If used in [method Control._gui_input] the position is relative to the current [Control] wich is under the mouse.
        /// </summary>
        public Vector2 Position
        {
            get
            {
                return GetPosition();
            }
            set
            {
                SetPosition(value);
            }
        }

        /// <summary>
        /// Mouse position relative to the current [Viewport] when used in [method Control._gui_input], otherwise is at 0,0.
        /// </summary>
        public Vector2 GlobalPosition
        {
            get
            {
                return GetGlobalPosition();
            }
            set
            {
                SetGlobalPosition(value);
            }
        }


        private const string nativeName = "InputEventMouse";

        internal InputEventMouse() {}

        internal InputEventMouse(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_button_mask");

        [GodotMethod("set_button_mask")]
        public void SetButtonMask(int buttonMask)
        {
            NativeCalls.godot_icall_1_0(method_bind_0, Object.GetPtr(this), buttonMask);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_button_mask");

        [GodotMethod("get_button_mask")]
        public int GetButtonMask()
        {
            return NativeCalls.godot_icall_0_1(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_position");

        [GodotMethod("set_position")]
        public void SetPosition(Vector2 position)
        {
            NativeCalls.godot_icall_1_45(method_bind_2, Object.GetPtr(this), ref position);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_position");

        [GodotMethod("get_position")]
        public Vector2 GetPosition()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_3, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_global_position");

        [GodotMethod("set_global_position")]
        public void SetGlobalPosition(Vector2 globalPosition)
        {
            NativeCalls.godot_icall_1_45(method_bind_4, Object.GetPtr(this), ref globalPosition);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_global_position");

        [GodotMethod("get_global_position")]
        public Vector2 GetGlobalPosition()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_5, Object.GetPtr(this));
            return (Vector2)ret;
        }
    }
}
