using System;

namespace Godot
{
    /// <summary>
    /// Contains mouse motion informations. Supports relative, absolute positions and speed. See [method Node._input].
    /// </summary>
    public class InputEventMouseMotion : InputEventMouse
    {
        /// <summary>
        /// Mouse position relative to the previous position (position at the last frame).
        /// </summary>
        public Vector2 Relative
        {
            get
            {
                return GetRelative();
            }
            set
            {
                SetRelative(value);
            }
        }

        /// <summary>
        /// Mouse speed.
        /// </summary>
        public Vector2 Speed
        {
            get
            {
                return GetSpeed();
            }
            set
            {
                SetSpeed(value);
            }
        }


        private const string nativeName = "InputEventMouseMotion";

        public InputEventMouseMotion() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_InputEventMouseMotion_Ctor(this);
        }

        internal InputEventMouseMotion(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_relative");

        [GodotMethod("set_relative")]
        public void SetRelative(Vector2 relative)
        {
            NativeCalls.godot_icall_1_45(method_bind_0, Object.GetPtr(this), ref relative);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_relative");

        [GodotMethod("get_relative")]
        public Vector2 GetRelative()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_1, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_speed");

        [GodotMethod("set_speed")]
        public void SetSpeed(Vector2 speed)
        {
            NativeCalls.godot_icall_1_45(method_bind_2, Object.GetPtr(this), ref speed);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_speed");

        [GodotMethod("get_speed")]
        public Vector2 GetSpeed()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_3, Object.GetPtr(this));
            return (Vector2)ret;
        }
    }
}
