using System;

namespace Godot
{
    /// <summary>
    /// Contains screen drag informations. See [method Node._input].
    /// </summary>
    public class InputEventScreenDrag : InputEvent
    {
        /// <summary>
        /// Drag event index in the case of a multi-drag event.
        /// </summary>
        public int Index
        {
            get
            {
                return GetIndex();
            }
            set
            {
                SetIndex(value);
            }
        }

        /// <summary>
        /// Drag position.
        /// </summary>
        public Vector2 Position
        {
            get
            {
                return GetPosition();
            }
            set
            {
                SetPosition(value);
            }
        }

        /// <summary>
        /// Drag position relative to its start position.
        /// </summary>
        public Vector2 Relative
        {
            get
            {
                return GetRelative();
            }
            set
            {
                SetRelative(value);
            }
        }

        /// <summary>
        /// Drag speed.
        /// </summary>
        public Vector2 Speed
        {
            get
            {
                return GetSpeed();
            }
            set
            {
                SetSpeed(value);
            }
        }


        private const string nativeName = "InputEventScreenDrag";

        public InputEventScreenDrag() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_InputEventScreenDrag_Ctor(this);
        }

        internal InputEventScreenDrag(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_index");

        [GodotMethod("set_index")]
        public void SetIndex(int index)
        {
            NativeCalls.godot_icall_1_0(method_bind_0, Object.GetPtr(this), index);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_index");

        [GodotMethod("get_index")]
        public int GetIndex()
        {
            return NativeCalls.godot_icall_0_1(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_position");

        [GodotMethod("set_position")]
        public void SetPosition(Vector2 position)
        {
            NativeCalls.godot_icall_1_45(method_bind_2, Object.GetPtr(this), ref position);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_position");

        [GodotMethod("get_position")]
        public Vector2 GetPosition()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_3, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_relative");

        [GodotMethod("set_relative")]
        public void SetRelative(Vector2 relative)
        {
            NativeCalls.godot_icall_1_45(method_bind_4, Object.GetPtr(this), ref relative);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_relative");

        [GodotMethod("get_relative")]
        public Vector2 GetRelative()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_5, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_speed");

        [GodotMethod("set_speed")]
        public void SetSpeed(Vector2 speed)
        {
            NativeCalls.godot_icall_1_45(method_bind_6, Object.GetPtr(this), ref speed);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_speed");

        [GodotMethod("get_speed")]
        public Vector2 GetSpeed()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_7, Object.GetPtr(this));
            return (Vector2)ret;
        }
    }
}
