using System;

namespace Godot
{
    /// <summary>
    /// Stores multi-touch press/release information. Supports touch press, touch release and [member index] for multi-touch count and order.
    /// </summary>
    public class InputEventScreenTouch : InputEvent
    {
        /// <summary>
        /// Touch index in the case of a multi-touch event. One index = one finger.
        /// </summary>
        public int Index
        {
            get
            {
                return GetIndex();
            }
            set
            {
                SetIndex(value);
            }
        }

        /// <summary>
        /// Touch position.
        /// </summary>
        public Vector2 Position
        {
            get
            {
                return GetPosition();
            }
            set
            {
                SetPosition(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the touch's state is pressed. If [code]false[/code] the touch's state is released.
        /// </summary>
        public bool Pressed
        {
            get
            {
                return IsPressed();
            }
            set
            {
                SetPressed(value);
            }
        }


        private const string nativeName = "InputEventScreenTouch";

        public InputEventScreenTouch() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_InputEventScreenTouch_Ctor(this);
        }

        internal InputEventScreenTouch(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_index");

        [GodotMethod("set_index")]
        public void SetIndex(int index)
        {
            NativeCalls.godot_icall_1_0(method_bind_0, Object.GetPtr(this), index);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_index");

        [GodotMethod("get_index")]
        public int GetIndex()
        {
            return NativeCalls.godot_icall_0_1(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_position");

        [GodotMethod("set_position")]
        public void SetPosition(Vector2 position)
        {
            NativeCalls.godot_icall_1_45(method_bind_2, Object.GetPtr(this), ref position);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_position");

        [GodotMethod("get_position")]
        public Vector2 GetPosition()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_3, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_pressed");

        [GodotMethod("set_pressed")]
        public void SetPressed(bool pressed)
        {
            NativeCalls.godot_icall_1_11(method_bind_4, Object.GetPtr(this), pressed);
        }
    }
}
