using System;

namespace Godot
{
    /// <summary>
    /// Contains keys events informations with modifiers support like [code]SHIFT[/code] or [code]ALT[/code]. See [method Node._input].
    /// </summary>
    public class InputEventWithModifiers : InputEvent
    {
        /// <summary>
        /// State of the Alt modifier.
        /// </summary>
        public bool Alt
        {
            get
            {
                return GetAlt();
            }
            set
            {
                SetAlt(value);
            }
        }

        /// <summary>
        /// State of the Shift modifier.
        /// </summary>
        public bool Shift
        {
            get
            {
                return GetShift();
            }
            set
            {
                SetShift(value);
            }
        }

        /// <summary>
        /// State of the Ctrl modifier.
        /// </summary>
        public bool Control
        {
            get
            {
                return GetControl();
            }
            set
            {
                SetControl(value);
            }
        }

        /// <summary>
        /// State of the Meta modifier.
        /// </summary>
        public bool Meta
        {
            get
            {
                return GetMetakey();
            }
            set
            {
                SetMetakey(value);
            }
        }

        /// <summary>
        /// State of the Command modifier.
        /// </summary>
        public bool Command
        {
            get
            {
                return GetCommand();
            }
            set
            {
                SetCommand(value);
            }
        }


        private const string nativeName = "InputEventWithModifiers";

        internal InputEventWithModifiers() {}

        internal InputEventWithModifiers(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_alt");

        [GodotMethod("set_alt")]
        public void SetAlt(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_0, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_alt");

        [GodotMethod("get_alt")]
        public bool GetAlt()
        {
            return NativeCalls.godot_icall_0_3(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_shift");

        [GodotMethod("set_shift")]
        public void SetShift(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_2, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_shift");

        [GodotMethod("get_shift")]
        public bool GetShift()
        {
            return NativeCalls.godot_icall_0_3(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_control");

        [GodotMethod("set_control")]
        public void SetControl(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_4, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_control");

        [GodotMethod("get_control")]
        public bool GetControl()
        {
            return NativeCalls.godot_icall_0_3(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_metakey");

        [GodotMethod("set_metakey")]
        public void SetMetakey(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_6, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_metakey");

        [GodotMethod("get_metakey")]
        public bool GetMetakey()
        {
            return NativeCalls.godot_icall_0_3(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_command");

        [GodotMethod("set_command")]
        public void SetCommand(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_8, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_command");

        [GodotMethod("get_command")]
        public bool GetCommand()
        {
            return NativeCalls.godot_icall_0_3(method_bind_9, Object.GetPtr(this));
        }
    }
}
