using System;

namespace Godot
{
    /// <summary>
    /// Manages all [InputEventAction] which can be created/modified from the project settings menu [code]Project &gt; Project Settings &gt; Input Map[/code] or in code with [method add_action] and [method action_add_event]. See [method Node._input].
    /// </summary>
    public static class InputMap
    {
        private const string nativeName = "InputMap";
        internal static IntPtr ptr = NativeCalls.godot_icall_InputMap_get_singleton();

        private static IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_action");

        /// <summary>
        /// Returns [code]true[/code] if the [code]InputMap[/code] has a registered action with the given name.
        /// </summary>
        [GodotMethod("has_action")]
        public static bool HasAction(string action)
        {
            return NativeCalls.godot_icall_1_70(method_bind_0, ptr, action);
        }

        private static IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_actions");

        /// <summary>
        /// Returns an array of all actions in the [code]InputMap[/code].
        /// </summary>
        [GodotMethod("get_actions")]
        public static object[] GetActions()
        {
            return NativeCalls.godot_icall_0_21(method_bind_1, ptr);
        }

        private static IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_action");

        /// <summary>
        /// Adds an (empty) action to the [code]InputMap[/code]. An [InputEvent] can then be added to this action with [method action_add_event].
        /// </summary>
        [GodotMethod("add_action")]
        public static void AddAction(string action)
        {
            NativeCalls.godot_icall_1_43(method_bind_2, ptr, action);
        }

        private static IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "erase_action");

        /// <summary>
        /// Removes an action from the [code]InputMap[/code].
        /// </summary>
        [GodotMethod("erase_action")]
        public static void EraseAction(string action)
        {
            NativeCalls.godot_icall_1_43(method_bind_3, ptr, action);
        }

        private static IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "action_add_event");

        /// <summary>
        /// Adds an [InputEvent] to an action. This [InputEvent] will trigger the action.
        /// </summary>
        [GodotMethod("action_add_event")]
        public static void ActionAddEvent(string action, InputEvent @event)
        {
            NativeCalls.godot_icall_2_89(method_bind_4, ptr, action, Object.GetPtr(@event));
        }

        private static IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "action_has_event");

        /// <summary>
        /// Returns [true] if an action has an [InputEvent] associated with it.
        /// </summary>
        [GodotMethod("action_has_event")]
        public static bool ActionHasEvent(string action, InputEvent @event)
        {
            return NativeCalls.godot_icall_2_445(method_bind_5, ptr, action, Object.GetPtr(@event));
        }

        private static IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "action_erase_event");

        /// <summary>
        /// Removes an [InputEvent] from an action.
        /// </summary>
        [GodotMethod("action_erase_event")]
        public static void ActionEraseEvent(string action, InputEvent @event)
        {
            NativeCalls.godot_icall_2_89(method_bind_6, ptr, action, Object.GetPtr(@event));
        }

        private static IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_action_list");

        /// <summary>
        /// Returns an array of [InputEvent]s associated with a given action.
        /// </summary>
        [GodotMethod("get_action_list")]
        public static object[] GetActionList(string action)
        {
            return NativeCalls.godot_icall_1_446(method_bind_7, ptr, action);
        }

        private static IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "event_is_action");

        /// <summary>
        /// Returns [true] if the given event is part of an existing action. This method ignores keyboard modifiers if the given [InputEvent] is not pressed (for proper release detection). See [method action_has_event] if you don't want this behavior.
        /// </summary>
        [GodotMethod("event_is_action")]
        public static bool EventIsAction(InputEvent @event, string action)
        {
            return NativeCalls.godot_icall_2_447(method_bind_8, ptr, Object.GetPtr(@event), action);
        }

        private static IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "load_from_globals");

        /// <summary>
        /// Clears all [InputEventAction] in the [code]InputMap[/code] and load it anew from [ProjectSettings].
        /// </summary>
        [GodotMethod("load_from_globals")]
        public static void LoadFromGlobals()
        {
            NativeCalls.godot_icall_0_12(method_bind_9, ptr);
        }
    }
}
