using System;
using System.Collections.Generic;

namespace Godot
{
    /// <summary>
    /// Turning on the option [b]Load As Placeholder[/b] for an instanced scene in the editor causes it to be replaced by an InstacePlaceholder when running the game. This makes it possible to delay actually loading the scene until calling [method replace_by_instance]. This is useful to avoid loading large scenes all at once by loading parts of it selectively.
    /// The InstancePlaceholder does not have a transform. This causes any child nodes to be positioned relatively to the Viewport from point (0,0), rather than their parent as displayed in the editor. Replacing the placeholder with a scene with a transform will transform children relatively to their parent again.
    /// </summary>
    public class InstancePlaceholder : Node
    {
        private const string nativeName = "InstancePlaceholder";

        internal InstancePlaceholder() {}

        internal InstancePlaceholder(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_stored_values");

        /// <summary>
        /// </summary>
        [GodotMethod("get_stored_values")]
        public Dictionary<object, object> GetStoredValues(bool withOrder = false)
        {
            return NativeCalls.godot_icall_1_448(method_bind_0, Object.GetPtr(this), withOrder);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "replace_by_instance");

        /// <summary>
        /// Replace this placeholder by the scene handed as an argument, or the original scene if no argument is given. As for all resources, the scene is loaded only if it's not loaded already. By manually loading the scene beforehand, delays caused by this function can be avoided.
        /// </summary>
        [GodotMethod("replace_by_instance")]
        public void ReplaceByInstance(PackedScene customScene = null)
        {
            NativeCalls.godot_icall_1_24(method_bind_1, Object.GetPtr(this), Object.GetPtr(customScene));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_instance_path");

        /// <summary>
        /// Retrieve the path to the [PackedScene] resource file that is loaded by default when calling [method replace_by_instance].
        /// </summary>
        [GodotMethod("get_instance_path")]
        public string GetInstancePath()
        {
            return NativeCalls.godot_icall_0_2(method_bind_2, Object.GetPtr(this));
        }
    }
}
