using System;

namespace Godot
{
    /// <summary>
    /// This control provides a selectable list of items that may be in a single (or multiple columns) with option of text, icons,
    /// or both text and icon.  Tooltips are supported and may be different for every item in the list.  Selectable items in the list
    /// may be selected or deselected and multiple selection may be enabled.  Selection with right mouse button may also be enabled
    /// to allow use of popup context menus.  Items may also be 'activated' with a double click (or Enter key).
    /// </summary>
    public class ItemList : Control
    {
        public enum SelectModeEnum
        {
            Single = 0,
            Multi = 1
        }

        public enum IconModeEnum
        {
            Top = 0,
            Left = 1
        }


        public object[] Items
        {
            get
            {
                return _GetItems();
            }
            set
            {
                _SetItems(value);
            }
        }

        /// <summary>
        /// </summary>
        public ItemList.SelectModeEnum SelectMode
        {
            get
            {
                return GetSelectMode();
            }
            set
            {
                SetSelectMode(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool AllowRmbSelect
        {
            get
            {
                return GetAllowRmbSelect();
            }
            set
            {
                SetAllowRmbSelect(value);
            }
        }

        /// <summary>
        /// </summary>
        public int MaxTextLines
        {
            get
            {
                return GetMaxTextLines();
            }
            set
            {
                SetMaxTextLines(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool AutoHeight
        {
            get
            {
                return HasAutoHeight();
            }
            set
            {
                SetAutoHeight(value);
            }
        }

        /// <summary>
        /// </summary>
        public int MaxColumns
        {
            get
            {
                return GetMaxColumns();
            }
            set
            {
                SetMaxColumns(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool SameColumnWidth
        {
            get
            {
                return IsSameColumnWidth();
            }
            set
            {
                SetSameColumnWidth(value);
            }
        }

        /// <summary>
        /// </summary>
        public int FixedColumnWidth
        {
            get
            {
                return GetFixedColumnWidth();
            }
            set
            {
                SetFixedColumnWidth(value);
            }
        }

        /// <summary>
        /// </summary>
        public ItemList.IconModeEnum IconMode
        {
            get
            {
                return GetIconMode();
            }
            set
            {
                SetIconMode(value);
            }
        }

        /// <summary>
        /// </summary>
        public float IconScale
        {
            get
            {
                return GetIconScale();
            }
            set
            {
                SetIconScale(value);
            }
        }

        /// <summary>
        /// </summary>
        public Vector2 FixedIconSize
        {
            get
            {
                return GetFixedIconSize();
            }
            set
            {
                SetFixedIconSize(value);
            }
        }


        private const string nativeName = "ItemList";

        public ItemList() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_ItemList_Ctor(this);
        }

        internal ItemList(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_item");

        /// <summary>
        /// Adds an item to the item list with specified text.  Specify an icon of null for a list item with no icon.
        /// If selectable is true the list item will be selectable.
        /// </summary>
        [GodotMethod("add_item")]
        public void AddItem(string text, Texture icon = null, bool selectable = true)
        {
            NativeCalls.godot_icall_3_449(method_bind_0, Object.GetPtr(this), text, Object.GetPtr(icon), selectable);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_icon_item");

        /// <summary>
        /// Adds an item to the item list with no text, only an icon.
        /// </summary>
        [GodotMethod("add_icon_item")]
        public void AddIconItem(Texture icon, bool selectable = true)
        {
            NativeCalls.godot_icall_2_450(method_bind_1, Object.GetPtr(this), Object.GetPtr(icon), selectable);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_text");

        /// <summary>
        /// Sets text of item at specified index.
        /// </summary>
        [GodotMethod("set_item_text")]
        public void SetItemText(int idx, string text)
        {
            NativeCalls.godot_icall_2_115(method_bind_2, Object.GetPtr(this), idx, text);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_text");

        /// <summary>
        /// Return the text for specified item index.
        /// </summary>
        [GodotMethod("get_item_text")]
        public string GetItemText(int idx)
        {
            return NativeCalls.godot_icall_1_108(method_bind_3, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_icon");

        /// <summary>
        /// Set (or replace) icon of the item at the specified index.
        /// </summary>
        [GodotMethod("set_item_icon")]
        public void SetItemIcon(int idx, Texture icon)
        {
            NativeCalls.godot_icall_2_67(method_bind_4, Object.GetPtr(this), idx, Object.GetPtr(icon));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_icon");

        /// <summary>
        /// </summary>
        [GodotMethod("get_item_icon")]
        public Texture GetItemIcon(int idx)
        {
            return NativeCalls.godot_icall_1_157(method_bind_5, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_icon_region");

        /// <summary>
        /// </summary>
        [GodotMethod("set_item_icon_region")]
        public void SetItemIconRegion(int idx, Rect2 rect)
        {
            NativeCalls.godot_icall_2_451(method_bind_6, Object.GetPtr(this), idx, ref rect);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_icon_region");

        /// <summary>
        /// </summary>
        [GodotMethod("get_item_icon_region")]
        public Rect2 GetItemIconRegion(int idx)
        {
            object ret = NativeCalls.godot_icall_1_452(method_bind_7, Object.GetPtr(this), idx);
            return (Rect2)ret;
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_selectable");

        /// <summary>
        /// Allow or disallow selection of the item at the specified index.
        /// </summary>
        [GodotMethod("set_item_selectable")]
        public void SetItemSelectable(int idx, bool selectable)
        {
            NativeCalls.godot_icall_2_51(method_bind_8, Object.GetPtr(this), idx, selectable);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_item_selectable");

        /// <summary>
        /// Returns whether or not the item at the specified index is selectable.
        /// </summary>
        [GodotMethod("is_item_selectable")]
        public bool IsItemSelectable(int idx)
        {
            return NativeCalls.godot_icall_1_29(method_bind_9, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_disabled");

        /// <summary>
        /// Disable (or enable) item at specified index.
        /// Disabled items are not be selectable and do not fire activation (Enter or double-click) signals.
        /// </summary>
        [GodotMethod("set_item_disabled")]
        public void SetItemDisabled(int idx, bool disabled)
        {
            NativeCalls.godot_icall_2_51(method_bind_10, Object.GetPtr(this), idx, disabled);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_item_disabled");

        /// <summary>
        /// Returns whether or not the item at the specified index is disabled
        /// </summary>
        [GodotMethod("is_item_disabled")]
        public bool IsItemDisabled(int idx)
        {
            return NativeCalls.godot_icall_1_29(method_bind_11, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_metadata");

        /// <summary>
        /// Sets a value (of any type) to be stored with the item at the specified index.
        /// </summary>
        [GodotMethod("set_item_metadata")]
        public void SetItemMetadata(int idx, object metadata)
        {
            NativeCalls.godot_icall_2_453(method_bind_12, Object.GetPtr(this), idx, metadata);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_metadata");

        /// <summary>
        /// </summary>
        [GodotMethod("get_item_metadata")]
        public object GetItemMetadata(int idx)
        {
            return NativeCalls.godot_icall_1_454(method_bind_13, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_custom_bg_color");

        /// <summary>
        /// </summary>
        [GodotMethod("set_item_custom_bg_color")]
        public void SetItemCustomBgColor(int idx, Color customBgColor)
        {
            NativeCalls.godot_icall_2_384(method_bind_14, Object.GetPtr(this), idx, ref customBgColor);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_custom_bg_color");

        /// <summary>
        /// </summary>
        [GodotMethod("get_item_custom_bg_color")]
        public Color GetItemCustomBgColor(int idx)
        {
            object ret = NativeCalls.godot_icall_1_385(method_bind_15, Object.GetPtr(this), idx);
            return (Color)ret;
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_tooltip_enabled");

        /// <summary>
        /// Sets whether the tooltip is enabled for specified item index.
        /// </summary>
        [GodotMethod("set_item_tooltip_enabled")]
        public void SetItemTooltipEnabled(int idx, bool enable)
        {
            NativeCalls.godot_icall_2_51(method_bind_16, Object.GetPtr(this), idx, enable);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_item_tooltip_enabled");

        /// <summary>
        /// Returns whether the tooltip is enabled for specified item index.
        /// </summary>
        [GodotMethod("is_item_tooltip_enabled")]
        public bool IsItemTooltipEnabled(int idx)
        {
            return NativeCalls.godot_icall_1_29(method_bind_17, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_tooltip");

        /// <summary>
        /// Sets tooltip hint for item at specified index.
        /// </summary>
        [GodotMethod("set_item_tooltip")]
        public void SetItemTooltip(int idx, string tooltip)
        {
            NativeCalls.godot_icall_2_115(method_bind_18, Object.GetPtr(this), idx, tooltip);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_tooltip");

        /// <summary>
        /// Return tooltip hint for specified item index.
        /// </summary>
        [GodotMethod("get_item_tooltip")]
        public string GetItemTooltip(int idx)
        {
            return NativeCalls.godot_icall_1_108(method_bind_19, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "select");

        /// <summary>
        /// Select the item at the specified index.
        /// Note:  This method does not trigger the item selection signal.
        /// </summary>
        [GodotMethod("select")]
        public void Select(int idx, bool single = true)
        {
            NativeCalls.godot_icall_2_51(method_bind_20, Object.GetPtr(this), idx, single);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "unselect");

        /// <summary>
        /// Ensure item at specified index is not selected.
        /// </summary>
        [GodotMethod("unselect")]
        public void Unselect(int idx)
        {
            NativeCalls.godot_icall_1_0(method_bind_21, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_selected");

        /// <summary>
        /// Returns whether or not item at the specified index is currently selected.
        /// </summary>
        [GodotMethod("is_selected")]
        public bool IsSelected(int idx)
        {
            return NativeCalls.godot_icall_1_29(method_bind_22, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_selected_items");

        /// <summary>
        /// Returns the list of selected indexes.
        /// </summary>
        [GodotMethod("get_selected_items")]
        public int[] GetSelectedItems()
        {
            return NativeCalls.godot_icall_0_175(method_bind_23, Object.GetPtr(this));
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_count");

        /// <summary>
        /// Return count of items currently in the item list.
        /// </summary>
        [GodotMethod("get_item_count")]
        public int GetItemCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_24, Object.GetPtr(this));
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_item");

        /// <summary>
        /// Remove item at specified index from the list.
        /// </summary>
        [GodotMethod("remove_item")]
        public void RemoveItem(int idx)
        {
            NativeCalls.godot_icall_1_0(method_bind_25, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear");

        /// <summary>
        /// Remove all items from the list.
        /// </summary>
        [GodotMethod("clear")]
        public void Clear()
        {
            NativeCalls.godot_icall_0_12(method_bind_26, Object.GetPtr(this));
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "sort_items_by_text");

        /// <summary>
        /// Sorts items in the list by their text.
        /// </summary>
        [GodotMethod("sort_items_by_text")]
        public void SortItemsByText()
        {
            NativeCalls.godot_icall_0_12(method_bind_27, Object.GetPtr(this));
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_fixed_column_width");

        [GodotMethod("set_fixed_column_width")]
        public void SetFixedColumnWidth(int width)
        {
            NativeCalls.godot_icall_1_0(method_bind_28, Object.GetPtr(this), width);
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_fixed_column_width");

        [GodotMethod("get_fixed_column_width")]
        public int GetFixedColumnWidth()
        {
            return NativeCalls.godot_icall_0_1(method_bind_29, Object.GetPtr(this));
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_same_column_width");

        [GodotMethod("set_same_column_width")]
        public void SetSameColumnWidth(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_30, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_same_column_width");

        [GodotMethod("is_same_column_width")]
        public bool IsSameColumnWidth()
        {
            return NativeCalls.godot_icall_0_3(method_bind_31, Object.GetPtr(this));
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_max_text_lines");

        [GodotMethod("set_max_text_lines")]
        public void SetMaxTextLines(int lines)
        {
            NativeCalls.godot_icall_1_0(method_bind_32, Object.GetPtr(this), lines);
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_max_text_lines");

        [GodotMethod("get_max_text_lines")]
        public int GetMaxTextLines()
        {
            return NativeCalls.godot_icall_0_1(method_bind_33, Object.GetPtr(this));
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_max_columns");

        [GodotMethod("set_max_columns")]
        public void SetMaxColumns(int amount)
        {
            NativeCalls.godot_icall_1_0(method_bind_34, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_max_columns");

        [GodotMethod("get_max_columns")]
        public int GetMaxColumns()
        {
            return NativeCalls.godot_icall_0_1(method_bind_35, Object.GetPtr(this));
        }

        private IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_select_mode");

        [GodotMethod("set_select_mode")]
        public void SetSelectMode(ItemList.SelectModeEnum mode)
        {
            NativeCalls.godot_icall_1_455(method_bind_36, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_select_mode");

        [GodotMethod("get_select_mode")]
        public ItemList.SelectModeEnum GetSelectMode()
        {
            return NativeCalls.godot_icall_0_456(method_bind_37, Object.GetPtr(this));
        }

        private IntPtr method_bind_38 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_icon_mode");

        [GodotMethod("set_icon_mode")]
        public void SetIconMode(ItemList.IconModeEnum mode)
        {
            NativeCalls.godot_icall_1_457(method_bind_38, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_39 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_icon_mode");

        [GodotMethod("get_icon_mode")]
        public ItemList.IconModeEnum GetIconMode()
        {
            return NativeCalls.godot_icall_0_458(method_bind_39, Object.GetPtr(this));
        }

        private IntPtr method_bind_40 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_fixed_icon_size");

        [GodotMethod("set_fixed_icon_size")]
        public void SetFixedIconSize(Vector2 size)
        {
            NativeCalls.godot_icall_1_45(method_bind_40, Object.GetPtr(this), ref size);
        }

        private IntPtr method_bind_41 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_fixed_icon_size");

        [GodotMethod("get_fixed_icon_size")]
        public Vector2 GetFixedIconSize()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_41, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_42 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_icon_scale");

        [GodotMethod("set_icon_scale")]
        public void SetIconScale(float scale)
        {
            NativeCalls.godot_icall_1_10(method_bind_42, Object.GetPtr(this), scale);
        }

        private IntPtr method_bind_43 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_icon_scale");

        [GodotMethod("get_icon_scale")]
        public float GetIconScale()
        {
            return NativeCalls.godot_icall_0_9(method_bind_43, Object.GetPtr(this));
        }

        private IntPtr method_bind_44 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_allow_rmb_select");

        [GodotMethod("set_allow_rmb_select")]
        public void SetAllowRmbSelect(bool allow)
        {
            NativeCalls.godot_icall_1_11(method_bind_44, Object.GetPtr(this), allow);
        }

        private IntPtr method_bind_45 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_allow_rmb_select");

        [GodotMethod("get_allow_rmb_select")]
        public bool GetAllowRmbSelect()
        {
            return NativeCalls.godot_icall_0_3(method_bind_45, Object.GetPtr(this));
        }

        private IntPtr method_bind_46 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_auto_height");

        [GodotMethod("set_auto_height")]
        public void SetAutoHeight(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_46, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_47 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_auto_height");

        [GodotMethod("has_auto_height")]
        public bool HasAutoHeight()
        {
            return NativeCalls.godot_icall_0_3(method_bind_47, Object.GetPtr(this));
        }

        private IntPtr method_bind_48 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_at_position");

        /// <summary>
        /// Given a position within the control return the item (if any) at that point.
        /// </summary>
        [GodotMethod("get_item_at_position")]
        public int GetItemAtPosition(Vector2 position, bool exact = false)
        {
            return NativeCalls.godot_icall_2_459(method_bind_48, Object.GetPtr(this), ref position, exact);
        }

        private IntPtr method_bind_49 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "ensure_current_is_visible");

        /// <summary>
        /// Ensure selection is visible, adjusting the scroll position as necessary.
        /// </summary>
        [GodotMethod("ensure_current_is_visible")]
        public void EnsureCurrentIsVisible()
        {
            NativeCalls.godot_icall_0_12(method_bind_49, Object.GetPtr(this));
        }

        private IntPtr method_bind_50 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_v_scroll");

        /// <summary>
        /// Returns the current vertical scroll bar for the List.
        /// </summary>
        [GodotMethod("get_v_scroll")]
        public VScrollBar GetVScroll()
        {
            return NativeCalls.godot_icall_0_460(method_bind_50, Object.GetPtr(this));
        }

        private IntPtr method_bind_51 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_items");

        internal void _SetItems(object[] arg0)
        {
            NativeCalls.godot_icall_1_154(method_bind_51, Object.GetPtr(this), arg0);
        }

        private IntPtr method_bind_52 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_items");

        internal object[] _GetItems()
        {
            return NativeCalls.godot_icall_0_21(method_bind_52, Object.GetPtr(this));
        }
    }
}
