using System;

namespace Godot
{
    /// <summary>
    /// All 3D joints link two nodes, has a priority, and can decide if the two bodies of the nodes should be able to collide with each other
    /// </summary>
    public class Joint : Spatial
    {
        /// <summary>
        /// The [Node], the first side of the Joint attaches to.
        /// </summary>
        public NodePath Nodes__nodeA
        {
            get
            {
                return GetNodeA();
            }
            set
            {
                SetNodeA(value);
            }
        }

        /// <summary>
        /// The [Node], the second side of the Joint attaches to.
        /// </summary>
        public NodePath Nodes__nodeB
        {
            get
            {
                return GetNodeB();
            }
            set
            {
                SetNodeB(value);
            }
        }

        /// <summary>
        /// The order in wich the solver is executed compared to the other [Joints], the lower, the earlier.
        /// </summary>
        public int Solver__priority
        {
            get
            {
                return GetSolverPriority();
            }
            set
            {
                SetSolverPriority(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the two bodies of the nodes are not able to collide with each other.
        /// </summary>
        public bool Collision__excludeNodes
        {
            get
            {
                return GetExcludeNodesFromCollision();
            }
            set
            {
                SetExcludeNodesFromCollision(value);
            }
        }


        private const string nativeName = "Joint";

        internal Joint() {}

        internal Joint(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_node_a");

        [GodotMethod("set_node_a")]
        public void SetNodeA(NodePath node)
        {
            NativeCalls.godot_icall_1_78(method_bind_0, Object.GetPtr(this), NodePath.GetPtr(node));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_node_a");

        [GodotMethod("get_node_a")]
        public NodePath GetNodeA()
        {
            IntPtr ret = NativeCalls.godot_icall_0_79(method_bind_1, Object.GetPtr(this));
            return new NodePath(ret);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_node_b");

        [GodotMethod("set_node_b")]
        public void SetNodeB(NodePath node)
        {
            NativeCalls.godot_icall_1_78(method_bind_2, Object.GetPtr(this), NodePath.GetPtr(node));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_node_b");

        [GodotMethod("get_node_b")]
        public NodePath GetNodeB()
        {
            IntPtr ret = NativeCalls.godot_icall_0_79(method_bind_3, Object.GetPtr(this));
            return new NodePath(ret);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_solver_priority");

        [GodotMethod("set_solver_priority")]
        public void SetSolverPriority(int priority)
        {
            NativeCalls.godot_icall_1_0(method_bind_4, Object.GetPtr(this), priority);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_solver_priority");

        [GodotMethod("get_solver_priority")]
        public int GetSolverPriority()
        {
            return NativeCalls.godot_icall_0_1(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_exclude_nodes_from_collision");

        [GodotMethod("set_exclude_nodes_from_collision")]
        public void SetExcludeNodesFromCollision(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_6, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_exclude_nodes_from_collision");

        [GodotMethod("get_exclude_nodes_from_collision")]
        public bool GetExcludeNodesFromCollision()
        {
            return NativeCalls.godot_icall_0_3(method_bind_7, Object.GetPtr(this));
        }
    }
}
