using System;

namespace Godot
{
    /// <summary>
    /// Base node for all joint constraints in 2D physics. Joints take 2 bodies and apply a custom constraint.
    /// </summary>
    public class Joint2D : Node2D
    {
        /// <summary>
        /// The first body attached to the joint. Must derive from [PhysicsBody2D].
        /// </summary>
        public NodePath NodeA
        {
            get
            {
                return GetNodeA();
            }
            set
            {
                SetNodeA(value);
            }
        }

        /// <summary>
        /// The second body attached to the joint. Must derive from [PhysicsBody2D].
        /// </summary>
        public NodePath NodeB
        {
            get
            {
                return GetNodeB();
            }
            set
            {
                SetNodeB(value);
            }
        }

        /// <summary>
        /// When [member node_a] and [member node_b] move in different directions the [code]bias[/code] controls how fast the joint pulls them back to their original position. The lower the [code]bias[/code] the more the two bodies can pull on the joint. Default value: [code]0[/code]
        /// </summary>
        public float Bias
        {
            get
            {
                return GetBias();
            }
            set
            {
                SetBias(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] [member node_a] and [member node_b] can collide. Default value: [code]false[/code].
        /// </summary>
        public bool DisableCollision
        {
            get
            {
                return GetExcludeNodesFromCollision();
            }
            set
            {
                SetExcludeNodesFromCollision(value);
            }
        }


        private const string nativeName = "Joint2D";

        internal Joint2D() {}

        internal Joint2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_node_a");

        [GodotMethod("set_node_a")]
        public void SetNodeA(NodePath node)
        {
            NativeCalls.godot_icall_1_78(method_bind_0, Object.GetPtr(this), NodePath.GetPtr(node));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_node_a");

        [GodotMethod("get_node_a")]
        public NodePath GetNodeA()
        {
            IntPtr ret = NativeCalls.godot_icall_0_79(method_bind_1, Object.GetPtr(this));
            return new NodePath(ret);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_node_b");

        [GodotMethod("set_node_b")]
        public void SetNodeB(NodePath node)
        {
            NativeCalls.godot_icall_1_78(method_bind_2, Object.GetPtr(this), NodePath.GetPtr(node));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_node_b");

        [GodotMethod("get_node_b")]
        public NodePath GetNodeB()
        {
            IntPtr ret = NativeCalls.godot_icall_0_79(method_bind_3, Object.GetPtr(this));
            return new NodePath(ret);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bias");

        [GodotMethod("set_bias")]
        public void SetBias(float bias)
        {
            NativeCalls.godot_icall_1_10(method_bind_4, Object.GetPtr(this), bias);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bias");

        [GodotMethod("get_bias")]
        public float GetBias()
        {
            return NativeCalls.godot_icall_0_9(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_exclude_nodes_from_collision");

        [GodotMethod("set_exclude_nodes_from_collision")]
        public void SetExcludeNodesFromCollision(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_6, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_exclude_nodes_from_collision");

        [GodotMethod("get_exclude_nodes_from_collision")]
        public bool GetExcludeNodesFromCollision()
        {
            return NativeCalls.godot_icall_0_3(method_bind_7, Object.GetPtr(this));
        }
    }
}
