using System;

namespace Godot
{
    /// <summary>
    /// A Texture capable of storing many smaller Textures with offsets.
    /// You can dynamically add pieces([Texture]) to this [code]LargeTexture[/code] using different offsets.
    /// </summary>
    public class LargeTexture : Texture
    {
        public object[] _Data
        {
            get
            {
                return _GetData();
            }
            set
            {
                _SetData(value);
            }
        }

        private const string nativeName = "LargeTexture";

        public LargeTexture() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_LargeTexture_Ctor(this);
        }

        internal LargeTexture(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_piece");

        /// <summary>
        /// Add another [Texture] to this [code]LargeTexture[/code], starting on offset "ofs".
        /// </summary>
        [GodotMethod("add_piece")]
        public int AddPiece(Vector2 ofs, Texture texture)
        {
            return NativeCalls.godot_icall_2_480(method_bind_0, Object.GetPtr(this), ref ofs, Object.GetPtr(texture));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_piece_offset");

        /// <summary>
        /// Sets the offset of the piece with index "idx" to "ofs".
        /// </summary>
        [GodotMethod("set_piece_offset")]
        public void SetPieceOffset(int idx, Vector2 ofs)
        {
            NativeCalls.godot_icall_2_288(method_bind_1, Object.GetPtr(this), idx, ref ofs);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_piece_texture");

        /// <summary>
        /// Sets the [Texture] of the piece with index "idx" to  "ofs".
        /// </summary>
        [GodotMethod("set_piece_texture")]
        public void SetPieceTexture(int idx, Texture texture)
        {
            NativeCalls.godot_icall_2_67(method_bind_2, Object.GetPtr(this), idx, Object.GetPtr(texture));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_size");

        /// <summary>
        /// Sets the size of this [code]LargeTexture[/code].
        /// </summary>
        [GodotMethod("set_size")]
        public void SetSize(Vector2 size)
        {
            NativeCalls.godot_icall_1_45(method_bind_3, Object.GetPtr(this), ref size);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear");

        /// <summary>
        /// Clears the [code]LargeTexture[/code].
        /// </summary>
        [GodotMethod("clear")]
        public void Clear()
        {
            NativeCalls.godot_icall_0_12(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_piece_count");

        /// <summary>
        /// Returns the number of pieces currently in this [code]LargeTexture[/code].
        /// </summary>
        [GodotMethod("get_piece_count")]
        public int GetPieceCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_piece_offset");

        /// <summary>
        /// Returns the offset of the piece with index "idx".
        /// </summary>
        [GodotMethod("get_piece_offset")]
        public Vector2 GetPieceOffset(int idx)
        {
            object ret = NativeCalls.godot_icall_1_282(method_bind_6, Object.GetPtr(this), idx);
            return (Vector2)ret;
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_piece_texture");

        /// <summary>
        /// Returns the [Texture] of the piece with index "idx".
        /// </summary>
        [GodotMethod("get_piece_texture")]
        public Texture GetPieceTexture(int idx)
        {
            return NativeCalls.godot_icall_1_157(method_bind_7, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_data");

        internal void _SetData(object[] data)
        {
            NativeCalls.godot_icall_1_154(method_bind_8, Object.GetPtr(this), data);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_data");

        internal object[] _GetData()
        {
            return NativeCalls.godot_icall_0_21(method_bind_9, Object.GetPtr(this));
        }
    }
}
