using System;

namespace Godot
{
    /// <summary>
    /// Light is the abstract base class for light nodes, so it shouldn't be used directly (It can't be instanced). Other types of light nodes inherit from it. Light contains the common variables and parameters used for lighting.
    /// </summary>
    public class Light : VisualInstance
    {
        public enum BakeMode
        {
            Disabled = 0,
            Indirect = 1,
            All = 2
        }

        public enum Param
        {
            Energy = 0,
            IndirectEnergy = 1,
            Specular = 2,
            Range = 3,
            Attenuation = 4,
            SpotAngle = 5,
            SpotAttenuation = 6,
            ContactShadowSize = 7,
            ShadowMaxDistance = 8,
            ShadowSplit1Offset = 9,
            ShadowSplit2Offset = 10,
            ShadowSplit3Offset = 11,
            ShadowNormalBias = 12,
            ShadowBias = 13,
            ShadowBiasSplitScale = 14,
            Max = 15
        }


        /// <summary>
        /// </summary>
        public Color LightColor
        {
            get
            {
                return GetColor();
            }
            set
            {
                SetColor(value);
            }
        }

        /// <summary>
        /// </summary>
        public float LightEnergy
        {
            get
            {
                return GetParam((Light.Param)0);
            }
            set
            {
                SetParam((Light.Param)0, value);
            }
        }

        /// <summary>
        /// </summary>
        public float LightIndirectEnergy
        {
            get
            {
                return GetParam((Light.Param)1);
            }
            set
            {
                SetParam((Light.Param)1, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool LightNegative
        {
            get
            {
                return IsNegative();
            }
            set
            {
                SetNegative(value);
            }
        }

        /// <summary>
        /// </summary>
        public float LightSpecular
        {
            get
            {
                return GetParam((Light.Param)2);
            }
            set
            {
                SetParam((Light.Param)2, value);
            }
        }

        /// <summary>
        /// </summary>
        public Light.BakeMode LightBakeMode
        {
            get
            {
                return GetBakeMode();
            }
            set
            {
                SetBakeMode(value);
            }
        }

        /// <summary>
        /// </summary>
        public int LightCullMask
        {
            get
            {
                return GetCullMask();
            }
            set
            {
                SetCullMask(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool ShadowEnabled
        {
            get
            {
                return HasShadow();
            }
            set
            {
                SetShadow(value);
            }
        }

        /// <summary>
        /// </summary>
        public Color ShadowColor
        {
            get
            {
                return GetShadowColor();
            }
            set
            {
                SetShadowColor(value);
            }
        }

        /// <summary>
        /// </summary>
        public float ShadowBias
        {
            get
            {
                return GetParam((Light.Param)13);
            }
            set
            {
                SetParam((Light.Param)13, value);
            }
        }

        /// <summary>
        /// </summary>
        public float ShadowContact
        {
            get
            {
                return GetParam((Light.Param)7);
            }
            set
            {
                SetParam((Light.Param)7, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool ShadowReverseCullFace
        {
            get
            {
                return GetShadowReverseCullFace();
            }
            set
            {
                SetShadowReverseCullFace(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool EditorOnly
        {
            get
            {
                return IsEditorOnly();
            }
            set
            {
                SetEditorOnly(value);
            }
        }


        private const string nativeName = "Light";

        internal Light() {}

        internal Light(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_editor_only");

        [GodotMethod("set_editor_only")]
        public void SetEditorOnly(bool editorOnly)
        {
            NativeCalls.godot_icall_1_11(method_bind_0, Object.GetPtr(this), editorOnly);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_editor_only");

        [GodotMethod("is_editor_only")]
        public bool IsEditorOnly()
        {
            return NativeCalls.godot_icall_0_3(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_param");

        [GodotMethod("set_param")]
        public void SetParam(Light.Param param, float value)
        {
            NativeCalls.godot_icall_2_481(method_bind_2, Object.GetPtr(this), param, value);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_param");

        [GodotMethod("get_param")]
        public float GetParam(Light.Param param)
        {
            return NativeCalls.godot_icall_1_482(method_bind_3, Object.GetPtr(this), param);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_shadow");

        [GodotMethod("set_shadow")]
        public void SetShadow(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_4, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_shadow");

        [GodotMethod("has_shadow")]
        public bool HasShadow()
        {
            return NativeCalls.godot_icall_0_3(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_negative");

        [GodotMethod("set_negative")]
        public void SetNegative(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_6, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_negative");

        [GodotMethod("is_negative")]
        public bool IsNegative()
        {
            return NativeCalls.godot_icall_0_3(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_cull_mask");

        [GodotMethod("set_cull_mask")]
        public void SetCullMask(int cullMask)
        {
            NativeCalls.godot_icall_1_0(method_bind_8, Object.GetPtr(this), cullMask);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_cull_mask");

        [GodotMethod("get_cull_mask")]
        public int GetCullMask()
        {
            return NativeCalls.godot_icall_0_1(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_color");

        [GodotMethod("set_color")]
        public void SetColor(Color color)
        {
            NativeCalls.godot_icall_1_202(method_bind_10, Object.GetPtr(this), ref color);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_color");

        [GodotMethod("get_color")]
        public Color GetColor()
        {
            object ret = NativeCalls.godot_icall_0_203(method_bind_11, Object.GetPtr(this));
            return (Color)ret;
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_shadow_reverse_cull_face");

        [GodotMethod("set_shadow_reverse_cull_face")]
        public void SetShadowReverseCullFace(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_12, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_shadow_reverse_cull_face");

        [GodotMethod("get_shadow_reverse_cull_face")]
        public bool GetShadowReverseCullFace()
        {
            return NativeCalls.godot_icall_0_3(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_shadow_color");

        [GodotMethod("set_shadow_color")]
        public void SetShadowColor(Color shadowColor)
        {
            NativeCalls.godot_icall_1_202(method_bind_14, Object.GetPtr(this), ref shadowColor);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_shadow_color");

        [GodotMethod("get_shadow_color")]
        public Color GetShadowColor()
        {
            object ret = NativeCalls.godot_icall_0_203(method_bind_15, Object.GetPtr(this));
            return (Color)ret;
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bake_mode");

        [GodotMethod("set_bake_mode")]
        public void SetBakeMode(Light.BakeMode bakeMode)
        {
            NativeCalls.godot_icall_1_483(method_bind_16, Object.GetPtr(this), bakeMode);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bake_mode");

        [GodotMethod("get_bake_mode")]
        public Light.BakeMode GetBakeMode()
        {
            return NativeCalls.godot_icall_0_484(method_bind_17, Object.GetPtr(this));
        }
    }
}
