using System;

namespace Godot
{
    /// <summary>
    /// Occludes light cast by a Light2D, casting shadows. The LightOccluder2D must be provided with an [OccluderPolygon2D] in order for the shadow to be computed.
    /// </summary>
    public class LightOccluder2D : Node2D
    {
        /// <summary>
        /// The [OccluderPolygon2D] used to compute the shadow.
        /// </summary>
        public OccluderPolygon2D Occluder
        {
            get
            {
                return GetOccluderPolygon();
            }
            set
            {
                SetOccluderPolygon(value);
            }
        }

        /// <summary>
        /// The LightOccluder2D's light mask. The LightOccluder2D will cast shadows only from Light2D(s) that have the same light mask(s).
        /// </summary>
        public int LightMask
        {
            get
            {
                return GetOccluderLightMask();
            }
            set
            {
                SetOccluderLightMask(value);
            }
        }


        private const string nativeName = "LightOccluder2D";

        public LightOccluder2D() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_LightOccluder2D_Ctor(this);
        }

        internal LightOccluder2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_occluder_polygon");

        [GodotMethod("set_occluder_polygon")]
        public void SetOccluderPolygon(OccluderPolygon2D polygon)
        {
            NativeCalls.godot_icall_1_24(method_bind_0, Object.GetPtr(this), Object.GetPtr(polygon));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_occluder_polygon");

        [GodotMethod("get_occluder_polygon")]
        public OccluderPolygon2D GetOccluderPolygon()
        {
            return NativeCalls.godot_icall_0_489(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_occluder_light_mask");

        [GodotMethod("set_occluder_light_mask")]
        public void SetOccluderLightMask(int mask)
        {
            NativeCalls.godot_icall_1_0(method_bind_2, Object.GetPtr(this), mask);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_occluder_light_mask");

        [GodotMethod("get_occluder_light_mask")]
        public int GetOccluderLightMask()
        {
            return NativeCalls.godot_icall_0_1(method_bind_3, Object.GetPtr(this));
        }
    }
}
