using System;

namespace Godot
{
    /// <summary>
    /// Line shape for 2D collisions. It works like a 2D plane and will not allow any body to go to the negative side. Not recommended for rigid bodies, and usually not recommended for static bodies either because it forces checks against it on every frame.
    /// </summary>
    public class LineShape2D : Shape2D
    {
        /// <summary>
        /// The line's normal.
        /// </summary>
        public Vector2 Normal
        {
            get
            {
                return GetNormal();
            }
            set
            {
                SetNormal(value);
            }
        }

        /// <summary>
        /// The line's distance from the origin.
        /// </summary>
        public float D
        {
            get
            {
                return GetD();
            }
            set
            {
                SetD(value);
            }
        }


        private const string nativeName = "LineShape2D";

        public LineShape2D() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_LineShape2D_Ctor(this);
        }

        internal LineShape2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_normal");

        [GodotMethod("set_normal")]
        public void SetNormal(Vector2 normal)
        {
            NativeCalls.godot_icall_1_45(method_bind_0, Object.GetPtr(this), ref normal);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_normal");

        [GodotMethod("get_normal")]
        public Vector2 GetNormal()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_1, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_d");

        [GodotMethod("set_d")]
        public void SetD(float d)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), d);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_d");

        [GodotMethod("get_d")]
        public float GetD()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }
    }
}
