using System;

namespace Godot
{
    /// <summary>
    /// This kind of buttons are primarily used when the interaction with the button causes a context change (like linking to a web page).
    /// </summary>
    public class LinkButton : BaseButton
    {
        public enum UnderlineMode
        {
            Always = 0,
            OnHover = 1,
            Never = 2
        }


        /// <summary>
        /// </summary>
        public string Text
        {
            get
            {
                return GetText();
            }
            set
            {
                SetText(value);
            }
        }

        /// <summary>
        /// </summary>
        public LinkButton.UnderlineMode Underline
        {
            get
            {
                return GetUnderlineMode();
            }
            set
            {
                SetUnderlineMode(value);
            }
        }


        private const string nativeName = "LinkButton";

        public LinkButton() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_LinkButton_Ctor(this);
        }

        internal LinkButton(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_text");

        [GodotMethod("set_text")]
        public void SetText(string text)
        {
            NativeCalls.godot_icall_1_43(method_bind_0, Object.GetPtr(this), text);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_text");

        [GodotMethod("get_text")]
        public string GetText()
        {
            return NativeCalls.godot_icall_0_2(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_underline_mode");

        [GodotMethod("set_underline_mode")]
        public void SetUnderlineMode(LinkButton.UnderlineMode underlineMode)
        {
            NativeCalls.godot_icall_1_499(method_bind_2, Object.GetPtr(this), underlineMode);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_underline_mode");

        [GodotMethod("get_underline_mode")]
        public LinkButton.UnderlineMode GetUnderlineMode()
        {
            return NativeCalls.godot_icall_0_500(method_bind_3, Object.GetPtr(this));
        }
    }
}
