using System;

namespace Godot
{
    /// <summary>
    /// Main loop is the abstract main loop base class. All other main loop classes are derived from it. Upon application start, a [code]MainLoop[/code] has to be provided to OS, else the application will exit. This happens automatically (and a [SceneTree] is created), unless a main [Script] is supplied, which may or not create and return a [code]MainLoop[/code].
    /// </summary>
    public class MainLoop : Object
    {
        public const int NotificationWmMouseEnter = 2;
        public const int NotificationWmMouseExit = 3;
        public const int NotificationWmFocusIn = 4;
        public const int NotificationWmFocusOut = 5;
        public const int NotificationWmQuitRequest = 6;
        public const int NotificationWmGoBackRequest = 7;
        public const int NotificationWmUnfocusRequest = 8;
        public const int NotificationOsMemoryWarning = 9;
        public const int NotificationTranslationChanged = 90;
        public const int NotificationWmAbout = 91;

        private const string nativeName = "MainLoop";

        public MainLoop() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_MainLoop_Ctor(this);
        }

        internal MainLoop(bool memoryOwn) : base(memoryOwn) {}

        /// <summary>
        /// </summary>
        [GodotMethod("_drop_files")]
        public virtual void _DropFiles(string[] files, int screen)
        {
            return;
        }

        /// <summary>
        /// Called before the program exits.
        /// </summary>
        [GodotMethod("_finalize")]
        public virtual void _Finalize()
        {
            return;
        }

        /// <summary>
        /// Called each idle frame with time since last call as an only argument.
        /// </summary>
        [GodotMethod("_idle")]
        public virtual void _Idle(float delta)
        {
            return;
        }

        /// <summary>
        /// Called once during initialization.
        /// </summary>
        [GodotMethod("_initialize")]
        public virtual void _Initialize()
        {
            return;
        }

        /// <summary>
        /// </summary>
        [GodotMethod("_input_event")]
        public virtual void _InputEvent(InputEvent ev)
        {
            return;
        }

        /// <summary>
        /// </summary>
        [GodotMethod("_input_text")]
        public virtual void _InputText(string text)
        {
            return;
        }

        /// <summary>
        /// </summary>
        [GodotMethod("_iteration")]
        public virtual void _Iteration(float delta)
        {
            return;
        }

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "input_event");

        /// <summary>
        /// </summary>
        [GodotMethod("input_event")]
        public void InputEvent(InputEvent ev)
        {
            NativeCalls.godot_icall_1_24(method_bind_0, Object.GetPtr(this), Object.GetPtr(ev));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "input_text");

        /// <summary>
        /// </summary>
        [GodotMethod("input_text")]
        public void InputText(string text)
        {
            NativeCalls.godot_icall_1_43(method_bind_1, Object.GetPtr(this), text);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "init");

        /// <summary>
        /// </summary>
        [GodotMethod("init")]
        public void Init()
        {
            NativeCalls.godot_icall_0_12(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "iteration");

        /// <summary>
        /// </summary>
        [GodotMethod("iteration")]
        public bool Iteration(float delta)
        {
            return NativeCalls.godot_icall_1_501(method_bind_3, Object.GetPtr(this), delta);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "idle");

        /// <summary>
        /// </summary>
        [GodotMethod("idle")]
        public bool Idle(float delta)
        {
            return NativeCalls.godot_icall_1_501(method_bind_4, Object.GetPtr(this), delta);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "finish");

        /// <summary>
        /// </summary>
        [GodotMethod("finish")]
        public void Finish()
        {
            NativeCalls.godot_icall_0_12(method_bind_5, Object.GetPtr(this));
        }
    }
}
