using System;

namespace Godot
{
    /// <summary>
    /// Provides data transformation and encoding utility functions.
    /// </summary>
    public static class Marshalls
    {
        private const string nativeName = "_Marshalls";
        internal static IntPtr ptr = NativeCalls.godot_icall__Marshalls_get_singleton();

        private static IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "variant_to_base64");

        /// <summary>
        /// Return base64 encoded String of a given [Variant].
        /// </summary>
        [GodotMethod("variant_to_base64")]
        public static string VariantToBase64(object variant)
        {
            return NativeCalls.godot_icall_1_728(method_bind_0, ptr, variant);
        }

        private static IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "base64_to_variant");

        /// <summary>
        /// Return [Variant] of a given base64 encoded String.
        /// </summary>
        [GodotMethod("base64_to_variant")]
        public static object Base64ToVariant(string base64Str)
        {
            return NativeCalls.godot_icall_1_338(method_bind_1, ptr, base64Str);
        }

        private static IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "raw_to_base64");

        /// <summary>
        /// Return base64 encoded String of a given [PoolByteArray].
        /// </summary>
        [GodotMethod("raw_to_base64")]
        public static string RawToBase64(byte[] array)
        {
            return NativeCalls.godot_icall_1_1062(method_bind_2, ptr, array);
        }

        private static IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "base64_to_raw");

        /// <summary>
        /// Return [PoolByteArray] of a given base64 encoded String.
        /// </summary>
        [GodotMethod("base64_to_raw")]
        public static byte[] Base64ToRaw(string base64Str)
        {
            return NativeCalls.godot_icall_1_1063(method_bind_3, ptr, base64Str);
        }

        private static IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "utf8_to_base64");

        /// <summary>
        /// Return base64 encoded String of a given utf8 String.
        /// </summary>
        [GodotMethod("utf8_to_base64")]
        public static string Utf8ToBase64(string utf8Str)
        {
            return NativeCalls.godot_icall_1_73(method_bind_4, ptr, utf8Str);
        }

        private static IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "base64_to_utf8");

        /// <summary>
        /// Return utf8 String of a given base64 encoded String.
        /// </summary>
        [GodotMethod("base64_to_utf8")]
        public static string Base64ToUtf8(string base64Str)
        {
            return NativeCalls.godot_icall_1_73(method_bind_5, ptr, base64Str);
        }
    }
}
