using System;

namespace Godot
{
    /// <summary>
    /// Material is a base [Resource] used for coloring and shading geometry. All materials inherit from it and almost all [VisualInstance] derived nodes carry a Material. A few flags and parameters are shared between all material types and are configured here.
    /// </summary>
    public class Material : Resource
    {
        public const int RenderPriorityMax = 127;
        public const int RenderPriorityMin = -128;

        /// <summary>
        /// </summary>
        public int RenderPriority
        {
            get
            {
                return GetRenderPriority();
            }
            set
            {
                SetRenderPriority(value);
            }
        }

        /// <summary>
        /// </summary>
        public Material NextPass
        {
            get
            {
                return GetNextPass();
            }
            set
            {
                SetNextPass(value);
            }
        }


        private const string nativeName = "Material";

        internal Material() {}

        internal Material(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_next_pass");

        [GodotMethod("set_next_pass")]
        public void SetNextPass(Material nextPass)
        {
            NativeCalls.godot_icall_1_24(method_bind_0, Object.GetPtr(this), Object.GetPtr(nextPass));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_next_pass");

        [GodotMethod("get_next_pass")]
        public Material GetNextPass()
        {
            return NativeCalls.godot_icall_0_222(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_render_priority");

        [GodotMethod("set_render_priority")]
        public void SetRenderPriority(int priority)
        {
            NativeCalls.godot_icall_1_0(method_bind_2, Object.GetPtr(this), priority);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_render_priority");

        [GodotMethod("get_render_priority")]
        public int GetRenderPriority()
        {
            return NativeCalls.godot_icall_0_1(method_bind_3, Object.GetPtr(this));
        }
    }
}
