using System;

namespace Godot
{
    /// <summary>
    /// Special button that brings up a [PopupMenu] when clicked. That's pretty much all it does, as it's just a helper class when building GUIs.
    /// </summary>
    public class MenuButton : Button
    {
        public object[] Items
        {
            get
            {
                return _GetItems();
            }
            set
            {
                _SetItems(value);
            }
        }

        private const string nativeName = "MenuButton";

        public MenuButton() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_MenuButton_Ctor(this);
        }

        internal MenuButton(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_popup");

        /// <summary>
        /// Return the [PopupMenu] contained in this button.
        /// </summary>
        [GodotMethod("get_popup")]
        public PopupMenu GetPopup()
        {
            return NativeCalls.godot_icall_0_498(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_items");

        internal void _SetItems(object[] arg0)
        {
            NativeCalls.godot_icall_1_154(method_bind_1, Object.GetPtr(this), arg0);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_items");

        internal object[] _GetItems()
        {
            return NativeCalls.godot_icall_0_21(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_disable_shortcuts");

        /// <summary>
        /// </summary>
        [GodotMethod("set_disable_shortcuts")]
        public void SetDisableShortcuts(bool disabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_3, Object.GetPtr(this), disabled);
        }
    }
}
