using System;

namespace Godot
{
    /// <summary>
    /// Library of meshes. Contains a list of [Mesh] resources, each with name and ID. Useful for GridMap or painting Terrain.
    /// </summary>
    public class MeshLibrary : Resource
    {
        private const string nativeName = "MeshLibrary";

        public MeshLibrary() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_MeshLibrary_Ctor(this);
        }

        internal MeshLibrary(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "create_item");

        /// <summary>
        /// Create a new item in the library, supplied an id.
        /// </summary>
        [GodotMethod("create_item")]
        public void CreateItem(int id)
        {
            NativeCalls.godot_icall_1_0(method_bind_0, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_name");

        /// <summary>
        /// Set the name of the item.
        /// </summary>
        [GodotMethod("set_item_name")]
        public void SetItemName(int id, string name)
        {
            NativeCalls.godot_icall_2_115(method_bind_1, Object.GetPtr(this), id, name);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_mesh");

        /// <summary>
        /// Set the mesh of the item.
        /// </summary>
        [GodotMethod("set_item_mesh")]
        public void SetItemMesh(int id, Mesh mesh)
        {
            NativeCalls.godot_icall_2_67(method_bind_2, Object.GetPtr(this), id, Object.GetPtr(mesh));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_navmesh");

        /// <summary>
        /// </summary>
        [GodotMethod("set_item_navmesh")]
        public void SetItemNavmesh(int id, NavigationMesh navmesh)
        {
            NativeCalls.godot_icall_2_67(method_bind_3, Object.GetPtr(this), id, Object.GetPtr(navmesh));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_shapes");

        /// <summary>
        /// </summary>
        [GodotMethod("set_item_shapes")]
        public void SetItemShapes(int id, object[] shapes)
        {
            NativeCalls.godot_icall_2_512(method_bind_4, Object.GetPtr(this), id, shapes);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_preview");

        /// <summary>
        /// </summary>
        [GodotMethod("set_item_preview")]
        public void SetItemPreview(int id, Texture texture)
        {
            NativeCalls.godot_icall_2_67(method_bind_5, Object.GetPtr(this), id, Object.GetPtr(texture));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_name");

        /// <summary>
        /// Return the name of the item.
        /// </summary>
        [GodotMethod("get_item_name")]
        public string GetItemName(int id)
        {
            return NativeCalls.godot_icall_1_108(method_bind_6, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_mesh");

        /// <summary>
        /// Return the mesh of the item.
        /// </summary>
        [GodotMethod("get_item_mesh")]
        public Mesh GetItemMesh(int id)
        {
            return NativeCalls.godot_icall_1_513(method_bind_7, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_navmesh");

        /// <summary>
        /// </summary>
        [GodotMethod("get_item_navmesh")]
        public NavigationMesh GetItemNavmesh(int id)
        {
            return NativeCalls.godot_icall_1_514(method_bind_8, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_shapes");

        /// <summary>
        /// </summary>
        [GodotMethod("get_item_shapes")]
        public object[] GetItemShapes(int id)
        {
            return NativeCalls.godot_icall_1_116(method_bind_9, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_preview");

        /// <summary>
        /// </summary>
        [GodotMethod("get_item_preview")]
        public Texture GetItemPreview(int id)
        {
            return NativeCalls.godot_icall_1_157(method_bind_10, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_item");

        /// <summary>
        /// Remove the item.
        /// </summary>
        [GodotMethod("remove_item")]
        public void RemoveItem(int id)
        {
            NativeCalls.godot_icall_1_0(method_bind_11, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "find_item_by_name");

        /// <summary>
        /// </summary>
        [GodotMethod("find_item_by_name")]
        public int FindItemByName(string name)
        {
            return NativeCalls.godot_icall_1_87(method_bind_12, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear");

        /// <summary>
        /// Clear the library.
        /// </summary>
        [GodotMethod("clear")]
        public void Clear()
        {
            NativeCalls.godot_icall_0_12(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_list");

        /// <summary>
        /// Return the list of items.
        /// </summary>
        [GodotMethod("get_item_list")]
        public int[] GetItemList()
        {
            return NativeCalls.godot_icall_0_175(method_bind_14, Object.GetPtr(this));
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_last_unused_item_id");

        /// <summary>
        /// Get an unused id for a new item.
        /// </summary>
        [GodotMethod("get_last_unused_item_id")]
        public int GetLastUnusedItemId()
        {
            return NativeCalls.godot_icall_0_1(method_bind_15, Object.GetPtr(this));
        }
    }
}
