using System;

namespace Godot
{
    /// <summary>
    /// A synchronization Mutex. Element used to synchronize multiple [Thread]s. Basically a binary [Semaphore]. Guarantees that only one thread can ever acquire this lock at a time. Can be used to protect a critical section. Be careful to avoid deadlocks.
    /// </summary>
    public class Mutex : Reference
    {
        private const string nativeName = "_Mutex";

        public Mutex() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Mutex_Ctor(this);
        }

        internal Mutex(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "lock");

        /// <summary>
        /// Lock this [code]Mutex[/code], blocks until it is unlocked by the current owner.
        /// </summary>
        [GodotMethod("lock")]
        public void Lock()
        {
            NativeCalls.godot_icall_0_12(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "try_lock");

        /// <summary>
        /// Try locking this [code]Mutex[/code], does not block. Returns [OK] on success, [ERR_BUSY] otherwise.
        /// </summary>
        [GodotMethod("try_lock")]
        public Error TryLock()
        {
            return NativeCalls.godot_icall_0_325(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "unlock");

        /// <summary>
        /// Unlock this [code]Mutex[/code], leaving it to other threads.
        /// </summary>
        [GodotMethod("unlock")]
        public void Unlock()
        {
            NativeCalls.godot_icall_0_12(method_bind_2, Object.GetPtr(this));
        }
    }
}
