using System;

namespace Godot
{
    /// <summary>
    /// A connection (or a listening server) that should be passed to [method SceneTree.set_network_peer]. Socket events can be handled by connecting to [SceneTree] signals.
    /// </summary>
    public class NetworkedMultiplayerENet : NetworkedMultiplayerPeer
    {
        public enum CompressionModeEnum
        {
            None = 0,
            RangeCoder = 1,
            Fastlz = 2,
            Zlib = 3,
            Zstd = 4
        }


        /// <summary>
        /// </summary>
        public NetworkedMultiplayerENet.CompressionModeEnum CompressionMode
        {
            get
            {
                return GetCompressionMode();
            }
            set
            {
                SetCompressionMode(value);
            }
        }


        private const string nativeName = "NetworkedMultiplayerENet";

        public NetworkedMultiplayerENet() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_NetworkedMultiplayerENet_Ctor(this);
        }

        internal NetworkedMultiplayerENet(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "create_server");

        /// <summary>
        /// Create server that listens to connections via [code]port[/code].
        /// </summary>
        [GodotMethod("create_server")]
        public Error CreateServer(int port, int maxClients = 32, int inBandwidth = 0, int outBandwidth = 0)
        {
            return NativeCalls.godot_icall_4_532(method_bind_0, Object.GetPtr(this), port, maxClients, inBandwidth, outBandwidth);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "create_client");

        /// <summary>
        /// Create client that connects to a server at address [code]ip[/code] using specified [code]port[/code].
        /// </summary>
        [GodotMethod("create_client")]
        public Error CreateClient(string ip, int port, int inBandwidth = 0, int outBandwidth = 0)
        {
            return NativeCalls.godot_icall_4_533(method_bind_1, Object.GetPtr(this), ip, port, inBandwidth, outBandwidth);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "close_connection");

        /// <summary>
        /// </summary>
        [GodotMethod("close_connection")]
        public void CloseConnection()
        {
            NativeCalls.godot_icall_0_12(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_compression_mode");

        [GodotMethod("set_compression_mode")]
        public void SetCompressionMode(NetworkedMultiplayerENet.CompressionModeEnum mode)
        {
            NativeCalls.godot_icall_1_534(method_bind_3, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_compression_mode");

        [GodotMethod("get_compression_mode")]
        public NetworkedMultiplayerENet.CompressionModeEnum GetCompressionMode()
        {
            return NativeCalls.godot_icall_0_535(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bind_ip");

        /// <summary>
        /// </summary>
        [GodotMethod("set_bind_ip")]
        public void SetBindIp(string ip)
        {
            NativeCalls.godot_icall_1_43(method_bind_5, Object.GetPtr(this), ip);
        }
    }
}
