using System;

namespace Godot
{
    /// <summary>
    /// Manages the connection to network peers. Assigns unique IDs to each client connected to the server.
    /// </summary>
    public class NetworkedMultiplayerPeer : PacketPeer
    {
        public const int TargetPeerBroadcast = 0;
        public const int TargetPeerServer = 1;

        public enum ConnectionStatus
        {
            Disconnected = 0,
            Connecting = 1,
            Connected = 2
        }

        public enum TransferModeEnum
        {
            Unreliable = 0,
            UnreliableOrdered = 1,
            Reliable = 2
        }


        /// <summary>
        /// If [code]true[/code] this [code]NetworkedMultiplayerPeer[/code] refuses new connections. Default value: [code]false[/code].
        /// </summary>
        public bool RefuseNewConnections
        {
            get
            {
                return IsRefusingNewConnections();
            }
            set
            {
                SetRefuseNewConnections(value);
            }
        }

        /// <summary>
        /// The manner in which to send packets to the [code]target_peer[/code]. See [enum TransferMode].
        /// </summary>
        public NetworkedMultiplayerPeer.TransferModeEnum TransferMode
        {
            get
            {
                return GetTransferMode();
            }
            set
            {
                SetTransferMode(value);
            }
        }


        private const string nativeName = "NetworkedMultiplayerPeer";

        internal NetworkedMultiplayerPeer() {}

        internal NetworkedMultiplayerPeer(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_transfer_mode");

        [GodotMethod("set_transfer_mode")]
        public void SetTransferMode(NetworkedMultiplayerPeer.TransferModeEnum mode)
        {
            NativeCalls.godot_icall_1_536(method_bind_0, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_transfer_mode");

        [GodotMethod("get_transfer_mode")]
        public NetworkedMultiplayerPeer.TransferModeEnum GetTransferMode()
        {
            return NativeCalls.godot_icall_0_537(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_target_peer");

        /// <summary>
        /// The peer to which packets will be sent. Default value: [code]0[/code].
        /// </summary>
        [GodotMethod("set_target_peer")]
        public void SetTargetPeer(int id)
        {
            NativeCalls.godot_icall_1_0(method_bind_2, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_packet_peer");

        /// <summary>
        /// Returns the ID of the [code]NetworkedMultiplayerPeer[/code] who sent the most recent packet.
        /// </summary>
        [GodotMethod("get_packet_peer")]
        public int GetPacketPeer()
        {
            return NativeCalls.godot_icall_0_1(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "poll");

        /// <summary>
        /// Waits up to 1 second to receive a new network event.
        /// </summary>
        [GodotMethod("poll")]
        public void Poll()
        {
            NativeCalls.godot_icall_0_12(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_connection_status");

        /// <summary>
        /// Returns the current state of the connection. See [enum ConnectionStatus].
        /// </summary>
        [GodotMethod("get_connection_status")]
        public NetworkedMultiplayerPeer.ConnectionStatus GetConnectionStatus()
        {
            return NativeCalls.godot_icall_0_538(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_unique_id");

        /// <summary>
        /// Returns the ID of this [code]NetworkedMultiplayerPeer[/code].
        /// </summary>
        [GodotMethod("get_unique_id")]
        public int GetUniqueId()
        {
            return NativeCalls.godot_icall_0_1(method_bind_6, Object.GetPtr(this));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_refuse_new_connections");

        [GodotMethod("set_refuse_new_connections")]
        public void SetRefuseNewConnections(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_7, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_refusing_new_connections");

        [GodotMethod("is_refusing_new_connections")]
        public bool IsRefusingNewConnections()
        {
            return NativeCalls.godot_icall_0_3(method_bind_8, Object.GetPtr(this));
        }
    }
}
