using System;

namespace Godot
{
    /// <summary>
    /// Editor facility that helps you draw a 2D polygon used as resource for [LightOccluder2D].
    /// </summary>
    public class OccluderPolygon2D : Resource
    {
        public enum CullModeEnum
        {
            Disabled = 0,
            Clockwise = 1,
            CounterClockwise = 2
        }


        /// <summary>
        /// If [code]true[/code] closes the polygon. A closed OccluderPolygon2D occludes the light coming from any direction. An opened OccluderPolygon2D occludes the light only at its outline's direction. Default value [code]true[/code].
        /// </summary>
        public bool Closed
        {
            get
            {
                return IsClosed();
            }
            set
            {
                SetClosed(value);
            }
        }

        /// <summary>
        /// Set the direction of the occlusion culling when not [code]CULL_DISABLED[/code]. Default value [code]DISABLED[/code].
        /// </summary>
        public OccluderPolygon2D.CullModeEnum CullMode
        {
            get
            {
                return GetCullMode();
            }
            set
            {
                SetCullMode(value);
            }
        }

        /// <summary>
        /// A [Vector2] array with the index for polygon's vertices positions.
        /// </summary>
        public Vector2[] Polygon
        {
            get
            {
                return GetPolygon();
            }
            set
            {
                SetPolygon(value);
            }
        }


        private const string nativeName = "OccluderPolygon2D";

        public OccluderPolygon2D() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_OccluderPolygon2D_Ctor(this);
        }

        internal OccluderPolygon2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_closed");

        [GodotMethod("set_closed")]
        public void SetClosed(bool closed)
        {
            NativeCalls.godot_icall_1_11(method_bind_0, Object.GetPtr(this), closed);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_closed");

        [GodotMethod("is_closed")]
        public bool IsClosed()
        {
            return NativeCalls.godot_icall_0_3(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_cull_mode");

        [GodotMethod("set_cull_mode")]
        public void SetCullMode(OccluderPolygon2D.CullModeEnum cullMode)
        {
            NativeCalls.godot_icall_1_568(method_bind_2, Object.GetPtr(this), cullMode);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_cull_mode");

        [GodotMethod("get_cull_mode")]
        public OccluderPolygon2D.CullModeEnum GetCullMode()
        {
            return NativeCalls.godot_icall_0_569(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_polygon");

        [GodotMethod("set_polygon")]
        public void SetPolygon(Vector2[] polygon)
        {
            NativeCalls.godot_icall_1_237(method_bind_4, Object.GetPtr(this), polygon);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_polygon");

        [GodotMethod("get_polygon")]
        public Vector2[] GetPolygon()
        {
            return NativeCalls.godot_icall_0_238(method_bind_5, Object.GetPtr(this));
        }
    }
}
