using System;

namespace Godot
{
    /// <summary>
    /// An OmniDirectional light is a type of [Light] node that emits lights in all directions. The light is attenuated through the distance and this attenuation can be configured by changing the energy, radius and attenuation parameters of [Light].
    /// </summary>
    public class OmniLight : Light
    {
        public enum ShadowMode
        {
            DualParaboloid = 0,
            Cube = 1
        }

        public enum ShadowDetail
        {
            Vertical = 0,
            Horizontal = 1
        }


        /// <summary>
        /// </summary>
        public float OmniRange
        {
            get
            {
                return GetParam((Light.Param)3);
            }
            set
            {
                SetParam((Light.Param)3, value);
            }
        }

        /// <summary>
        /// </summary>
        public float OmniAttenuation
        {
            get
            {
                return GetParam((Light.Param)4);
            }
            set
            {
                SetParam((Light.Param)4, value);
            }
        }

        /// <summary>
        /// </summary>
        public OmniLight.ShadowMode OmniShadowMode
        {
            get
            {
                return GetShadowMode();
            }
            set
            {
                SetShadowMode(value);
            }
        }

        /// <summary>
        /// </summary>
        public OmniLight.ShadowDetail OmniShadowDetail
        {
            get
            {
                return GetShadowDetail();
            }
            set
            {
                SetShadowDetail(value);
            }
        }


        private const string nativeName = "OmniLight";

        public OmniLight() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_OmniLight_Ctor(this);
        }

        internal OmniLight(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_shadow_mode");

        [GodotMethod("set_shadow_mode")]
        public void SetShadowMode(OmniLight.ShadowMode mode)
        {
            NativeCalls.godot_icall_1_570(method_bind_0, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_shadow_mode");

        [GodotMethod("get_shadow_mode")]
        public OmniLight.ShadowMode GetShadowMode()
        {
            return NativeCalls.godot_icall_0_571(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_shadow_detail");

        [GodotMethod("set_shadow_detail")]
        public void SetShadowDetail(OmniLight.ShadowDetail detail)
        {
            NativeCalls.godot_icall_1_572(method_bind_2, Object.GetPtr(this), detail);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_shadow_detail");

        [GodotMethod("get_shadow_detail")]
        public OmniLight.ShadowDetail GetShadowDetail()
        {
            return NativeCalls.godot_icall_0_573(method_bind_3, Object.GetPtr(this));
        }
    }
}
