using System;

namespace Godot
{
    /// <summary>
    /// OptionButton is a type button that provides a selectable list of items when pressed. The item selected becomes the "current" item and is displayed as the button text.
    /// </summary>
    public class OptionButton : Button
    {
        /// <summary>
        /// </summary>
        public int Selected
        {
            get
            {
                return GetSelected();
            }
            set
            {
                _SelectInt(value);
            }
        }

        public object[] Items
        {
            get
            {
                return _GetItems();
            }
            set
            {
                _SetItems(value);
            }
        }


        private const string nativeName = "OptionButton";

        public OptionButton() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_OptionButton_Ctor(this);
        }

        internal OptionButton(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_item");

        /// <summary>
        /// Add an item, with text "label" and (optionally) id. If no "id" is passed, "id" becomes the item index. New items are appended at the end.
        /// </summary>
        [GodotMethod("add_item")]
        public void AddItem(string label, int id = -1)
        {
            NativeCalls.godot_icall_2_95(method_bind_0, Object.GetPtr(this), label, id);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_icon_item");

        /// <summary>
        /// Add an item, with a "texture" icon, text "label" and (optionally) id. If no "id" is passed, "id" becomes the item index. New items are appended at the end.
        /// </summary>
        [GodotMethod("add_icon_item")]
        public void AddIconItem(Texture texture, string label, int id)
        {
            NativeCalls.godot_icall_3_574(method_bind_1, Object.GetPtr(this), Object.GetPtr(texture), label, id);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_text");

        /// <summary>
        /// Set the text of an item at index "idx".
        /// </summary>
        [GodotMethod("set_item_text")]
        public void SetItemText(int idx, string text)
        {
            NativeCalls.godot_icall_2_115(method_bind_2, Object.GetPtr(this), idx, text);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_icon");

        /// <summary>
        /// Set the icon of an item at index "idx".
        /// </summary>
        [GodotMethod("set_item_icon")]
        public void SetItemIcon(int idx, Texture texture)
        {
            NativeCalls.godot_icall_2_67(method_bind_3, Object.GetPtr(this), idx, Object.GetPtr(texture));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_disabled");

        /// <summary>
        /// </summary>
        [GodotMethod("set_item_disabled")]
        public void SetItemDisabled(int idx, bool disabled)
        {
            NativeCalls.godot_icall_2_51(method_bind_4, Object.GetPtr(this), idx, disabled);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_id");

        /// <summary>
        /// Set the ID of an item at index "idx".
        /// </summary>
        [GodotMethod("set_item_id")]
        public void SetItemId(int idx, int id)
        {
            NativeCalls.godot_icall_2_32(method_bind_5, Object.GetPtr(this), idx, id);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_metadata");

        /// <summary>
        /// </summary>
        [GodotMethod("set_item_metadata")]
        public void SetItemMetadata(int idx, object metadata)
        {
            NativeCalls.godot_icall_2_453(method_bind_6, Object.GetPtr(this), idx, metadata);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_text");

        /// <summary>
        /// Return the text of the item at index "idx".
        /// </summary>
        [GodotMethod("get_item_text")]
        public string GetItemText(int idx)
        {
            return NativeCalls.godot_icall_1_108(method_bind_7, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_icon");

        /// <summary>
        /// Return the icon of the item at index "idx".
        /// </summary>
        [GodotMethod("get_item_icon")]
        public Texture GetItemIcon(int idx)
        {
            return NativeCalls.godot_icall_1_157(method_bind_8, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_id");

        /// <summary>
        /// Return the ID of the item at index "idx".
        /// </summary>
        [GodotMethod("get_item_id")]
        public int GetItemId(int idx)
        {
            return NativeCalls.godot_icall_1_6(method_bind_9, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_metadata");

        /// <summary>
        /// </summary>
        [GodotMethod("get_item_metadata")]
        public object GetItemMetadata(int idx)
        {
            return NativeCalls.godot_icall_1_454(method_bind_10, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_item_disabled");

        /// <summary>
        /// </summary>
        [GodotMethod("is_item_disabled")]
        public bool IsItemDisabled(int idx)
        {
            return NativeCalls.godot_icall_1_29(method_bind_11, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_count");

        /// <summary>
        /// Return the amount of items in the OptionButton.
        /// </summary>
        [GodotMethod("get_item_count")]
        public int GetItemCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_12, Object.GetPtr(this));
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_separator");

        /// <summary>
        /// Add a separator to the list of items. Separators help to group items. Separator also takes up an index and is appended at the end.
        /// </summary>
        [GodotMethod("add_separator")]
        public void AddSeparator()
        {
            NativeCalls.godot_icall_0_12(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear");

        /// <summary>
        /// Clear all the items in the [code]OptionButton[/code].
        /// </summary>
        [GodotMethod("clear")]
        public void Clear()
        {
            NativeCalls.godot_icall_0_12(method_bind_14, Object.GetPtr(this));
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "select");

        /// <summary>
        /// Select an item by index and make it the current item.
        /// </summary>
        [GodotMethod("select")]
        public void Select(int idx)
        {
            NativeCalls.godot_icall_1_0(method_bind_15, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_selected");

        [GodotMethod("get_selected")]
        public int GetSelected()
        {
            return NativeCalls.godot_icall_0_1(method_bind_16, Object.GetPtr(this));
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_selected_id");

        /// <summary>
        /// </summary>
        [GodotMethod("get_selected_id")]
        public int GetSelectedId()
        {
            return NativeCalls.godot_icall_0_1(method_bind_17, Object.GetPtr(this));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_selected_metadata");

        /// <summary>
        /// </summary>
        [GodotMethod("get_selected_metadata")]
        public object GetSelectedMetadata()
        {
            return NativeCalls.godot_icall_0_461(method_bind_18, Object.GetPtr(this));
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_item");

        /// <summary>
        /// </summary>
        [GodotMethod("remove_item")]
        public void RemoveItem(int idx)
        {
            NativeCalls.godot_icall_1_0(method_bind_19, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_select_int");

        internal void _SelectInt(int arg0)
        {
            NativeCalls.godot_icall_1_0(method_bind_20, Object.GetPtr(this), arg0);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_popup");

        /// <summary>
        /// Return the [PopupMenu] contained in this button.
        /// </summary>
        [GodotMethod("get_popup")]
        public PopupMenu GetPopup()
        {
            return NativeCalls.godot_icall_0_498(method_bind_21, Object.GetPtr(this));
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_items");

        internal void _SetItems(object[] arg0)
        {
            NativeCalls.godot_icall_1_154(method_bind_22, Object.GetPtr(this), arg0);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_items");

        internal object[] _GetItems()
        {
            return NativeCalls.godot_icall_0_21(method_bind_23, Object.GetPtr(this));
        }
    }
}
