using System;

namespace Godot
{
    /// <summary>
    /// PacketPeer is an abstraction and base class for packet-based protocols (such as UDP). It provides an API for sending and receiving packets both as raw data or variables. This makes it easy to transfer data over a protocol, without having to encode data as low level bytes or having to worry about network ordering.
    /// </summary>
    public class PacketPeer : Reference
    {
        /// <summary>
        /// </summary>
        public bool AllowObjectDecoding
        {
            get
            {
                return IsObjectDecodingAllowed();
            }
            set
            {
                SetAllowObjectDecoding(value);
            }
        }


        private const string nativeName = "PacketPeer";

        internal PacketPeer() {}

        internal PacketPeer(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_var");

        /// <summary>
        /// Get a Variant.
        /// </summary>
        [GodotMethod("get_var")]
        public object GetVar()
        {
            return NativeCalls.godot_icall_0_461(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "put_var");

        /// <summary>
        /// Send a Variant as a packet.
        /// </summary>
        [GodotMethod("put_var")]
        public Error PutVar(object var)
        {
            return NativeCalls.godot_icall_1_577(method_bind_1, Object.GetPtr(this), var);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_packet");

        /// <summary>
        /// Get a raw packet.
        /// </summary>
        [GodotMethod("get_packet")]
        public byte[] GetPacket()
        {
            return NativeCalls.godot_icall_0_132(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "put_packet");

        /// <summary>
        /// Send a raw packet.
        /// </summary>
        [GodotMethod("put_packet")]
        public Error PutPacket(byte[] buffer)
        {
            return NativeCalls.godot_icall_1_430(method_bind_3, Object.GetPtr(this), buffer);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_packet_error");

        /// <summary>
        /// Return the error state of the last packet received (via [method get_packet] and [method get_var]).
        /// </summary>
        [GodotMethod("get_packet_error")]
        public Error GetPacketError()
        {
            return NativeCalls.godot_icall_0_325(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_available_packet_count");

        /// <summary>
        /// Return the number of packets currently available in the ring-buffer.
        /// </summary>
        [GodotMethod("get_available_packet_count")]
        public int GetAvailablePacketCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_allow_object_decoding");

        [GodotMethod("set_allow_object_decoding")]
        public void SetAllowObjectDecoding(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_6, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_object_decoding_allowed");

        [GodotMethod("is_object_decoding_allowed")]
        public bool IsObjectDecodingAllowed()
        {
            return NativeCalls.godot_icall_0_3(method_bind_7, Object.GetPtr(this));
        }
    }
}
