using System;

namespace Godot
{
    /// <summary>
    /// PacketStreamPeer provides a wrapper for working using packets over a stream. This allows for using packet based code with StreamPeers. PacketPeerStream implements a custom protocol over the StreamPeer, so the user should not read or write to the wrapped StreamPeer directly.
    /// </summary>
    public class PacketPeerStream : PacketPeer
    {
        /// <summary>
        /// </summary>
        public int InputBufferMaxSize
        {
            get
            {
                return GetInputBufferMaxSize();
            }
            set
            {
                SetInputBufferMaxSize(value);
            }
        }

        /// <summary>
        /// </summary>
        public int OutputBufferMaxSize
        {
            get
            {
                return GetOutputBufferMaxSize();
            }
            set
            {
                SetOutputBufferMaxSize(value);
            }
        }

        /// <summary>
        /// The wrapped [StreamPeer] object.
        /// </summary>
        public StreamPeer StreamPeer
        {
            get
            {
                return GetStreamPeer();
            }
            set
            {
                SetStreamPeer(value);
            }
        }


        private const string nativeName = "PacketPeerStream";

        public PacketPeerStream() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_PacketPeerStream_Ctor(this);
        }

        internal PacketPeerStream(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_stream_peer");

        [GodotMethod("set_stream_peer")]
        public void SetStreamPeer(StreamPeer peer)
        {
            NativeCalls.godot_icall_1_24(method_bind_0, Object.GetPtr(this), Object.GetPtr(peer));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_stream_peer");

        [GodotMethod("get_stream_peer")]
        public StreamPeer GetStreamPeer()
        {
            return NativeCalls.godot_icall_0_405(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_input_buffer_max_size");

        [GodotMethod("set_input_buffer_max_size")]
        public void SetInputBufferMaxSize(int maxSizeBytes)
        {
            NativeCalls.godot_icall_1_0(method_bind_2, Object.GetPtr(this), maxSizeBytes);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_output_buffer_max_size");

        [GodotMethod("set_output_buffer_max_size")]
        public void SetOutputBufferMaxSize(int maxSizeBytes)
        {
            NativeCalls.godot_icall_1_0(method_bind_3, Object.GetPtr(this), maxSizeBytes);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_input_buffer_max_size");

        [GodotMethod("get_input_buffer_max_size")]
        public int GetInputBufferMaxSize()
        {
            return NativeCalls.godot_icall_0_1(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_output_buffer_max_size");

        [GodotMethod("get_output_buffer_max_size")]
        public int GetOutputBufferMaxSize()
        {
            return NativeCalls.godot_icall_0_1(method_bind_5, Object.GetPtr(this));
        }
    }
}
