using System;

namespace Godot
{
    /// <summary>
    /// UDP packet peer. Can be used to send raw UDP packets as well as [Variant]s.
    /// </summary>
    public class PacketPeerUDP : PacketPeer
    {
        private const string nativeName = "PacketPeerUDP";

        public PacketPeerUDP() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_PacketPeerUDP_Ctor(this);
        }

        internal PacketPeerUDP(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "listen");

        /// <summary>
        /// Make this [code]PacketPeerUDP[/code] listen on the "port" binding to "bind_address" with a buffer size "recv_buf_size".
        /// If "bind_address" is set as "*" (default), the peer will listen on all available addresses (both IPv4 and IPv6).
        /// If "bind_address" is set as "0.0.0.0" (for IPv4) or "::" (for IPv6), the peer will listen on all available addresses matching that IP type.
        /// If "bind_address" is set to any valid address (e.g. "192.168.1.101", "::1", etc), the peer will only listen on the interface with that addresses (or fail if no interface with the given address exists).
        /// </summary>
        [GodotMethod("listen")]
        public Error Listen(int port, string bindAddress = "*", int recvBufSize = 65536)
        {
            return NativeCalls.godot_icall_3_581(method_bind_0, Object.GetPtr(this), port, bindAddress, recvBufSize);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "close");

        /// <summary>
        /// Close the UDP socket the [code]PacketPeerUDP[/code] is currently listening on.
        /// </summary>
        [GodotMethod("close")]
        public void Close()
        {
            NativeCalls.godot_icall_0_12(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "wait");

        /// <summary>
        /// Wait for a packet to arrive on the listening port, see [method listen].
        /// </summary>
        [GodotMethod("wait")]
        public Error Wait()
        {
            return NativeCalls.godot_icall_0_325(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_listening");

        /// <summary>
        /// Return whether this [code]PacketPeerUDP[/code] is listening.
        /// </summary>
        [GodotMethod("is_listening")]
        public bool IsListening()
        {
            return NativeCalls.godot_icall_0_3(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_packet_ip");

        /// <summary>
        /// Return the IP of the remote peer that sent the last packet(that was received with [method get_packet] or [method get_var]).
        /// </summary>
        [GodotMethod("get_packet_ip")]
        public string GetPacketIp()
        {
            return NativeCalls.godot_icall_0_2(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_packet_port");

        /// <summary>
        /// Return the port of the remote peer that sent the last packet(that was received with [method get_packet] or [method get_var]).
        /// </summary>
        [GodotMethod("get_packet_port")]
        public int GetPacketPort()
        {
            return NativeCalls.godot_icall_0_1(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_dest_address");

        /// <summary>
        /// Set the destination address and port for sending packets and variables, a hostname will be resolved using if valid.
        /// </summary>
        [GodotMethod("set_dest_address")]
        public Error SetDestAddress(string host, int port)
        {
            return NativeCalls.godot_icall_2_575(method_bind_6, Object.GetPtr(this), host, port);
        }
    }
}
