using System;

namespace Godot
{
    /// <summary>
    /// A ParallaxLayer must be the child of a [ParallaxBackground] node. Each ParallaxLayer can be set to move at different speeds relative to the camera movement or the [member ParallaxBackground.scroll_offset] value.
    /// This node's children will be affected by its scroll offset.
    /// </summary>
    public class ParallaxLayer : Node2D
    {
        /// <summary>
        /// Multiplies the ParallaxLayer's motion. If an axis is set to [code]0[/code] it will not scroll.
        /// </summary>
        public Vector2 MotionScale
        {
            get
            {
                return GetMotionScale();
            }
            set
            {
                SetMotionScale(value);
            }
        }

        /// <summary>
        /// The ParallaxLayer's offset relative to the parent ParallaxBackground's [member ParallaxBackground.scroll_offset].
        /// </summary>
        public Vector2 MotionOffset
        {
            get
            {
                return GetMotionOffset();
            }
            set
            {
                SetMotionOffset(value);
            }
        }

        /// <summary>
        /// The ParallaxLayer's [Texture] mirroring. Useful for creating an infinite scrolling background. If an axis is set to [code]0[/code] the [Texture] will not be mirrored. Default value: [code](0, 0)[/code].
        /// </summary>
        public Vector2 MotionMirroring
        {
            get
            {
                return GetMirroring();
            }
            set
            {
                SetMirroring(value);
            }
        }


        private const string nativeName = "ParallaxLayer";

        public ParallaxLayer() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_ParallaxLayer_Ctor(this);
        }

        internal ParallaxLayer(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_motion_scale");

        [GodotMethod("set_motion_scale")]
        public void SetMotionScale(Vector2 scale)
        {
            NativeCalls.godot_icall_1_45(method_bind_0, Object.GetPtr(this), ref scale);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_motion_scale");

        [GodotMethod("get_motion_scale")]
        public Vector2 GetMotionScale()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_1, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_motion_offset");

        [GodotMethod("set_motion_offset")]
        public void SetMotionOffset(Vector2 offset)
        {
            NativeCalls.godot_icall_1_45(method_bind_2, Object.GetPtr(this), ref offset);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_motion_offset");

        [GodotMethod("get_motion_offset")]
        public Vector2 GetMotionOffset()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_3, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_mirroring");

        [GodotMethod("set_mirroring")]
        public void SetMirroring(Vector2 mirror)
        {
            NativeCalls.godot_icall_1_45(method_bind_4, Object.GetPtr(this), ref mirror);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_mirroring");

        [GodotMethod("get_mirroring")]
        public Vector2 GetMirroring()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_5, Object.GetPtr(this));
            return (Vector2)ret;
        }
    }
}
