using System;

namespace Godot
{
    /// <summary>
    /// This class provides access to a number of different monitors related to performance, such as memory usage, draw calls, and FPS. These are the same as the values displayed in the [i]Monitor[/i] tab in the editor's [i]Debugger[/i] panel. By using the [method get_monitor] method of this class, you can access this data from your code. Note that a few of these monitors are only available in debug mode and will always return 0 when used in a release build.
    /// Many of these monitors are not updated in real-time, so there may be a short delay between changes.
    /// </summary>
    public static class Performance
    {
        public enum Monitor
        {
            TimeFps = 0,
            TimeProcess = 1,
            TimePhysicsProcess = 2,
            MemoryStatic = 3,
            MemoryDynamic = 4,
            MemoryStaticMax = 5,
            MemoryDynamicMax = 6,
            MemoryMessageBufferMax = 7,
            ObjectCount = 8,
            ObjectResourceCount = 9,
            ObjectNodeCount = 10,
            RenderObjectsInFrame = 11,
            RenderVerticesInFrame = 12,
            RenderMaterialChangesInFrame = 13,
            RenderShaderChangesInFrame = 14,
            RenderSurfaceChangesInFrame = 15,
            RenderDrawCallsInFrame = 16,
            RenderVideoMemUsed = 17,
            RenderTextureMemUsed = 18,
            RenderVertexMemUsed = 19,
            RenderUsageVideoMemTotal = 20,
            Physics2dActiveObjects = 21,
            Physics2dCollisionPairs = 22,
            Physics2dIslandCount = 23,
            Physics3dActiveObjects = 24,
            Physics3dCollisionPairs = 25,
            Physics3dIslandCount = 26,
            MonitorMax = 27
        }


        private const string nativeName = "Performance";
        internal static IntPtr ptr = NativeCalls.godot_icall_Performance_get_singleton();

        private static IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_monitor");

        /// <summary>
        /// Returns the value of one of the available monitors. You should provide one of this class's constants as the argument, like this:
        /// [codeblock]
        /// print(Performance.get_monitor(Performance.TIME_FPS)) # Prints the FPS to the console
        /// [/codeblock]
        /// </summary>
        [GodotMethod("get_monitor")]
        public static float GetMonitor(Performance.Monitor monitor)
        {
            return NativeCalls.godot_icall_1_600(method_bind_0, ptr, monitor);
        }
    }
}
