using System;

namespace Godot
{
    /// <summary>
    /// Physics 2D Server is the server responsible for all 2D physics. It can create many kinds of physics objects, but does not insert them on the node tree.
    /// </summary>
    public static class Physics2DServer
    {
        public enum ProcessInfo
        {
            ActiveObjects = 0,
            CollisionPairs = 1,
            IslandCount = 2
        }

        public enum AreaBodyStatus
        {
            Added = 0,
            Removed = 1
        }

        public enum DampedStringParam
        {
            RestLength = 0,
            Stiffness = 1,
            Damping = 2
        }

        public enum BodyMode
        {
            Static = 0,
            Kinematic = 1,
            Rigid = 2,
            Character = 3
        }

        public enum ShapeType
        {
            Line = 0,
            Ray = 1,
            Segment = 2,
            Circle = 3,
            Rectangle = 4,
            Capsule = 5,
            ConvexPolygon = 6,
            ConcavePolygon = 7,
            Custom = 8
        }

        public enum JointParam
        {
            Bias = 0,
            MaxBias = 1,
            MaxForce = 2
        }

        public enum SpaceParameter
        {
            ContactRecycleRadius = 0,
            ContactMaxSeparation = 1,
            BodyMaxAllowedPenetration = 2,
            BodyLinearVelocitySleepThreshold = 3,
            BodyAngularVelocitySleepThreshold = 4,
            BodyTimeToSleep = 5,
            ConstraintDefaultBias = 6
        }

        public enum JointType
        {
            Pin = 0,
            Groove = 1,
            DampedSpring = 2
        }

        public enum CCDMode
        {
            Disabled = 0,
            CastRay = 1,
            CastShape = 2
        }

        public enum BodyState
        {
            Transform = 0,
            LinearVelocity = 1,
            AngularVelocity = 2,
            Sleeping = 3,
            CanSleep = 4
        }

        public enum BodyParameter
        {
            Bounce = 0,
            Friction = 1,
            Mass = 2,
            Inertia = 3,
            GravityScale = 4,
            LinearDamp = 5,
            AngularDamp = 6,
            Max = 7
        }

        public enum AreaSpaceOverrideMode
        {
            Disabled = 0,
            Combine = 1,
            CombineReplace = 2,
            Replace = 3,
            ReplaceCombine = 4
        }

        public enum AreaParameter
        {
            Gravity = 0,
            GravityVector = 1,
            GravityIsPoint = 2,
            GravityDistanceScale = 3,
            GravityPointAttenuation = 4,
            LinearDamp = 5,
            AngularDamp = 6,
            Priority = 7
        }


        private const string nativeName = "Physics2DServer";
        internal static IntPtr ptr = NativeCalls.godot_icall_Physics2DServer_get_singleton();

        private static IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "line_shape_create");

        /// <summary>
        /// </summary>
        [GodotMethod("line_shape_create")]
        public static RID LineShapeCreate()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_0, ptr);
            return new RID(ret);
        }

        private static IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "ray_shape_create");

        /// <summary>
        /// </summary>
        [GodotMethod("ray_shape_create")]
        public static RID RayShapeCreate()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_1, ptr);
            return new RID(ret);
        }

        private static IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "segment_shape_create");

        /// <summary>
        /// </summary>
        [GodotMethod("segment_shape_create")]
        public static RID SegmentShapeCreate()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_2, ptr);
            return new RID(ret);
        }

        private static IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "circle_shape_create");

        /// <summary>
        /// </summary>
        [GodotMethod("circle_shape_create")]
        public static RID CircleShapeCreate()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_3, ptr);
            return new RID(ret);
        }

        private static IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "rectangle_shape_create");

        /// <summary>
        /// </summary>
        [GodotMethod("rectangle_shape_create")]
        public static RID RectangleShapeCreate()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_4, ptr);
            return new RID(ret);
        }

        private static IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "capsule_shape_create");

        /// <summary>
        /// </summary>
        [GodotMethod("capsule_shape_create")]
        public static RID CapsuleShapeCreate()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_5, ptr);
            return new RID(ret);
        }

        private static IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "convex_polygon_shape_create");

        /// <summary>
        /// </summary>
        [GodotMethod("convex_polygon_shape_create")]
        public static RID ConvexPolygonShapeCreate()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_6, ptr);
            return new RID(ret);
        }

        private static IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "concave_polygon_shape_create");

        /// <summary>
        /// </summary>
        [GodotMethod("concave_polygon_shape_create")]
        public static RID ConcavePolygonShapeCreate()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_7, ptr);
            return new RID(ret);
        }

        private static IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "shape_set_data");

        /// <summary>
        /// Sets the shape data that defines its shape and size. The data to be passed depends on the kind of shape created [method shape_get_type].
        /// </summary>
        [GodotMethod("shape_set_data")]
        public static void ShapeSetData(RID shape, object data)
        {
            NativeCalls.godot_icall_2_607(method_bind_8, ptr, RID.GetPtr(shape), data);
        }

        private static IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "shape_get_type");

        /// <summary>
        /// Returns the type of shape (see SHAPE_* constants).
        /// </summary>
        [GodotMethod("shape_get_type")]
        public static Physics2DServer.ShapeType ShapeGetType(RID shape)
        {
            return NativeCalls.godot_icall_1_608(method_bind_9, ptr, RID.GetPtr(shape));
        }

        private static IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "shape_get_data");

        /// <summary>
        /// Returns the shape data.
        /// </summary>
        [GodotMethod("shape_get_data")]
        public static object ShapeGetData(RID shape)
        {
            return NativeCalls.godot_icall_1_609(method_bind_10, ptr, RID.GetPtr(shape));
        }

        private static IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "space_create");

        /// <summary>
        /// Creates a space. A space is a collection of parameters for the physics engine that can be assigned to an area or a body. It can be assigned to an area with [method area_set_space], or to a body with [method body_set_space].
        /// </summary>
        [GodotMethod("space_create")]
        public static RID SpaceCreate()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_11, ptr);
            return new RID(ret);
        }

        private static IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "space_set_active");

        /// <summary>
        /// Marks a space as active. It will not have an effect, unless it is assigned to an area or body.
        /// </summary>
        [GodotMethod("space_set_active")]
        public static void SpaceSetActive(RID space, bool active)
        {
            NativeCalls.godot_icall_2_610(method_bind_12, ptr, RID.GetPtr(space), active);
        }

        private static IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "space_is_active");

        /// <summary>
        /// Returns whether the space is active.
        /// </summary>
        [GodotMethod("space_is_active")]
        public static bool SpaceIsActive(RID space)
        {
            return NativeCalls.godot_icall_1_611(method_bind_13, ptr, RID.GetPtr(space));
        }

        private static IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "space_set_param");

        /// <summary>
        /// Sets the value for a space parameter. A list of available parameters is on the SPACE_PARAM_* constants.
        /// </summary>
        [GodotMethod("space_set_param")]
        public static void SpaceSetParam(RID space, Physics2DServer.SpaceParameter param, float value)
        {
            NativeCalls.godot_icall_3_612(method_bind_14, ptr, RID.GetPtr(space), param, value);
        }

        private static IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "space_get_param");

        /// <summary>
        /// Returns the value of a space parameter.
        /// </summary>
        [GodotMethod("space_get_param")]
        public static float SpaceGetParam(RID space, Physics2DServer.SpaceParameter param)
        {
            return NativeCalls.godot_icall_2_613(method_bind_15, ptr, RID.GetPtr(space), param);
        }

        private static IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "space_get_direct_state");

        /// <summary>
        /// Returns the state of a space, a [Physics2DDirectSpaceState]. This object can be used to make collision/intersection queries.
        /// </summary>
        [GodotMethod("space_get_direct_state")]
        public static Physics2DDirectSpaceState SpaceGetDirectState(RID space)
        {
            return NativeCalls.godot_icall_1_614(method_bind_16, ptr, RID.GetPtr(space));
        }

        private static IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_create");

        /// <summary>
        /// Creates an [Area2D].
        /// </summary>
        [GodotMethod("area_create")]
        public static RID AreaCreate()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_17, ptr);
            return new RID(ret);
        }

        private static IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_set_space");

        /// <summary>
        /// Assigns a space to the area.
        /// </summary>
        [GodotMethod("area_set_space")]
        public static void AreaSetSpace(RID area, RID space)
        {
            NativeCalls.godot_icall_2_615(method_bind_18, ptr, RID.GetPtr(area), RID.GetPtr(space));
        }

        private static IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_get_space");

        /// <summary>
        /// Returns the space assigned to the area.
        /// </summary>
        [GodotMethod("area_get_space")]
        public static RID AreaGetSpace(RID area)
        {
            IntPtr ret = NativeCalls.godot_icall_1_616(method_bind_19, ptr, RID.GetPtr(area));
            return new RID(ret);
        }

        private static IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_set_space_override_mode");

        /// <summary>
        /// Sets the space override mode for the area. The modes are described in the constants AREA_SPACE_OVERRIDE_*.
        /// </summary>
        [GodotMethod("area_set_space_override_mode")]
        public static void AreaSetSpaceOverrideMode(RID area, Physics2DServer.AreaSpaceOverrideMode mode)
        {
            NativeCalls.godot_icall_2_617(method_bind_20, ptr, RID.GetPtr(area), mode);
        }

        private static IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_get_space_override_mode");

        /// <summary>
        /// Returns the space override mode for the area.
        /// </summary>
        [GodotMethod("area_get_space_override_mode")]
        public static Physics2DServer.AreaSpaceOverrideMode AreaGetSpaceOverrideMode(RID area)
        {
            return NativeCalls.godot_icall_1_618(method_bind_21, ptr, RID.GetPtr(area));
        }

        private static IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_add_shape");

        /// <summary>
        /// Adds a shape to the area, along with a transform matrix. Shapes are usually referenced by their index, so you should track which shape has a given index.
        /// &lt;param name="transform"&gt;If the param is null, then the default value is Transform2D.Identity&lt;/param&gt;
        /// </summary>
        [GodotMethod("area_add_shape")]
        public static void AreaAddShape(RID area, RID shape, Nullable<Transform2D> transform = null)
        {
            Transform2D transform_in = transform.HasValue ? transform.Value : Transform2D.Identity;
            NativeCalls.godot_icall_3_619(method_bind_22, ptr, RID.GetPtr(area), RID.GetPtr(shape), ref transform_in);
        }

        private static IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_set_shape");

        /// <summary>
        /// Substitutes a given area shape by another. The old shape is selected by its index, the new one by its [RID].
        /// </summary>
        [GodotMethod("area_set_shape")]
        public static void AreaSetShape(RID area, int shapeIdx, RID shape)
        {
            NativeCalls.godot_icall_3_620(method_bind_23, ptr, RID.GetPtr(area), shapeIdx, RID.GetPtr(shape));
        }

        private static IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_set_shape_transform");

        /// <summary>
        /// Sets the transform matrix for an area shape.
        /// </summary>
        [GodotMethod("area_set_shape_transform")]
        public static void AreaSetShapeTransform(RID area, int shapeIdx, Transform2D transform)
        {
            NativeCalls.godot_icall_3_621(method_bind_24, ptr, RID.GetPtr(area), shapeIdx, ref transform);
        }

        private static IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_set_shape_disabled");

        /// <summary>
        /// Disables a given shape in an area.
        /// </summary>
        [GodotMethod("area_set_shape_disabled")]
        public static void AreaSetShapeDisabled(RID area, int shapeIdx, bool disable)
        {
            NativeCalls.godot_icall_3_622(method_bind_25, ptr, RID.GetPtr(area), shapeIdx, disable);
        }

        private static IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_get_shape_count");

        /// <summary>
        /// Returns the number of shapes assigned to an area.
        /// </summary>
        [GodotMethod("area_get_shape_count")]
        public static int AreaGetShapeCount(RID area)
        {
            return NativeCalls.godot_icall_1_623(method_bind_26, ptr, RID.GetPtr(area));
        }

        private static IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_get_shape");

        /// <summary>
        /// Returns the [RID] of the nth shape of an area.
        /// </summary>
        [GodotMethod("area_get_shape")]
        public static RID AreaGetShape(RID area, int shapeIdx)
        {
            IntPtr ret = NativeCalls.godot_icall_2_624(method_bind_27, ptr, RID.GetPtr(area), shapeIdx);
            return new RID(ret);
        }

        private static IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_get_shape_transform");

        /// <summary>
        /// Returns the transform matrix of a shape within an area.
        /// </summary>
        [GodotMethod("area_get_shape_transform")]
        public static Transform2D AreaGetShapeTransform(RID area, int shapeIdx)
        {
            object ret = NativeCalls.godot_icall_2_625(method_bind_28, ptr, RID.GetPtr(area), shapeIdx);
            return (Transform2D)ret;
        }

        private static IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_remove_shape");

        /// <summary>
        /// Removes a shape from an area. It does not delete the shape, so it can be reassigned later.
        /// </summary>
        [GodotMethod("area_remove_shape")]
        public static void AreaRemoveShape(RID area, int shapeIdx)
        {
            NativeCalls.godot_icall_2_626(method_bind_29, ptr, RID.GetPtr(area), shapeIdx);
        }

        private static IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_clear_shapes");

        /// <summary>
        /// Removes all shapes from an area. It does not delete the shapes, so they can be reassigned later.
        /// </summary>
        [GodotMethod("area_clear_shapes")]
        public static void AreaClearShapes(RID area)
        {
            NativeCalls.godot_icall_1_627(method_bind_30, ptr, RID.GetPtr(area));
        }

        private static IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_set_collision_layer");

        /// <summary>
        /// Assigns the area to one or many physics layers.
        /// </summary>
        [GodotMethod("area_set_collision_layer")]
        public static void AreaSetCollisionLayer(RID area, int layer)
        {
            NativeCalls.godot_icall_2_626(method_bind_31, ptr, RID.GetPtr(area), layer);
        }

        private static IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_set_collision_mask");

        /// <summary>
        /// Sets which physics layers the area will monitor.
        /// </summary>
        [GodotMethod("area_set_collision_mask")]
        public static void AreaSetCollisionMask(RID area, int mask)
        {
            NativeCalls.godot_icall_2_626(method_bind_32, ptr, RID.GetPtr(area), mask);
        }

        private static IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_set_param");

        /// <summary>
        /// Sets the value for an area parameter. A list of available parameters is on the AREA_PARAM_* constants.
        /// </summary>
        [GodotMethod("area_set_param")]
        public static void AreaSetParam(RID area, Physics2DServer.AreaParameter param, object value)
        {
            NativeCalls.godot_icall_3_628(method_bind_33, ptr, RID.GetPtr(area), param, value);
        }

        private static IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_set_transform");

        /// <summary>
        /// Sets the transform matrix for an area.
        /// </summary>
        [GodotMethod("area_set_transform")]
        public static void AreaSetTransform(RID area, Transform2D transform)
        {
            NativeCalls.godot_icall_2_629(method_bind_34, ptr, RID.GetPtr(area), ref transform);
        }

        private static IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_get_param");

        /// <summary>
        /// Returns an area parameter value. A list of available parameters is on the AREA_PARAM_* constants.
        /// </summary>
        [GodotMethod("area_get_param")]
        public static object AreaGetParam(RID area, Physics2DServer.AreaParameter param)
        {
            return NativeCalls.godot_icall_2_630(method_bind_35, ptr, RID.GetPtr(area), param);
        }

        private static IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_get_transform");

        /// <summary>
        /// Returns the transform matrix for an area.
        /// </summary>
        [GodotMethod("area_get_transform")]
        public static Transform2D AreaGetTransform(RID area)
        {
            object ret = NativeCalls.godot_icall_1_631(method_bind_36, ptr, RID.GetPtr(area));
            return (Transform2D)ret;
        }

        private static IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_attach_object_instance_id");

        /// <summary>
        /// Assigns the area to a descendant of [Object], so it can exist in the node tree.
        /// </summary>
        [GodotMethod("area_attach_object_instance_id")]
        public static void AreaAttachObjectInstanceId(RID area, int id)
        {
            NativeCalls.godot_icall_2_626(method_bind_37, ptr, RID.GetPtr(area), id);
        }

        private static IntPtr method_bind_38 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_get_object_instance_id");

        /// <summary>
        /// Gets the instance ID of the object the area is assigned to.
        /// </summary>
        [GodotMethod("area_get_object_instance_id")]
        public static int AreaGetObjectInstanceId(RID area)
        {
            return NativeCalls.godot_icall_1_623(method_bind_38, ptr, RID.GetPtr(area));
        }

        private static IntPtr method_bind_39 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_set_monitor_callback");

        /// <summary>
        /// Sets the function to call when any body/area enters or exits the area. This callback will be called for any object interacting with the area, and takes five parameters:
        /// 1: AREA_BODY_ADDED or AREA_BODY_REMOVED, depending on whether the object entered or exited the area.
        /// 2: [RID] of the object that entered/exited the area.
        /// 3: Instance ID of the object that entered/exited the area.
        /// 4: The shape index of the object that entered/exited the area.
        /// 5: The shape index of the area where the object entered/exited.
        /// </summary>
        [GodotMethod("area_set_monitor_callback")]
        public static void AreaSetMonitorCallback(RID area, Object receiver, string method)
        {
            NativeCalls.godot_icall_3_632(method_bind_39, ptr, RID.GetPtr(area), Object.GetPtr(receiver), method);
        }

        private static IntPtr method_bind_40 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_create");

        /// <summary>
        /// Creates a physics body. The first parameter can be any value from constants BODY_MODE*, for the type of body created. Additionally, the body can be created in sleeping state to save processing time.
        /// </summary>
        [GodotMethod("body_create")]
        public static RID BodyCreate()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_40, ptr);
            return new RID(ret);
        }

        private static IntPtr method_bind_41 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_space");

        /// <summary>
        /// Assigns a space to the body (see [method create_space]).
        /// </summary>
        [GodotMethod("body_set_space")]
        public static void BodySetSpace(RID body, RID space)
        {
            NativeCalls.godot_icall_2_615(method_bind_41, ptr, RID.GetPtr(body), RID.GetPtr(space));
        }

        private static IntPtr method_bind_42 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_get_space");

        /// <summary>
        /// Returns the [RID] of the space assigned to a body.
        /// </summary>
        [GodotMethod("body_get_space")]
        public static RID BodyGetSpace(RID body)
        {
            IntPtr ret = NativeCalls.godot_icall_1_616(method_bind_42, ptr, RID.GetPtr(body));
            return new RID(ret);
        }

        private static IntPtr method_bind_43 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_mode");

        /// <summary>
        /// Sets the body mode, from one of the constants BODY_MODE*.
        /// </summary>
        [GodotMethod("body_set_mode")]
        public static void BodySetMode(RID body, Physics2DServer.BodyMode mode)
        {
            NativeCalls.godot_icall_2_633(method_bind_43, ptr, RID.GetPtr(body), mode);
        }

        private static IntPtr method_bind_44 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_get_mode");

        /// <summary>
        /// Returns the body mode.
        /// </summary>
        [GodotMethod("body_get_mode")]
        public static Physics2DServer.BodyMode BodyGetMode(RID body)
        {
            return NativeCalls.godot_icall_1_634(method_bind_44, ptr, RID.GetPtr(body));
        }

        private static IntPtr method_bind_45 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_add_shape");

        /// <summary>
        /// Adds a shape to the body, along with a transform matrix. Shapes are usually referenced by their index, so you should track which shape has a given index.
        /// &lt;param name="transform"&gt;If the param is null, then the default value is Transform2D.Identity&lt;/param&gt;
        /// </summary>
        [GodotMethod("body_add_shape")]
        public static void BodyAddShape(RID body, RID shape, Nullable<Transform2D> transform = null)
        {
            Transform2D transform_in = transform.HasValue ? transform.Value : Transform2D.Identity;
            NativeCalls.godot_icall_3_619(method_bind_45, ptr, RID.GetPtr(body), RID.GetPtr(shape), ref transform_in);
        }

        private static IntPtr method_bind_46 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_shape");

        /// <summary>
        /// Substitutes a given body shape by another. The old shape is selected by its index, the new one by its [RID].
        /// </summary>
        [GodotMethod("body_set_shape")]
        public static void BodySetShape(RID body, int shapeIdx, RID shape)
        {
            NativeCalls.godot_icall_3_620(method_bind_46, ptr, RID.GetPtr(body), shapeIdx, RID.GetPtr(shape));
        }

        private static IntPtr method_bind_47 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_shape_transform");

        /// <summary>
        /// Sets the transform matrix for a body shape.
        /// </summary>
        [GodotMethod("body_set_shape_transform")]
        public static void BodySetShapeTransform(RID body, int shapeIdx, Transform2D transform)
        {
            NativeCalls.godot_icall_3_621(method_bind_47, ptr, RID.GetPtr(body), shapeIdx, ref transform);
        }

        private static IntPtr method_bind_48 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_shape_metadata");

        /// <summary>
        /// Sets metadata of a shape within a body. This metadata is different from [method Object.set_meta], and can be retrieved on shape queries.
        /// </summary>
        [GodotMethod("body_set_shape_metadata")]
        public static void BodySetShapeMetadata(RID body, int shapeIdx, object metadata)
        {
            NativeCalls.godot_icall_3_635(method_bind_48, ptr, RID.GetPtr(body), shapeIdx, metadata);
        }

        private static IntPtr method_bind_49 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_get_shape_count");

        /// <summary>
        /// Returns the number of shapes assigned to a body.
        /// </summary>
        [GodotMethod("body_get_shape_count")]
        public static int BodyGetShapeCount(RID body)
        {
            return NativeCalls.godot_icall_1_623(method_bind_49, ptr, RID.GetPtr(body));
        }

        private static IntPtr method_bind_50 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_get_shape");

        /// <summary>
        /// Returns the [RID] of the nth shape of a body.
        /// </summary>
        [GodotMethod("body_get_shape")]
        public static RID BodyGetShape(RID body, int shapeIdx)
        {
            IntPtr ret = NativeCalls.godot_icall_2_624(method_bind_50, ptr, RID.GetPtr(body), shapeIdx);
            return new RID(ret);
        }

        private static IntPtr method_bind_51 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_get_shape_transform");

        /// <summary>
        /// Returns the transform matrix of a body shape.
        /// </summary>
        [GodotMethod("body_get_shape_transform")]
        public static Transform2D BodyGetShapeTransform(RID body, int shapeIdx)
        {
            object ret = NativeCalls.godot_icall_2_625(method_bind_51, ptr, RID.GetPtr(body), shapeIdx);
            return (Transform2D)ret;
        }

        private static IntPtr method_bind_52 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_get_shape_metadata");

        /// <summary>
        /// Returns the metadata of a shape of a body.
        /// </summary>
        [GodotMethod("body_get_shape_metadata")]
        public static object BodyGetShapeMetadata(RID body, int shapeIdx)
        {
            return NativeCalls.godot_icall_2_636(method_bind_52, ptr, RID.GetPtr(body), shapeIdx);
        }

        private static IntPtr method_bind_53 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_remove_shape");

        /// <summary>
        /// Removes a shape from a body. The shape is not deleted, so it can be reused afterwards.
        /// </summary>
        [GodotMethod("body_remove_shape")]
        public static void BodyRemoveShape(RID body, int shapeIdx)
        {
            NativeCalls.godot_icall_2_626(method_bind_53, ptr, RID.GetPtr(body), shapeIdx);
        }

        private static IntPtr method_bind_54 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_clear_shapes");

        /// <summary>
        /// Removes all shapes from a body.
        /// </summary>
        [GodotMethod("body_clear_shapes")]
        public static void BodyClearShapes(RID body)
        {
            NativeCalls.godot_icall_1_627(method_bind_54, ptr, RID.GetPtr(body));
        }

        private static IntPtr method_bind_55 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_shape_disabled");

        /// <summary>
        /// Disables shape in body if [code]disable[/code] is [code]true[/code].
        /// </summary>
        [GodotMethod("body_set_shape_disabled")]
        public static void BodySetShapeDisabled(RID body, int shapeIdx, bool disable)
        {
            NativeCalls.godot_icall_3_622(method_bind_55, ptr, RID.GetPtr(body), shapeIdx, disable);
        }

        private static IntPtr method_bind_56 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_shape_as_one_way_collision");

        /// <summary>
        /// Enables one way collision on body if [code]enable[/code] is [code]true[/code].
        /// </summary>
        [GodotMethod("body_set_shape_as_one_way_collision")]
        public static void BodySetShapeAsOneWayCollision(RID body, int shapeIdx, bool enable)
        {
            NativeCalls.godot_icall_3_622(method_bind_56, ptr, RID.GetPtr(body), shapeIdx, enable);
        }

        private static IntPtr method_bind_57 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_attach_object_instance_id");

        /// <summary>
        /// Assigns the area to a descendant of [Object], so it can exist in the node tree.
        /// </summary>
        [GodotMethod("body_attach_object_instance_id")]
        public static void BodyAttachObjectInstanceId(RID body, int id)
        {
            NativeCalls.godot_icall_2_626(method_bind_57, ptr, RID.GetPtr(body), id);
        }

        private static IntPtr method_bind_58 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_get_object_instance_id");

        /// <summary>
        /// Gets the instance ID of the object the area is assigned to.
        /// </summary>
        [GodotMethod("body_get_object_instance_id")]
        public static int BodyGetObjectInstanceId(RID body)
        {
            return NativeCalls.godot_icall_1_623(method_bind_58, ptr, RID.GetPtr(body));
        }

        private static IntPtr method_bind_59 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_continuous_collision_detection_mode");

        /// <summary>
        /// Sets the continuous collision detection mode from any of the CCD_MODE_* constants.
        /// Continuous collision detection tries to predict where a moving body will collide, instead of moving it and correcting its movement if it collided.
        /// </summary>
        [GodotMethod("body_set_continuous_collision_detection_mode")]
        public static void BodySetContinuousCollisionDetectionMode(RID body, Physics2DServer.CCDMode mode)
        {
            NativeCalls.godot_icall_2_637(method_bind_59, ptr, RID.GetPtr(body), mode);
        }

        private static IntPtr method_bind_60 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_get_continuous_collision_detection_mode");

        /// <summary>
        /// Returns the continuous collision detection mode.
        /// </summary>
        [GodotMethod("body_get_continuous_collision_detection_mode")]
        public static Physics2DServer.CCDMode BodyGetContinuousCollisionDetectionMode(RID body)
        {
            return NativeCalls.godot_icall_1_638(method_bind_60, ptr, RID.GetPtr(body));
        }

        private static IntPtr method_bind_61 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_collision_layer");

        /// <summary>
        /// Sets the physics layer or layers a body belongs to.
        /// </summary>
        [GodotMethod("body_set_collision_layer")]
        public static void BodySetCollisionLayer(RID body, int layer)
        {
            NativeCalls.godot_icall_2_626(method_bind_61, ptr, RID.GetPtr(body), layer);
        }

        private static IntPtr method_bind_62 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_get_collision_layer");

        /// <summary>
        /// Returns the physics layer or layers a body belongs to.
        /// </summary>
        [GodotMethod("body_get_collision_layer")]
        public static int BodyGetCollisionLayer(RID body)
        {
            return NativeCalls.godot_icall_1_623(method_bind_62, ptr, RID.GetPtr(body));
        }

        private static IntPtr method_bind_63 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_collision_mask");

        /// <summary>
        /// Sets the physics layer or layers a body can collide with.
        /// </summary>
        [GodotMethod("body_set_collision_mask")]
        public static void BodySetCollisionMask(RID body, int mask)
        {
            NativeCalls.godot_icall_2_626(method_bind_63, ptr, RID.GetPtr(body), mask);
        }

        private static IntPtr method_bind_64 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_get_collision_mask");

        /// <summary>
        /// Returns the physics layer or layers a body can collide with.
        /// </summary>
        [GodotMethod("body_get_collision_mask")]
        public static int BodyGetCollisionMask(RID body)
        {
            return NativeCalls.godot_icall_1_623(method_bind_64, ptr, RID.GetPtr(body));
        }

        private static IntPtr method_bind_65 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_param");

        /// <summary>
        /// Sets a body parameter. A list of available parameters is on the BODY_PARAM_* constants.
        /// </summary>
        [GodotMethod("body_set_param")]
        public static void BodySetParam(RID body, Physics2DServer.BodyParameter param, float value)
        {
            NativeCalls.godot_icall_3_639(method_bind_65, ptr, RID.GetPtr(body), param, value);
        }

        private static IntPtr method_bind_66 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_get_param");

        /// <summary>
        /// Returns the value of a body parameter. A list of available parameters is on the BODY_PARAM_* constants.
        /// </summary>
        [GodotMethod("body_get_param")]
        public static float BodyGetParam(RID body, Physics2DServer.BodyParameter param)
        {
            return NativeCalls.godot_icall_2_640(method_bind_66, ptr, RID.GetPtr(body), param);
        }

        private static IntPtr method_bind_67 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_state");

        /// <summary>
        /// Sets a body state (see BODY_STATE* constants).
        /// </summary>
        [GodotMethod("body_set_state")]
        public static void BodySetState(RID body, Physics2DServer.BodyState state, object value)
        {
            NativeCalls.godot_icall_3_641(method_bind_67, ptr, RID.GetPtr(body), state, value);
        }

        private static IntPtr method_bind_68 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_get_state");

        /// <summary>
        /// Returns a body state.
        /// </summary>
        [GodotMethod("body_get_state")]
        public static object BodyGetState(RID body, Physics2DServer.BodyState state)
        {
            return NativeCalls.godot_icall_2_642(method_bind_68, ptr, RID.GetPtr(body), state);
        }

        private static IntPtr method_bind_69 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_apply_impulse");

        /// <summary>
        /// Adds a positioned impulse to the applied force and torque. Both the force and the offset from the body origin are in global coordinates.
        /// </summary>
        [GodotMethod("body_apply_impulse")]
        public static void BodyApplyImpulse(RID body, Vector2 position, Vector2 impulse)
        {
            NativeCalls.godot_icall_3_643(method_bind_69, ptr, RID.GetPtr(body), ref position, ref impulse);
        }

        private static IntPtr method_bind_70 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_add_force");

        /// <summary>
        /// Adds a positioned force to the applied force and torque. As with [method body_apply_impulse], both the force and the offset from the body origin are in global coordinates. A force differs from an impulse in that, while the two are forces, the impulse clears itself after being applied.
        /// </summary>
        [GodotMethod("body_add_force")]
        public static void BodyAddForce(RID body, Vector2 offset, Vector2 force)
        {
            NativeCalls.godot_icall_3_643(method_bind_70, ptr, RID.GetPtr(body), ref offset, ref force);
        }

        private static IntPtr method_bind_71 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_axis_velocity");

        /// <summary>
        /// Sets an axis velocity. The velocity in the given vector axis will be set as the given vector length. This is useful for jumping behavior.
        /// </summary>
        [GodotMethod("body_set_axis_velocity")]
        public static void BodySetAxisVelocity(RID body, Vector2 axisVelocity)
        {
            NativeCalls.godot_icall_2_644(method_bind_71, ptr, RID.GetPtr(body), ref axisVelocity);
        }

        private static IntPtr method_bind_72 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_add_collision_exception");

        /// <summary>
        /// Adds a body to the list of bodies exempt from collisions.
        /// </summary>
        [GodotMethod("body_add_collision_exception")]
        public static void BodyAddCollisionException(RID body, RID exceptedBody)
        {
            NativeCalls.godot_icall_2_615(method_bind_72, ptr, RID.GetPtr(body), RID.GetPtr(exceptedBody));
        }

        private static IntPtr method_bind_73 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_remove_collision_exception");

        /// <summary>
        /// Removes a body from the list of bodies exempt from collisions.
        /// </summary>
        [GodotMethod("body_remove_collision_exception")]
        public static void BodyRemoveCollisionException(RID body, RID exceptedBody)
        {
            NativeCalls.godot_icall_2_615(method_bind_73, ptr, RID.GetPtr(body), RID.GetPtr(exceptedBody));
        }

        private static IntPtr method_bind_74 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_max_contacts_reported");

        /// <summary>
        /// Sets the maximum contacts to report. Bodies can keep a log of the contacts with other bodies, this is enabled by setting the maximum amount of contacts reported to a number greater than 0.
        /// </summary>
        [GodotMethod("body_set_max_contacts_reported")]
        public static void BodySetMaxContactsReported(RID body, int amount)
        {
            NativeCalls.godot_icall_2_626(method_bind_74, ptr, RID.GetPtr(body), amount);
        }

        private static IntPtr method_bind_75 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_get_max_contacts_reported");

        /// <summary>
        /// Returns the maximum contacts that can be reported. See [method body_set_max_contacts_reported].
        /// </summary>
        [GodotMethod("body_get_max_contacts_reported")]
        public static int BodyGetMaxContactsReported(RID body)
        {
            return NativeCalls.godot_icall_1_623(method_bind_75, ptr, RID.GetPtr(body));
        }

        private static IntPtr method_bind_76 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_omit_force_integration");

        /// <summary>
        /// Sets whether a body uses a callback function to calculate its own physics (see [method body_set_force_integration_callback]).
        /// </summary>
        [GodotMethod("body_set_omit_force_integration")]
        public static void BodySetOmitForceIntegration(RID body, bool enable)
        {
            NativeCalls.godot_icall_2_610(method_bind_76, ptr, RID.GetPtr(body), enable);
        }

        private static IntPtr method_bind_77 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_is_omitting_force_integration");

        /// <summary>
        /// Returns whether a body uses a callback function to calculate its own physics (see [method body_set_force_integration_callback]).
        /// </summary>
        [GodotMethod("body_is_omitting_force_integration")]
        public static bool BodyIsOmittingForceIntegration(RID body)
        {
            return NativeCalls.godot_icall_1_611(method_bind_77, ptr, RID.GetPtr(body));
        }

        private static IntPtr method_bind_78 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_force_integration_callback");

        /// <summary>
        /// Sets the function used to calculate physics for an object, if that object allows it (see [method body_set_omit_force integration]).
        /// </summary>
        [GodotMethod("body_set_force_integration_callback")]
        public static void BodySetForceIntegrationCallback(RID body, Object receiver, string method, object userdata = null)
        {
            NativeCalls.godot_icall_4_645(method_bind_78, ptr, RID.GetPtr(body), Object.GetPtr(receiver), method, userdata);
        }

        private static IntPtr method_bind_79 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_test_motion");

        /// <summary>
        /// Returns whether a body can move from a given point in a given direction. Apart from the boolean return value, a [Physics2DTestMotionResult] can be passed to return additional information in.
        /// </summary>
        [GodotMethod("body_test_motion")]
        public static bool BodyTestMotion(RID body, Transform2D from, Vector2 motion, float margin = 0.08f, Physics2DTestMotionResult result = null)
        {
            return NativeCalls.godot_icall_5_646(method_bind_79, ptr, RID.GetPtr(body), ref from, ref motion, margin, Object.GetPtr(result));
        }

        private static IntPtr method_bind_80 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_get_direct_state");

        /// <summary>
        /// Returns the [Physics2DDirectBodyState] of the body.
        /// </summary>
        [GodotMethod("body_get_direct_state")]
        public static Physics2DDirectBodyState BodyGetDirectState(RID body)
        {
            return NativeCalls.godot_icall_1_647(method_bind_80, ptr, RID.GetPtr(body));
        }

        private static IntPtr method_bind_81 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "joint_set_param");

        /// <summary>
        /// Sets a joint parameter. Parameters are explained in the JOINT_PARAM* constants.
        /// </summary>
        [GodotMethod("joint_set_param")]
        public static void JointSetParam(RID joint, Physics2DServer.JointParam param, float value)
        {
            NativeCalls.godot_icall_3_648(method_bind_81, ptr, RID.GetPtr(joint), param, value);
        }

        private static IntPtr method_bind_82 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "joint_get_param");

        /// <summary>
        /// Returns the value of a joint parameter.
        /// </summary>
        [GodotMethod("joint_get_param")]
        public static float JointGetParam(RID joint, Physics2DServer.JointParam param)
        {
            return NativeCalls.godot_icall_2_649(method_bind_82, ptr, RID.GetPtr(joint), param);
        }

        private static IntPtr method_bind_83 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "pin_joint_create");

        /// <summary>
        /// Creates a pin joint between two bodies. If not specified, the second body is assumed to be the joint itself.
        /// &lt;param name="bodyB"&gt;If the param is null, then the default value is new RID()&lt;/param&gt;
        /// </summary>
        [GodotMethod("pin_joint_create")]
        public static RID PinJointCreate(Vector2 anchor, RID bodyA, RID bodyB = null)
        {
            RID bodyB_in = bodyB != null ? bodyB : new RID();
            IntPtr ret = NativeCalls.godot_icall_3_650(method_bind_83, ptr, ref anchor, RID.GetPtr(bodyA), RID.GetPtr(bodyB_in));
            return new RID(ret);
        }

        private static IntPtr method_bind_84 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "groove_joint_create");

        /// <summary>
        /// Creates a groove joint between two bodies. If not specified, the bodyies are assumed to be the joint itself.
        /// &lt;param name="bodyA"&gt;If the param is null, then the default value is new RID()&lt;/param&gt;
        /// &lt;param name="bodyB"&gt;If the param is null, then the default value is new RID()&lt;/param&gt;
        /// </summary>
        [GodotMethod("groove_joint_create")]
        public static RID GrooveJointCreate(Vector2 groove1A, Vector2 groove2A, Vector2 anchorB, RID bodyA = null, RID bodyB = null)
        {
            RID bodyA_in = bodyA != null ? bodyA : new RID();
            RID bodyB_in = bodyB != null ? bodyB : new RID();
            IntPtr ret = NativeCalls.godot_icall_5_651(method_bind_84, ptr, ref groove1A, ref groove2A, ref anchorB, RID.GetPtr(bodyA_in), RID.GetPtr(bodyB_in));
            return new RID(ret);
        }

        private static IntPtr method_bind_85 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "damped_spring_joint_create");

        /// <summary>
        /// Creates a damped spring joint between two bodies. If not specified, the second body is assumed to be the joint itself.
        /// &lt;param name="bodyB"&gt;If the param is null, then the default value is new RID()&lt;/param&gt;
        /// </summary>
        [GodotMethod("damped_spring_joint_create")]
        public static RID DampedSpringJointCreate(Vector2 anchorA, Vector2 anchorB, RID bodyA, RID bodyB = null)
        {
            RID bodyB_in = bodyB != null ? bodyB : new RID();
            IntPtr ret = NativeCalls.godot_icall_4_652(method_bind_85, ptr, ref anchorA, ref anchorB, RID.GetPtr(bodyA), RID.GetPtr(bodyB_in));
            return new RID(ret);
        }

        private static IntPtr method_bind_86 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "damped_string_joint_set_param");

        /// <summary>
        /// Sets a damped spring joint parameter. Parameters are explained in the DAMPED_STRING* constants.
        /// </summary>
        [GodotMethod("damped_string_joint_set_param")]
        public static void DampedStringJointSetParam(RID joint, Physics2DServer.DampedStringParam param, float value)
        {
            NativeCalls.godot_icall_3_653(method_bind_86, ptr, RID.GetPtr(joint), param, value);
        }

        private static IntPtr method_bind_87 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "damped_string_joint_get_param");

        /// <summary>
        /// Returns the value of a damped spring joint parameter.
        /// </summary>
        [GodotMethod("damped_string_joint_get_param")]
        public static float DampedStringJointGetParam(RID joint, Physics2DServer.DampedStringParam param)
        {
            return NativeCalls.godot_icall_2_654(method_bind_87, ptr, RID.GetPtr(joint), param);
        }

        private static IntPtr method_bind_88 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "joint_get_type");

        /// <summary>
        /// Returns the type of a joint (see JOINT_* constants).
        /// </summary>
        [GodotMethod("joint_get_type")]
        public static Physics2DServer.JointType JointGetType(RID joint)
        {
            return NativeCalls.godot_icall_1_655(method_bind_88, ptr, RID.GetPtr(joint));
        }

        private static IntPtr method_bind_89 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "free_rid");

        /// <summary>
        /// Destroys any of the objects created by Physics2DServer. If the [RID] passed is not one of the objects that can be created by Physics2DServer, an error will be sent to the console.
        /// </summary>
        [GodotMethod("free_rid")]
        public static void FreeRid(RID rid)
        {
            NativeCalls.godot_icall_1_627(method_bind_89, ptr, RID.GetPtr(rid));
        }

        private static IntPtr method_bind_90 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_active");

        /// <summary>
        /// Activates or deactivates the 2D physics engine.
        /// </summary>
        [GodotMethod("set_active")]
        public static void SetActive(bool active)
        {
            NativeCalls.godot_icall_1_11(method_bind_90, ptr, active);
        }

        private static IntPtr method_bind_91 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_process_info");

        /// <summary>
        /// Returns information about the current state of the 2D physics engine. The states are listed under the INFO_* constants.
        /// </summary>
        [GodotMethod("get_process_info")]
        public static int GetProcessInfo(Physics2DServer.ProcessInfo processInfo)
        {
            return NativeCalls.godot_icall_1_656(method_bind_91, ptr, processInfo);
        }
    }
}
