using System;

namespace Godot
{
    /// <summary>
    /// This class contains the shape and other parameters for intersection/collision queries.
    /// </summary>
    public class Physics2DShapeQueryParameters : Reference
    {
        /// <summary>
        /// The physics layer the query should be made on.
        /// </summary>
        public int CollisionLayer
        {
            get
            {
                return GetCollisionLayer();
            }
            set
            {
                SetCollisionLayer(value);
            }
        }

        /// <summary>
        /// The list of objects or object [RID]s, that will be excluded from collisions.
        /// </summary>
        public object[] Exclude
        {
            get
            {
                return GetExclude();
            }
            set
            {
                SetExclude(value);
            }
        }

        /// <summary>
        /// The collision margin for the shape.
        /// </summary>
        public float Margin
        {
            get
            {
                return GetMargin();
            }
            set
            {
                SetMargin(value);
            }
        }

        /// <summary>
        /// The motion of the shape being queried for.
        /// </summary>
        public Vector2 Motion
        {
            get
            {
                return GetMotion();
            }
            set
            {
                SetMotion(value);
            }
        }

        /// <summary>
        /// The [RID] of the queried shape. See [method set_shape] also.
        /// </summary>
        public RID ShapeRid
        {
            get
            {
                return GetShapeRid();
            }
            set
            {
                SetShapeRid(value);
            }
        }

        /// <summary>
        /// the transform matrix of the queried shape.
        /// </summary>
        public Transform2D Transform
        {
            get
            {
                return GetTransform();
            }
            set
            {
                SetTransform(value);
            }
        }


        private const string nativeName = "Physics2DShapeQueryParameters";

        public Physics2DShapeQueryParameters() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Physics2DShapeQueryParameters_Ctor(this);
        }

        internal Physics2DShapeQueryParameters(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_shape");

        /// <summary>
        /// Set the [Shape2D] that will be used for collision/intersection queries.
        /// </summary>
        [GodotMethod("set_shape")]
        public void SetShape(Resource shape)
        {
            NativeCalls.godot_icall_1_24(method_bind_0, Object.GetPtr(this), Object.GetPtr(shape));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_shape_rid");

        [GodotMethod("set_shape_rid")]
        public void SetShapeRid(RID shape)
        {
            NativeCalls.godot_icall_1_627(method_bind_1, Object.GetPtr(this), RID.GetPtr(shape));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_shape_rid");

        [GodotMethod("get_shape_rid")]
        public RID GetShapeRid()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_2, Object.GetPtr(this));
            return new RID(ret);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_transform");

        [GodotMethod("set_transform")]
        public void SetTransform(Transform2D transform)
        {
            NativeCalls.godot_icall_1_219(method_bind_3, Object.GetPtr(this), ref transform);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_transform");

        [GodotMethod("get_transform")]
        public Transform2D GetTransform()
        {
            object ret = NativeCalls.godot_icall_0_220(method_bind_4, Object.GetPtr(this));
            return (Transform2D)ret;
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_motion");

        [GodotMethod("set_motion")]
        public void SetMotion(Vector2 motion)
        {
            NativeCalls.godot_icall_1_45(method_bind_5, Object.GetPtr(this), ref motion);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_motion");

        [GodotMethod("get_motion")]
        public Vector2 GetMotion()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_6, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_margin");

        [GodotMethod("set_margin")]
        public void SetMargin(float margin)
        {
            NativeCalls.godot_icall_1_10(method_bind_7, Object.GetPtr(this), margin);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_margin");

        [GodotMethod("get_margin")]
        public float GetMargin()
        {
            return NativeCalls.godot_icall_0_9(method_bind_8, Object.GetPtr(this));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_collision_layer");

        [GodotMethod("set_collision_layer")]
        public void SetCollisionLayer(int collisionLayer)
        {
            NativeCalls.godot_icall_1_0(method_bind_9, Object.GetPtr(this), collisionLayer);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collision_layer");

        [GodotMethod("get_collision_layer")]
        public int GetCollisionLayer()
        {
            return NativeCalls.godot_icall_0_1(method_bind_10, Object.GetPtr(this));
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_exclude");

        [GodotMethod("set_exclude")]
        public void SetExclude(object[] exclude)
        {
            NativeCalls.godot_icall_1_154(method_bind_11, Object.GetPtr(this), exclude);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_exclude");

        [GodotMethod("get_exclude")]
        public object[] GetExclude()
        {
            return NativeCalls.godot_icall_0_21(method_bind_12, Object.GetPtr(this));
        }
    }
}
