using System;

namespace Godot
{
    public class PhysicsDirectBodyState : Object
    {
        /// <summary>
        /// The timestep (delta) used for the simulation.
        /// </summary>
        public float Step
        {
            get
            {
                return GetStep();
            }
        }

        /// <summary>
        /// The inverse of the mass of the body.
        /// </summary>
        public float InverseMass
        {
            get
            {
                return GetInverseMass();
            }
        }

        /// <summary>
        /// The rate at which the body stops rotating, if there are not any other forces moving it.
        /// </summary>
        public float TotalAngularDamp
        {
            get
            {
                return GetTotalAngularDamp();
            }
        }

        /// <summary>
        /// The rate at which the body stops moving, if there are not any other forces moving it.
        /// </summary>
        public float TotalLinearDamp
        {
            get
            {
                return GetTotalLinearDamp();
            }
        }

        /// <summary>
        /// The inverse of the inertia of the body.
        /// </summary>
        public Vector3 InverseInertia
        {
            get
            {
                return GetInverseInertia();
            }
        }

        /// <summary>
        /// The total gravity vector being currently applied to this body.
        /// </summary>
        public Vector3 TotalGravity
        {
            get
            {
                return GetTotalGravity();
            }
        }

        /// <summary>
        /// </summary>
        public Vector3 CenterOfMass
        {
            get
            {
                return GetCenterOfMass();
            }
        }

        /// <summary>
        /// </summary>
        public Basis PrincipalInertiaAxes
        {
            get
            {
                return GetPrincipalInertiaAxes();
            }
        }

        /// <summary>
        /// The angular velocity of the body.
        /// </summary>
        public Vector3 AngularVelocity
        {
            get
            {
                return GetAngularVelocity();
            }
            set
            {
                SetAngularVelocity(value);
            }
        }

        /// <summary>
        /// The linear velocity of the body.
        /// </summary>
        public Vector3 LinearVelocity
        {
            get
            {
                return GetLinearVelocity();
            }
            set
            {
                SetLinearVelocity(value);
            }
        }

        /// <summary>
        /// [code]true[/code] if this body is currently sleeping (not active).
        /// </summary>
        public bool Sleeping
        {
            get
            {
                return IsSleeping();
            }
            set
            {
                SetSleepState(value);
            }
        }

        /// <summary>
        /// The transformation matrix of the body.
        /// </summary>
        public Transform Transform
        {
            get
            {
                return GetTransform();
            }
            set
            {
                SetTransform(value);
            }
        }


        private const string nativeName = "PhysicsDirectBodyState";

        internal PhysicsDirectBodyState() {}

        internal PhysicsDirectBodyState(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_total_gravity");

        [GodotMethod("get_total_gravity")]
        public Vector3 GetTotalGravity()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_0, Object.GetPtr(this));
            return (Vector3)ret;
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_total_linear_damp");

        [GodotMethod("get_total_linear_damp")]
        public float GetTotalLinearDamp()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_total_angular_damp");

        [GodotMethod("get_total_angular_damp")]
        public float GetTotalAngularDamp()
        {
            return NativeCalls.godot_icall_0_9(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_center_of_mass");

        [GodotMethod("get_center_of_mass")]
        public Vector3 GetCenterOfMass()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_3, Object.GetPtr(this));
            return (Vector3)ret;
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_principal_inertia_axes");

        [GodotMethod("get_principal_inertia_axes")]
        public Basis GetPrincipalInertiaAxes()
        {
            object ret = NativeCalls.godot_icall_0_16(method_bind_4, Object.GetPtr(this));
            return (Basis)ret;
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_inverse_mass");

        [GodotMethod("get_inverse_mass")]
        public float GetInverseMass()
        {
            return NativeCalls.godot_icall_0_9(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_inverse_inertia");

        [GodotMethod("get_inverse_inertia")]
        public Vector3 GetInverseInertia()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_6, Object.GetPtr(this));
            return (Vector3)ret;
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_linear_velocity");

        [GodotMethod("set_linear_velocity")]
        public void SetLinearVelocity(Vector3 velocity)
        {
            NativeCalls.godot_icall_1_104(method_bind_7, Object.GetPtr(this), ref velocity);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_linear_velocity");

        [GodotMethod("get_linear_velocity")]
        public Vector3 GetLinearVelocity()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_8, Object.GetPtr(this));
            return (Vector3)ret;
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_angular_velocity");

        [GodotMethod("set_angular_velocity")]
        public void SetAngularVelocity(Vector3 velocity)
        {
            NativeCalls.godot_icall_1_104(method_bind_9, Object.GetPtr(this), ref velocity);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_angular_velocity");

        [GodotMethod("get_angular_velocity")]
        public Vector3 GetAngularVelocity()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_10, Object.GetPtr(this));
            return (Vector3)ret;
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_transform");

        [GodotMethod("set_transform")]
        public void SetTransform(Transform transform)
        {
            NativeCalls.godot_icall_1_155(method_bind_11, Object.GetPtr(this), ref transform);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_transform");

        [GodotMethod("get_transform")]
        public Transform GetTransform()
        {
            object ret = NativeCalls.godot_icall_0_18(method_bind_12, Object.GetPtr(this));
            return (Transform)ret;
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_force");

        /// <summary>
        /// </summary>
        [GodotMethod("add_force")]
        public void AddForce(Vector3 force, Vector3 position)
        {
            NativeCalls.godot_icall_2_657(method_bind_13, Object.GetPtr(this), ref force, ref position);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "apply_impulse");

        /// <summary>
        /// </summary>
        [GodotMethod("apply_impulse")]
        public void ApplyImpulse(Vector3 position, Vector3 j)
        {
            NativeCalls.godot_icall_2_657(method_bind_14, Object.GetPtr(this), ref position, ref j);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "apply_torqe_impulse");

        /// <summary>
        /// </summary>
        [GodotMethod("apply_torqe_impulse")]
        public void ApplyTorqeImpulse(Vector3 j)
        {
            NativeCalls.godot_icall_1_104(method_bind_15, Object.GetPtr(this), ref j);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_sleep_state");

        [GodotMethod("set_sleep_state")]
        public void SetSleepState(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_16, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_sleeping");

        [GodotMethod("is_sleeping")]
        public bool IsSleeping()
        {
            return NativeCalls.godot_icall_0_3(method_bind_17, Object.GetPtr(this));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_contact_count");

        /// <summary>
        /// </summary>
        [GodotMethod("get_contact_count")]
        public int GetContactCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_18, Object.GetPtr(this));
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_contact_local_position");

        /// <summary>
        /// </summary>
        [GodotMethod("get_contact_local_position")]
        public Vector3 GetContactLocalPosition(int contactIdx)
        {
            object ret = NativeCalls.godot_icall_1_26(method_bind_19, Object.GetPtr(this), contactIdx);
            return (Vector3)ret;
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_contact_local_normal");

        /// <summary>
        /// </summary>
        [GodotMethod("get_contact_local_normal")]
        public Vector3 GetContactLocalNormal(int contactIdx)
        {
            object ret = NativeCalls.godot_icall_1_26(method_bind_20, Object.GetPtr(this), contactIdx);
            return (Vector3)ret;
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_contact_local_shape");

        /// <summary>
        /// </summary>
        [GodotMethod("get_contact_local_shape")]
        public int GetContactLocalShape(int contactIdx)
        {
            return NativeCalls.godot_icall_1_6(method_bind_21, Object.GetPtr(this), contactIdx);
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_contact_collider");

        /// <summary>
        /// </summary>
        [GodotMethod("get_contact_collider")]
        public RID GetContactCollider(int contactIdx)
        {
            IntPtr ret = NativeCalls.godot_icall_1_402(method_bind_22, Object.GetPtr(this), contactIdx);
            return new RID(ret);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_contact_collider_position");

        /// <summary>
        /// </summary>
        [GodotMethod("get_contact_collider_position")]
        public Vector3 GetContactColliderPosition(int contactIdx)
        {
            object ret = NativeCalls.godot_icall_1_26(method_bind_23, Object.GetPtr(this), contactIdx);
            return (Vector3)ret;
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_contact_collider_id");

        /// <summary>
        /// </summary>
        [GodotMethod("get_contact_collider_id")]
        public int GetContactColliderId(int contactIdx)
        {
            return NativeCalls.godot_icall_1_6(method_bind_24, Object.GetPtr(this), contactIdx);
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_contact_collider_object");

        /// <summary>
        /// </summary>
        [GodotMethod("get_contact_collider_object")]
        public Object GetContactColliderObject(int contactIdx)
        {
            return NativeCalls.godot_icall_1_232(method_bind_25, Object.GetPtr(this), contactIdx);
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_contact_collider_shape");

        /// <summary>
        /// </summary>
        [GodotMethod("get_contact_collider_shape")]
        public int GetContactColliderShape(int contactIdx)
        {
            return NativeCalls.godot_icall_1_6(method_bind_26, Object.GetPtr(this), contactIdx);
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_contact_collider_velocity_at_position");

        /// <summary>
        /// </summary>
        [GodotMethod("get_contact_collider_velocity_at_position")]
        public Vector3 GetContactColliderVelocityAtPosition(int contactIdx)
        {
            object ret = NativeCalls.godot_icall_1_26(method_bind_27, Object.GetPtr(this), contactIdx);
            return (Vector3)ret;
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_step");

        [GodotMethod("get_step")]
        public float GetStep()
        {
            return NativeCalls.godot_icall_0_9(method_bind_28, Object.GetPtr(this));
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "integrate_forces");

        /// <summary>
        /// </summary>
        [GodotMethod("integrate_forces")]
        public void IntegrateForces()
        {
            NativeCalls.godot_icall_0_12(method_bind_29, Object.GetPtr(this));
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_space_state");

        /// <summary>
        /// </summary>
        [GodotMethod("get_space_state")]
        public PhysicsDirectSpaceState GetSpaceState()
        {
            return NativeCalls.godot_icall_0_658(method_bind_30, Object.GetPtr(this));
        }
    }
}
