using System;

namespace Godot
{
    /// <summary>
    /// Everything related to physics in 3D.
    /// </summary>
    public static class PhysicsServer
    {
        public enum BodyAxis
        {
            LinearX = 1,
            LinearY = 2,
            LinearZ = 4,
            AngularX = 8,
            AngularY = 16,
            AngularZ = 32
        }

        public enum ProcessInfo
        {
            ActiveObjects = 0,
            CollisionPairs = 1,
            IslandCount = 2
        }

        public enum AreaBodyStatus
        {
            Added = 0,
            Removed = 1
        }

        public enum BodyMode
        {
            Static = 0,
            Kinematic = 1,
            Rigid = 2,
            Soft = 3,
            Character = 4
        }

        public enum ShapeType
        {
            Plane = 0,
            Ray = 1,
            Sphere = 2,
            Box = 3,
            Capsule = 4,
            ConvexPolygon = 5,
            ConcavePolygon = 6,
            Heightmap = 7,
            Custom = 8
        }

        public enum PinJointParam
        {
            Bias = 0,
            Damping = 1,
            ImpulseClamp = 2
        }

        public enum SpaceParameter
        {
            ContactRecycleRadius = 0,
            ContactMaxSeparation = 1,
            BodyMaxAllowedPenetration = 2,
            BodyLinearVelocitySleepThreshold = 3,
            BodyAngularVelocitySleepThreshold = 4,
            BodyTimeToSleep = 5,
            BodyAngularVelocityDampRatio = 6,
            ConstraintDefaultBias = 7
        }

        public enum ConeTwistJointParam
        {
            SwingSpan = 0,
            TwistSpan = 1,
            Bias = 2,
            Softness = 3,
            Relaxation = 4
        }

        public enum JointType
        {
            Pin = 0,
            Hinge = 1,
            Slider = 2,
            ConeTwist = 3,
            Joint6dof = 4
        }

        public enum BodyState
        {
            Transform = 0,
            LinearVelocity = 1,
            AngularVelocity = 2,
            Sleeping = 3,
            CanSleep = 4
        }

        public enum BodyParameter
        {
            Bounce = 0,
            Friction = 1,
            Mass = 2,
            GravityScale = 3,
            LinearDamp = 4,
            AngularDamp = 5,
            Max = 6
        }

        public enum G6DOFJointAxisParam
        {
            LinearLowerLimit = 0,
            LinearUpperLimit = 1,
            LinearLimitSoftness = 2,
            LinearRestitution = 3,
            LinearDamping = 4,
            AngularLowerLimit = 5,
            AngularUpperLimit = 6,
            AngularLimitSoftness = 7,
            AngularDamping = 8,
            AngularRestitution = 9,
            AngularForceLimit = 10,
            AngularErp = 11,
            AngularMotorTargetVelocity = 12,
            AngularMotorForceLimit = 13
        }

        public enum SliderJointParam
        {
            LinearLimitUpper = 0,
            LinearLimitLower = 1,
            LinearLimitSoftness = 2,
            LinearLimitRestitution = 3,
            LinearLimitDamping = 4,
            LinearMotionSoftness = 5,
            LinearMotionRestitution = 6,
            LinearMotionDamping = 7,
            LinearOrthogonalSoftness = 8,
            LinearOrthogonalRestitution = 9,
            LinearOrthogonalDamping = 10,
            AngularLimitUpper = 11,
            AngularLimitLower = 12,
            AngularLimitSoftness = 13,
            AngularLimitRestitution = 14,
            AngularLimitDamping = 15,
            AngularMotionSoftness = 16,
            AngularMotionRestitution = 17,
            AngularMotionDamping = 18,
            AngularOrthogonalSoftness = 19,
            AngularOrthogonalRestitution = 20,
            AngularOrthogonalDamping = 21,
            Max = 22
        }

        public enum HingeJointParam
        {
            Bias = 0,
            LimitUpper = 1,
            LimitLower = 2,
            LimitBias = 3,
            LimitSoftness = 4,
            LimitRelaxation = 5,
            MotorTargetVelocity = 6,
            MotorMaxImpulse = 7
        }

        public enum G6DOFJointAxisFlag
        {
            LinearLimit = 0,
            AngularLimit = 1,
            Motor = 2
        }

        public enum HingeJointFlag
        {
            UseLimit = 0,
            EnableMotor = 1
        }

        public enum AreaSpaceOverrideMode
        {
            Disabled = 0,
            Combine = 1,
            CombineReplace = 2,
            Replace = 3,
            ReplaceCombine = 4
        }

        public enum AreaParameter
        {
            Gravity = 0,
            GravityVector = 1,
            GravityIsPoint = 2,
            GravityDistanceScale = 3,
            GravityPointAttenuation = 4,
            LinearDamp = 5,
            AngularDamp = 6,
            Priority = 7
        }


        private const string nativeName = "PhysicsServer";
        internal static IntPtr ptr = NativeCalls.godot_icall_PhysicsServer_get_singleton();

        private static IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "shape_create");

        /// <summary>
        /// Creates a shape of type SHAPE_*. Does not assign it to a body or an area. To do so, you must use [method area_set_shape] or [method body_set_shape].
        /// </summary>
        [GodotMethod("shape_create")]
        public static RID ShapeCreate(PhysicsServer.ShapeType type)
        {
            IntPtr ret = NativeCalls.godot_icall_1_661(method_bind_0, ptr, type);
            return new RID(ret);
        }

        private static IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "shape_set_data");

        /// <summary>
        /// Sets the shape data that defines its shape and size. The data to be passed depends on the kind of shape created [method shape_get_type].
        /// </summary>
        [GodotMethod("shape_set_data")]
        public static void ShapeSetData(RID shape, object data)
        {
            NativeCalls.godot_icall_2_607(method_bind_1, ptr, RID.GetPtr(shape), data);
        }

        private static IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "shape_get_type");

        /// <summary>
        /// Returns the type of shape (see SHAPE_* constants).
        /// </summary>
        [GodotMethod("shape_get_type")]
        public static PhysicsServer.ShapeType ShapeGetType(RID shape)
        {
            return NativeCalls.godot_icall_1_662(method_bind_2, ptr, RID.GetPtr(shape));
        }

        private static IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "shape_get_data");

        /// <summary>
        /// Returns the shape data.
        /// </summary>
        [GodotMethod("shape_get_data")]
        public static object ShapeGetData(RID shape)
        {
            return NativeCalls.godot_icall_1_609(method_bind_3, ptr, RID.GetPtr(shape));
        }

        private static IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "space_create");

        /// <summary>
        /// Creates a space. A space is a collection of parameters for the physics engine that can be assigned to an area or a body. It can be assigned to an area with [method area_set_space], or to a body with [method body_set_space].
        /// </summary>
        [GodotMethod("space_create")]
        public static RID SpaceCreate()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_4, ptr);
            return new RID(ret);
        }

        private static IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "space_set_active");

        /// <summary>
        /// Marks a space as active. It will not have an effect, unless it is assigned to an area or body.
        /// </summary>
        [GodotMethod("space_set_active")]
        public static void SpaceSetActive(RID space, bool active)
        {
            NativeCalls.godot_icall_2_610(method_bind_5, ptr, RID.GetPtr(space), active);
        }

        private static IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "space_is_active");

        /// <summary>
        /// Returns whether the space is active.
        /// </summary>
        [GodotMethod("space_is_active")]
        public static bool SpaceIsActive(RID space)
        {
            return NativeCalls.godot_icall_1_611(method_bind_6, ptr, RID.GetPtr(space));
        }

        private static IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "space_set_param");

        /// <summary>
        /// Sets the value for a space parameter. A list of available parameters is on the SPACE_PARAM_* constants.
        /// </summary>
        [GodotMethod("space_set_param")]
        public static void SpaceSetParam(RID space, PhysicsServer.SpaceParameter param, float value)
        {
            NativeCalls.godot_icall_3_663(method_bind_7, ptr, RID.GetPtr(space), param, value);
        }

        private static IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "space_get_param");

        /// <summary>
        /// Returns the value of a space parameter.
        /// </summary>
        [GodotMethod("space_get_param")]
        public static float SpaceGetParam(RID space, PhysicsServer.SpaceParameter param)
        {
            return NativeCalls.godot_icall_2_664(method_bind_8, ptr, RID.GetPtr(space), param);
        }

        private static IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "space_get_direct_state");

        /// <summary>
        /// Returns the state of a space, a [PhysicsDirectSpaceState]. This object can be used to make collision/intersection queries.
        /// </summary>
        [GodotMethod("space_get_direct_state")]
        public static PhysicsDirectSpaceState SpaceGetDirectState(RID space)
        {
            return NativeCalls.godot_icall_1_665(method_bind_9, ptr, RID.GetPtr(space));
        }

        private static IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_create");

        /// <summary>
        /// Creates an [Area].
        /// </summary>
        [GodotMethod("area_create")]
        public static RID AreaCreate()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_10, ptr);
            return new RID(ret);
        }

        private static IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_set_space");

        /// <summary>
        /// Assigns a space to the area.
        /// </summary>
        [GodotMethod("area_set_space")]
        public static void AreaSetSpace(RID area, RID space)
        {
            NativeCalls.godot_icall_2_615(method_bind_11, ptr, RID.GetPtr(area), RID.GetPtr(space));
        }

        private static IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_get_space");

        /// <summary>
        /// Returns the space assigned to the area.
        /// </summary>
        [GodotMethod("area_get_space")]
        public static RID AreaGetSpace(RID area)
        {
            IntPtr ret = NativeCalls.godot_icall_1_616(method_bind_12, ptr, RID.GetPtr(area));
            return new RID(ret);
        }

        private static IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_set_space_override_mode");

        /// <summary>
        /// Sets the space override mode for the area. The modes are described in the constants AREA_SPACE_OVERRIDE_*.
        /// </summary>
        [GodotMethod("area_set_space_override_mode")]
        public static void AreaSetSpaceOverrideMode(RID area, PhysicsServer.AreaSpaceOverrideMode mode)
        {
            NativeCalls.godot_icall_2_666(method_bind_13, ptr, RID.GetPtr(area), mode);
        }

        private static IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_get_space_override_mode");

        /// <summary>
        /// Returns the space override mode for the area.
        /// </summary>
        [GodotMethod("area_get_space_override_mode")]
        public static PhysicsServer.AreaSpaceOverrideMode AreaGetSpaceOverrideMode(RID area)
        {
            return NativeCalls.godot_icall_1_667(method_bind_14, ptr, RID.GetPtr(area));
        }

        private static IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_add_shape");

        /// <summary>
        /// Adds a shape to the area, along with a transform matrix. Shapes are usually referenced by their index, so you should track which shape has a given index.
        /// &lt;param name="transform"&gt;If the param is null, then the default value is new Transform()&lt;/param&gt;
        /// </summary>
        [GodotMethod("area_add_shape")]
        public static void AreaAddShape(RID area, RID shape, Nullable<Transform> transform = null)
        {
            Transform transform_in = transform.HasValue ? transform.Value : new Transform();
            NativeCalls.godot_icall_3_668(method_bind_15, ptr, RID.GetPtr(area), RID.GetPtr(shape), ref transform_in);
        }

        private static IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_set_shape");

        /// <summary>
        /// Substitutes a given area shape by another. The old shape is selected by its index, the new one by its [RID].
        /// </summary>
        [GodotMethod("area_set_shape")]
        public static void AreaSetShape(RID area, int shapeIdx, RID shape)
        {
            NativeCalls.godot_icall_3_620(method_bind_16, ptr, RID.GetPtr(area), shapeIdx, RID.GetPtr(shape));
        }

        private static IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_set_shape_transform");

        /// <summary>
        /// Sets the transform matrix for an area shape.
        /// </summary>
        [GodotMethod("area_set_shape_transform")]
        public static void AreaSetShapeTransform(RID area, int shapeIdx, Transform transform)
        {
            NativeCalls.godot_icall_3_669(method_bind_17, ptr, RID.GetPtr(area), shapeIdx, ref transform);
        }

        private static IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_get_shape_count");

        /// <summary>
        /// Returns the number of shapes assigned to an area.
        /// </summary>
        [GodotMethod("area_get_shape_count")]
        public static int AreaGetShapeCount(RID area)
        {
            return NativeCalls.godot_icall_1_623(method_bind_18, ptr, RID.GetPtr(area));
        }

        private static IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_get_shape");

        /// <summary>
        /// Returns the [RID] of the nth shape of an area.
        /// </summary>
        [GodotMethod("area_get_shape")]
        public static RID AreaGetShape(RID area, int shapeIdx)
        {
            IntPtr ret = NativeCalls.godot_icall_2_624(method_bind_19, ptr, RID.GetPtr(area), shapeIdx);
            return new RID(ret);
        }

        private static IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_get_shape_transform");

        /// <summary>
        /// Returns the transform matrix of a shape within an area.
        /// </summary>
        [GodotMethod("area_get_shape_transform")]
        public static Transform AreaGetShapeTransform(RID area, int shapeIdx)
        {
            object ret = NativeCalls.godot_icall_2_670(method_bind_20, ptr, RID.GetPtr(area), shapeIdx);
            return (Transform)ret;
        }

        private static IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_remove_shape");

        /// <summary>
        /// Removes a shape from an area. It does not delete the shape, so it can be reassigned later.
        /// </summary>
        [GodotMethod("area_remove_shape")]
        public static void AreaRemoveShape(RID area, int shapeIdx)
        {
            NativeCalls.godot_icall_2_626(method_bind_21, ptr, RID.GetPtr(area), shapeIdx);
        }

        private static IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_clear_shapes");

        /// <summary>
        /// Removes all shapes from an area. It does not delete the shapes, so they can be reassigned later.
        /// </summary>
        [GodotMethod("area_clear_shapes")]
        public static void AreaClearShapes(RID area)
        {
            NativeCalls.godot_icall_1_627(method_bind_22, ptr, RID.GetPtr(area));
        }

        private static IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_set_collision_layer");

        /// <summary>
        /// Assigns the area to one or many physics layers.
        /// </summary>
        [GodotMethod("area_set_collision_layer")]
        public static void AreaSetCollisionLayer(RID area, int layer)
        {
            NativeCalls.godot_icall_2_626(method_bind_23, ptr, RID.GetPtr(area), layer);
        }

        private static IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_set_collision_mask");

        /// <summary>
        /// Sets which physics layers the area will monitor.
        /// </summary>
        [GodotMethod("area_set_collision_mask")]
        public static void AreaSetCollisionMask(RID area, int mask)
        {
            NativeCalls.godot_icall_2_626(method_bind_24, ptr, RID.GetPtr(area), mask);
        }

        private static IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_set_param");

        /// <summary>
        /// Sets the value for an area parameter. A list of available parameters is on the AREA_PARAM_* constants.
        /// </summary>
        [GodotMethod("area_set_param")]
        public static void AreaSetParam(RID area, PhysicsServer.AreaParameter param, object value)
        {
            NativeCalls.godot_icall_3_671(method_bind_25, ptr, RID.GetPtr(area), param, value);
        }

        private static IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_set_transform");

        /// <summary>
        /// Sets the transform matrix for an area.
        /// </summary>
        [GodotMethod("area_set_transform")]
        public static void AreaSetTransform(RID area, Transform transform)
        {
            NativeCalls.godot_icall_2_672(method_bind_26, ptr, RID.GetPtr(area), ref transform);
        }

        private static IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_get_param");

        /// <summary>
        /// Returns an area parameter value. A list of available parameters is on the AREA_PARAM_* constants.
        /// </summary>
        [GodotMethod("area_get_param")]
        public static object AreaGetParam(RID area, PhysicsServer.AreaParameter param)
        {
            return NativeCalls.godot_icall_2_673(method_bind_27, ptr, RID.GetPtr(area), param);
        }

        private static IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_get_transform");

        /// <summary>
        /// Returns the transform matrix for an area.
        /// </summary>
        [GodotMethod("area_get_transform")]
        public static Transform AreaGetTransform(RID area)
        {
            object ret = NativeCalls.godot_icall_1_674(method_bind_28, ptr, RID.GetPtr(area));
            return (Transform)ret;
        }

        private static IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_attach_object_instance_id");

        /// <summary>
        /// Assigns the area to a descendant of [Object], so it can exist in the node tree.
        /// </summary>
        [GodotMethod("area_attach_object_instance_id")]
        public static void AreaAttachObjectInstanceId(RID area, int id)
        {
            NativeCalls.godot_icall_2_626(method_bind_29, ptr, RID.GetPtr(area), id);
        }

        private static IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_get_object_instance_id");

        /// <summary>
        /// Gets the instance ID of the object the area is assigned to.
        /// </summary>
        [GodotMethod("area_get_object_instance_id")]
        public static int AreaGetObjectInstanceId(RID area)
        {
            return NativeCalls.godot_icall_1_623(method_bind_30, ptr, RID.GetPtr(area));
        }

        private static IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_set_monitor_callback");

        /// <summary>
        /// Sets the function to call when any body/area enters or exits the area. This callback will be called for any object interacting with the area, and takes five parameters:
        /// 1: AREA_BODY_ADDED or AREA_BODY_REMOVED, depending on whether the object entered or exited the area.
        /// 2: [RID] of the object that entered/exited the area.
        /// 3: Instance ID of the object that entered/exited the area.
        /// 4: The shape index of the object that entered/exited the area.
        /// 5: The shape index of the area where the object entered/exited.
        /// </summary>
        [GodotMethod("area_set_monitor_callback")]
        public static void AreaSetMonitorCallback(RID area, Object receiver, string method)
        {
            NativeCalls.godot_icall_3_632(method_bind_31, ptr, RID.GetPtr(area), Object.GetPtr(receiver), method);
        }

        private static IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_set_ray_pickable");

        /// <summary>
        /// Sets object pickable with rays.
        /// </summary>
        [GodotMethod("area_set_ray_pickable")]
        public static void AreaSetRayPickable(RID area, bool enable)
        {
            NativeCalls.godot_icall_2_610(method_bind_32, ptr, RID.GetPtr(area), enable);
        }

        private static IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "area_is_ray_pickable");

        /// <summary>
        /// If [code]true[/code] area collides with rays.
        /// </summary>
        [GodotMethod("area_is_ray_pickable")]
        public static bool AreaIsRayPickable(RID area)
        {
            return NativeCalls.godot_icall_1_611(method_bind_33, ptr, RID.GetPtr(area));
        }

        private static IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_create");

        /// <summary>
        /// Creates a physics body. The first parameter can be any value from constants BODY_MODE*, for the type of body created. Additionally, the body can be created in sleeping state to save processing time.
        /// </summary>
        [GodotMethod("body_create")]
        public static RID BodyCreate(PhysicsServer.BodyMode mode = (PhysicsServer.BodyMode)2, bool initSleeping = false)
        {
            IntPtr ret = NativeCalls.godot_icall_2_675(method_bind_34, ptr, mode, initSleeping);
            return new RID(ret);
        }

        private static IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_space");

        /// <summary>
        /// Assigns a space to the body (see [method create_space]).
        /// </summary>
        [GodotMethod("body_set_space")]
        public static void BodySetSpace(RID body, RID space)
        {
            NativeCalls.godot_icall_2_615(method_bind_35, ptr, RID.GetPtr(body), RID.GetPtr(space));
        }

        private static IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_get_space");

        /// <summary>
        /// Returns the [RID] of the space assigned to a body.
        /// </summary>
        [GodotMethod("body_get_space")]
        public static RID BodyGetSpace(RID body)
        {
            IntPtr ret = NativeCalls.godot_icall_1_616(method_bind_36, ptr, RID.GetPtr(body));
            return new RID(ret);
        }

        private static IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_mode");

        /// <summary>
        /// Sets the body mode, from one of the constants BODY_MODE*.
        /// </summary>
        [GodotMethod("body_set_mode")]
        public static void BodySetMode(RID body, PhysicsServer.BodyMode mode)
        {
            NativeCalls.godot_icall_2_676(method_bind_37, ptr, RID.GetPtr(body), mode);
        }

        private static IntPtr method_bind_38 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_get_mode");

        /// <summary>
        /// Returns the body mode.
        /// </summary>
        [GodotMethod("body_get_mode")]
        public static PhysicsServer.BodyMode BodyGetMode(RID body)
        {
            return NativeCalls.godot_icall_1_677(method_bind_38, ptr, RID.GetPtr(body));
        }

        private static IntPtr method_bind_39 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_collision_layer");

        /// <summary>
        /// Sets the physics layer or layers a body belongs to.
        /// </summary>
        [GodotMethod("body_set_collision_layer")]
        public static void BodySetCollisionLayer(RID body, int layer)
        {
            NativeCalls.godot_icall_2_626(method_bind_39, ptr, RID.GetPtr(body), layer);
        }

        private static IntPtr method_bind_40 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_get_collision_layer");

        /// <summary>
        /// Returns the physics layer or layers a body belongs to.
        /// </summary>
        [GodotMethod("body_get_collision_layer")]
        public static int BodyGetCollisionLayer(RID body)
        {
            return NativeCalls.godot_icall_1_623(method_bind_40, ptr, RID.GetPtr(body));
        }

        private static IntPtr method_bind_41 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_collision_mask");

        /// <summary>
        /// Sets the physics layer or layers a body can collide with.
        /// </summary>
        [GodotMethod("body_set_collision_mask")]
        public static void BodySetCollisionMask(RID body, int mask)
        {
            NativeCalls.godot_icall_2_626(method_bind_41, ptr, RID.GetPtr(body), mask);
        }

        private static IntPtr method_bind_42 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_get_collision_mask");

        /// <summary>
        /// Returns the physics layer or layers a body can collide with.
        /// -
        /// </summary>
        [GodotMethod("body_get_collision_mask")]
        public static int BodyGetCollisionMask(RID body)
        {
            return NativeCalls.godot_icall_1_623(method_bind_42, ptr, RID.GetPtr(body));
        }

        private static IntPtr method_bind_43 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_add_shape");

        /// <summary>
        /// Adds a shape to the body, along with a transform matrix. Shapes are usually referenced by their index, so you should track which shape has a given index.
        /// &lt;param name="transform"&gt;If the param is null, then the default value is new Transform()&lt;/param&gt;
        /// </summary>
        [GodotMethod("body_add_shape")]
        public static void BodyAddShape(RID body, RID shape, Nullable<Transform> transform = null)
        {
            Transform transform_in = transform.HasValue ? transform.Value : new Transform();
            NativeCalls.godot_icall_3_668(method_bind_43, ptr, RID.GetPtr(body), RID.GetPtr(shape), ref transform_in);
        }

        private static IntPtr method_bind_44 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_shape");

        /// <summary>
        /// Substitutes a given body shape by another. The old shape is selected by its index, the new one by its [RID].
        /// </summary>
        [GodotMethod("body_set_shape")]
        public static void BodySetShape(RID body, int shapeIdx, RID shape)
        {
            NativeCalls.godot_icall_3_620(method_bind_44, ptr, RID.GetPtr(body), shapeIdx, RID.GetPtr(shape));
        }

        private static IntPtr method_bind_45 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_shape_transform");

        /// <summary>
        /// Sets the transform matrix for a body shape.
        /// </summary>
        [GodotMethod("body_set_shape_transform")]
        public static void BodySetShapeTransform(RID body, int shapeIdx, Transform transform)
        {
            NativeCalls.godot_icall_3_669(method_bind_45, ptr, RID.GetPtr(body), shapeIdx, ref transform);
        }

        private static IntPtr method_bind_46 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_get_shape_count");

        /// <summary>
        /// Returns the number of shapes assigned to a body.
        /// </summary>
        [GodotMethod("body_get_shape_count")]
        public static int BodyGetShapeCount(RID body)
        {
            return NativeCalls.godot_icall_1_623(method_bind_46, ptr, RID.GetPtr(body));
        }

        private static IntPtr method_bind_47 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_get_shape");

        /// <summary>
        /// Returns the [RID] of the nth shape of a body.
        /// </summary>
        [GodotMethod("body_get_shape")]
        public static RID BodyGetShape(RID body, int shapeIdx)
        {
            IntPtr ret = NativeCalls.godot_icall_2_624(method_bind_47, ptr, RID.GetPtr(body), shapeIdx);
            return new RID(ret);
        }

        private static IntPtr method_bind_48 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_get_shape_transform");

        /// <summary>
        /// Returns the transform matrix of a body shape.
        /// </summary>
        [GodotMethod("body_get_shape_transform")]
        public static Transform BodyGetShapeTransform(RID body, int shapeIdx)
        {
            object ret = NativeCalls.godot_icall_2_670(method_bind_48, ptr, RID.GetPtr(body), shapeIdx);
            return (Transform)ret;
        }

        private static IntPtr method_bind_49 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_remove_shape");

        /// <summary>
        /// Removes a shape from a body. The shape is not deleted, so it can be reused afterwards.
        /// </summary>
        [GodotMethod("body_remove_shape")]
        public static void BodyRemoveShape(RID body, int shapeIdx)
        {
            NativeCalls.godot_icall_2_626(method_bind_49, ptr, RID.GetPtr(body), shapeIdx);
        }

        private static IntPtr method_bind_50 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_clear_shapes");

        /// <summary>
        /// Removes all shapes from a body.
        /// </summary>
        [GodotMethod("body_clear_shapes")]
        public static void BodyClearShapes(RID body)
        {
            NativeCalls.godot_icall_1_627(method_bind_50, ptr, RID.GetPtr(body));
        }

        private static IntPtr method_bind_51 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_attach_object_instance_id");

        /// <summary>
        /// Assigns the area to a descendant of [Object], so it can exist in the node tree.
        /// </summary>
        [GodotMethod("body_attach_object_instance_id")]
        public static void BodyAttachObjectInstanceId(RID body, int id)
        {
            NativeCalls.godot_icall_2_626(method_bind_51, ptr, RID.GetPtr(body), id);
        }

        private static IntPtr method_bind_52 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_get_object_instance_id");

        /// <summary>
        /// Gets the instance ID of the object the area is assigned to.
        /// </summary>
        [GodotMethod("body_get_object_instance_id")]
        public static int BodyGetObjectInstanceId(RID body)
        {
            return NativeCalls.godot_icall_1_623(method_bind_52, ptr, RID.GetPtr(body));
        }

        private static IntPtr method_bind_53 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_enable_continuous_collision_detection");

        /// <summary>
        /// If [code]true[/code] the continuous collision detection mode is enabled.
        /// Continuous collision detection tries to predict where a moving body will collide, instead of moving it and correcting its movement if it collided.
        /// </summary>
        [GodotMethod("body_set_enable_continuous_collision_detection")]
        public static void BodySetEnableContinuousCollisionDetection(RID body, bool enable)
        {
            NativeCalls.godot_icall_2_610(method_bind_53, ptr, RID.GetPtr(body), enable);
        }

        private static IntPtr method_bind_54 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_is_continuous_collision_detection_enabled");

        /// <summary>
        /// If [code]true[/code] the continuous collision detection mode is enabled.
        /// </summary>
        [GodotMethod("body_is_continuous_collision_detection_enabled")]
        public static bool BodyIsContinuousCollisionDetectionEnabled(RID body)
        {
            return NativeCalls.godot_icall_1_611(method_bind_54, ptr, RID.GetPtr(body));
        }

        private static IntPtr method_bind_55 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_param");

        /// <summary>
        /// Sets a body parameter. A list of available parameters is on the BODY_PARAM_* constants.
        /// </summary>
        [GodotMethod("body_set_param")]
        public static void BodySetParam(RID body, PhysicsServer.BodyParameter param, float value)
        {
            NativeCalls.godot_icall_3_678(method_bind_55, ptr, RID.GetPtr(body), param, value);
        }

        private static IntPtr method_bind_56 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_get_param");

        /// <summary>
        /// Returns the value of a body parameter. A list of available parameters is on the BODY_PARAM_* constants.
        /// </summary>
        [GodotMethod("body_get_param")]
        public static float BodyGetParam(RID body, PhysicsServer.BodyParameter param)
        {
            return NativeCalls.godot_icall_2_679(method_bind_56, ptr, RID.GetPtr(body), param);
        }

        private static IntPtr method_bind_57 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_kinematic_safe_margin");

        /// <summary>
        /// </summary>
        [GodotMethod("body_set_kinematic_safe_margin")]
        public static void BodySetKinematicSafeMargin(RID body, float margin)
        {
            NativeCalls.godot_icall_2_680(method_bind_57, ptr, RID.GetPtr(body), margin);
        }

        private static IntPtr method_bind_58 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_get_kinematic_safe_margin");

        /// <summary>
        /// </summary>
        [GodotMethod("body_get_kinematic_safe_margin")]
        public static float BodyGetKinematicSafeMargin(RID body)
        {
            return NativeCalls.godot_icall_1_681(method_bind_58, ptr, RID.GetPtr(body));
        }

        private static IntPtr method_bind_59 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_state");

        /// <summary>
        /// Sets a body state (see BODY_STATE* constants).
        /// </summary>
        [GodotMethod("body_set_state")]
        public static void BodySetState(RID body, PhysicsServer.BodyState state, object value)
        {
            NativeCalls.godot_icall_3_682(method_bind_59, ptr, RID.GetPtr(body), state, value);
        }

        private static IntPtr method_bind_60 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_get_state");

        /// <summary>
        /// Returns a body state.
        /// </summary>
        [GodotMethod("body_get_state")]
        public static object BodyGetState(RID body, PhysicsServer.BodyState state)
        {
            return NativeCalls.godot_icall_2_683(method_bind_60, ptr, RID.GetPtr(body), state);
        }

        private static IntPtr method_bind_61 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_apply_impulse");

        /// <summary>
        /// Gives the body a push at a [code]position[/code] in the direction of the [code]impulse[/code].
        /// </summary>
        [GodotMethod("body_apply_impulse")]
        public static void BodyApplyImpulse(RID body, Vector3 position, Vector3 impulse)
        {
            NativeCalls.godot_icall_3_684(method_bind_61, ptr, RID.GetPtr(body), ref position, ref impulse);
        }

        private static IntPtr method_bind_62 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_apply_torque_impulse");

        /// <summary>
        /// Gives the body a push to rotate it.
        /// </summary>
        [GodotMethod("body_apply_torque_impulse")]
        public static void BodyApplyTorqueImpulse(RID body, Vector3 impulse)
        {
            NativeCalls.godot_icall_2_685(method_bind_62, ptr, RID.GetPtr(body), ref impulse);
        }

        private static IntPtr method_bind_63 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_axis_velocity");

        /// <summary>
        /// Sets an axis velocity. The velocity in the given vector axis will be set as the given vector length. This is useful for jumping behavior.
        /// </summary>
        [GodotMethod("body_set_axis_velocity")]
        public static void BodySetAxisVelocity(RID body, Vector3 axisVelocity)
        {
            NativeCalls.godot_icall_2_685(method_bind_63, ptr, RID.GetPtr(body), ref axisVelocity);
        }

        private static IntPtr method_bind_64 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_axis_lock");

        /// <summary>
        /// </summary>
        [GodotMethod("body_set_axis_lock")]
        public static void BodySetAxisLock(RID body, PhysicsServer.BodyAxis axis, bool @lock)
        {
            NativeCalls.godot_icall_3_686(method_bind_64, ptr, RID.GetPtr(body), axis, @lock);
        }

        private static IntPtr method_bind_65 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_is_axis_locked");

        /// <summary>
        /// </summary>
        [GodotMethod("body_is_axis_locked")]
        public static bool BodyIsAxisLocked(RID body, PhysicsServer.BodyAxis axis)
        {
            return NativeCalls.godot_icall_2_687(method_bind_65, ptr, RID.GetPtr(body), axis);
        }

        private static IntPtr method_bind_66 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_add_collision_exception");

        /// <summary>
        /// Adds a body to the list of bodies exempt from collisions.
        /// </summary>
        [GodotMethod("body_add_collision_exception")]
        public static void BodyAddCollisionException(RID body, RID exceptedBody)
        {
            NativeCalls.godot_icall_2_615(method_bind_66, ptr, RID.GetPtr(body), RID.GetPtr(exceptedBody));
        }

        private static IntPtr method_bind_67 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_remove_collision_exception");

        /// <summary>
        /// Removes a body from the list of bodies exempt from collisions.
        /// Continuous collision detection tries to predict where a moving body will collide, instead of moving it and correcting its movement if it collided.
        /// </summary>
        [GodotMethod("body_remove_collision_exception")]
        public static void BodyRemoveCollisionException(RID body, RID exceptedBody)
        {
            NativeCalls.godot_icall_2_615(method_bind_67, ptr, RID.GetPtr(body), RID.GetPtr(exceptedBody));
        }

        private static IntPtr method_bind_68 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_max_contacts_reported");

        /// <summary>
        /// Sets the maximum contacts to report. Bodies can keep a log of the contacts with other bodies, this is enabled by setting the maximum amount of contacts reported to a number greater than 0.
        /// </summary>
        [GodotMethod("body_set_max_contacts_reported")]
        public static void BodySetMaxContactsReported(RID body, int amount)
        {
            NativeCalls.godot_icall_2_626(method_bind_68, ptr, RID.GetPtr(body), amount);
        }

        private static IntPtr method_bind_69 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_get_max_contacts_reported");

        /// <summary>
        /// Returns the maximum contacts that can be reported. See [method body_set_max_contacts_reported].
        /// </summary>
        [GodotMethod("body_get_max_contacts_reported")]
        public static int BodyGetMaxContactsReported(RID body)
        {
            return NativeCalls.godot_icall_1_623(method_bind_69, ptr, RID.GetPtr(body));
        }

        private static IntPtr method_bind_70 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_omit_force_integration");

        /// <summary>
        /// Sets whether a body uses a callback function to calculate its own physics (see [method body_set_force_integration_callback]).
        /// </summary>
        [GodotMethod("body_set_omit_force_integration")]
        public static void BodySetOmitForceIntegration(RID body, bool enable)
        {
            NativeCalls.godot_icall_2_610(method_bind_70, ptr, RID.GetPtr(body), enable);
        }

        private static IntPtr method_bind_71 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_is_omitting_force_integration");

        /// <summary>
        /// Returns whether a body uses a callback function to calculate its own physics (see [method body_set_force_integration_callback]).
        /// </summary>
        [GodotMethod("body_is_omitting_force_integration")]
        public static bool BodyIsOmittingForceIntegration(RID body)
        {
            return NativeCalls.godot_icall_1_611(method_bind_71, ptr, RID.GetPtr(body));
        }

        private static IntPtr method_bind_72 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_force_integration_callback");

        /// <summary>
        /// Sets the function used to calculate physics for an object, if that object allows it (see [method body_set_omit_force integration]).
        /// </summary>
        [GodotMethod("body_set_force_integration_callback")]
        public static void BodySetForceIntegrationCallback(RID body, Object receiver, string method, object userdata = null)
        {
            NativeCalls.godot_icall_4_645(method_bind_72, ptr, RID.GetPtr(body), Object.GetPtr(receiver), method, userdata);
        }

        private static IntPtr method_bind_73 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_set_ray_pickable");

        /// <summary>
        /// Sets the body pickable with rays if [code]enabled[/code] is set.
        /// </summary>
        [GodotMethod("body_set_ray_pickable")]
        public static void BodySetRayPickable(RID body, bool enable)
        {
            NativeCalls.godot_icall_2_610(method_bind_73, ptr, RID.GetPtr(body), enable);
        }

        private static IntPtr method_bind_74 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_is_ray_pickable");

        /// <summary>
        /// If [code]true[/code] the body can be detected by rays
        /// </summary>
        [GodotMethod("body_is_ray_pickable")]
        public static bool BodyIsRayPickable(RID body)
        {
            return NativeCalls.godot_icall_1_611(method_bind_74, ptr, RID.GetPtr(body));
        }

        private static IntPtr method_bind_75 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "body_get_direct_state");

        /// <summary>
        /// Returns the [PhysicsDirectBodyState] of the body.
        /// </summary>
        [GodotMethod("body_get_direct_state")]
        public static PhysicsDirectBodyState BodyGetDirectState(RID body)
        {
            return NativeCalls.godot_icall_1_688(method_bind_75, ptr, RID.GetPtr(body));
        }

        private static IntPtr method_bind_76 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "joint_create_pin");

        /// <summary>
        /// Creates a [PinJoint].
        /// </summary>
        [GodotMethod("joint_create_pin")]
        public static RID JointCreatePin(RID bodyA, Vector3 localA, RID bodyB, Vector3 localB)
        {
            IntPtr ret = NativeCalls.godot_icall_4_689(method_bind_76, ptr, RID.GetPtr(bodyA), ref localA, RID.GetPtr(bodyB), ref localB);
            return new RID(ret);
        }

        private static IntPtr method_bind_77 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "pin_joint_set_param");

        /// <summary>
        /// Sets a pin_joint parameter (see PIN_JOINT* constants).
        /// </summary>
        [GodotMethod("pin_joint_set_param")]
        public static void PinJointSetParam(RID joint, PhysicsServer.PinJointParam param, float value)
        {
            NativeCalls.godot_icall_3_690(method_bind_77, ptr, RID.GetPtr(joint), param, value);
        }

        private static IntPtr method_bind_78 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "pin_joint_get_param");

        /// <summary>
        /// Gets a pin_joint parameter (see PIN_JOINT* constants).
        /// </summary>
        [GodotMethod("pin_joint_get_param")]
        public static float PinJointGetParam(RID joint, PhysicsServer.PinJointParam param)
        {
            return NativeCalls.godot_icall_2_691(method_bind_78, ptr, RID.GetPtr(joint), param);
        }

        private static IntPtr method_bind_79 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "pin_joint_set_local_a");

        /// <summary>
        /// Sets position of the joint in the local space of body a of the joint.
        /// </summary>
        [GodotMethod("pin_joint_set_local_a")]
        public static void PinJointSetLocalA(RID joint, Vector3 localA)
        {
            NativeCalls.godot_icall_2_685(method_bind_79, ptr, RID.GetPtr(joint), ref localA);
        }

        private static IntPtr method_bind_80 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "pin_joint_get_local_a");

        /// <summary>
        /// Returns position of the joint in the local space of body a of the joint.
        /// </summary>
        [GodotMethod("pin_joint_get_local_a")]
        public static Vector3 PinJointGetLocalA(RID joint)
        {
            object ret = NativeCalls.godot_icall_1_692(method_bind_80, ptr, RID.GetPtr(joint));
            return (Vector3)ret;
        }

        private static IntPtr method_bind_81 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "pin_joint_set_local_b");

        /// <summary>
        /// Sets position of the joint in the local space of body b of the joint.
        /// </summary>
        [GodotMethod("pin_joint_set_local_b")]
        public static void PinJointSetLocalB(RID joint, Vector3 localB)
        {
            NativeCalls.godot_icall_2_685(method_bind_81, ptr, RID.GetPtr(joint), ref localB);
        }

        private static IntPtr method_bind_82 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "pin_joint_get_local_b");

        /// <summary>
        /// Returns position of the joint in the local space of body b of the joint.
        /// </summary>
        [GodotMethod("pin_joint_get_local_b")]
        public static Vector3 PinJointGetLocalB(RID joint)
        {
            object ret = NativeCalls.godot_icall_1_692(method_bind_82, ptr, RID.GetPtr(joint));
            return (Vector3)ret;
        }

        private static IntPtr method_bind_83 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "joint_create_hinge");

        /// <summary>
        /// Creates a [HingeJoint].
        /// </summary>
        [GodotMethod("joint_create_hinge")]
        public static RID JointCreateHinge(RID bodyA, Transform hingeA, RID bodyB, Transform hingeB)
        {
            IntPtr ret = NativeCalls.godot_icall_4_693(method_bind_83, ptr, RID.GetPtr(bodyA), ref hingeA, RID.GetPtr(bodyB), ref hingeB);
            return new RID(ret);
        }

        private static IntPtr method_bind_84 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "hinge_joint_set_param");

        /// <summary>
        /// Sets a hinge_joint parameter (see HINGE_JOINT* constants without the HINGE_JOINT_FLAG*).
        /// </summary>
        [GodotMethod("hinge_joint_set_param")]
        public static void HingeJointSetParam(RID joint, PhysicsServer.HingeJointParam param, float value)
        {
            NativeCalls.godot_icall_3_694(method_bind_84, ptr, RID.GetPtr(joint), param, value);
        }

        private static IntPtr method_bind_85 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "hinge_joint_get_param");

        /// <summary>
        /// Gets a hinge_joint parameter (see HINGE_JOINT* constants without the HINGE_JOINT_FLAG*).
        /// </summary>
        [GodotMethod("hinge_joint_get_param")]
        public static float HingeJointGetParam(RID joint, PhysicsServer.HingeJointParam param)
        {
            return NativeCalls.godot_icall_2_695(method_bind_85, ptr, RID.GetPtr(joint), param);
        }

        private static IntPtr method_bind_86 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "hinge_joint_set_flag");

        /// <summary>
        /// Sets a hinge_joint flag (see HINGE_JOINT_FLAG* constants).
        /// </summary>
        [GodotMethod("hinge_joint_set_flag")]
        public static void HingeJointSetFlag(RID joint, PhysicsServer.HingeJointFlag flag, bool enabled)
        {
            NativeCalls.godot_icall_3_696(method_bind_86, ptr, RID.GetPtr(joint), flag, enabled);
        }

        private static IntPtr method_bind_87 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "hinge_joint_get_flag");

        /// <summary>
        /// Gets a hinge_joint flag (see HINGE_JOINT_FLAG* constants).
        /// </summary>
        [GodotMethod("hinge_joint_get_flag")]
        public static bool HingeJointGetFlag(RID joint, PhysicsServer.HingeJointFlag flag)
        {
            return NativeCalls.godot_icall_2_697(method_bind_87, ptr, RID.GetPtr(joint), flag);
        }

        private static IntPtr method_bind_88 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "joint_create_slider");

        /// <summary>
        /// Creates a [SliderJoint].
        /// </summary>
        [GodotMethod("joint_create_slider")]
        public static RID JointCreateSlider(RID bodyA, Transform localRefA, RID bodyB, Transform localRefB)
        {
            IntPtr ret = NativeCalls.godot_icall_4_693(method_bind_88, ptr, RID.GetPtr(bodyA), ref localRefA, RID.GetPtr(bodyB), ref localRefB);
            return new RID(ret);
        }

        private static IntPtr method_bind_89 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "slider_joint_set_param");

        /// <summary>
        /// Gets a slider_joint parameter (see SLIDER_JOINT* constants).
        /// </summary>
        [GodotMethod("slider_joint_set_param")]
        public static void SliderJointSetParam(RID joint, PhysicsServer.SliderJointParam param, float value)
        {
            NativeCalls.godot_icall_3_698(method_bind_89, ptr, RID.GetPtr(joint), param, value);
        }

        private static IntPtr method_bind_90 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "slider_joint_get_param");

        /// <summary>
        /// Gets a slider_joint parameter (see SLIDER_JOINT* constants).
        /// </summary>
        [GodotMethod("slider_joint_get_param")]
        public static float SliderJointGetParam(RID joint, PhysicsServer.SliderJointParam param)
        {
            return NativeCalls.godot_icall_2_699(method_bind_90, ptr, RID.GetPtr(joint), param);
        }

        private static IntPtr method_bind_91 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "joint_create_cone_twist");

        /// <summary>
        /// Creates a [ConeTwistJoint].
        /// </summary>
        [GodotMethod("joint_create_cone_twist")]
        public static RID JointCreateConeTwist(RID bodyA, Transform localRefA, RID bodyB, Transform localRefB)
        {
            IntPtr ret = NativeCalls.godot_icall_4_693(method_bind_91, ptr, RID.GetPtr(bodyA), ref localRefA, RID.GetPtr(bodyB), ref localRefB);
            return new RID(ret);
        }

        private static IntPtr method_bind_92 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "cone_twist_joint_set_param");

        /// <summary>
        /// Sets a cone_twist_joint parameter (see CONE_TWIST_JOINT* constants).
        /// </summary>
        [GodotMethod("cone_twist_joint_set_param")]
        public static void ConeTwistJointSetParam(RID joint, PhysicsServer.ConeTwistJointParam param, float value)
        {
            NativeCalls.godot_icall_3_700(method_bind_92, ptr, RID.GetPtr(joint), param, value);
        }

        private static IntPtr method_bind_93 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "cone_twist_joint_get_param");

        /// <summary>
        /// Gets a cone_twist_joint parameter (see CONE_TWIST_JOINT* constants).
        /// </summary>
        [GodotMethod("cone_twist_joint_get_param")]
        public static float ConeTwistJointGetParam(RID joint, PhysicsServer.ConeTwistJointParam param)
        {
            return NativeCalls.godot_icall_2_701(method_bind_93, ptr, RID.GetPtr(joint), param);
        }

        private static IntPtr method_bind_94 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "joint_get_type");

        /// <summary>
        /// Returns the type of the Joint.
        /// </summary>
        [GodotMethod("joint_get_type")]
        public static PhysicsServer.JointType JointGetType(RID joint)
        {
            return NativeCalls.godot_icall_1_702(method_bind_94, ptr, RID.GetPtr(joint));
        }

        private static IntPtr method_bind_95 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "joint_set_solver_priority");

        /// <summary>
        /// Sets the priority value of the Joint.
        /// </summary>
        [GodotMethod("joint_set_solver_priority")]
        public static void JointSetSolverPriority(RID joint, int priority)
        {
            NativeCalls.godot_icall_2_626(method_bind_95, ptr, RID.GetPtr(joint), priority);
        }

        private static IntPtr method_bind_96 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "joint_get_solver_priority");

        /// <summary>
        /// Gets the priority value of the Joint.
        /// </summary>
        [GodotMethod("joint_get_solver_priority")]
        public static int JointGetSolverPriority(RID joint)
        {
            return NativeCalls.godot_icall_1_623(method_bind_96, ptr, RID.GetPtr(joint));
        }

        private static IntPtr method_bind_97 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "joint_create_generic_6dof");

        /// <summary>
        /// Creates a [Generic6DOFJoint].
        /// </summary>
        [GodotMethod("joint_create_generic_6dof")]
        public static RID JointCreateGeneric6dof(RID bodyA, Transform localRefA, RID bodyB, Transform localRefB)
        {
            IntPtr ret = NativeCalls.godot_icall_4_693(method_bind_97, ptr, RID.GetPtr(bodyA), ref localRefA, RID.GetPtr(bodyB), ref localRefB);
            return new RID(ret);
        }

        private static IntPtr method_bind_98 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "generic_6dof_joint_set_param");

        /// <summary>
        /// Sets a generic_6_DOF_joint parameter (see G6DOF_JOINT* constants without the G6DOF_JOINT_FLAG*).
        /// </summary>
        [GodotMethod("generic_6dof_joint_set_param")]
        public static void Generic6dofJointSetParam(RID joint, Vector3.Axis axis, PhysicsServer.G6DOFJointAxisParam param, float value)
        {
            NativeCalls.godot_icall_4_703(method_bind_98, ptr, RID.GetPtr(joint), axis, param, value);
        }

        private static IntPtr method_bind_99 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "generic_6dof_joint_get_param");

        /// <summary>
        /// Gets a generic_6_DOF_joint parameter (see G6DOF_JOINT* constants without the G6DOF_JOINT_FLAG*).
        /// </summary>
        [GodotMethod("generic_6dof_joint_get_param")]
        public static float Generic6dofJointGetParam(RID joint, Vector3.Axis axis, PhysicsServer.G6DOFJointAxisParam param)
        {
            return NativeCalls.godot_icall_3_704(method_bind_99, ptr, RID.GetPtr(joint), axis, param);
        }

        private static IntPtr method_bind_100 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "generic_6dof_joint_set_flag");

        /// <summary>
        /// Sets a generic_6_DOF_joint flag (see G6DOF_JOINT_FLAG* constants).
        /// </summary>
        [GodotMethod("generic_6dof_joint_set_flag")]
        public static void Generic6dofJointSetFlag(RID joint, Vector3.Axis axis, PhysicsServer.G6DOFJointAxisFlag flag, bool enable)
        {
            NativeCalls.godot_icall_4_705(method_bind_100, ptr, RID.GetPtr(joint), axis, flag, enable);
        }

        private static IntPtr method_bind_101 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "generic_6dof_joint_get_flag");

        /// <summary>
        /// Gets a generic_6_DOF_joint flag (see G6DOF_JOINT_FLAG* constants).
        /// </summary>
        [GodotMethod("generic_6dof_joint_get_flag")]
        public static bool Generic6dofJointGetFlag(RID joint, Vector3.Axis axis, PhysicsServer.G6DOFJointAxisFlag flag)
        {
            return NativeCalls.godot_icall_3_706(method_bind_101, ptr, RID.GetPtr(joint), axis, flag);
        }

        private static IntPtr method_bind_102 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "free_rid");

        /// <summary>
        /// Destroys any of the objects created by PhysicsServer. If the [RID] passed is not one of the objects that can be created by PhysicsServer, an error will be sent to the console.
        /// </summary>
        [GodotMethod("free_rid")]
        public static void FreeRid(RID rid)
        {
            NativeCalls.godot_icall_1_627(method_bind_102, ptr, RID.GetPtr(rid));
        }

        private static IntPtr method_bind_103 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_active");

        /// <summary>
        /// Activates or deactivates the 3D physics engine.
        /// </summary>
        [GodotMethod("set_active")]
        public static void SetActive(bool active)
        {
            NativeCalls.godot_icall_1_11(method_bind_103, ptr, active);
        }

        private static IntPtr method_bind_104 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_process_info");

        /// <summary>
        /// Returns an Info defined by the [ProcessInfo] input given.
        /// </summary>
        [GodotMethod("get_process_info")]
        public static int GetProcessInfo(PhysicsServer.ProcessInfo processInfo)
        {
            return NativeCalls.godot_icall_1_707(method_bind_104, ptr, processInfo);
        }
    }
}
