using System;

namespace Godot
{
    /// <summary>
    /// Pin Joint for 3D Rigid Bodies. It pins 2 bodies (rigid or static) together.
    /// </summary>
    public class PinJoint : Joint
    {
        public enum Param
        {
            Bias = 0,
            Damping = 1,
            ImpulseClamp = 2
        }


        /// <summary>
        /// The force with wich the pinned objects stay in positional relation to each other.
        /// The higher, the stronger.
        /// </summary>
        public float Params__bias
        {
            get
            {
                return GetParam((PinJoint.Param)0);
            }
            set
            {
                SetParam((PinJoint.Param)0, value);
            }
        }

        /// <summary>
        /// The force with wich the pinned objects stay in velocity relation to each other.
        /// The higher, the stronger.
        /// </summary>
        public float Params__damping
        {
            get
            {
                return GetParam((PinJoint.Param)1);
            }
            set
            {
                SetParam((PinJoint.Param)1, value);
            }
        }

        /// <summary>
        /// If above 0, this value is the maximum value for an impulse that this Joint produces.
        /// </summary>
        public float Params__impulseClamp
        {
            get
            {
                return GetParam((PinJoint.Param)2);
            }
            set
            {
                SetParam((PinJoint.Param)2, value);
            }
        }


        private const string nativeName = "PinJoint";

        public PinJoint() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_PinJoint_Ctor(this);
        }

        internal PinJoint(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_param");

        [GodotMethod("set_param")]
        public void SetParam(PinJoint.Param param, float value)
        {
            NativeCalls.godot_icall_2_708(method_bind_0, Object.GetPtr(this), param, value);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_param");

        [GodotMethod("get_param")]
        public float GetParam(PinJoint.Param param)
        {
            return NativeCalls.godot_icall_1_709(method_bind_1, Object.GetPtr(this), param);
        }
    }
}
