using System;

namespace Godot
{
    /// <summary>
    /// Pin Joint for 2D Rigid Bodies. It pins 2 bodies (rigid or static) together, or a single body to a fixed position in space.
    /// </summary>
    public class PinJoint2D : Joint2D
    {
        /// <summary>
        /// The higher this value, the more the bond to the pinned partner can flex.
        /// </summary>
        public float Softness
        {
            get
            {
                return GetSoftness();
            }
            set
            {
                SetSoftness(value);
            }
        }


        private const string nativeName = "PinJoint2D";

        public PinJoint2D() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_PinJoint2D_Ctor(this);
        }

        internal PinJoint2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_softness");

        [GodotMethod("set_softness")]
        public void SetSoftness(float softness)
        {
            NativeCalls.godot_icall_1_10(method_bind_0, Object.GetPtr(this), softness);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_softness");

        [GodotMethod("get_softness")]
        public float GetSoftness()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }
    }
}
