using System;

namespace Godot
{
    /// <summary>
    /// Class representing a planar [PrimitiveMesh]. This flat mesh does not have a thickness.
    /// </summary>
    public class PlaneMesh : PrimitiveMesh
    {
        /// <summary>
        /// Size of the generated plane. Defaults to (2.0, 2.0).
        /// </summary>
        public Vector2 Size
        {
            get
            {
                return GetSize();
            }
            set
            {
                SetSize(value);
            }
        }

        /// <summary>
        /// Number of subdivision along the x-axis. Defaults to 0.
        /// </summary>
        public int SubdivideWidth
        {
            get
            {
                return GetSubdivideWidth();
            }
            set
            {
                SetSubdivideWidth(value);
            }
        }

        /// <summary>
        /// Number of subdivision along the z-axis. Defaults to 0.
        /// </summary>
        public int SubdivideDepth
        {
            get
            {
                return GetSubdivideDepth();
            }
            set
            {
                SetSubdivideDepth(value);
            }
        }


        private const string nativeName = "PlaneMesh";

        public PlaneMesh() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_PlaneMesh_Ctor(this);
        }

        internal PlaneMesh(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_size");

        [GodotMethod("set_size")]
        public void SetSize(Vector2 size)
        {
            NativeCalls.godot_icall_1_45(method_bind_0, Object.GetPtr(this), ref size);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_size");

        [GodotMethod("get_size")]
        public Vector2 GetSize()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_1, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_subdivide_width");

        [GodotMethod("set_subdivide_width")]
        public void SetSubdivideWidth(int subdivide)
        {
            NativeCalls.godot_icall_1_0(method_bind_2, Object.GetPtr(this), subdivide);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_subdivide_width");

        [GodotMethod("get_subdivide_width")]
        public int GetSubdivideWidth()
        {
            return NativeCalls.godot_icall_0_1(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_subdivide_depth");

        [GodotMethod("set_subdivide_depth")]
        public void SetSubdivideDepth(int subdivide)
        {
            NativeCalls.godot_icall_1_0(method_bind_4, Object.GetPtr(this), subdivide);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_subdivide_depth");

        [GodotMethod("get_subdivide_depth")]
        public int GetSubdivideDepth()
        {
            return NativeCalls.godot_icall_0_1(method_bind_5, Object.GetPtr(this));
        }
    }
}
