using System;

namespace Godot
{
    /// <summary>
    /// Popup is a base [Control] used to show dialogs and popups. It's a subwindow and modal by default (see [Control]) and has helpers for custom popup behavior.
    /// </summary>
    public class Popup : Control
    {
        public const int NotificationPostPopup = 80;
        public const int NotificationPopupHide = 81;

        /// <summary>
        /// </summary>
        public bool PopupExclusive
        {
            get
            {
                return IsExclusive();
            }
            set
            {
                SetExclusive(value);
            }
        }


        private const string nativeName = "Popup";

        public Popup() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Popup_Ctor(this);
        }

        internal Popup(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "popup_centered");

        /// <summary>
        /// Popup (show the control in modal form) in the center of the screen, at the current size, or at a size determined by "size".
        /// &lt;param name="size"&gt;If the param is null, then the default value is new Vector2(0, 0)&lt;/param&gt;
        /// </summary>
        [GodotMethod("popup_centered")]
        public void PopupCentered(Nullable<Vector2> size = null)
        {
            Vector2 size_in = size.HasValue ? size.Value : new Vector2(0, 0);
            NativeCalls.godot_icall_1_45(method_bind_0, Object.GetPtr(this), ref size_in);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "popup_centered_ratio");

        /// <summary>
        /// Popup (show the control in modal form) in the center of the screen, scaled at a ratio of size of the screen.
        /// </summary>
        [GodotMethod("popup_centered_ratio")]
        public void PopupCenteredRatio(float ratio = 0.75f)
        {
            NativeCalls.godot_icall_1_10(method_bind_1, Object.GetPtr(this), ratio);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "popup_centered_minsize");

        /// <summary>
        /// Popup (show the control in modal form) in the center of the screen, ensuring the size is never smaller than [code]minsize[/code].
        /// &lt;param name="minsize"&gt;If the param is null, then the default value is new Vector2(0, 0)&lt;/param&gt;
        /// </summary>
        [GodotMethod("popup_centered_minsize")]
        public void PopupCenteredMinsize(Nullable<Vector2> minsize = null)
        {
            Vector2 minsize_in = minsize.HasValue ? minsize.Value : new Vector2(0, 0);
            NativeCalls.godot_icall_1_45(method_bind_2, Object.GetPtr(this), ref minsize_in);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "popup");

        /// <summary>
        /// Popup (show the control in modal form).
        /// &lt;param name="bounds"&gt;If the param is null, then the default value is new Rect2(0, 0, 0, 0)&lt;/param&gt;
        /// </summary>
        [GodotMethod("popup")]
        public void Popup_(Nullable<Rect2> bounds = null)
        {
            Rect2 bounds_in = bounds.HasValue ? bounds.Value : new Rect2(0, 0, 0, 0);
            NativeCalls.godot_icall_1_120(method_bind_3, Object.GetPtr(this), ref bounds_in);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_exclusive");

        [GodotMethod("set_exclusive")]
        public void SetExclusive(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_4, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_exclusive");

        [GodotMethod("is_exclusive")]
        public bool IsExclusive()
        {
            return NativeCalls.godot_icall_0_3(method_bind_5, Object.GetPtr(this));
        }
    }
}
