using System;

namespace Godot
{
    /// <summary>
    /// PopupMenu is the typical Control that displays a list of options. They are popular in toolbars or context menus.
    /// </summary>
    public class PopupMenu : Popup
    {
        public object[] Items
        {
            get
            {
                return _GetItems();
            }
            set
            {
                _SetItems(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool HideOnItemSelection
        {
            get
            {
                return IsHideOnItemSelection();
            }
            set
            {
                SetHideOnItemSelection(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool HideOnCheckableItemSelection
        {
            get
            {
                return IsHideOnCheckableItemSelection();
            }
            set
            {
                SetHideOnCheckableItemSelection(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool HideOnStateItemSelection
        {
            get
            {
                return IsHideOnStateItemSelection();
            }
            set
            {
                SetHideOnStateItemSelection(value);
            }
        }


        private const string nativeName = "PopupMenu";

        public PopupMenu() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_PopupMenu_Ctor(this);
        }

        internal PopupMenu(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_icon_item");

        /// <summary>
        /// Add a new item with text "label" and icon "texture". An id can optionally be provided, as well as an accelerator keybinding. If no id is provided, one will be created from the index.
        /// </summary>
        [GodotMethod("add_icon_item")]
        public void AddIconItem(Texture texture, string label, int id = -1, int accel = 0)
        {
            NativeCalls.godot_icall_4_712(method_bind_0, Object.GetPtr(this), Object.GetPtr(texture), label, id, accel);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_item");

        /// <summary>
        /// Add a new item with text "label". An id can optionally be provided, as well as an accelerator keybinding. If no id is provided, one will be created from the index.
        /// </summary>
        [GodotMethod("add_item")]
        public void AddItem(string label, int id = -1, int accel = 0)
        {
            NativeCalls.godot_icall_3_713(method_bind_1, Object.GetPtr(this), label, id, accel);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_icon_check_item");

        /// <summary>
        /// Add a new checkable item with text "label" and icon "texture". An id can optionally be provided, as well as an accelerator. If no id is provided, one will be
        /// created from the index. Note that checkable items just display a checkmark, but don't have any built-in checking behavior and must be checked/unchecked manually.
        /// </summary>
        [GodotMethod("add_icon_check_item")]
        public void AddIconCheckItem(Texture texture, string label, int id = -1, int accel = 0)
        {
            NativeCalls.godot_icall_4_712(method_bind_2, Object.GetPtr(this), Object.GetPtr(texture), label, id, accel);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_check_item");

        /// <summary>
        /// Add a new checkable item with text "label". An id can optionally be provided, as well as an accelerator. If no id is provided, one will be created from the index. Note that checkable items just display a checkmark, but don't have any built-in checking behavior and must be checked/unchecked manually.
        /// </summary>
        [GodotMethod("add_check_item")]
        public void AddCheckItem(string label, int id = -1, int accel = 0)
        {
            NativeCalls.godot_icall_3_713(method_bind_3, Object.GetPtr(this), label, id, accel);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_submenu_item");

        /// <summary>
        /// Adds an item with a submenu. The submenu is the name of a child PopupMenu node that would be shown when the item is clicked. An id can optionally be provided, but if is isn't provided, one will be created from the index.
        /// </summary>
        [GodotMethod("add_submenu_item")]
        public void AddSubmenuItem(string label, string submenu, int id = -1)
        {
            NativeCalls.godot_icall_3_714(method_bind_4, Object.GetPtr(this), label, submenu, id);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_icon_shortcut");

        /// <summary>
        /// </summary>
        [GodotMethod("add_icon_shortcut")]
        public void AddIconShortcut(Texture texture, ShortCut shortcut, int id = -1, bool global = false)
        {
            NativeCalls.godot_icall_4_715(method_bind_5, Object.GetPtr(this), Object.GetPtr(texture), Object.GetPtr(shortcut), id, global);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_shortcut");

        /// <summary>
        /// </summary>
        [GodotMethod("add_shortcut")]
        public void AddShortcut(ShortCut shortcut, int id = -1, bool global = false)
        {
            NativeCalls.godot_icall_3_716(method_bind_6, Object.GetPtr(this), Object.GetPtr(shortcut), id, global);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_icon_check_shortcut");

        /// <summary>
        /// </summary>
        [GodotMethod("add_icon_check_shortcut")]
        public void AddIconCheckShortcut(Texture texture, ShortCut shortcut, int id = -1, bool global = false)
        {
            NativeCalls.godot_icall_4_715(method_bind_7, Object.GetPtr(this), Object.GetPtr(texture), Object.GetPtr(shortcut), id, global);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_check_shortcut");

        /// <summary>
        /// </summary>
        [GodotMethod("add_check_shortcut")]
        public void AddCheckShortcut(ShortCut shortcut, int id = -1, bool global = false)
        {
            NativeCalls.godot_icall_3_716(method_bind_8, Object.GetPtr(this), Object.GetPtr(shortcut), id, global);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_text");

        /// <summary>
        /// Set the text of the item at index "idx".
        /// </summary>
        [GodotMethod("set_item_text")]
        public void SetItemText(int idx, string text)
        {
            NativeCalls.godot_icall_2_115(method_bind_9, Object.GetPtr(this), idx, text);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_icon");

        /// <summary>
        /// </summary>
        [GodotMethod("set_item_icon")]
        public void SetItemIcon(int idx, Texture icon)
        {
            NativeCalls.godot_icall_2_67(method_bind_10, Object.GetPtr(this), idx, Object.GetPtr(icon));
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_checked");

        /// <summary>
        /// Set the checkstate status of the item at index "idx".
        /// </summary>
        [GodotMethod("set_item_checked")]
        public void SetItemChecked(int idx, bool @checked)
        {
            NativeCalls.godot_icall_2_51(method_bind_11, Object.GetPtr(this), idx, @checked);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_id");

        /// <summary>
        /// Set the id of the item at index "idx".
        /// </summary>
        [GodotMethod("set_item_id")]
        public void SetItemId(int idx, int id)
        {
            NativeCalls.godot_icall_2_32(method_bind_12, Object.GetPtr(this), idx, id);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_accelerator");

        /// <summary>
        /// Set the accelerator of the item at index "idx". Accelerators are special combinations of keys that activate the item, no matter which control is focused.
        /// </summary>
        [GodotMethod("set_item_accelerator")]
        public void SetItemAccelerator(int idx, int accel)
        {
            NativeCalls.godot_icall_2_32(method_bind_13, Object.GetPtr(this), idx, accel);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_metadata");

        /// <summary>
        /// Sets the metadata of an item, which might be of any type. You can later get it with [method get_item_metadata], which provides a simple way of assigning context data to items.
        /// </summary>
        [GodotMethod("set_item_metadata")]
        public void SetItemMetadata(int idx, object metadata)
        {
            NativeCalls.godot_icall_2_453(method_bind_14, Object.GetPtr(this), idx, metadata);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_disabled");

        /// <summary>
        /// Sets whether the item at index "idx" is disabled or not. When it is disabled it can't be selected, or its action invoked.
        /// </summary>
        [GodotMethod("set_item_disabled")]
        public void SetItemDisabled(int idx, bool disabled)
        {
            NativeCalls.godot_icall_2_51(method_bind_15, Object.GetPtr(this), idx, disabled);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_submenu");

        /// <summary>
        /// Sets the submenu of the item at index "idx". The submenu is the name of a child PopupMenu node that would be shown when the item is clicked.
        /// </summary>
        [GodotMethod("set_item_submenu")]
        public void SetItemSubmenu(int idx, string submenu)
        {
            NativeCalls.godot_icall_2_115(method_bind_16, Object.GetPtr(this), idx, submenu);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_as_separator");

        /// <summary>
        /// Mark the item at index "idx" as a seperator, which means that it would be displayed as a mere line.
        /// </summary>
        [GodotMethod("set_item_as_separator")]
        public void SetItemAsSeparator(int idx, bool enable)
        {
            NativeCalls.godot_icall_2_51(method_bind_17, Object.GetPtr(this), idx, enable);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_as_checkable");

        /// <summary>
        /// Set whether the item at index "idx" has a checkbox. Note that checkable items just display a checkmark, but don't have any built-in checking behavior and must be checked/unchecked manually.
        /// </summary>
        [GodotMethod("set_item_as_checkable")]
        public void SetItemAsCheckable(int idx, bool enable)
        {
            NativeCalls.godot_icall_2_51(method_bind_18, Object.GetPtr(this), idx, enable);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_tooltip");

        /// <summary>
        /// </summary>
        [GodotMethod("set_item_tooltip")]
        public void SetItemTooltip(int idx, string tooltip)
        {
            NativeCalls.godot_icall_2_115(method_bind_19, Object.GetPtr(this), idx, tooltip);
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_shortcut");

        /// <summary>
        /// </summary>
        [GodotMethod("set_item_shortcut")]
        public void SetItemShortcut(int idx, ShortCut shortcut, bool global = false)
        {
            NativeCalls.godot_icall_3_717(method_bind_20, Object.GetPtr(this), idx, Object.GetPtr(shortcut), global);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_multistate");

        /// <summary>
        /// </summary>
        [GodotMethod("set_item_multistate")]
        public void SetItemMultistate(int idx, int state)
        {
            NativeCalls.godot_icall_2_32(method_bind_21, Object.GetPtr(this), idx, state);
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "toggle_item_checked");

        /// <summary>
        /// </summary>
        [GodotMethod("toggle_item_checked")]
        public void ToggleItemChecked(int idx)
        {
            NativeCalls.godot_icall_1_0(method_bind_22, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "toggle_item_multistate");

        /// <summary>
        /// </summary>
        [GodotMethod("toggle_item_multistate")]
        public void ToggleItemMultistate(int idx)
        {
            NativeCalls.godot_icall_1_0(method_bind_23, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_text");

        /// <summary>
        /// Return the text of the item at index "idx".
        /// </summary>
        [GodotMethod("get_item_text")]
        public string GetItemText(int idx)
        {
            return NativeCalls.godot_icall_1_108(method_bind_24, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_icon");

        /// <summary>
        /// Return the icon of the item at index "idx".
        /// </summary>
        [GodotMethod("get_item_icon")]
        public Texture GetItemIcon(int idx)
        {
            return NativeCalls.godot_icall_1_157(method_bind_25, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_item_checked");

        /// <summary>
        /// Return the checkstate status of the item at index "idx".
        /// </summary>
        [GodotMethod("is_item_checked")]
        public bool IsItemChecked(int idx)
        {
            return NativeCalls.godot_icall_1_29(method_bind_26, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_id");

        /// <summary>
        /// Return the id of the item at index "idx".
        /// </summary>
        [GodotMethod("get_item_id")]
        public int GetItemId(int idx)
        {
            return NativeCalls.godot_icall_1_6(method_bind_27, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_index");

        /// <summary>
        /// Find and return the index of the item containing a given id.
        /// </summary>
        [GodotMethod("get_item_index")]
        public int GetItemIndex(int id)
        {
            return NativeCalls.godot_icall_1_6(method_bind_28, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_accelerator");

        /// <summary>
        /// Return the accelerator of the item at index "idx". Accelerators are special combinations of keys that activate the item, no matter which control is focused.
        /// </summary>
        [GodotMethod("get_item_accelerator")]
        public int GetItemAccelerator(int idx)
        {
            return NativeCalls.godot_icall_1_6(method_bind_29, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_metadata");

        /// <summary>
        /// Return the metadata of an item, which might be of any type. You can set it with [method set_item_metadata], which provides a simple way of assigning context data to items.
        /// </summary>
        [GodotMethod("get_item_metadata")]
        public object GetItemMetadata(int idx)
        {
            return NativeCalls.godot_icall_1_454(method_bind_30, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_item_disabled");

        /// <summary>
        /// Return whether the item at index "idx" is disabled. When it is disabled it can't be selected, or its action invoked.
        /// </summary>
        [GodotMethod("is_item_disabled")]
        public bool IsItemDisabled(int idx)
        {
            return NativeCalls.godot_icall_1_29(method_bind_31, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_submenu");

        /// <summary>
        /// Return the submenu name of the item at index "idx".
        /// </summary>
        [GodotMethod("get_item_submenu")]
        public string GetItemSubmenu(int idx)
        {
            return NativeCalls.godot_icall_1_108(method_bind_32, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_item_separator");

        /// <summary>
        /// Return whether the item is a seperator. If it is, it would be displayed as a line.
        /// </summary>
        [GodotMethod("is_item_separator")]
        public bool IsItemSeparator(int idx)
        {
            return NativeCalls.godot_icall_1_29(method_bind_33, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_item_checkable");

        /// <summary>
        /// Return whether the item at index "idx" has a checkbox. Note that checkable items just display a checkmark, but don't have any built-in checking behavior and must be checked/unchecked manually.
        /// </summary>
        [GodotMethod("is_item_checkable")]
        public bool IsItemCheckable(int idx)
        {
            return NativeCalls.godot_icall_1_29(method_bind_34, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_tooltip");

        /// <summary>
        /// </summary>
        [GodotMethod("get_item_tooltip")]
        public string GetItemTooltip(int idx)
        {
            return NativeCalls.godot_icall_1_108(method_bind_35, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_shortcut");

        /// <summary>
        /// </summary>
        [GodotMethod("get_item_shortcut")]
        public ShortCut GetItemShortcut(int idx)
        {
            return NativeCalls.godot_icall_1_718(method_bind_36, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_count");

        /// <summary>
        /// Return the amount of items.
        /// </summary>
        [GodotMethod("get_item_count")]
        public int GetItemCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_37, Object.GetPtr(this));
        }

        private IntPtr method_bind_38 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_item");

        /// <summary>
        /// Removes the item at index "idx" from the menu. Note that the indexes of items after the removed item are going to be shifted by one.
        /// </summary>
        [GodotMethod("remove_item")]
        public void RemoveItem(int idx)
        {
            NativeCalls.godot_icall_1_0(method_bind_38, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_39 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_separator");

        /// <summary>
        /// Add a separator between items. Separators also occupy an index.
        /// </summary>
        [GodotMethod("add_separator")]
        public void AddSeparator()
        {
            NativeCalls.godot_icall_0_12(method_bind_39, Object.GetPtr(this));
        }

        private IntPtr method_bind_40 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear");

        /// <summary>
        /// Clear the popup menu, in effect removing all items.
        /// </summary>
        [GodotMethod("clear")]
        public void Clear()
        {
            NativeCalls.godot_icall_0_12(method_bind_40, Object.GetPtr(this));
        }

        private IntPtr method_bind_41 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_items");

        internal void _SetItems(object[] arg0)
        {
            NativeCalls.godot_icall_1_154(method_bind_41, Object.GetPtr(this), arg0);
        }

        private IntPtr method_bind_42 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_items");

        internal object[] _GetItems()
        {
            return NativeCalls.godot_icall_0_21(method_bind_42, Object.GetPtr(this));
        }

        private IntPtr method_bind_43 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_hide_on_item_selection");

        [GodotMethod("set_hide_on_item_selection")]
        public void SetHideOnItemSelection(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_43, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_44 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_hide_on_item_selection");

        [GodotMethod("is_hide_on_item_selection")]
        public bool IsHideOnItemSelection()
        {
            return NativeCalls.godot_icall_0_3(method_bind_44, Object.GetPtr(this));
        }

        private IntPtr method_bind_45 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_hide_on_checkable_item_selection");

        [GodotMethod("set_hide_on_checkable_item_selection")]
        public void SetHideOnCheckableItemSelection(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_45, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_46 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_hide_on_checkable_item_selection");

        [GodotMethod("is_hide_on_checkable_item_selection")]
        public bool IsHideOnCheckableItemSelection()
        {
            return NativeCalls.godot_icall_0_3(method_bind_46, Object.GetPtr(this));
        }

        private IntPtr method_bind_47 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_hide_on_state_item_selection");

        [GodotMethod("set_hide_on_state_item_selection")]
        public void SetHideOnStateItemSelection(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_47, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_48 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_hide_on_state_item_selection");

        [GodotMethod("is_hide_on_state_item_selection")]
        public bool IsHideOnStateItemSelection()
        {
            return NativeCalls.godot_icall_0_3(method_bind_48, Object.GetPtr(this));
        }
    }
}
