using System;

namespace Godot
{
    /// <summary>
    /// Base class for all primitive meshes. Handles applying a [Material] to a primitive mesh.
    /// </summary>
    public class PrimitiveMesh : Mesh
    {
        /// <summary>
        /// The current [Material] of the primitive mesh.
        /// </summary>
        public Material Material
        {
            get
            {
                return GetMaterial();
            }
            set
            {
                SetMaterial(value);
            }
        }


        private const string nativeName = "PrimitiveMesh";

        internal PrimitiveMesh() {}

        internal PrimitiveMesh(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_material");

        [GodotMethod("set_material")]
        public void SetMaterial(Material material)
        {
            NativeCalls.godot_icall_1_24(method_bind_0, Object.GetPtr(this), Object.GetPtr(material));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_material");

        [GodotMethod("get_material")]
        public Material GetMaterial()
        {
            return NativeCalls.godot_icall_0_222(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_mesh_arrays");

        /// <summary>
        /// </summary>
        [GodotMethod("get_mesh_arrays")]
        public object[] GetMeshArrays()
        {
            return NativeCalls.godot_icall_0_21(method_bind_2, Object.GetPtr(this));
        }
    }
}
