using System;

namespace Godot
{
    /// <summary>
    /// Class representing a prism-shaped [PrimitiveMesh].
    /// </summary>
    public class PrismMesh : PrimitiveMesh
    {
        /// <summary>
        /// Displacement of of the upper edge along the x-axis. 0.0 positions edge straight above the bottome left edge. Defaults to 0.5 (positioned on the midpoint).
        /// </summary>
        public float LeftToRight
        {
            get
            {
                return GetLeftToRight();
            }
            set
            {
                SetLeftToRight(value);
            }
        }

        /// <summary>
        /// Size of the prism. Defaults to (2.0, 2.0, 2.0).
        /// </summary>
        public Vector3 Size
        {
            get
            {
                return GetSize();
            }
            set
            {
                SetSize(value);
            }
        }

        /// <summary>
        /// Number of added edge loops along the x-axis. Defaults to 0.
        /// </summary>
        public int SubdivideWidth
        {
            get
            {
                return GetSubdivideWidth();
            }
            set
            {
                SetSubdivideWidth(value);
            }
        }

        /// <summary>
        /// Number of added edge loops along the y-axis. Defaults to 0.
        /// </summary>
        public int SubdivideHeight
        {
            get
            {
                return GetSubdivideHeight();
            }
            set
            {
                SetSubdivideHeight(value);
            }
        }

        /// <summary>
        /// Number of added edge loops along the z-axis. Defaults to 0.
        /// </summary>
        public int SubdivideDepth
        {
            get
            {
                return GetSubdivideDepth();
            }
            set
            {
                SetSubdivideDepth(value);
            }
        }


        private const string nativeName = "PrismMesh";

        public PrismMesh() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_PrismMesh_Ctor(this);
        }

        internal PrismMesh(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_left_to_right");

        [GodotMethod("set_left_to_right")]
        public void SetLeftToRight(float leftToRight)
        {
            NativeCalls.godot_icall_1_10(method_bind_0, Object.GetPtr(this), leftToRight);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_left_to_right");

        [GodotMethod("get_left_to_right")]
        public float GetLeftToRight()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_size");

        [GodotMethod("set_size")]
        public void SetSize(Vector3 size)
        {
            NativeCalls.godot_icall_1_104(method_bind_2, Object.GetPtr(this), ref size);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_size");

        [GodotMethod("get_size")]
        public Vector3 GetSize()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_3, Object.GetPtr(this));
            return (Vector3)ret;
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_subdivide_width");

        [GodotMethod("set_subdivide_width")]
        public void SetSubdivideWidth(int segments)
        {
            NativeCalls.godot_icall_1_0(method_bind_4, Object.GetPtr(this), segments);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_subdivide_width");

        [GodotMethod("get_subdivide_width")]
        public int GetSubdivideWidth()
        {
            return NativeCalls.godot_icall_0_1(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_subdivide_height");

        [GodotMethod("set_subdivide_height")]
        public void SetSubdivideHeight(int segments)
        {
            NativeCalls.godot_icall_1_0(method_bind_6, Object.GetPtr(this), segments);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_subdivide_height");

        [GodotMethod("get_subdivide_height")]
        public int GetSubdivideHeight()
        {
            return NativeCalls.godot_icall_0_1(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_subdivide_depth");

        [GodotMethod("set_subdivide_depth")]
        public void SetSubdivideDepth(int segments)
        {
            NativeCalls.godot_icall_1_0(method_bind_8, Object.GetPtr(this), segments);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_subdivide_depth");

        [GodotMethod("get_subdivide_depth")]
        public int GetSubdivideDepth()
        {
            return NativeCalls.godot_icall_0_1(method_bind_9, Object.GetPtr(this));
        }
    }
}
