using System;
using System.Collections.Generic;

namespace Godot
{
    /// <summary>
    /// Contains global variables accessible from everywhere. Use the normal [Object] API, such as "ProjectSettings.get(variable)", "ProjectSettings.set(variable,value)" or "ProjectSettings.has(variable)" to access them. Variables stored in project.godot are also loaded into ProjectSettings, making this object very useful for reading custom game configuration options.
    /// </summary>
    public static class ProjectSettings
    {
        private const string nativeName = "ProjectSettings";
        internal static IntPtr ptr = NativeCalls.godot_icall_ProjectSettings_get_singleton();

        private static IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_setting");

        /// <summary>
        /// Return true if a configuration value is present.
        /// </summary>
        [GodotMethod("has_setting")]
        public static bool HasSetting(string name)
        {
            return NativeCalls.godot_icall_1_70(method_bind_0, ptr, name);
        }

        private static IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_setting");

        /// <summary>
        /// </summary>
        [GodotMethod("set_setting")]
        public static void SetSetting(string name, object value)
        {
            NativeCalls.godot_icall_2_337(method_bind_1, ptr, name, value);
        }

        private static IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_setting");

        /// <summary>
        /// </summary>
        [GodotMethod("get_setting")]
        public static object GetSetting(string name)
        {
            return NativeCalls.godot_icall_1_338(method_bind_2, ptr, name);
        }

        private static IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_order");

        /// <summary>
        /// Set the order of a configuration value (influences when saved to the config file).
        /// </summary>
        [GodotMethod("set_order")]
        public static void SetOrder(string name, int position)
        {
            NativeCalls.godot_icall_2_95(method_bind_3, ptr, name, position);
        }

        private static IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_order");

        /// <summary>
        /// Return the order of a configuration value (influences when saved to the config file).
        /// </summary>
        [GodotMethod("get_order")]
        public static int GetOrder(string name)
        {
            return NativeCalls.godot_icall_1_87(method_bind_4, ptr, name);
        }

        private static IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_initial_value");

        /// <summary>
        /// </summary>
        [GodotMethod("set_initial_value")]
        public static void SetInitialValue(string name, object value)
        {
            NativeCalls.godot_icall_2_337(method_bind_5, ptr, name, value);
        }

        private static IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_property_info");

        /// <summary>
        /// Add a custom property info to a property. The dictionary must contain: name:[String](the name of the property) and type:[int](see TYPE_* in [@GlobalScope]), and optionally hint:[int](see PROPERTY_HINT_* in [@GlobalScope]), hint_string:[String].
        /// Example:
        /// [codeblock]
        /// ProjectSettings.set("category/property_name", 0)
        /// var property_info = {
        /// "name": "category/property_name",
        /// "type": TYPE_INT,
        /// "hint": PROPERTY_HINT_ENUM,
        /// "hint_string": "one,two,three"
        /// }
        /// ProjectSettings.add_property_info(property_info)
        /// [/codeblock]
        /// </summary>
        [GodotMethod("add_property_info")]
        public static void AddPropertyInfo(Dictionary<object, object> hint)
        {
            NativeCalls.godot_icall_1_168(method_bind_6, ptr, hint);
        }

        private static IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear");

        /// <summary>
        /// Clear the whole configuration (not recommended, may break things).
        /// </summary>
        [GodotMethod("clear")]
        public static void Clear(string name)
        {
            NativeCalls.godot_icall_1_43(method_bind_7, ptr, name);
        }

        private static IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "localize_path");

        /// <summary>
        /// Convert a path to a localized path (res:// path).
        /// </summary>
        [GodotMethod("localize_path")]
        public static string LocalizePath(string path)
        {
            return NativeCalls.godot_icall_1_73(method_bind_8, ptr, path);
        }

        private static IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "globalize_path");

        /// <summary>
        /// Convert a localized path (res://) to a full native OS path.
        /// </summary>
        [GodotMethod("globalize_path")]
        public static string GlobalizePath(string path)
        {
            return NativeCalls.godot_icall_1_73(method_bind_9, ptr, path);
        }

        private static IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "save");

        /// <summary>
        /// </summary>
        [GodotMethod("save")]
        public static Error Save()
        {
            return NativeCalls.godot_icall_0_325(method_bind_10, ptr);
        }

        private static IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "load_resource_pack");

        /// <summary>
        /// </summary>
        [GodotMethod("load_resource_pack")]
        public static bool LoadResourcePack(string pack)
        {
            return NativeCalls.godot_icall_1_70(method_bind_11, ptr, pack);
        }

        private static IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "property_can_revert");

        /// <summary>
        /// </summary>
        [GodotMethod("property_can_revert")]
        public static bool PropertyCanRevert(string name)
        {
            return NativeCalls.godot_icall_1_70(method_bind_12, ptr, name);
        }

        private static IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "property_get_revert");

        /// <summary>
        /// </summary>
        [GodotMethod("property_get_revert")]
        public static object PropertyGetRevert(string name)
        {
            return NativeCalls.godot_icall_1_338(method_bind_13, ptr, name);
        }

        private static IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "save_custom");

        /// <summary>
        /// </summary>
        [GodotMethod("save_custom")]
        public static Error SaveCustom(string file)
        {
            return NativeCalls.godot_icall_1_170(method_bind_14, ptr, file);
        }
    }
}
