using System;

namespace Godot
{
    /// <summary>
    /// Ray shape for 3D collisions, which can be set into a [PhysicsBody] or [Area]. A ray is not really a collision body, instead it tries to separate itself from whatever is touching its far endpoint. It's often useful for characters.
    /// </summary>
    public class RayShape : Shape
    {
        /// <summary>
        /// The ray's length.
        /// </summary>
        public float Length
        {
            get
            {
                return GetLength();
            }
            set
            {
                SetLength(value);
            }
        }


        private const string nativeName = "RayShape";

        public RayShape() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_RayShape_Ctor(this);
        }

        internal RayShape(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_length");

        [GodotMethod("set_length")]
        public void SetLength(float length)
        {
            NativeCalls.godot_icall_1_10(method_bind_0, Object.GetPtr(this), length);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_length");

        [GodotMethod("get_length")]
        public float GetLength()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }
    }
}
