using System;

namespace Godot
{
    /// <summary>
    /// Rectangle shape for 2D collisions. This shape is useful for modeling box-like 2D objects.
    /// </summary>
    public class RectangleShape2D : Shape2D
    {
        /// <summary>
        /// The rectangle's half extents. The width and height of this shape is twice the half extents.
        /// </summary>
        public Vector2 Extents
        {
            get
            {
                return GetExtents();
            }
            set
            {
                SetExtents(value);
            }
        }


        private const string nativeName = "RectangleShape2D";

        public RectangleShape2D() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_RectangleShape2D_Ctor(this);
        }

        internal RectangleShape2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_extents");

        [GodotMethod("set_extents")]
        public void SetExtents(Vector2 extents)
        {
            NativeCalls.godot_icall_1_45(method_bind_0, Object.GetPtr(this), ref extents);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_extents");

        [GodotMethod("get_extents")]
        public Vector2 GetExtents()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_1, Object.GetPtr(this));
            return (Vector2)ret;
        }
    }
}
