using System;

namespace Godot
{
    /// <summary>
    /// Base class for anything that keeps a reference count. Resource and many other helper objects inherit this. References keep an internal reference counter so they are only released when no longer in use.
    /// </summary>
    public class Reference : Object
    {
        private const string nativeName = "Reference";

        public Reference() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Reference_Ctor(this);
        }

        internal Reference(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "init_ref");

        /// <summary>
        /// </summary>
        [GodotMethod("init_ref")]
        public bool InitRef()
        {
            return NativeCalls.godot_icall_0_3(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "reference");

        /// <summary>
        /// Increase the internal reference counter. Use this only if you really know what you are doing.
        /// </summary>
        [GodotMethod("reference")]
        public bool Reference_()
        {
            return NativeCalls.godot_icall_0_3(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "unreference");

        /// <summary>
        /// Decrease the internal reference counter. Use this only if you really know what you are doing.
        /// </summary>
        [GodotMethod("unreference")]
        public bool Unreference()
        {
            return NativeCalls.godot_icall_0_3(method_bind_2, Object.GetPtr(this));
        }
    }
}
