using System;
using System.Collections.Generic;

namespace Godot
{
    /// <summary>
    /// Contains the results of a single regex match returned by [method RegEx.search] and [method RegEx.search_all]. It can be used to find the position and range of the match and its capturing groups, and it can extract its sub-string for you.
    /// </summary>
    public class RegExMatch : Reference
    {
        /// <summary>
        /// The source string used with the search pattern to find this matching result.
        /// </summary>
        public string Subject
        {
            get
            {
                return GetSubject();
            }
        }

        /// <summary>
        /// A dictionary of named groups and its corresponding group number. Only groups with that were matched are included. If multiple groups have the same name, that name would refer to the first matching one.
        /// </summary>
        public Dictionary<object, object> Names
        {
            get
            {
                return GetNames();
            }
        }

        /// <summary>
        /// An [Array] of the match and its capturing groups.
        /// </summary>
        public object[] Strings
        {
            get
            {
                return GetStrings();
            }
        }


        private const string nativeName = "RegExMatch";

        public RegExMatch() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_RegExMatch_Ctor(this);
        }

        internal RegExMatch(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_subject");

        [GodotMethod("get_subject")]
        public string GetSubject()
        {
            return NativeCalls.godot_icall_0_2(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_group_count");

        /// <summary>
        /// Returns the number of capturing groups.
        /// </summary>
        [GodotMethod("get_group_count")]
        public int GetGroupCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_names");

        [GodotMethod("get_names")]
        public Dictionary<object, object> GetNames()
        {
            return NativeCalls.godot_icall_0_169(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_strings");

        [GodotMethod("get_strings")]
        public object[] GetStrings()
        {
            return NativeCalls.godot_icall_0_21(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_string");

        /// <summary>
        /// Returns the substring of the match from the source string. Capturing groups can be retrieved by providing its group number as an integer or its string name (if it's a named group). The default value of 0 refers to the whole pattern.
        /// Returns an empty string if the group did not match or doesn't exist.
        /// &lt;param name="name"&gt;If the param is null, then the default value is (object)0&lt;/param&gt;
        /// </summary>
        [GodotMethod("get_string")]
        public string GetString(object name = null)
        {
            object name_in = name != null ? name : (object)0;
            return NativeCalls.godot_icall_1_728(method_bind_4, Object.GetPtr(this), name_in);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_start");

        /// <summary>
        /// Returns the starting position of the match within the source string. The starting position of capturing groups can be retrieved by providing its group number as an integer or its string name (if it's a named group). The default value of 0 refers to the whole pattern.
        /// Returns -1 if the group did not match or doesn't exist.
        /// &lt;param name="name"&gt;If the param is null, then the default value is (object)0&lt;/param&gt;
        /// </summary>
        [GodotMethod("get_start")]
        public int GetStart(object name = null)
        {
            object name_in = name != null ? name : (object)0;
            return NativeCalls.godot_icall_1_729(method_bind_5, Object.GetPtr(this), name_in);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_end");

        /// <summary>
        /// Returns the end position of the match within the source string. The end position of capturing groups can be retrieved by providing its group number as an integer or its string name (if it's a named group). The default value of 0 refers to the whole pattern.
        /// Returns -1 if the group did not match or doesn't exist.
        /// &lt;param name="name"&gt;If the param is null, then the default value is (object)0&lt;/param&gt;
        /// </summary>
        [GodotMethod("get_end")]
        public int GetEnd(object name = null)
        {
            object name_in = name != null ? name : (object)0;
            return NativeCalls.godot_icall_1_729(method_bind_6, Object.GetPtr(this), name_in);
        }
    }
}
