using System;

namespace Godot
{
    /// <summary>
    /// Resource Loader. This is a static object accessible as [code]ResourceLoader[/code]. GDScript has a simplified load() function, though.
    /// </summary>
    public static class ResourceLoader
    {
        private const string nativeName = "_ResourceLoader";
        internal static IntPtr ptr = NativeCalls.godot_icall__ResourceLoader_get_singleton();

        private static IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "load_interactive");

        /// <summary>
        /// Load a resource interactively, the returned object allows to load with high granularity.
        /// </summary>
        [GodotMethod("load_interactive")]
        public static ResourceInteractiveLoader LoadInteractive(string path, string typeHint = "")
        {
            return NativeCalls.godot_icall_2_1071(method_bind_0, ptr, path, typeHint);
        }

        private static IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "load");

        /// <summary>
        /// </summary>
        [GodotMethod("load")]
        public static Resource Load(string path, string typeHint = "", bool pNoCache = false)
        {
            return NativeCalls.godot_icall_3_1072(method_bind_1, ptr, path, typeHint, pNoCache);
        }

        private static IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_recognized_extensions_for_type");

        /// <summary>
        /// Return the list of recognized extensions for a resource type.
        /// </summary>
        [GodotMethod("get_recognized_extensions_for_type")]
        public static string[] GetRecognizedExtensionsForType(string type)
        {
            return NativeCalls.godot_icall_1_252(method_bind_2, ptr, type);
        }

        private static IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_abort_on_missing_resources");

        /// <summary>
        /// Change the behavior on missing sub-resources. Default is to abort load.
        /// </summary>
        [GodotMethod("set_abort_on_missing_resources")]
        public static void SetAbortOnMissingResources(bool abort)
        {
            NativeCalls.godot_icall_1_11(method_bind_3, ptr, abort);
        }

        private static IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_dependencies");

        /// <summary>
        /// </summary>
        [GodotMethod("get_dependencies")]
        public static string[] GetDependencies(string path)
        {
            return NativeCalls.godot_icall_1_252(method_bind_4, ptr, path);
        }

        private static IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has");

        /// <summary>
        /// </summary>
        [GodotMethod("has")]
        public static bool Has(string path)
        {
            return NativeCalls.godot_icall_1_70(method_bind_5, ptr, path);
        }
    }
}
