using System;

namespace Godot
{
    /// <summary>
    /// Resource Preloader Node. This node is used to preload sub-resources inside a scene, so when the scene is loaded all the resources are ready to use and be retrieved from here.
    /// </summary>
    public class ResourcePreloader : Node
    {
        public object[] Resources
        {
            get
            {
                return _GetResources();
            }
            set
            {
                _SetResources(value);
            }
        }

        private const string nativeName = "ResourcePreloader";

        public ResourcePreloader() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_ResourcePreloader_Ctor(this);
        }

        internal ResourcePreloader(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_resources");

        internal void _SetResources(object[] arg0)
        {
            NativeCalls.godot_icall_1_154(method_bind_0, Object.GetPtr(this), arg0);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_resources");

        internal object[] _GetResources()
        {
            return NativeCalls.godot_icall_0_21(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_resource");

        /// <summary>
        /// </summary>
        [GodotMethod("add_resource")]
        public void AddResource(string name, Resource resource)
        {
            NativeCalls.godot_icall_2_89(method_bind_2, Object.GetPtr(this), name, Object.GetPtr(resource));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_resource");

        /// <summary>
        /// Remove a resource from the preloader by text id.
        /// </summary>
        [GodotMethod("remove_resource")]
        public void RemoveResource(string name)
        {
            NativeCalls.godot_icall_1_43(method_bind_3, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "rename_resource");

        /// <summary>
        /// Rename a resource inside the preloader, from a text-id to a new text-id.
        /// </summary>
        [GodotMethod("rename_resource")]
        public void RenameResource(string name, string newname)
        {
            NativeCalls.godot_icall_2_69(method_bind_4, Object.GetPtr(this), name, newname);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_resource");

        /// <summary>
        /// Return true if the preloader has a given resource.
        /// </summary>
        [GodotMethod("has_resource")]
        public bool HasResource(string name)
        {
            return NativeCalls.godot_icall_1_70(method_bind_5, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_resource");

        /// <summary>
        /// Return the resource given a text-id.
        /// </summary>
        [GodotMethod("get_resource")]
        public Resource GetResource(string name)
        {
            return NativeCalls.godot_icall_1_732(method_bind_6, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_resource_list");

        /// <summary>
        /// Return the list of resources inside the preloader.
        /// </summary>
        [GodotMethod("get_resource_list")]
        public string[] GetResourceList()
        {
            return NativeCalls.godot_icall_0_72(method_bind_7, Object.GetPtr(this));
        }
    }
}
