using System;

namespace Godot
{
    /// <summary>
    /// Resource Saving Interface. This interface is used for saving resources to disk.
    /// </summary>
    public static class ResourceSaver
    {
        public enum SaverFlags
        {
            RelativePaths = 1,
            BundleResources = 2,
            ChangePath = 4,
            OmitEditorProperties = 8,
            SaveBigEndian = 16,
            Compress = 32
        }


        private const string nativeName = "_ResourceSaver";
        internal static IntPtr ptr = NativeCalls.godot_icall__ResourceSaver_get_singleton();

        private static IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "save");

        /// <summary>
        /// Save a resource to disk, to a given path.
        /// </summary>
        [GodotMethod("save")]
        public static Error Save(string path, Resource resource, int flags = 0)
        {
            return NativeCalls.godot_icall_3_1073(method_bind_0, ptr, path, Object.GetPtr(resource), flags);
        }

        private static IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_recognized_extensions");

        /// <summary>
        /// Return the list of extensions available for saving a resource of a given type.
        /// </summary>
        [GodotMethod("get_recognized_extensions")]
        public static string[] GetRecognizedExtensions(Resource type)
        {
            return NativeCalls.godot_icall_1_1074(method_bind_1, ptr, Object.GetPtr(type));
        }
    }
}
