using System;

namespace Godot
{
    /// <summary>
    /// Maintains a list of resources, nodes, exported and overridden properties, and built-in scripts associated with a scene.
    /// </summary>
    public class SceneState : Reference
    {
        public enum GenEditState
        {
            Disabled = 0,
            Instance = 1,
            Main = 2
        }


        private const string nativeName = "SceneState";

        internal SceneState() {}

        internal SceneState(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_node_count");

        /// <summary>
        /// Returns the number of nodes in the scene.
        /// </summary>
        [GodotMethod("get_node_count")]
        public int GetNodeCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_node_type");

        /// <summary>
        /// Returns the type of the node at [code]idx[/code].
        /// </summary>
        [GodotMethod("get_node_type")]
        public string GetNodeType(int idx)
        {
            return NativeCalls.godot_icall_1_108(method_bind_1, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_node_name");

        /// <summary>
        /// Returns the name of the node at [code]idx[/code].
        /// </summary>
        [GodotMethod("get_node_name")]
        public string GetNodeName(int idx)
        {
            return NativeCalls.godot_icall_1_108(method_bind_2, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_node_path");

        /// <summary>
        /// Returns the path to the node at [code]idx[/code].
        /// </summary>
        [GodotMethod("get_node_path")]
        public NodePath GetNodePath(int idx, bool forParent = false)
        {
            IntPtr ret = NativeCalls.godot_icall_2_744(method_bind_3, Object.GetPtr(this), idx, forParent);
            return new NodePath(ret);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_node_owner_path");

        /// <summary>
        /// Returns the path to the owner of the node at [code]idx[/code], relative to the root node.
        /// </summary>
        [GodotMethod("get_node_owner_path")]
        public NodePath GetNodeOwnerPath(int idx)
        {
            IntPtr ret = NativeCalls.godot_icall_1_48(method_bind_4, Object.GetPtr(this), idx);
            return new NodePath(ret);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_node_instance_placeholder");

        /// <summary>
        /// Returns [code]true[/code] if the node at [code]idx[/code] is an [InstancePlaceholder].
        /// </summary>
        [GodotMethod("is_node_instance_placeholder")]
        public bool IsNodeInstancePlaceholder(int idx)
        {
            return NativeCalls.godot_icall_1_29(method_bind_5, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_node_instance_placeholder");

        /// <summary>
        /// Returns the path to the represented scene file if the node at [code]idx[/code] is an [InstancePlaceholder].
        /// </summary>
        [GodotMethod("get_node_instance_placeholder")]
        public string GetNodeInstancePlaceholder(int idx)
        {
            return NativeCalls.godot_icall_1_108(method_bind_6, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_node_instance");

        /// <summary>
        /// Returns the scene for the node at [code]idx[/code] or [code]null[/code] if the node is not an instance.
        /// </summary>
        [GodotMethod("get_node_instance")]
        public PackedScene GetNodeInstance(int idx)
        {
            return NativeCalls.godot_icall_1_745(method_bind_7, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_node_groups");

        /// <summary>
        /// Returns the list of group names associated with the node at [code]idx[/code].
        /// </summary>
        [GodotMethod("get_node_groups")]
        public string[] GetNodeGroups(int idx)
        {
            return NativeCalls.godot_icall_1_746(method_bind_8, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_node_index");

        /// <summary>
        /// </summary>
        [GodotMethod("get_node_index")]
        public int GetNodeIndex(int idx)
        {
            return NativeCalls.godot_icall_1_6(method_bind_9, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_node_property_count");

        /// <summary>
        /// Returns the number of exported or overridden properties for the node at [code]idx[/code].
        /// </summary>
        [GodotMethod("get_node_property_count")]
        public int GetNodePropertyCount(int idx)
        {
            return NativeCalls.godot_icall_1_6(method_bind_10, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_node_property_name");

        /// <summary>
        /// Returns the name of the property at [code]prop_idx[/code] for the node at [code]idx[/code].
        /// </summary>
        [GodotMethod("get_node_property_name")]
        public string GetNodePropertyName(int idx, int propIdx)
        {
            return NativeCalls.godot_icall_2_65(method_bind_11, Object.GetPtr(this), idx, propIdx);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_node_property_value");

        /// <summary>
        /// Returns the value of the property at [code]prop_idx[/code] for the node at [code]idx[/code].
        /// </summary>
        [GodotMethod("get_node_property_value")]
        public object GetNodePropertyValue(int idx, int propIdx)
        {
            return NativeCalls.godot_icall_2_57(method_bind_12, Object.GetPtr(this), idx, propIdx);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_connection_count");

        /// <summary>
        /// Returns the number of signal connections in the scene.
        /// </summary>
        [GodotMethod("get_connection_count")]
        public int GetConnectionCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_connection_source");

        /// <summary>
        /// Returns the path to the node that owns the signal at [code]idx[/code], relative to the root node.
        /// </summary>
        [GodotMethod("get_connection_source")]
        public NodePath GetConnectionSource(int idx)
        {
            IntPtr ret = NativeCalls.godot_icall_1_48(method_bind_14, Object.GetPtr(this), idx);
            return new NodePath(ret);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_connection_signal");

        /// <summary>
        /// Returns the name of the signal at [code]idx[/code].
        /// </summary>
        [GodotMethod("get_connection_signal")]
        public string GetConnectionSignal(int idx)
        {
            return NativeCalls.godot_icall_1_108(method_bind_15, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_connection_target");

        /// <summary>
        /// Returns the path to the node that owns the method connected to the signal at [code]idx[/code], relative to the root node.
        /// </summary>
        [GodotMethod("get_connection_target")]
        public NodePath GetConnectionTarget(int idx)
        {
            IntPtr ret = NativeCalls.godot_icall_1_48(method_bind_16, Object.GetPtr(this), idx);
            return new NodePath(ret);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_connection_method");

        /// <summary>
        /// Returns the method connected to the signal at [code]idx[/code].
        /// </summary>
        [GodotMethod("get_connection_method")]
        public string GetConnectionMethod(int idx)
        {
            return NativeCalls.godot_icall_1_108(method_bind_17, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_connection_flags");

        /// <summary>
        /// Returns the flags for the signal at [code]idx[/code]. See [Object]'s [code]CONNECT_*[/code] flags.
        /// </summary>
        [GodotMethod("get_connection_flags")]
        public int GetConnectionFlags(int idx)
        {
            return NativeCalls.godot_icall_1_6(method_bind_18, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_connection_binds");

        /// <summary>
        /// Returns the list of bound parameters for the signal at [code]idx[/code].
        /// </summary>
        [GodotMethod("get_connection_binds")]
        public object[] GetConnectionBinds(int idx)
        {
            return NativeCalls.godot_icall_1_116(method_bind_19, Object.GetPtr(this), idx);
        }
    }
}
