using System;

namespace Godot
{
    public class SceneTree : MainLoop
    {
        public enum StretchAspect
        {
            Ignore = 0,
            Keep = 1,
            KeepWidth = 2,
            KeepHeight = 3,
            Expand = 4
        }

        public enum GroupCallFlags
        {
            Default = 0,
            Reverse = 1,
            Realtime = 2,
            Unique = 4
        }

        public enum StretchMode
        {
            Disabled = 0,
            Mode2d = 1,
            Viewport = 2
        }


        /// <summary>
        /// </summary>
        public bool DebugCollisionsHint
        {
            get
            {
                return IsDebuggingCollisionsHint();
            }
            set
            {
                SetDebugCollisionsHint(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool DebugNavigationHint
        {
            get
            {
                return IsDebuggingNavigationHint();
            }
            set
            {
                SetDebugNavigationHint(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool Paused
        {
            get
            {
                return IsPaused();
            }
            set
            {
                SetPause(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool RefuseNewNetworkConnections
        {
            get
            {
                return IsRefusingNewNetworkConnections();
            }
            set
            {
                SetRefuseNewNetworkConnections(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool UseFontOversampling
        {
            get
            {
                return IsUsingFontOversampling();
            }
            set
            {
                SetUseFontOversampling(value);
            }
        }

        /// <summary>
        /// </summary>
        public Node EditedSceneRoot
        {
            get
            {
                return GetEditedSceneRoot();
            }
            set
            {
                SetEditedSceneRoot(value);
            }
        }

        /// <summary>
        /// </summary>
        public Node CurrentScene
        {
            get
            {
                return GetCurrentScene();
            }
            set
            {
                SetCurrentScene(value);
            }
        }

        /// <summary>
        /// The peer object to handle the RPC system (effectively enabling networking when set). Depending on the peer itself, the SceneTree will become a network server (check with [method is_network_server()]) and will set root node's network mode to master (see NETWORK_MODE_* constants in [Node]), or it will become a regular peer with root node set to slave. All child nodes are set to inherit the network mode by default. Handling of networking-related events (connection, disconnection, new clients) is done by connecting to SceneTree's signals.
        /// </summary>
        public NetworkedMultiplayerPeer NetworkPeer
        {
            get
            {
                return GetNetworkPeer();
            }
            set
            {
                SetNetworkPeer(value);
            }
        }

        /// <summary>
        /// </summary>
        public Viewport Root
        {
            get
            {
                return GetRoot();
            }
        }


        private const string nativeName = "SceneTree";

        public SceneTree() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_SceneTree_Ctor(this);
        }

        internal SceneTree(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_root");

        [GodotMethod("get_root")]
        public Viewport GetRoot()
        {
            return NativeCalls.godot_icall_0_553(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_group");

        /// <summary>
        /// </summary>
        [GodotMethod("has_group")]
        public bool HasGroup(string name)
        {
            return NativeCalls.godot_icall_1_70(method_bind_1, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_auto_accept_quit");

        /// <summary>
        /// </summary>
        [GodotMethod("set_auto_accept_quit")]
        public void SetAutoAcceptQuit(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_2, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_quit_on_go_back");

        /// <summary>
        /// </summary>
        [GodotMethod("set_quit_on_go_back")]
        public void SetQuitOnGoBack(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_3, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_debug_collisions_hint");

        [GodotMethod("set_debug_collisions_hint")]
        public void SetDebugCollisionsHint(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_4, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_debugging_collisions_hint");

        [GodotMethod("is_debugging_collisions_hint")]
        public bool IsDebuggingCollisionsHint()
        {
            return NativeCalls.godot_icall_0_3(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_debug_navigation_hint");

        [GodotMethod("set_debug_navigation_hint")]
        public void SetDebugNavigationHint(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_6, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_debugging_navigation_hint");

        [GodotMethod("is_debugging_navigation_hint")]
        public bool IsDebuggingNavigationHint()
        {
            return NativeCalls.godot_icall_0_3(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_edited_scene_root");

        [GodotMethod("set_edited_scene_root")]
        public void SetEditedSceneRoot(Node scene)
        {
            NativeCalls.godot_icall_1_42(method_bind_8, Object.GetPtr(this), Object.GetPtr(scene));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_edited_scene_root");

        [GodotMethod("get_edited_scene_root")]
        public Node GetEditedSceneRoot()
        {
            return NativeCalls.godot_icall_0_200(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_pause");

        [GodotMethod("set_pause")]
        public void SetPause(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_10, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_paused");

        [GodotMethod("is_paused")]
        public bool IsPaused()
        {
            return NativeCalls.godot_icall_0_3(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_input_as_handled");

        /// <summary>
        /// </summary>
        [GodotMethod("set_input_as_handled")]
        public void SetInputAsHandled()
        {
            NativeCalls.godot_icall_0_12(method_bind_12, Object.GetPtr(this));
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_input_handled");

        /// <summary>
        /// </summary>
        [GodotMethod("is_input_handled")]
        public bool IsInputHandled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "create_timer");

        /// <summary>
        /// </summary>
        [GodotMethod("create_timer")]
        public SceneTreeTimer CreateTimer(float timeSec, bool pauseModeProcess = true)
        {
            return NativeCalls.godot_icall_2_747(method_bind_14, Object.GetPtr(this), timeSec, pauseModeProcess);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_node_count");

        /// <summary>
        /// </summary>
        [GodotMethod("get_node_count")]
        public int GetNodeCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_frame");

        /// <summary>
        /// </summary>
        [GodotMethod("get_frame")]
        public int GetFrame()
        {
            return NativeCalls.godot_icall_0_1(method_bind_16, Object.GetPtr(this));
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "quit");

        /// <summary>
        /// </summary>
        [GodotMethod("quit")]
        public void Quit()
        {
            NativeCalls.godot_icall_0_12(method_bind_17, Object.GetPtr(this));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_screen_stretch");

        /// <summary>
        /// </summary>
        [GodotMethod("set_screen_stretch")]
        public void SetScreenStretch(SceneTree.StretchMode mode, SceneTree.StretchAspect aspect, Vector2 minsize, float shrink = (float)1)
        {
            NativeCalls.godot_icall_4_748(method_bind_18, Object.GetPtr(this), mode, aspect, ref minsize, shrink);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "queue_delete");

        /// <summary>
        /// </summary>
        [GodotMethod("queue_delete")]
        public void QueueDelete(Object obj)
        {
            NativeCalls.godot_icall_1_42(method_bind_19, Object.GetPtr(this), Object.GetPtr(obj));
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "call_group_flags");

        /// <summary>
        /// </summary>
        [GodotMethod("call_group_flags")]
        public object CallGroupFlags(int flags, string group, string method, params object[] @args)
        {
            return NativeCalls.godot_icall_4_749(method_bind_20, Object.GetPtr(this), flags, group, method, @args);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "notify_group_flags");

        /// <summary>
        /// </summary>
        [GodotMethod("notify_group_flags")]
        public void NotifyGroupFlags(int callFlags, string group, int notification)
        {
            NativeCalls.godot_icall_3_750(method_bind_21, Object.GetPtr(this), callFlags, group, notification);
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_group_flags");

        /// <summary>
        /// </summary>
        [GodotMethod("set_group_flags")]
        public void SetGroupFlags(int callFlags, string group, string property, object value)
        {
            NativeCalls.godot_icall_4_751(method_bind_22, Object.GetPtr(this), callFlags, group, property, value);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "call_group");

        /// <summary>
        /// </summary>
        [GodotMethod("call_group")]
        public object CallGroup(string group, string method, params object[] @args)
        {
            return NativeCalls.godot_icall_3_752(method_bind_23, Object.GetPtr(this), group, method, @args);
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "notify_group");

        /// <summary>
        /// </summary>
        [GodotMethod("notify_group")]
        public void NotifyGroup(string group, int notification)
        {
            NativeCalls.godot_icall_2_95(method_bind_24, Object.GetPtr(this), group, notification);
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_group");

        /// <summary>
        /// </summary>
        [GodotMethod("set_group")]
        public void SetGroup(string group, string property, object value)
        {
            NativeCalls.godot_icall_3_249(method_bind_25, Object.GetPtr(this), group, property, value);
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_nodes_in_group");

        /// <summary>
        /// </summary>
        [GodotMethod("get_nodes_in_group")]
        public object[] GetNodesInGroup(string group)
        {
            return NativeCalls.godot_icall_1_446(method_bind_26, Object.GetPtr(this), group);
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_current_scene");

        [GodotMethod("set_current_scene")]
        public void SetCurrentScene(Node childNode)
        {
            NativeCalls.godot_icall_1_42(method_bind_27, Object.GetPtr(this), Object.GetPtr(childNode));
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_current_scene");

        [GodotMethod("get_current_scene")]
        public Node GetCurrentScene()
        {
            return NativeCalls.godot_icall_0_200(method_bind_28, Object.GetPtr(this));
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "change_scene");

        /// <summary>
        /// </summary>
        [GodotMethod("change_scene")]
        public Error ChangeScene(string path)
        {
            return NativeCalls.godot_icall_1_170(method_bind_29, Object.GetPtr(this), path);
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "change_scene_to");

        /// <summary>
        /// </summary>
        [GodotMethod("change_scene_to")]
        public Error ChangeSceneTo(PackedScene packedScene)
        {
            return NativeCalls.godot_icall_1_505(method_bind_30, Object.GetPtr(this), Object.GetPtr(packedScene));
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "reload_current_scene");

        /// <summary>
        /// </summary>
        [GodotMethod("reload_current_scene")]
        public Error ReloadCurrentScene()
        {
            return NativeCalls.godot_icall_0_325(method_bind_31, Object.GetPtr(this));
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_network_peer");

        [GodotMethod("set_network_peer")]
        public void SetNetworkPeer(NetworkedMultiplayerPeer peer)
        {
            NativeCalls.godot_icall_1_24(method_bind_32, Object.GetPtr(this), Object.GetPtr(peer));
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_network_peer");

        [GodotMethod("get_network_peer")]
        public NetworkedMultiplayerPeer GetNetworkPeer()
        {
            return NativeCalls.godot_icall_0_753(method_bind_33, Object.GetPtr(this));
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_network_server");

        /// <summary>
        /// Returns true if this SceneTree's [NetworkedMultiplayerPeer] is in server mode (listening for connections).
        /// </summary>
        [GodotMethod("is_network_server")]
        public bool IsNetworkServer()
        {
            return NativeCalls.godot_icall_0_3(method_bind_34, Object.GetPtr(this));
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_network_peer");

        /// <summary>
        /// Returns true if there is a [NetworkedMultiplayerPeer] set (with [method SceneTree.set_network_peer]).
        /// </summary>
        [GodotMethod("has_network_peer")]
        public bool HasNetworkPeer()
        {
            return NativeCalls.godot_icall_0_3(method_bind_35, Object.GetPtr(this));
        }

        private IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_network_connected_peers");

        /// <summary>
        /// </summary>
        [GodotMethod("get_network_connected_peers")]
        public int[] GetNetworkConnectedPeers()
        {
            return NativeCalls.godot_icall_0_175(method_bind_36, Object.GetPtr(this));
        }

        private IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_network_unique_id");

        /// <summary>
        /// </summary>
        [GodotMethod("get_network_unique_id")]
        public int GetNetworkUniqueId()
        {
            return NativeCalls.godot_icall_0_1(method_bind_37, Object.GetPtr(this));
        }

        private IntPtr method_bind_38 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_rpc_sender_id");

        /// <summary>
        /// </summary>
        [GodotMethod("get_rpc_sender_id")]
        public int GetRpcSenderId()
        {
            return NativeCalls.godot_icall_0_1(method_bind_38, Object.GetPtr(this));
        }

        private IntPtr method_bind_39 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_refuse_new_network_connections");

        [GodotMethod("set_refuse_new_network_connections")]
        public void SetRefuseNewNetworkConnections(bool refuse)
        {
            NativeCalls.godot_icall_1_11(method_bind_39, Object.GetPtr(this), refuse);
        }

        private IntPtr method_bind_40 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_refusing_new_network_connections");

        [GodotMethod("is_refusing_new_network_connections")]
        public bool IsRefusingNewNetworkConnections()
        {
            return NativeCalls.godot_icall_0_3(method_bind_40, Object.GetPtr(this));
        }

        private IntPtr method_bind_41 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_use_font_oversampling");

        [GodotMethod("set_use_font_oversampling")]
        public void SetUseFontOversampling(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_41, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_42 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_using_font_oversampling");

        [GodotMethod("is_using_font_oversampling")]
        public bool IsUsingFontOversampling()
        {
            return NativeCalls.godot_icall_0_3(method_bind_42, Object.GetPtr(this));
        }
    }
}
