using System;

namespace Godot
{
    /// <summary>
    /// Segment shape for 2D collisions. Consists of two points, [code]a[/code] and [code]b[/code].
    /// </summary>
    public class SegmentShape2D : Shape2D
    {
        /// <summary>
        /// The segment's first point position.
        /// </summary>
        public Vector2 A
        {
            get
            {
                return GetA();
            }
            set
            {
                SetA(value);
            }
        }

        /// <summary>
        /// The segment's second point position.
        /// </summary>
        public Vector2 B
        {
            get
            {
                return GetB();
            }
            set
            {
                SetB(value);
            }
        }


        private const string nativeName = "SegmentShape2D";

        public SegmentShape2D() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_SegmentShape2D_Ctor(this);
        }

        internal SegmentShape2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_a");

        [GodotMethod("set_a")]
        public void SetA(Vector2 a)
        {
            NativeCalls.godot_icall_1_45(method_bind_0, Object.GetPtr(this), ref a);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_a");

        [GodotMethod("get_a")]
        public Vector2 GetA()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_1, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_b");

        [GodotMethod("set_b")]
        public void SetB(Vector2 b)
        {
            NativeCalls.godot_icall_1_45(method_bind_2, Object.GetPtr(this), ref b);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_b");

        [GodotMethod("get_b")]
        public Vector2 GetB()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_3, Object.GetPtr(this));
            return (Vector2)ret;
        }
    }
}
