using System;

namespace Godot
{
    /// <summary>
    /// A synchronization Semaphore. Element used to synchronize multiple [Thread]s. Initialized to zero on creation. Be careful to avoid deadlocks. For a binary version, see [Mutex].
    /// </summary>
    public class Semaphore : Reference
    {
        private const string nativeName = "_Semaphore";

        public Semaphore() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Semaphore_Ctor(this);
        }

        internal Semaphore(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "wait");

        /// <summary>
        /// Tries to wait for the [code]Semaphore[/code], if its value is zero, blocks until non-zero. Returns [OK] on success, [ERR_BUSY] otherwise.
        /// </summary>
        [GodotMethod("wait")]
        public Error Wait()
        {
            return NativeCalls.godot_icall_0_325(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "post");

        /// <summary>
        /// Lowers the [code]Semaphore[/code], allowing one more thread in. Returns [OK] on success, [ERR_BUSY] otherwise.
        /// </summary>
        [GodotMethod("post")]
        public Error Post()
        {
            return NativeCalls.godot_icall_0_325(method_bind_1, Object.GetPtr(this));
        }
    }
}
