using System;

namespace Godot
{
    /// <summary>
    /// Base class for all 2D Shapes. All 2D shape types inherit from this.
    /// </summary>
    public class Shape2D : Resource
    {
        /// <summary>
        /// </summary>
        public float CustomSolverBias
        {
            get
            {
                return GetCustomSolverBias();
            }
            set
            {
                SetCustomSolverBias(value);
            }
        }


        private const string nativeName = "Shape2D";

        internal Shape2D() {}

        internal Shape2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_custom_solver_bias");

        [GodotMethod("set_custom_solver_bias")]
        public void SetCustomSolverBias(float bias)
        {
            NativeCalls.godot_icall_1_10(method_bind_0, Object.GetPtr(this), bias);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_custom_solver_bias");

        [GodotMethod("get_custom_solver_bias")]
        public float GetCustomSolverBias()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "collide");

        /// <summary>
        /// Return whether this shape is colliding with another.
        /// This method needs the transformation matrix for this shape ([code]local_xform[/code]), the shape to check collisions with ([code]with_shape[/code]), and the transformation matrix of that shape ([code]shape_xform[/code]).
        /// </summary>
        [GodotMethod("collide")]
        public bool Collide(Transform2D localXform, Shape2D withShape, Transform2D shapeXform)
        {
            return NativeCalls.godot_icall_3_761(method_bind_2, Object.GetPtr(this), ref localXform, Object.GetPtr(withShape), ref shapeXform);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "collide_with_motion");

        /// <summary>
        /// Return whether this shape would collide with another, if a given movement was applied.
        /// This method needs the transformation matrix for this shape ([code]local_xform[/code]), the movement to test on this shape ([code]local_motion[/code]), the shape to check collisions with ([code]with_shape[/code]), the transformation matrix of that shape ([code]shape_xform[/code]), and the movement to test onto the other object ([code]shape_motion[/code]).
        /// </summary>
        [GodotMethod("collide_with_motion")]
        public bool CollideWithMotion(Transform2D localXform, Vector2 localMotion, Shape2D withShape, Transform2D shapeXform, Vector2 shapeMotion)
        {
            return NativeCalls.godot_icall_5_762(method_bind_3, Object.GetPtr(this), ref localXform, ref localMotion, Object.GetPtr(withShape), ref shapeXform, ref shapeMotion);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "collide_and_get_contacts");

        /// <summary>
        /// Return a list of the points where this shape touches another. If there are no collisions, the list is empty.
        /// This method needs the transformation matrix for this shape ([code]local_xform[/code]), the shape to check collisions with ([code]with_shape[/code]), and the transformation matrix of that shape ([code]shape_xform[/code]).
        /// </summary>
        [GodotMethod("collide_and_get_contacts")]
        public object CollideAndGetContacts(Transform2D localXform, Shape2D withShape, Transform2D shapeXform)
        {
            return NativeCalls.godot_icall_3_763(method_bind_4, Object.GetPtr(this), ref localXform, Object.GetPtr(withShape), ref shapeXform);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "collide_with_motion_and_get_contacts");

        /// <summary>
        /// Return a list of the points where this shape would touch another, if a given movement was applied. If there are no collisions, the list is empty.
        /// This method needs the transformation matrix for this shape ([code]local_xform[/code]), the movement to test on this shape ([code]local_motion[/code]), the shape to check collisions with ([code]with_shape[/code]), the transformation matrix of that shape ([code]shape_xform[/code]), and the movement to test onto the other object ([code]shape_motion[/code]).
        /// </summary>
        [GodotMethod("collide_with_motion_and_get_contacts")]
        public object CollideWithMotionAndGetContacts(Transform2D localXform, Vector2 localMotion, Shape2D withShape, Transform2D shapeXform, Vector2 shapeMotion)
        {
            return NativeCalls.godot_icall_5_764(method_bind_5, Object.GetPtr(this), ref localXform, ref localMotion, Object.GetPtr(withShape), ref shapeXform, ref shapeMotion);
        }
    }
}
