using System;

namespace Godot
{
    /// <summary>
    /// A shortcut for binding input.
    /// Shortcuts are commonly used for interacting with a [Control] element from a [InputEvent].
    /// </summary>
    public class ShortCut : Resource
    {
        /// <summary>
        /// The Shortcut's [InputEvent].
        /// Generally the [InputEvent] is a keyboard key, though it can be any [InputEvent].
        /// </summary>
        public InputEvent Shortcut
        {
            get
            {
                return GetShortcut();
            }
            set
            {
                SetShortcut(value);
            }
        }


        private const string nativeName = "ShortCut";

        public ShortCut() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_ShortCut_Ctor(this);
        }

        internal ShortCut(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_shortcut");

        [GodotMethod("set_shortcut")]
        public void SetShortcut(InputEvent @event)
        {
            NativeCalls.godot_icall_1_24(method_bind_0, Object.GetPtr(this), Object.GetPtr(@event));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_shortcut");

        [GodotMethod("get_shortcut")]
        public InputEvent GetShortcut()
        {
            return NativeCalls.godot_icall_0_765(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_valid");

        /// <summary>
        /// If [code]true[/code] this Shortcut is valid.
        /// </summary>
        [GodotMethod("is_valid")]
        public bool IsValid()
        {
            return NativeCalls.godot_icall_0_3(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_shortcut");

        /// <summary>
        /// Returns [code]true[/code] if the Shortcut's [InputEvent] equals [code]event[/code].
        /// </summary>
        [GodotMethod("is_shortcut")]
        public bool IsShortcut(InputEvent @event)
        {
            return NativeCalls.godot_icall_1_443(method_bind_3, Object.GetPtr(this), Object.GetPtr(@event));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_as_text");

        /// <summary>
        /// Returns the Shortcut's [InputEvent] as a [String].
        /// </summary>
        [GodotMethod("get_as_text")]
        public string GetAsText()
        {
            return NativeCalls.godot_icall_0_2(method_bind_4, Object.GetPtr(this));
        }
    }
}
