using System;

namespace Godot
{
    /// <summary>
    /// The base class for [PanoramaSky] and [ProceduralSky].
    /// </summary>
    public class Sky : Resource
    {
        public enum RadianceSizeEnum
        {
            Size32 = 0,
            Size64 = 1,
            Size128 = 2,
            Size256 = 3,
            Size512 = 4,
            Size1024 = 5,
            Size2048 = 6,
            Max = 7
        }


        /// <summary>
        /// The Sky's radiance map size.
        /// The higher the radiance map size, the more detailed the lighting from the Sky will be.
        /// See RADIANCE_SIZE_* constants for values. Default size is RADIANCE_SIZE_512.
        /// </summary>
        public Sky.RadianceSizeEnum RadianceSize
        {
            get
            {
                return GetRadianceSize();
            }
            set
            {
                SetRadianceSize(value);
            }
        }


        private const string nativeName = "Sky";

        internal Sky() {}

        internal Sky(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_radiance_size");

        [GodotMethod("set_radiance_size")]
        public void SetRadianceSize(Sky.RadianceSizeEnum size)
        {
            NativeCalls.godot_icall_1_767(method_bind_0, Object.GetPtr(this), size);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_radiance_size");

        [GodotMethod("get_radiance_size")]
        public Sky.RadianceSizeEnum GetRadianceSize()
        {
            return NativeCalls.godot_icall_0_768(method_bind_1, Object.GetPtr(this));
        }
    }
}
