using System;

namespace Godot
{
    /// <summary>
    /// Slides across the x-axis of the [Pivot] object.
    /// </summary>
    public class SliderJoint : Joint
    {
        public enum Param
        {
            LinearLimitUpper = 0,
            LinearLimitLower = 1,
            LinearLimitSoftness = 2,
            LinearLimitRestitution = 3,
            LinearLimitDamping = 4,
            LinearMotionSoftness = 5,
            LinearMotionRestitution = 6,
            LinearMotionDamping = 7,
            LinearOrthogonalSoftness = 8,
            LinearOrthogonalRestitution = 9,
            LinearOrthogonalDamping = 10,
            AngularLimitUpper = 11,
            AngularLimitLower = 12,
            AngularLimitSoftness = 13,
            AngularLimitRestitution = 14,
            AngularLimitDamping = 15,
            AngularMotionSoftness = 16,
            AngularMotionRestitution = 17,
            AngularMotionDamping = 18,
            AngularOrthogonalSoftness = 19,
            AngularOrthogonalRestitution = 20,
            AngularOrthogonalDamping = 21,
            Max = 22
        }


        /// <summary>
        /// The maximum difference between the pivot points on their x-axis before damping happens.
        /// </summary>
        public float LinearLimit__upperDistance
        {
            get
            {
                return GetParam((SliderJoint.Param)0);
            }
            set
            {
                SetParam((SliderJoint.Param)0, value);
            }
        }

        /// <summary>
        /// The minimum difference between the pivot points on their x-axis before damping happens.
        /// </summary>
        public float LinearLimit__lowerDistance
        {
            get
            {
                return GetParam((SliderJoint.Param)1);
            }
            set
            {
                SetParam((SliderJoint.Param)1, value);
            }
        }

        /// <summary>
        /// A factor applied to the movement accross the slider axis once the limits get surpassed. The lower, the slower the movement.
        /// </summary>
        public float LinearLimit__softness
        {
            get
            {
                return GetParam((SliderJoint.Param)2);
            }
            set
            {
                SetParam((SliderJoint.Param)2, value);
            }
        }

        /// <summary>
        /// The amount of restitution once the limits are surpassed. The lower, the more velocity-energy gets lost.
        /// </summary>
        public float LinearLimit__restitution
        {
            get
            {
                return GetParam((SliderJoint.Param)3);
            }
            set
            {
                SetParam((SliderJoint.Param)3, value);
            }
        }

        /// <summary>
        /// The amount of damping that happens once the limit defined by [member linear_limit/lower_distance] and [member linear_limit/upper_distance] is surpassed.
        /// </summary>
        public float LinearLimit__damping
        {
            get
            {
                return GetParam((SliderJoint.Param)4);
            }
            set
            {
                SetParam((SliderJoint.Param)4, value);
            }
        }

        /// <summary>
        /// A factor applied to the movement accross the slider axis as long as the slider is in the limits. The lower, the slower the movement.
        /// </summary>
        public float LinearMotion__softness
        {
            get
            {
                return GetParam((SliderJoint.Param)5);
            }
            set
            {
                SetParam((SliderJoint.Param)5, value);
            }
        }

        /// <summary>
        /// The amount of restitution inside the slider limits.
        /// </summary>
        public float LinearMotion__restitution
        {
            get
            {
                return GetParam((SliderJoint.Param)6);
            }
            set
            {
                SetParam((SliderJoint.Param)6, value);
            }
        }

        /// <summary>
        /// The amount of damping inside the slider limits.
        /// </summary>
        public float LinearMotion__damping
        {
            get
            {
                return GetParam((SliderJoint.Param)7);
            }
            set
            {
                SetParam((SliderJoint.Param)7, value);
            }
        }

        /// <summary>
        /// A factor applied to the movement accross axes orthogonal to the slider.
        /// </summary>
        public float LinearOrtho__softness
        {
            get
            {
                return GetParam((SliderJoint.Param)8);
            }
            set
            {
                SetParam((SliderJoint.Param)8, value);
            }
        }

        /// <summary>
        /// The amount of restitution when movement is accross axes orthogonal to the slider.
        /// </summary>
        public float LinearOrtho__restitution
        {
            get
            {
                return GetParam((SliderJoint.Param)9);
            }
            set
            {
                SetParam((SliderJoint.Param)9, value);
            }
        }

        /// <summary>
        /// The amount of damping when movement is accross axes orthogonal to the slider.
        /// </summary>
        public float LinearOrtho__damping
        {
            get
            {
                return GetParam((SliderJoint.Param)10);
            }
            set
            {
                SetParam((SliderJoint.Param)10, value);
            }
        }

        /// <summary>
        /// The upper limit of rotation in the slider.
        /// </summary>
        public float AngularLimit__upperAngle
        {
            get
            {
                return _GetUpperLimitAngular();
            }
            set
            {
                _SetUpperLimitAngular(value);
            }
        }

        /// <summary>
        /// The lower limit of rotation in the slider.
        /// </summary>
        public float AngularLimit__lowerAngle
        {
            get
            {
                return _GetLowerLimitAngular();
            }
            set
            {
                _SetLowerLimitAngular(value);
            }
        }

        /// <summary>
        /// A factor applied to the all rotation once the limit is surpassed.
        /// Makes all rotation slower when between 0 and 1.
        /// </summary>
        public float AngularLimit__softness
        {
            get
            {
                return GetParam((SliderJoint.Param)13);
            }
            set
            {
                SetParam((SliderJoint.Param)13, value);
            }
        }

        /// <summary>
        /// The amount of restitution of the rotation when the limit is surpassed.
        /// Does not affect damping.
        /// </summary>
        public float AngularLimit__restitution
        {
            get
            {
                return GetParam((SliderJoint.Param)14);
            }
            set
            {
                SetParam((SliderJoint.Param)14, value);
            }
        }

        /// <summary>
        /// The amount of damping of the rotation when the limit is surpassed.
        /// A lower damping value allows a rotation initiated by body A to travel to body B slower.
        /// </summary>
        public float AngularLimit__damping
        {
            get
            {
                return GetParam((SliderJoint.Param)15);
            }
            set
            {
                SetParam((SliderJoint.Param)15, value);
            }
        }

        /// <summary>
        /// A factor applied to the all rotation in the limits.
        /// </summary>
        public float AngularMotion__softness
        {
            get
            {
                return GetParam((SliderJoint.Param)16);
            }
            set
            {
                SetParam((SliderJoint.Param)16, value);
            }
        }

        /// <summary>
        /// The amount of restitution of the rotation in the limits.
        /// </summary>
        public float AngularMotion__restitution
        {
            get
            {
                return GetParam((SliderJoint.Param)17);
            }
            set
            {
                SetParam((SliderJoint.Param)17, value);
            }
        }

        /// <summary>
        /// The amount of damping of the rotation in the limits.
        /// </summary>
        public float AngularMotion__damping
        {
            get
            {
                return GetParam((SliderJoint.Param)18);
            }
            set
            {
                SetParam((SliderJoint.Param)18, value);
            }
        }

        /// <summary>
        /// A factor applied to the all rotation across axes orthogonal to the slider.
        /// </summary>
        public float AngularOrtho__softness
        {
            get
            {
                return GetParam((SliderJoint.Param)19);
            }
            set
            {
                SetParam((SliderJoint.Param)19, value);
            }
        }

        /// <summary>
        /// The amount of restitution of the rotation across axes orthogonal to the slider.
        /// </summary>
        public float AngularOrtho__restitution
        {
            get
            {
                return GetParam((SliderJoint.Param)20);
            }
            set
            {
                SetParam((SliderJoint.Param)20, value);
            }
        }

        /// <summary>
        /// The amount of damping of the rotation across axes orthogonal to the slider.
        /// </summary>
        public float AngularOrtho__damping
        {
            get
            {
                return GetParam((SliderJoint.Param)21);
            }
            set
            {
                SetParam((SliderJoint.Param)21, value);
            }
        }


        private const string nativeName = "SliderJoint";

        public SliderJoint() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_SliderJoint_Ctor(this);
        }

        internal SliderJoint(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_param");

        [GodotMethod("set_param")]
        public void SetParam(SliderJoint.Param param, float value)
        {
            NativeCalls.godot_icall_2_769(method_bind_0, Object.GetPtr(this), param, value);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_param");

        [GodotMethod("get_param")]
        public float GetParam(SliderJoint.Param param)
        {
            return NativeCalls.godot_icall_1_770(method_bind_1, Object.GetPtr(this), param);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_upper_limit_angular");

        internal void _SetUpperLimitAngular(float upperLimitAngular)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), upperLimitAngular);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_upper_limit_angular");

        internal float _GetUpperLimitAngular()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_lower_limit_angular");

        internal void _SetLowerLimitAngular(float lowerLimitAngular)
        {
            NativeCalls.godot_icall_1_10(method_bind_4, Object.GetPtr(this), lowerLimitAngular);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_lower_limit_angular");

        internal float _GetLowerLimitAngular()
        {
            return NativeCalls.godot_icall_0_9(method_bind_5, Object.GetPtr(this));
        }
    }
}
