using System;

namespace Godot
{
    public class SpatialMaterial : Material
    {
        public enum EmissionOperatorEnum
        {
            Add = 0,
            Multiply = 1
        }

        public enum DiffuseMode
        {
            Burley = 0,
            Lambert = 1,
            LambertWrap = 2,
            OrenNayar = 3,
            Toon = 4
        }

        public enum SpecularMode
        {
            SchlickGgx = 0,
            Blinn = 1,
            Phong = 2,
            Toon = 3,
            Disabled = 4
        }

        public enum Feature
        {
            Transparent = 0,
            Emission = 1,
            NormalMapping = 2,
            Rim = 3,
            Clearcoat = 4,
            Anisotropy = 5,
            AmbientOcclusion = 6,
            DepthMapping = 7,
            SubsuraceScattering = 8,
            Transmission = 9,
            Refraction = 10,
            Detail = 11,
            Max = 12
        }

        public enum Flags
        {
            Unshaded = 0,
            UseVertexLighting = 1,
            DisableDepthTest = 2,
            AlbedoFromVertexColor = 3,
            SrgbVertexColor = 4,
            UsePointSize = 5,
            FixedSize = 6,
            Uv1UseTriplanar = 7,
            Uv2UseTriplanar = 8,
            AoOnUv2 = 10,
            EmissionOnUv2 = 11,
            UseAlphaScissor = 12,
            TriplanarUseWorld = 9,
            AlbedoTextureForceSrgb = 13,
            Max = 14
        }

        public enum CullMode
        {
            Back = 0,
            Front = 1,
            Disabled = 2
        }

        public enum DetailUV
        {
            Uv1 = 0,
            Uv2 = 1
        }

        public enum BillboardMode
        {
            Disabled = 0,
            Enabled = 1,
            FixedY = 2,
            Particles = 3
        }

        public enum DepthDrawMode
        {
            OpaqueOnly = 0,
            Always = 1,
            Disabled = 2,
            AlphaOpaquePrepass = 3
        }

        public enum TextureChannel
        {
            Red = 0,
            Green = 1,
            Blue = 2,
            Alpha = 3,
            Grayscale = 4
        }

        public enum BlendMode
        {
            Mix = 0,
            Add = 1,
            Sub = 2,
            Mul = 3
        }

        public enum TextureParam
        {
            Albedo = 0,
            Metallic = 1,
            Roughness = 2,
            Emission = 3,
            Normal = 4,
            Rim = 5,
            Clearcoat = 6,
            Flowmap = 7,
            AmbientOcclusion = 8,
            Depth = 9,
            SubsurfaceScattering = 10,
            Transmission = 11,
            Refraction = 12,
            DetailMask = 13,
            DetailAlbedo = 14,
            DetailNormal = 15,
            Max = 16
        }


        /// <summary>
        /// </summary>
        public bool FlagsTransparent
        {
            get
            {
                return GetFeature((SpatialMaterial.Feature)0);
            }
            set
            {
                SetFeature((SpatialMaterial.Feature)0, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool FlagsUnshaded
        {
            get
            {
                return GetFlag((SpatialMaterial.Flags)0);
            }
            set
            {
                SetFlag((SpatialMaterial.Flags)0, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool FlagsVertexLighting
        {
            get
            {
                return GetFlag((SpatialMaterial.Flags)1);
            }
            set
            {
                SetFlag((SpatialMaterial.Flags)1, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool FlagsNoDepthTest
        {
            get
            {
                return GetFlag((SpatialMaterial.Flags)2);
            }
            set
            {
                SetFlag((SpatialMaterial.Flags)2, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool FlagsUsePointSize
        {
            get
            {
                return GetFlag((SpatialMaterial.Flags)5);
            }
            set
            {
                SetFlag((SpatialMaterial.Flags)5, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool FlagsWorldTriplanar
        {
            get
            {
                return GetFlag((SpatialMaterial.Flags)9);
            }
            set
            {
                SetFlag((SpatialMaterial.Flags)9, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool FlagsFixedSize
        {
            get
            {
                return GetFlag((SpatialMaterial.Flags)6);
            }
            set
            {
                SetFlag((SpatialMaterial.Flags)6, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool FlagsAlbedoTexForceSrgb
        {
            get
            {
                return GetFlag((SpatialMaterial.Flags)13);
            }
            set
            {
                SetFlag((SpatialMaterial.Flags)13, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool VertexColorUseAsAlbedo
        {
            get
            {
                return GetFlag((SpatialMaterial.Flags)3);
            }
            set
            {
                SetFlag((SpatialMaterial.Flags)3, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool VertexColorIsSrgb
        {
            get
            {
                return GetFlag((SpatialMaterial.Flags)4);
            }
            set
            {
                SetFlag((SpatialMaterial.Flags)4, value);
            }
        }

        /// <summary>
        /// </summary>
        public SpatialMaterial.DiffuseMode ParamsDiffuseMode
        {
            get
            {
                return GetDiffuseMode();
            }
            set
            {
                SetDiffuseMode(value);
            }
        }

        /// <summary>
        /// </summary>
        public SpatialMaterial.SpecularMode ParamsSpecularMode
        {
            get
            {
                return GetSpecularMode();
            }
            set
            {
                SetSpecularMode(value);
            }
        }

        /// <summary>
        /// </summary>
        public SpatialMaterial.BlendMode ParamsBlendMode
        {
            get
            {
                return GetBlendMode();
            }
            set
            {
                SetBlendMode(value);
            }
        }

        /// <summary>
        /// </summary>
        public SpatialMaterial.CullMode ParamsCullMode
        {
            get
            {
                return GetCullMode();
            }
            set
            {
                SetCullMode(value);
            }
        }

        /// <summary>
        /// </summary>
        public SpatialMaterial.DepthDrawMode ParamsDepthDrawMode
        {
            get
            {
                return GetDepthDrawMode();
            }
            set
            {
                SetDepthDrawMode(value);
            }
        }

        /// <summary>
        /// </summary>
        public float ParamsLineWidth
        {
            get
            {
                return GetLineWidth();
            }
            set
            {
                SetLineWidth(value);
            }
        }

        /// <summary>
        /// </summary>
        public float ParamsPointSize
        {
            get
            {
                return GetPointSize();
            }
            set
            {
                SetPointSize(value);
            }
        }

        /// <summary>
        /// </summary>
        public SpatialMaterial.BillboardMode ParamsBillboardMode
        {
            get
            {
                return GetBillboardMode();
            }
            set
            {
                SetBillboardMode(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool ParamsGrow
        {
            get
            {
                return IsGrowEnabled();
            }
            set
            {
                SetGrowEnabled(value);
            }
        }

        /// <summary>
        /// </summary>
        public float ParamsGrowAmount
        {
            get
            {
                return GetGrow();
            }
            set
            {
                SetGrow(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool ParamsUseAlphaScissor
        {
            get
            {
                return GetFlag((SpatialMaterial.Flags)12);
            }
            set
            {
                SetFlag((SpatialMaterial.Flags)12, value);
            }
        }

        /// <summary>
        /// </summary>
        public float ParamsAlphaScissorThreshold
        {
            get
            {
                return GetAlphaScissorThreshold();
            }
            set
            {
                SetAlphaScissorThreshold(value);
            }
        }

        /// <summary>
        /// </summary>
        public int ParticlesAnimHFrames
        {
            get
            {
                return GetParticlesAnimHFrames();
            }
            set
            {
                SetParticlesAnimHFrames(value);
            }
        }

        /// <summary>
        /// </summary>
        public int ParticlesAnimVFrames
        {
            get
            {
                return GetParticlesAnimVFrames();
            }
            set
            {
                SetParticlesAnimVFrames(value);
            }
        }

        /// <summary>
        /// </summary>
        public int ParticlesAnimLoop
        {
            get
            {
                return GetParticlesAnimLoop();
            }
            set
            {
                SetParticlesAnimLoop(value);
            }
        }

        /// <summary>
        /// </summary>
        public Color AlbedoColor
        {
            get
            {
                return GetAlbedo();
            }
            set
            {
                SetAlbedo(value);
            }
        }

        /// <summary>
        /// </summary>
        public Texture AlbedoTexture
        {
            get
            {
                return GetTexture((SpatialMaterial.TextureParam)0);
            }
            set
            {
                SetTexture((SpatialMaterial.TextureParam)0, value);
            }
        }

        /// <summary>
        /// </summary>
        public float Metallic
        {
            get
            {
                return GetMetallic();
            }
            set
            {
                SetMetallic(value);
            }
        }

        /// <summary>
        /// </summary>
        public float MetallicSpecular
        {
            get
            {
                return GetSpecular();
            }
            set
            {
                SetSpecular(value);
            }
        }

        /// <summary>
        /// </summary>
        public Texture MetallicTexture
        {
            get
            {
                return GetTexture((SpatialMaterial.TextureParam)1);
            }
            set
            {
                SetTexture((SpatialMaterial.TextureParam)1, value);
            }
        }

        /// <summary>
        /// </summary>
        public SpatialMaterial.TextureChannel MetallicTextureChannel
        {
            get
            {
                return GetMetallicTextureChannel();
            }
            set
            {
                SetMetallicTextureChannel(value);
            }
        }

        /// <summary>
        /// </summary>
        public float Roughness
        {
            get
            {
                return GetRoughness();
            }
            set
            {
                SetRoughness(value);
            }
        }

        /// <summary>
        /// </summary>
        public Texture RoughnessTexture
        {
            get
            {
                return GetTexture((SpatialMaterial.TextureParam)2);
            }
            set
            {
                SetTexture((SpatialMaterial.TextureParam)2, value);
            }
        }

        /// <summary>
        /// </summary>
        public SpatialMaterial.TextureChannel RoughnessTextureChannel
        {
            get
            {
                return GetRoughnessTextureChannel();
            }
            set
            {
                SetRoughnessTextureChannel(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool EmissionEnabled
        {
            get
            {
                return GetFeature((SpatialMaterial.Feature)1);
            }
            set
            {
                SetFeature((SpatialMaterial.Feature)1, value);
            }
        }

        /// <summary>
        /// </summary>
        public Color Emission
        {
            get
            {
                return GetEmission();
            }
            set
            {
                SetEmission(value);
            }
        }

        /// <summary>
        /// </summary>
        public float EmissionEnergy
        {
            get
            {
                return GetEmissionEnergy();
            }
            set
            {
                SetEmissionEnergy(value);
            }
        }

        /// <summary>
        /// </summary>
        public SpatialMaterial.EmissionOperatorEnum EmissionOperator
        {
            get
            {
                return GetEmissionOperator();
            }
            set
            {
                SetEmissionOperator(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool EmissionOnUv2
        {
            get
            {
                return GetFlag((SpatialMaterial.Flags)11);
            }
            set
            {
                SetFlag((SpatialMaterial.Flags)11, value);
            }
        }

        /// <summary>
        /// </summary>
        public Texture EmissionTexture
        {
            get
            {
                return GetTexture((SpatialMaterial.TextureParam)3);
            }
            set
            {
                SetTexture((SpatialMaterial.TextureParam)3, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool NormalEnabled
        {
            get
            {
                return GetFeature((SpatialMaterial.Feature)2);
            }
            set
            {
                SetFeature((SpatialMaterial.Feature)2, value);
            }
        }

        /// <summary>
        /// </summary>
        public float NormalScale
        {
            get
            {
                return GetNormalScale();
            }
            set
            {
                SetNormalScale(value);
            }
        }

        /// <summary>
        /// </summary>
        public Texture NormalTexture
        {
            get
            {
                return GetTexture((SpatialMaterial.TextureParam)4);
            }
            set
            {
                SetTexture((SpatialMaterial.TextureParam)4, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool RimEnabled
        {
            get
            {
                return GetFeature((SpatialMaterial.Feature)3);
            }
            set
            {
                SetFeature((SpatialMaterial.Feature)3, value);
            }
        }

        /// <summary>
        /// </summary>
        public float Rim
        {
            get
            {
                return GetRim();
            }
            set
            {
                SetRim(value);
            }
        }

        /// <summary>
        /// </summary>
        public float RimTint
        {
            get
            {
                return GetRimTint();
            }
            set
            {
                SetRimTint(value);
            }
        }

        /// <summary>
        /// </summary>
        public Texture RimTexture
        {
            get
            {
                return GetTexture((SpatialMaterial.TextureParam)5);
            }
            set
            {
                SetTexture((SpatialMaterial.TextureParam)5, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool ClearcoatEnabled
        {
            get
            {
                return GetFeature((SpatialMaterial.Feature)4);
            }
            set
            {
                SetFeature((SpatialMaterial.Feature)4, value);
            }
        }

        /// <summary>
        /// </summary>
        public float Clearcoat
        {
            get
            {
                return GetClearcoat();
            }
            set
            {
                SetClearcoat(value);
            }
        }

        /// <summary>
        /// </summary>
        public float ClearcoatGloss
        {
            get
            {
                return GetClearcoatGloss();
            }
            set
            {
                SetClearcoatGloss(value);
            }
        }

        /// <summary>
        /// </summary>
        public Texture ClearcoatTexture
        {
            get
            {
                return GetTexture((SpatialMaterial.TextureParam)6);
            }
            set
            {
                SetTexture((SpatialMaterial.TextureParam)6, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool AnisotropyEnabled
        {
            get
            {
                return GetFeature((SpatialMaterial.Feature)5);
            }
            set
            {
                SetFeature((SpatialMaterial.Feature)5, value);
            }
        }

        /// <summary>
        /// </summary>
        public float Anisotropy
        {
            get
            {
                return GetAnisotropy();
            }
            set
            {
                SetAnisotropy(value);
            }
        }

        /// <summary>
        /// </summary>
        public Texture AnisotropyFlowmap
        {
            get
            {
                return GetTexture((SpatialMaterial.TextureParam)7);
            }
            set
            {
                SetTexture((SpatialMaterial.TextureParam)7, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool AoEnabled
        {
            get
            {
                return GetFeature((SpatialMaterial.Feature)6);
            }
            set
            {
                SetFeature((SpatialMaterial.Feature)6, value);
            }
        }

        /// <summary>
        /// </summary>
        public float AoLightAffect
        {
            get
            {
                return GetAoLightAffect();
            }
            set
            {
                SetAoLightAffect(value);
            }
        }

        /// <summary>
        /// </summary>
        public Texture AoTexture
        {
            get
            {
                return GetTexture((SpatialMaterial.TextureParam)8);
            }
            set
            {
                SetTexture((SpatialMaterial.TextureParam)8, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool AoOnUv2
        {
            get
            {
                return GetFlag((SpatialMaterial.Flags)10);
            }
            set
            {
                SetFlag((SpatialMaterial.Flags)10, value);
            }
        }

        /// <summary>
        /// </summary>
        public SpatialMaterial.TextureChannel AoTextureChannel
        {
            get
            {
                return GetAoTextureChannel();
            }
            set
            {
                SetAoTextureChannel(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool DepthEnabled
        {
            get
            {
                return GetFeature((SpatialMaterial.Feature)7);
            }
            set
            {
                SetFeature((SpatialMaterial.Feature)7, value);
            }
        }

        /// <summary>
        /// </summary>
        public float DepthScale
        {
            get
            {
                return GetDepthScale();
            }
            set
            {
                SetDepthScale(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool DepthDeepParallax
        {
            get
            {
                return IsDepthDeepParallaxEnabled();
            }
            set
            {
                SetDepthDeepParallax(value);
            }
        }

        /// <summary>
        /// </summary>
        public int DepthMinLayers
        {
            get
            {
                return GetDepthDeepParallaxMinLayers();
            }
            set
            {
                SetDepthDeepParallaxMinLayers(value);
            }
        }

        /// <summary>
        /// </summary>
        public int DepthMaxLayers
        {
            get
            {
                return GetDepthDeepParallaxMaxLayers();
            }
            set
            {
                SetDepthDeepParallaxMaxLayers(value);
            }
        }

        /// <summary>
        /// </summary>
        public Texture DepthTexture
        {
            get
            {
                return GetTexture((SpatialMaterial.TextureParam)9);
            }
            set
            {
                SetTexture((SpatialMaterial.TextureParam)9, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool SubsurfScatterEnabled
        {
            get
            {
                return GetFeature((SpatialMaterial.Feature)8);
            }
            set
            {
                SetFeature((SpatialMaterial.Feature)8, value);
            }
        }

        /// <summary>
        /// </summary>
        public float SubsurfScatterStrength
        {
            get
            {
                return GetSubsurfaceScatteringStrength();
            }
            set
            {
                SetSubsurfaceScatteringStrength(value);
            }
        }

        /// <summary>
        /// </summary>
        public Texture SubsurfScatterTexture
        {
            get
            {
                return GetTexture((SpatialMaterial.TextureParam)10);
            }
            set
            {
                SetTexture((SpatialMaterial.TextureParam)10, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool TransmissionEnabled
        {
            get
            {
                return GetFeature((SpatialMaterial.Feature)9);
            }
            set
            {
                SetFeature((SpatialMaterial.Feature)9, value);
            }
        }

        /// <summary>
        /// </summary>
        public Color Transmission
        {
            get
            {
                return GetTransmission();
            }
            set
            {
                SetTransmission(value);
            }
        }

        /// <summary>
        /// </summary>
        public Texture TransmissionTexture
        {
            get
            {
                return GetTexture((SpatialMaterial.TextureParam)11);
            }
            set
            {
                SetTexture((SpatialMaterial.TextureParam)11, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool RefractionEnabled
        {
            get
            {
                return GetFeature((SpatialMaterial.Feature)10);
            }
            set
            {
                SetFeature((SpatialMaterial.Feature)10, value);
            }
        }

        /// <summary>
        /// </summary>
        public float RefractionScale
        {
            get
            {
                return GetRefraction();
            }
            set
            {
                SetRefraction(value);
            }
        }

        /// <summary>
        /// </summary>
        public Texture RefractionTexture
        {
            get
            {
                return GetTexture((SpatialMaterial.TextureParam)12);
            }
            set
            {
                SetTexture((SpatialMaterial.TextureParam)12, value);
            }
        }

        /// <summary>
        /// </summary>
        public SpatialMaterial.TextureChannel RefractionTextureChannel
        {
            get
            {
                return GetRefractionTextureChannel();
            }
            set
            {
                SetRefractionTextureChannel(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool DetailEnabled
        {
            get
            {
                return GetFeature((SpatialMaterial.Feature)11);
            }
            set
            {
                SetFeature((SpatialMaterial.Feature)11, value);
            }
        }

        /// <summary>
        /// </summary>
        public Texture DetailMask
        {
            get
            {
                return GetTexture((SpatialMaterial.TextureParam)13);
            }
            set
            {
                SetTexture((SpatialMaterial.TextureParam)13, value);
            }
        }

        /// <summary>
        /// </summary>
        public SpatialMaterial.BlendMode DetailBlendMode
        {
            get
            {
                return GetDetailBlendMode();
            }
            set
            {
                SetDetailBlendMode(value);
            }
        }

        /// <summary>
        /// </summary>
        public SpatialMaterial.DetailUV DetailUvLayer
        {
            get
            {
                return GetDetailUv();
            }
            set
            {
                SetDetailUv(value);
            }
        }

        /// <summary>
        /// </summary>
        public Texture DetailAlbedo
        {
            get
            {
                return GetTexture((SpatialMaterial.TextureParam)14);
            }
            set
            {
                SetTexture((SpatialMaterial.TextureParam)14, value);
            }
        }

        /// <summary>
        /// </summary>
        public Texture DetailNormal
        {
            get
            {
                return GetTexture((SpatialMaterial.TextureParam)15);
            }
            set
            {
                SetTexture((SpatialMaterial.TextureParam)15, value);
            }
        }

        /// <summary>
        /// </summary>
        public Vector3 Uv1Scale
        {
            get
            {
                return GetUv1Scale();
            }
            set
            {
                SetUv1Scale(value);
            }
        }

        /// <summary>
        /// </summary>
        public Vector3 Uv1Offset
        {
            get
            {
                return GetUv1Offset();
            }
            set
            {
                SetUv1Offset(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool Uv1Triplanar
        {
            get
            {
                return GetFlag((SpatialMaterial.Flags)7);
            }
            set
            {
                SetFlag((SpatialMaterial.Flags)7, value);
            }
        }

        /// <summary>
        /// </summary>
        public float Uv1TriplanarSharpness
        {
            get
            {
                return GetUv1TriplanarBlendSharpness();
            }
            set
            {
                SetUv1TriplanarBlendSharpness(value);
            }
        }

        /// <summary>
        /// </summary>
        public Vector3 Uv2Scale
        {
            get
            {
                return GetUv2Scale();
            }
            set
            {
                SetUv2Scale(value);
            }
        }

        /// <summary>
        /// </summary>
        public Vector3 Uv2Offset
        {
            get
            {
                return GetUv2Offset();
            }
            set
            {
                SetUv2Offset(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool Uv2Triplanar
        {
            get
            {
                return GetFlag((SpatialMaterial.Flags)8);
            }
            set
            {
                SetFlag((SpatialMaterial.Flags)8, value);
            }
        }

        /// <summary>
        /// </summary>
        public float Uv2TriplanarSharpness
        {
            get
            {
                return GetUv2TriplanarBlendSharpness();
            }
            set
            {
                SetUv2TriplanarBlendSharpness(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool ProximityFadeEnable
        {
            get
            {
                return IsProximityFadeEnabled();
            }
            set
            {
                SetProximityFade(value);
            }
        }

        /// <summary>
        /// </summary>
        public float ProximityFadeDistance
        {
            get
            {
                return GetProximityFadeDistance();
            }
            set
            {
                SetProximityFadeDistance(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool DistanceFadeEnable
        {
            get
            {
                return IsDistanceFadeEnabled();
            }
            set
            {
                SetDistanceFade(value);
            }
        }

        /// <summary>
        /// </summary>
        public float DistanceFadeMinDistance
        {
            get
            {
                return GetDistanceFadeMinDistance();
            }
            set
            {
                SetDistanceFadeMinDistance(value);
            }
        }

        /// <summary>
        /// </summary>
        public float DistanceFadeMaxDistance
        {
            get
            {
                return GetDistanceFadeMaxDistance();
            }
            set
            {
                SetDistanceFadeMaxDistance(value);
            }
        }


        private const string nativeName = "SpatialMaterial";

        public SpatialMaterial() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_SpatialMaterial_Ctor(this);
        }

        internal SpatialMaterial(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_albedo");

        [GodotMethod("set_albedo")]
        public void SetAlbedo(Color albedo)
        {
            NativeCalls.godot_icall_1_202(method_bind_0, Object.GetPtr(this), ref albedo);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_albedo");

        [GodotMethod("get_albedo")]
        public Color GetAlbedo()
        {
            object ret = NativeCalls.godot_icall_0_203(method_bind_1, Object.GetPtr(this));
            return (Color)ret;
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_specular");

        [GodotMethod("set_specular")]
        public void SetSpecular(float specular)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), specular);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_specular");

        [GodotMethod("get_specular")]
        public float GetSpecular()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_metallic");

        [GodotMethod("set_metallic")]
        public void SetMetallic(float metallic)
        {
            NativeCalls.godot_icall_1_10(method_bind_4, Object.GetPtr(this), metallic);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_metallic");

        [GodotMethod("get_metallic")]
        public float GetMetallic()
        {
            return NativeCalls.godot_icall_0_9(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_roughness");

        [GodotMethod("set_roughness")]
        public void SetRoughness(float roughness)
        {
            NativeCalls.godot_icall_1_10(method_bind_6, Object.GetPtr(this), roughness);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_roughness");

        [GodotMethod("get_roughness")]
        public float GetRoughness()
        {
            return NativeCalls.godot_icall_0_9(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_emission");

        [GodotMethod("set_emission")]
        public void SetEmission(Color emission)
        {
            NativeCalls.godot_icall_1_202(method_bind_8, Object.GetPtr(this), ref emission);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_emission");

        [GodotMethod("get_emission")]
        public Color GetEmission()
        {
            object ret = NativeCalls.godot_icall_0_203(method_bind_9, Object.GetPtr(this));
            return (Color)ret;
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_emission_energy");

        [GodotMethod("set_emission_energy")]
        public void SetEmissionEnergy(float emissionEnergy)
        {
            NativeCalls.godot_icall_1_10(method_bind_10, Object.GetPtr(this), emissionEnergy);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_emission_energy");

        [GodotMethod("get_emission_energy")]
        public float GetEmissionEnergy()
        {
            return NativeCalls.godot_icall_0_9(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_normal_scale");

        [GodotMethod("set_normal_scale")]
        public void SetNormalScale(float normalScale)
        {
            NativeCalls.godot_icall_1_10(method_bind_12, Object.GetPtr(this), normalScale);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_normal_scale");

        [GodotMethod("get_normal_scale")]
        public float GetNormalScale()
        {
            return NativeCalls.godot_icall_0_9(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_rim");

        [GodotMethod("set_rim")]
        public void SetRim(float rim)
        {
            NativeCalls.godot_icall_1_10(method_bind_14, Object.GetPtr(this), rim);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_rim");

        [GodotMethod("get_rim")]
        public float GetRim()
        {
            return NativeCalls.godot_icall_0_9(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_rim_tint");

        [GodotMethod("set_rim_tint")]
        public void SetRimTint(float rimTint)
        {
            NativeCalls.godot_icall_1_10(method_bind_16, Object.GetPtr(this), rimTint);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_rim_tint");

        [GodotMethod("get_rim_tint")]
        public float GetRimTint()
        {
            return NativeCalls.godot_icall_0_9(method_bind_17, Object.GetPtr(this));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_clearcoat");

        [GodotMethod("set_clearcoat")]
        public void SetClearcoat(float clearcoat)
        {
            NativeCalls.godot_icall_1_10(method_bind_18, Object.GetPtr(this), clearcoat);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_clearcoat");

        [GodotMethod("get_clearcoat")]
        public float GetClearcoat()
        {
            return NativeCalls.godot_icall_0_9(method_bind_19, Object.GetPtr(this));
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_clearcoat_gloss");

        [GodotMethod("set_clearcoat_gloss")]
        public void SetClearcoatGloss(float clearcoatGloss)
        {
            NativeCalls.godot_icall_1_10(method_bind_20, Object.GetPtr(this), clearcoatGloss);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_clearcoat_gloss");

        [GodotMethod("get_clearcoat_gloss")]
        public float GetClearcoatGloss()
        {
            return NativeCalls.godot_icall_0_9(method_bind_21, Object.GetPtr(this));
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_anisotropy");

        [GodotMethod("set_anisotropy")]
        public void SetAnisotropy(float anisotropy)
        {
            NativeCalls.godot_icall_1_10(method_bind_22, Object.GetPtr(this), anisotropy);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_anisotropy");

        [GodotMethod("get_anisotropy")]
        public float GetAnisotropy()
        {
            return NativeCalls.godot_icall_0_9(method_bind_23, Object.GetPtr(this));
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_depth_scale");

        [GodotMethod("set_depth_scale")]
        public void SetDepthScale(float depthScale)
        {
            NativeCalls.godot_icall_1_10(method_bind_24, Object.GetPtr(this), depthScale);
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_depth_scale");

        [GodotMethod("get_depth_scale")]
        public float GetDepthScale()
        {
            return NativeCalls.godot_icall_0_9(method_bind_25, Object.GetPtr(this));
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_subsurface_scattering_strength");

        [GodotMethod("set_subsurface_scattering_strength")]
        public void SetSubsurfaceScatteringStrength(float strength)
        {
            NativeCalls.godot_icall_1_10(method_bind_26, Object.GetPtr(this), strength);
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_subsurface_scattering_strength");

        [GodotMethod("get_subsurface_scattering_strength")]
        public float GetSubsurfaceScatteringStrength()
        {
            return NativeCalls.godot_icall_0_9(method_bind_27, Object.GetPtr(this));
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_transmission");

        [GodotMethod("set_transmission")]
        public void SetTransmission(Color transmission)
        {
            NativeCalls.godot_icall_1_202(method_bind_28, Object.GetPtr(this), ref transmission);
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_transmission");

        [GodotMethod("get_transmission")]
        public Color GetTransmission()
        {
            object ret = NativeCalls.godot_icall_0_203(method_bind_29, Object.GetPtr(this));
            return (Color)ret;
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_refraction");

        [GodotMethod("set_refraction")]
        public void SetRefraction(float refraction)
        {
            NativeCalls.godot_icall_1_10(method_bind_30, Object.GetPtr(this), refraction);
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_refraction");

        [GodotMethod("get_refraction")]
        public float GetRefraction()
        {
            return NativeCalls.godot_icall_0_9(method_bind_31, Object.GetPtr(this));
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_line_width");

        [GodotMethod("set_line_width")]
        public void SetLineWidth(float lineWidth)
        {
            NativeCalls.godot_icall_1_10(method_bind_32, Object.GetPtr(this), lineWidth);
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_line_width");

        [GodotMethod("get_line_width")]
        public float GetLineWidth()
        {
            return NativeCalls.godot_icall_0_9(method_bind_33, Object.GetPtr(this));
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_point_size");

        [GodotMethod("set_point_size")]
        public void SetPointSize(float pointSize)
        {
            NativeCalls.godot_icall_1_10(method_bind_34, Object.GetPtr(this), pointSize);
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_point_size");

        [GodotMethod("get_point_size")]
        public float GetPointSize()
        {
            return NativeCalls.godot_icall_0_9(method_bind_35, Object.GetPtr(this));
        }

        private IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_detail_uv");

        [GodotMethod("set_detail_uv")]
        public void SetDetailUv(SpatialMaterial.DetailUV detailUv)
        {
            NativeCalls.godot_icall_1_776(method_bind_36, Object.GetPtr(this), detailUv);
        }

        private IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_detail_uv");

        [GodotMethod("get_detail_uv")]
        public SpatialMaterial.DetailUV GetDetailUv()
        {
            return NativeCalls.godot_icall_0_777(method_bind_37, Object.GetPtr(this));
        }

        private IntPtr method_bind_38 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_blend_mode");

        [GodotMethod("set_blend_mode")]
        public void SetBlendMode(SpatialMaterial.BlendMode blendMode)
        {
            NativeCalls.godot_icall_1_778(method_bind_38, Object.GetPtr(this), blendMode);
        }

        private IntPtr method_bind_39 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_blend_mode");

        [GodotMethod("get_blend_mode")]
        public SpatialMaterial.BlendMode GetBlendMode()
        {
            return NativeCalls.godot_icall_0_779(method_bind_39, Object.GetPtr(this));
        }

        private IntPtr method_bind_40 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_depth_draw_mode");

        [GodotMethod("set_depth_draw_mode")]
        public void SetDepthDrawMode(SpatialMaterial.DepthDrawMode depthDrawMode)
        {
            NativeCalls.godot_icall_1_780(method_bind_40, Object.GetPtr(this), depthDrawMode);
        }

        private IntPtr method_bind_41 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_depth_draw_mode");

        [GodotMethod("get_depth_draw_mode")]
        public SpatialMaterial.DepthDrawMode GetDepthDrawMode()
        {
            return NativeCalls.godot_icall_0_781(method_bind_41, Object.GetPtr(this));
        }

        private IntPtr method_bind_42 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_cull_mode");

        [GodotMethod("set_cull_mode")]
        public void SetCullMode(SpatialMaterial.CullMode cullMode)
        {
            NativeCalls.godot_icall_1_782(method_bind_42, Object.GetPtr(this), cullMode);
        }

        private IntPtr method_bind_43 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_cull_mode");

        [GodotMethod("get_cull_mode")]
        public SpatialMaterial.CullMode GetCullMode()
        {
            return NativeCalls.godot_icall_0_783(method_bind_43, Object.GetPtr(this));
        }

        private IntPtr method_bind_44 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_diffuse_mode");

        [GodotMethod("set_diffuse_mode")]
        public void SetDiffuseMode(SpatialMaterial.DiffuseMode diffuseMode)
        {
            NativeCalls.godot_icall_1_784(method_bind_44, Object.GetPtr(this), diffuseMode);
        }

        private IntPtr method_bind_45 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_diffuse_mode");

        [GodotMethod("get_diffuse_mode")]
        public SpatialMaterial.DiffuseMode GetDiffuseMode()
        {
            return NativeCalls.godot_icall_0_785(method_bind_45, Object.GetPtr(this));
        }

        private IntPtr method_bind_46 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_specular_mode");

        [GodotMethod("set_specular_mode")]
        public void SetSpecularMode(SpatialMaterial.SpecularMode specularMode)
        {
            NativeCalls.godot_icall_1_786(method_bind_46, Object.GetPtr(this), specularMode);
        }

        private IntPtr method_bind_47 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_specular_mode");

        [GodotMethod("get_specular_mode")]
        public SpatialMaterial.SpecularMode GetSpecularMode()
        {
            return NativeCalls.godot_icall_0_787(method_bind_47, Object.GetPtr(this));
        }

        private IntPtr method_bind_48 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_flag");

        [GodotMethod("set_flag")]
        public void SetFlag(SpatialMaterial.Flags flag, bool enable)
        {
            NativeCalls.godot_icall_2_788(method_bind_48, Object.GetPtr(this), flag, enable);
        }

        private IntPtr method_bind_49 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_flag");

        [GodotMethod("get_flag")]
        public bool GetFlag(SpatialMaterial.Flags flag)
        {
            return NativeCalls.godot_icall_1_789(method_bind_49, Object.GetPtr(this), flag);
        }

        private IntPtr method_bind_50 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_feature");

        [GodotMethod("set_feature")]
        public void SetFeature(SpatialMaterial.Feature feature, bool enable)
        {
            NativeCalls.godot_icall_2_790(method_bind_50, Object.GetPtr(this), feature, enable);
        }

        private IntPtr method_bind_51 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_feature");

        [GodotMethod("get_feature")]
        public bool GetFeature(SpatialMaterial.Feature feature)
        {
            return NativeCalls.godot_icall_1_791(method_bind_51, Object.GetPtr(this), feature);
        }

        private IntPtr method_bind_52 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_texture");

        [GodotMethod("set_texture")]
        public void SetTexture(SpatialMaterial.TextureParam param, Texture texture)
        {
            NativeCalls.godot_icall_2_792(method_bind_52, Object.GetPtr(this), param, Object.GetPtr(texture));
        }

        private IntPtr method_bind_53 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_texture");

        [GodotMethod("get_texture")]
        public Texture GetTexture(SpatialMaterial.TextureParam param)
        {
            return NativeCalls.godot_icall_1_793(method_bind_53, Object.GetPtr(this), param);
        }

        private IntPtr method_bind_54 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_detail_blend_mode");

        [GodotMethod("set_detail_blend_mode")]
        public void SetDetailBlendMode(SpatialMaterial.BlendMode detailBlendMode)
        {
            NativeCalls.godot_icall_1_778(method_bind_54, Object.GetPtr(this), detailBlendMode);
        }

        private IntPtr method_bind_55 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_detail_blend_mode");

        [GodotMethod("get_detail_blend_mode")]
        public SpatialMaterial.BlendMode GetDetailBlendMode()
        {
            return NativeCalls.godot_icall_0_779(method_bind_55, Object.GetPtr(this));
        }

        private IntPtr method_bind_56 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_uv1_scale");

        [GodotMethod("set_uv1_scale")]
        public void SetUv1Scale(Vector3 scale)
        {
            NativeCalls.godot_icall_1_104(method_bind_56, Object.GetPtr(this), ref scale);
        }

        private IntPtr method_bind_57 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_uv1_scale");

        [GodotMethod("get_uv1_scale")]
        public Vector3 GetUv1Scale()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_57, Object.GetPtr(this));
            return (Vector3)ret;
        }

        private IntPtr method_bind_58 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_uv1_offset");

        [GodotMethod("set_uv1_offset")]
        public void SetUv1Offset(Vector3 offset)
        {
            NativeCalls.godot_icall_1_104(method_bind_58, Object.GetPtr(this), ref offset);
        }

        private IntPtr method_bind_59 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_uv1_offset");

        [GodotMethod("get_uv1_offset")]
        public Vector3 GetUv1Offset()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_59, Object.GetPtr(this));
            return (Vector3)ret;
        }

        private IntPtr method_bind_60 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_uv1_triplanar_blend_sharpness");

        [GodotMethod("set_uv1_triplanar_blend_sharpness")]
        public void SetUv1TriplanarBlendSharpness(float sharpness)
        {
            NativeCalls.godot_icall_1_10(method_bind_60, Object.GetPtr(this), sharpness);
        }

        private IntPtr method_bind_61 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_uv1_triplanar_blend_sharpness");

        [GodotMethod("get_uv1_triplanar_blend_sharpness")]
        public float GetUv1TriplanarBlendSharpness()
        {
            return NativeCalls.godot_icall_0_9(method_bind_61, Object.GetPtr(this));
        }

        private IntPtr method_bind_62 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_uv2_scale");

        [GodotMethod("set_uv2_scale")]
        public void SetUv2Scale(Vector3 scale)
        {
            NativeCalls.godot_icall_1_104(method_bind_62, Object.GetPtr(this), ref scale);
        }

        private IntPtr method_bind_63 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_uv2_scale");

        [GodotMethod("get_uv2_scale")]
        public Vector3 GetUv2Scale()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_63, Object.GetPtr(this));
            return (Vector3)ret;
        }

        private IntPtr method_bind_64 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_uv2_offset");

        [GodotMethod("set_uv2_offset")]
        public void SetUv2Offset(Vector3 offset)
        {
            NativeCalls.godot_icall_1_104(method_bind_64, Object.GetPtr(this), ref offset);
        }

        private IntPtr method_bind_65 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_uv2_offset");

        [GodotMethod("get_uv2_offset")]
        public Vector3 GetUv2Offset()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_65, Object.GetPtr(this));
            return (Vector3)ret;
        }

        private IntPtr method_bind_66 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_uv2_triplanar_blend_sharpness");

        [GodotMethod("set_uv2_triplanar_blend_sharpness")]
        public void SetUv2TriplanarBlendSharpness(float sharpness)
        {
            NativeCalls.godot_icall_1_10(method_bind_66, Object.GetPtr(this), sharpness);
        }

        private IntPtr method_bind_67 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_uv2_triplanar_blend_sharpness");

        [GodotMethod("get_uv2_triplanar_blend_sharpness")]
        public float GetUv2TriplanarBlendSharpness()
        {
            return NativeCalls.godot_icall_0_9(method_bind_67, Object.GetPtr(this));
        }

        private IntPtr method_bind_68 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_billboard_mode");

        [GodotMethod("set_billboard_mode")]
        public void SetBillboardMode(SpatialMaterial.BillboardMode mode)
        {
            NativeCalls.godot_icall_1_794(method_bind_68, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_69 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_billboard_mode");

        [GodotMethod("get_billboard_mode")]
        public SpatialMaterial.BillboardMode GetBillboardMode()
        {
            return NativeCalls.godot_icall_0_795(method_bind_69, Object.GetPtr(this));
        }

        private IntPtr method_bind_70 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_particles_anim_h_frames");

        [GodotMethod("set_particles_anim_h_frames")]
        public void SetParticlesAnimHFrames(int frames)
        {
            NativeCalls.godot_icall_1_0(method_bind_70, Object.GetPtr(this), frames);
        }

        private IntPtr method_bind_71 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_particles_anim_h_frames");

        [GodotMethod("get_particles_anim_h_frames")]
        public int GetParticlesAnimHFrames()
        {
            return NativeCalls.godot_icall_0_1(method_bind_71, Object.GetPtr(this));
        }

        private IntPtr method_bind_72 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_particles_anim_v_frames");

        [GodotMethod("set_particles_anim_v_frames")]
        public void SetParticlesAnimVFrames(int frames)
        {
            NativeCalls.godot_icall_1_0(method_bind_72, Object.GetPtr(this), frames);
        }

        private IntPtr method_bind_73 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_particles_anim_v_frames");

        [GodotMethod("get_particles_anim_v_frames")]
        public int GetParticlesAnimVFrames()
        {
            return NativeCalls.godot_icall_0_1(method_bind_73, Object.GetPtr(this));
        }

        private IntPtr method_bind_74 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_particles_anim_loop");

        [GodotMethod("set_particles_anim_loop")]
        public void SetParticlesAnimLoop(int frames)
        {
            NativeCalls.godot_icall_1_0(method_bind_74, Object.GetPtr(this), frames);
        }

        private IntPtr method_bind_75 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_particles_anim_loop");

        [GodotMethod("get_particles_anim_loop")]
        public int GetParticlesAnimLoop()
        {
            return NativeCalls.godot_icall_0_1(method_bind_75, Object.GetPtr(this));
        }

        private IntPtr method_bind_76 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_depth_deep_parallax");

        [GodotMethod("set_depth_deep_parallax")]
        public void SetDepthDeepParallax(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_76, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_77 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_depth_deep_parallax_enabled");

        [GodotMethod("is_depth_deep_parallax_enabled")]
        public bool IsDepthDeepParallaxEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_77, Object.GetPtr(this));
        }

        private IntPtr method_bind_78 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_depth_deep_parallax_min_layers");

        [GodotMethod("set_depth_deep_parallax_min_layers")]
        public void SetDepthDeepParallaxMinLayers(int layer)
        {
            NativeCalls.godot_icall_1_0(method_bind_78, Object.GetPtr(this), layer);
        }

        private IntPtr method_bind_79 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_depth_deep_parallax_min_layers");

        [GodotMethod("get_depth_deep_parallax_min_layers")]
        public int GetDepthDeepParallaxMinLayers()
        {
            return NativeCalls.godot_icall_0_1(method_bind_79, Object.GetPtr(this));
        }

        private IntPtr method_bind_80 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_depth_deep_parallax_max_layers");

        [GodotMethod("set_depth_deep_parallax_max_layers")]
        public void SetDepthDeepParallaxMaxLayers(int layer)
        {
            NativeCalls.godot_icall_1_0(method_bind_80, Object.GetPtr(this), layer);
        }

        private IntPtr method_bind_81 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_depth_deep_parallax_max_layers");

        [GodotMethod("get_depth_deep_parallax_max_layers")]
        public int GetDepthDeepParallaxMaxLayers()
        {
            return NativeCalls.godot_icall_0_1(method_bind_81, Object.GetPtr(this));
        }

        private IntPtr method_bind_82 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_grow");

        [GodotMethod("set_grow")]
        public void SetGrow(float amount)
        {
            NativeCalls.godot_icall_1_10(method_bind_82, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_83 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_grow");

        [GodotMethod("get_grow")]
        public float GetGrow()
        {
            return NativeCalls.godot_icall_0_9(method_bind_83, Object.GetPtr(this));
        }

        private IntPtr method_bind_84 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_emission_operator");

        [GodotMethod("set_emission_operator")]
        public void SetEmissionOperator(SpatialMaterial.EmissionOperatorEnum @operator)
        {
            NativeCalls.godot_icall_1_796(method_bind_84, Object.GetPtr(this), @operator);
        }

        private IntPtr method_bind_85 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_emission_operator");

        [GodotMethod("get_emission_operator")]
        public SpatialMaterial.EmissionOperatorEnum GetEmissionOperator()
        {
            return NativeCalls.godot_icall_0_797(method_bind_85, Object.GetPtr(this));
        }

        private IntPtr method_bind_86 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_ao_light_affect");

        [GodotMethod("set_ao_light_affect")]
        public void SetAoLightAffect(float amount)
        {
            NativeCalls.godot_icall_1_10(method_bind_86, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_87 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_ao_light_affect");

        [GodotMethod("get_ao_light_affect")]
        public float GetAoLightAffect()
        {
            return NativeCalls.godot_icall_0_9(method_bind_87, Object.GetPtr(this));
        }

        private IntPtr method_bind_88 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_alpha_scissor_threshold");

        [GodotMethod("set_alpha_scissor_threshold")]
        public void SetAlphaScissorThreshold(float threshold)
        {
            NativeCalls.godot_icall_1_10(method_bind_88, Object.GetPtr(this), threshold);
        }

        private IntPtr method_bind_89 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_alpha_scissor_threshold");

        [GodotMethod("get_alpha_scissor_threshold")]
        public float GetAlphaScissorThreshold()
        {
            return NativeCalls.godot_icall_0_9(method_bind_89, Object.GetPtr(this));
        }

        private IntPtr method_bind_90 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_grow_enabled");

        [GodotMethod("set_grow_enabled")]
        public void SetGrowEnabled(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_90, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_91 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_grow_enabled");

        [GodotMethod("is_grow_enabled")]
        public bool IsGrowEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_91, Object.GetPtr(this));
        }

        private IntPtr method_bind_92 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_metallic_texture_channel");

        [GodotMethod("set_metallic_texture_channel")]
        public void SetMetallicTextureChannel(SpatialMaterial.TextureChannel channel)
        {
            NativeCalls.godot_icall_1_798(method_bind_92, Object.GetPtr(this), channel);
        }

        private IntPtr method_bind_93 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_metallic_texture_channel");

        [GodotMethod("get_metallic_texture_channel")]
        public SpatialMaterial.TextureChannel GetMetallicTextureChannel()
        {
            return NativeCalls.godot_icall_0_799(method_bind_93, Object.GetPtr(this));
        }

        private IntPtr method_bind_94 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_roughness_texture_channel");

        [GodotMethod("set_roughness_texture_channel")]
        public void SetRoughnessTextureChannel(SpatialMaterial.TextureChannel channel)
        {
            NativeCalls.godot_icall_1_798(method_bind_94, Object.GetPtr(this), channel);
        }

        private IntPtr method_bind_95 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_roughness_texture_channel");

        [GodotMethod("get_roughness_texture_channel")]
        public SpatialMaterial.TextureChannel GetRoughnessTextureChannel()
        {
            return NativeCalls.godot_icall_0_799(method_bind_95, Object.GetPtr(this));
        }

        private IntPtr method_bind_96 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_ao_texture_channel");

        [GodotMethod("set_ao_texture_channel")]
        public void SetAoTextureChannel(SpatialMaterial.TextureChannel channel)
        {
            NativeCalls.godot_icall_1_798(method_bind_96, Object.GetPtr(this), channel);
        }

        private IntPtr method_bind_97 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_ao_texture_channel");

        [GodotMethod("get_ao_texture_channel")]
        public SpatialMaterial.TextureChannel GetAoTextureChannel()
        {
            return NativeCalls.godot_icall_0_799(method_bind_97, Object.GetPtr(this));
        }

        private IntPtr method_bind_98 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_refraction_texture_channel");

        [GodotMethod("set_refraction_texture_channel")]
        public void SetRefractionTextureChannel(SpatialMaterial.TextureChannel channel)
        {
            NativeCalls.godot_icall_1_798(method_bind_98, Object.GetPtr(this), channel);
        }

        private IntPtr method_bind_99 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_refraction_texture_channel");

        [GodotMethod("get_refraction_texture_channel")]
        public SpatialMaterial.TextureChannel GetRefractionTextureChannel()
        {
            return NativeCalls.godot_icall_0_799(method_bind_99, Object.GetPtr(this));
        }

        private IntPtr method_bind_100 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_proximity_fade");

        [GodotMethod("set_proximity_fade")]
        public void SetProximityFade(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_100, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_101 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_proximity_fade_enabled");

        [GodotMethod("is_proximity_fade_enabled")]
        public bool IsProximityFadeEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_101, Object.GetPtr(this));
        }

        private IntPtr method_bind_102 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_proximity_fade_distance");

        [GodotMethod("set_proximity_fade_distance")]
        public void SetProximityFadeDistance(float distance)
        {
            NativeCalls.godot_icall_1_10(method_bind_102, Object.GetPtr(this), distance);
        }

        private IntPtr method_bind_103 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_proximity_fade_distance");

        [GodotMethod("get_proximity_fade_distance")]
        public float GetProximityFadeDistance()
        {
            return NativeCalls.godot_icall_0_9(method_bind_103, Object.GetPtr(this));
        }

        private IntPtr method_bind_104 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_distance_fade");

        [GodotMethod("set_distance_fade")]
        public void SetDistanceFade(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_104, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_105 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_distance_fade_enabled");

        [GodotMethod("is_distance_fade_enabled")]
        public bool IsDistanceFadeEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_105, Object.GetPtr(this));
        }

        private IntPtr method_bind_106 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_distance_fade_max_distance");

        [GodotMethod("set_distance_fade_max_distance")]
        public void SetDistanceFadeMaxDistance(float distance)
        {
            NativeCalls.godot_icall_1_10(method_bind_106, Object.GetPtr(this), distance);
        }

        private IntPtr method_bind_107 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_distance_fade_max_distance");

        [GodotMethod("get_distance_fade_max_distance")]
        public float GetDistanceFadeMaxDistance()
        {
            return NativeCalls.godot_icall_0_9(method_bind_107, Object.GetPtr(this));
        }

        private IntPtr method_bind_108 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_distance_fade_min_distance");

        [GodotMethod("set_distance_fade_min_distance")]
        public void SetDistanceFadeMinDistance(float distance)
        {
            NativeCalls.godot_icall_1_10(method_bind_108, Object.GetPtr(this), distance);
        }

        private IntPtr method_bind_109 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_distance_fade_min_distance");

        [GodotMethod("get_distance_fade_min_distance")]
        public float GetDistanceFadeMinDistance()
        {
            return NativeCalls.godot_icall_0_9(method_bind_109, Object.GetPtr(this));
        }
    }
}
