using System;

namespace Godot
{
    /// <summary>
    /// Class representing a spherical [PrimitiveMesh].
    /// </summary>
    public class SphereMesh : PrimitiveMesh
    {
        /// <summary>
        /// Radius of sphere. Defaults to 1.0.
        /// </summary>
        public float Radius
        {
            get
            {
                return GetRadius();
            }
            set
            {
                SetRadius(value);
            }
        }

        /// <summary>
        /// Full height of the sphere. Defaults to 2.0.
        /// </summary>
        public float Height
        {
            get
            {
                return GetHeight();
            }
            set
            {
                SetHeight(value);
            }
        }

        /// <summary>
        /// Number of radial segments on the sphere. Defaults to 64.
        /// </summary>
        public int RadialSegments
        {
            get
            {
                return GetRadialSegments();
            }
            set
            {
                SetRadialSegments(value);
            }
        }

        /// <summary>
        /// Number of segments along the height of the sphere. Defaults to 32.
        /// </summary>
        public int Rings
        {
            get
            {
                return GetRings();
            }
            set
            {
                SetRings(value);
            }
        }

        /// <summary>
        /// Determines whether a full sphere or a hemisphere is created. Attention: To get a regular hemisphere the height and radius of the sphere have to equal. Defaults to false.
        /// </summary>
        public bool IsHemisphere
        {
            get
            {
                return GetIsHemisphere();
            }
            set
            {
                SetIsHemisphere(value);
            }
        }


        private const string nativeName = "SphereMesh";

        public SphereMesh() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_SphereMesh_Ctor(this);
        }

        internal SphereMesh(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_radius");

        [GodotMethod("set_radius")]
        public void SetRadius(float radius)
        {
            NativeCalls.godot_icall_1_10(method_bind_0, Object.GetPtr(this), radius);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_radius");

        [GodotMethod("get_radius")]
        public float GetRadius()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_height");

        [GodotMethod("set_height")]
        public void SetHeight(float height)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), height);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_height");

        [GodotMethod("get_height")]
        public float GetHeight()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_radial_segments");

        [GodotMethod("set_radial_segments")]
        public void SetRadialSegments(int radialSegments)
        {
            NativeCalls.godot_icall_1_0(method_bind_4, Object.GetPtr(this), radialSegments);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_radial_segments");

        [GodotMethod("get_radial_segments")]
        public int GetRadialSegments()
        {
            return NativeCalls.godot_icall_0_1(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_rings");

        [GodotMethod("set_rings")]
        public void SetRings(int rings)
        {
            NativeCalls.godot_icall_1_0(method_bind_6, Object.GetPtr(this), rings);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_rings");

        [GodotMethod("get_rings")]
        public int GetRings()
        {
            return NativeCalls.godot_icall_0_1(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_is_hemisphere");

        [GodotMethod("set_is_hemisphere")]
        public void SetIsHemisphere(bool isHemisphere)
        {
            NativeCalls.godot_icall_1_11(method_bind_8, Object.GetPtr(this), isHemisphere);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_is_hemisphere");

        [GodotMethod("get_is_hemisphere")]
        public bool GetIsHemisphere()
        {
            return NativeCalls.godot_icall_0_3(method_bind_9, Object.GetPtr(this));
        }
    }
}
