using System;

namespace Godot
{
    /// <summary>
    /// Sphere shape for 3D collisions, which can be set into a [PhysicsBody] or [Area]. This shape is useful for modeling sphere-like 3D objects.
    /// </summary>
    public class SphereShape : Shape
    {
        /// <summary>
        /// The sphere's radius. The shape's diameter is double the radius.
        /// </summary>
        public float Radius
        {
            get
            {
                return GetRadius();
            }
            set
            {
                SetRadius(value);
            }
        }


        private const string nativeName = "SphereShape";

        public SphereShape() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_SphereShape_Ctor(this);
        }

        internal SphereShape(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_radius");

        [GodotMethod("set_radius")]
        public void SetRadius(float radius)
        {
            NativeCalls.godot_icall_1_10(method_bind_0, Object.GetPtr(this), radius);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_radius");

        [GodotMethod("get_radius")]
        public float GetRadius()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }
    }
}
