using System;

namespace Godot
{
    /// <summary>
    /// Container for splitting two controls vertically or horizontally, with a grabber that allows adjusting the split offset or ratio.
    /// </summary>
    public class SplitContainer : Container
    {
        public enum DraggerVisibilityEnum
        {
            Visible = 0,
            Hidden = 1,
            HiddenCollapsed = 2
        }


        /// <summary>
        /// </summary>
        public int SplitOffset
        {
            get
            {
                return GetSplitOffset();
            }
            set
            {
                SetSplitOffset(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool Collapsed
        {
            get
            {
                return IsCollapsed();
            }
            set
            {
                SetCollapsed(value);
            }
        }

        /// <summary>
        /// </summary>
        public SplitContainer.DraggerVisibilityEnum DraggerVisibility
        {
            get
            {
                return GetDraggerVisibility();
            }
            set
            {
                SetDraggerVisibility(value);
            }
        }


        private const string nativeName = "SplitContainer";

        internal SplitContainer() {}

        internal SplitContainer(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_split_offset");

        [GodotMethod("set_split_offset")]
        public void SetSplitOffset(int offset)
        {
            NativeCalls.godot_icall_1_0(method_bind_0, Object.GetPtr(this), offset);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_split_offset");

        [GodotMethod("get_split_offset")]
        public int GetSplitOffset()
        {
            return NativeCalls.godot_icall_0_1(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_collapsed");

        [GodotMethod("set_collapsed")]
        public void SetCollapsed(bool collapsed)
        {
            NativeCalls.godot_icall_1_11(method_bind_2, Object.GetPtr(this), collapsed);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_collapsed");

        [GodotMethod("is_collapsed")]
        public bool IsCollapsed()
        {
            return NativeCalls.godot_icall_0_3(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_dragger_visibility");

        [GodotMethod("set_dragger_visibility")]
        public void SetDraggerVisibility(SplitContainer.DraggerVisibilityEnum mode)
        {
            NativeCalls.godot_icall_1_801(method_bind_4, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_dragger_visibility");

        [GodotMethod("get_dragger_visibility")]
        public SplitContainer.DraggerVisibilityEnum GetDraggerVisibility()
        {
            return NativeCalls.godot_icall_0_802(method_bind_5, Object.GetPtr(this));
        }
    }
}
