using System;

namespace Godot
{
    /// <summary>
    /// A node that displays a 2D texture in a 3D environment. The texture displayed can be a region from a larger atlas texture, or a frame from a sprite sheet animation.
    /// </summary>
    public class Sprite3D : SpriteBase3D
    {
        /// <summary>
        /// [Texture] object to draw.
        /// </summary>
        public Texture Texture
        {
            get
            {
                return GetTexture();
            }
            set
            {
                SetTexture(value);
            }
        }

        /// <summary>
        /// The number of rows in the sprite sheet.
        /// </summary>
        public int Vframes
        {
            get
            {
                return GetVframes();
            }
            set
            {
                SetVframes(value);
            }
        }

        /// <summary>
        /// The number of columns in the sprite sheet.
        /// </summary>
        public int Hframes
        {
            get
            {
                return GetHframes();
            }
            set
            {
                SetHframes(value);
            }
        }

        /// <summary>
        /// Current frame to display from sprite sheet. [member vframes] or [member hframes] must be greater than 1.
        /// </summary>
        public int Frame
        {
            get
            {
                return GetFrame();
            }
            set
            {
                SetFrame(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] texture will be cut from a larger atlas texture. See [member region_rect]. Default value: [code]false[/code].
        /// </summary>
        public bool RegionEnabled
        {
            get
            {
                return IsRegion();
            }
            set
            {
                SetRegion(value);
            }
        }

        /// <summary>
        /// The region of the atlas texture to display. [member region_enabled] must be [code]true[/code].
        /// </summary>
        public Rect2 RegionRect
        {
            get
            {
                return GetRegionRect();
            }
            set
            {
                SetRegionRect(value);
            }
        }


        private const string nativeName = "Sprite3D";

        public Sprite3D() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Sprite3D_Ctor(this);
        }

        internal Sprite3D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_texture");

        [GodotMethod("set_texture")]
        public void SetTexture(Texture texture)
        {
            NativeCalls.godot_icall_1_24(method_bind_0, Object.GetPtr(this), Object.GetPtr(texture));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_texture");

        [GodotMethod("get_texture")]
        public Texture GetTexture()
        {
            return NativeCalls.godot_icall_0_119(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_region");

        [GodotMethod("set_region")]
        public void SetRegion(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_2, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_region");

        [GodotMethod("is_region")]
        public bool IsRegion()
        {
            return NativeCalls.godot_icall_0_3(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_region_rect");

        [GodotMethod("set_region_rect")]
        public void SetRegionRect(Rect2 rect)
        {
            NativeCalls.godot_icall_1_120(method_bind_4, Object.GetPtr(this), ref rect);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_region_rect");

        [GodotMethod("get_region_rect")]
        public Rect2 GetRegionRect()
        {
            object ret = NativeCalls.godot_icall_0_121(method_bind_5, Object.GetPtr(this));
            return (Rect2)ret;
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_frame");

        [GodotMethod("set_frame")]
        public void SetFrame(int frame)
        {
            NativeCalls.godot_icall_1_0(method_bind_6, Object.GetPtr(this), frame);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_frame");

        [GodotMethod("get_frame")]
        public int GetFrame()
        {
            return NativeCalls.godot_icall_0_1(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_vframes");

        [GodotMethod("set_vframes")]
        public void SetVframes(int vframes)
        {
            NativeCalls.godot_icall_1_0(method_bind_8, Object.GetPtr(this), vframes);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_vframes");

        [GodotMethod("get_vframes")]
        public int GetVframes()
        {
            return NativeCalls.godot_icall_0_1(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_hframes");

        [GodotMethod("set_hframes")]
        public void SetHframes(int hframes)
        {
            NativeCalls.godot_icall_1_0(method_bind_10, Object.GetPtr(this), hframes);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_hframes");

        [GodotMethod("get_hframes")]
        public int GetHframes()
        {
            return NativeCalls.godot_icall_0_1(method_bind_11, Object.GetPtr(this));
        }
    }
}
