using System;

namespace Godot
{
    /// <summary>
    /// Sprite frame library for [AnimatedSprite]. Contains frames and animation data for playback.
    /// </summary>
    public class SpriteFrames : Resource
    {
        /// <summary>
        /// </summary>
        public object[] Frames
        {
            get
            {
                return _GetFrames();
            }
            set
            {
                _SetFrames(value);
            }
        }

        public object[] Animations
        {
            get
            {
                return _GetAnimations();
            }
            set
            {
                _SetAnimations(value);
            }
        }


        private const string nativeName = "SpriteFrames";

        public SpriteFrames() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_SpriteFrames_Ctor(this);
        }

        internal SpriteFrames(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_animation");

        /// <summary>
        /// Adds a new animation to the the library.
        /// </summary>
        [GodotMethod("add_animation")]
        public void AddAnimation(string anim)
        {
            NativeCalls.godot_icall_1_43(method_bind_0, Object.GetPtr(this), anim);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_animation");

        /// <summary>
        /// If [code]true[/code] the named animation exists.
        /// </summary>
        [GodotMethod("has_animation")]
        public bool HasAnimation(string anim)
        {
            return NativeCalls.godot_icall_1_70(method_bind_1, Object.GetPtr(this), anim);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_animation");

        /// <summary>
        /// Removes the given animation.
        /// </summary>
        [GodotMethod("remove_animation")]
        public void RemoveAnimation(string anim)
        {
            NativeCalls.godot_icall_1_43(method_bind_2, Object.GetPtr(this), anim);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "rename_animation");

        /// <summary>
        /// Changes the animation's name to [code]newname[/code].
        /// </summary>
        [GodotMethod("rename_animation")]
        public void RenameAnimation(string anim, string newname)
        {
            NativeCalls.godot_icall_2_69(method_bind_3, Object.GetPtr(this), anim, newname);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_animation_speed");

        /// <summary>
        /// The animation's speed in frames per second.
        /// </summary>
        [GodotMethod("set_animation_speed")]
        public void SetAnimationSpeed(string anim, float speed)
        {
            NativeCalls.godot_icall_2_77(method_bind_4, Object.GetPtr(this), anim, speed);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_animation_speed");

        /// <summary>
        /// The animation's speed in frames per second.
        /// </summary>
        [GodotMethod("get_animation_speed")]
        public float GetAnimationSpeed(string anim)
        {
            return NativeCalls.godot_icall_1_91(method_bind_5, Object.GetPtr(this), anim);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_animation_loop");

        /// <summary>
        /// If [code]true[/code] the animation will loop.
        /// </summary>
        [GodotMethod("set_animation_loop")]
        public void SetAnimationLoop(string anim, bool loop)
        {
            NativeCalls.godot_icall_2_92(method_bind_6, Object.GetPtr(this), anim, loop);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_animation_loop");

        /// <summary>
        /// If [code]true[/code] the given animation will loop.
        /// </summary>
        [GodotMethod("get_animation_loop")]
        public bool GetAnimationLoop(string anim)
        {
            return NativeCalls.godot_icall_1_70(method_bind_7, Object.GetPtr(this), anim);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_frame");

        /// <summary>
        /// Adds a frame to the given animation.
        /// </summary>
        [GodotMethod("add_frame")]
        public void AddFrame(string anim, Texture frame, int atPosition = -1)
        {
            NativeCalls.godot_icall_3_809(method_bind_8, Object.GetPtr(this), anim, Object.GetPtr(frame), atPosition);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_frame_count");

        /// <summary>
        /// Returns the number of frames in the animation.
        /// </summary>
        [GodotMethod("get_frame_count")]
        public int GetFrameCount(string anim)
        {
            return NativeCalls.godot_icall_1_87(method_bind_9, Object.GetPtr(this), anim);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_frame");

        /// <summary>
        /// Returns the animation's selected frame.
        /// </summary>
        [GodotMethod("get_frame")]
        public Texture GetFrame(string anim, int idx)
        {
            return NativeCalls.godot_icall_2_810(method_bind_10, Object.GetPtr(this), anim, idx);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_frame");

        /// <summary>
        /// Sets the texture of the given frame.
        /// </summary>
        [GodotMethod("set_frame")]
        public void SetFrame(string anim, int idx, Texture txt)
        {
            NativeCalls.godot_icall_3_811(method_bind_11, Object.GetPtr(this), anim, idx, Object.GetPtr(txt));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_frame");

        /// <summary>
        /// Removes the animation's selected frame.
        /// </summary>
        [GodotMethod("remove_frame")]
        public void RemoveFrame(string anim, int idx)
        {
            NativeCalls.godot_icall_2_95(method_bind_12, Object.GetPtr(this), anim, idx);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear");

        /// <summary>
        /// Removes all frames from the given animation.
        /// </summary>
        [GodotMethod("clear")]
        public void Clear(string anim)
        {
            NativeCalls.godot_icall_1_43(method_bind_13, Object.GetPtr(this), anim);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear_all");

        /// <summary>
        /// Removes all animations. A "default" animation will be created.
        /// </summary>
        [GodotMethod("clear_all")]
        public void ClearAll()
        {
            NativeCalls.godot_icall_0_12(method_bind_14, Object.GetPtr(this));
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_frames");

        internal void _SetFrames(object[] arg0)
        {
            NativeCalls.godot_icall_1_154(method_bind_15, Object.GetPtr(this), arg0);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_frames");

        internal object[] _GetFrames()
        {
            return NativeCalls.godot_icall_0_21(method_bind_16, Object.GetPtr(this));
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_animations");

        internal void _SetAnimations(object[] arg0)
        {
            NativeCalls.godot_icall_1_154(method_bind_17, Object.GetPtr(this), arg0);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_animations");

        internal object[] _GetAnimations()
        {
            return NativeCalls.godot_icall_0_21(method_bind_18, Object.GetPtr(this));
        }
    }
}
