using System;

namespace Godot
{
    /// <summary>
    /// StreamPeer is an abstraction and base class for stream-based protocols (such as TCP or Unix Sockets). It provides an API for sending and receiving data through streams as raw data or strings.
    /// </summary>
    public class StreamPeer : Reference
    {
        /// <summary>
        /// If [code]true[/code], this [code]StreamPeer[/code] will using big-endian format for encoding and decoding.
        /// </summary>
        public bool BigEndian
        {
            get
            {
                return IsBigEndianEnabled();
            }
            set
            {
                SetBigEndian(value);
            }
        }


        private const string nativeName = "StreamPeer";

        internal StreamPeer() {}

        internal StreamPeer(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "put_data");

        /// <summary>
        /// Send a chunk of data through the connection, blocking if necessary until the data is done sending. This function returns an Error code.
        /// </summary>
        [GodotMethod("put_data")]
        public Error PutData(byte[] data)
        {
            return NativeCalls.godot_icall_1_430(method_bind_0, Object.GetPtr(this), data);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "put_partial_data");

        /// <summary>
        /// Send a chunk of data through the connection, if all the data could not be sent at once, only part of it will. This function returns two values, an Error code and an integer, describing how much data was actually sent.
        /// </summary>
        [GodotMethod("put_partial_data")]
        public object[] PutPartialData(byte[] data)
        {
            return NativeCalls.godot_icall_1_812(method_bind_1, Object.GetPtr(this), data);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_data");

        /// <summary>
        /// Return a chunk data with the received bytes. The amount of bytes to be received can be requested in the "bytes" argument. If not enough bytes are available, the function will block until the desired amount is received. This function returns two values, an Error code and a data array.
        /// </summary>
        [GodotMethod("get_data")]
        public object[] GetData(int bytes)
        {
            return NativeCalls.godot_icall_1_116(method_bind_2, Object.GetPtr(this), bytes);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_partial_data");

        /// <summary>
        /// Return a chunk data with the received bytes. The amount of bytes to be received can be requested in the "bytes" argument. If not enough bytes are available, the function will return how many were actually received. This function returns two values, an Error code, and a data array.
        /// </summary>
        [GodotMethod("get_partial_data")]
        public object[] GetPartialData(int bytes)
        {
            return NativeCalls.godot_icall_1_116(method_bind_3, Object.GetPtr(this), bytes);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_available_bytes");

        /// <summary>
        /// Return the amount of bytes this [code]StreamPeer[/code] has available.
        /// </summary>
        [GodotMethod("get_available_bytes")]
        public int GetAvailableBytes()
        {
            return NativeCalls.godot_icall_0_1(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_big_endian");

        [GodotMethod("set_big_endian")]
        public void SetBigEndian(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_5, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_big_endian_enabled");

        [GodotMethod("is_big_endian_enabled")]
        public bool IsBigEndianEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_6, Object.GetPtr(this));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "put_8");

        /// <summary>
        /// Put a signed byte into the stream.
        /// </summary>
        [GodotMethod("put_8")]
        public void Put8(int val)
        {
            NativeCalls.godot_icall_1_0(method_bind_7, Object.GetPtr(this), val);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "put_u8");

        /// <summary>
        /// Put an unsigned byte into the stream.
        /// </summary>
        [GodotMethod("put_u8")]
        public void PutU8(int val)
        {
            NativeCalls.godot_icall_1_0(method_bind_8, Object.GetPtr(this), val);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "put_16");

        /// <summary>
        /// Put a signed 16 bit value into the stream.
        /// </summary>
        [GodotMethod("put_16")]
        public void Put16(int val)
        {
            NativeCalls.godot_icall_1_0(method_bind_9, Object.GetPtr(this), val);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "put_u16");

        /// <summary>
        /// Put an unsigned 16 bit value into the stream.
        /// </summary>
        [GodotMethod("put_u16")]
        public void PutU16(int val)
        {
            NativeCalls.godot_icall_1_0(method_bind_10, Object.GetPtr(this), val);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "put_32");

        /// <summary>
        /// Put a signed 32 bit value into the stream.
        /// </summary>
        [GodotMethod("put_32")]
        public void Put32(int val)
        {
            NativeCalls.godot_icall_1_0(method_bind_11, Object.GetPtr(this), val);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "put_u32");

        /// <summary>
        /// Put an unsigned 32 bit value into the stream.
        /// </summary>
        [GodotMethod("put_u32")]
        public void PutU32(int val)
        {
            NativeCalls.godot_icall_1_0(method_bind_12, Object.GetPtr(this), val);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "put_64");

        /// <summary>
        /// Put a signed 64 bit value into the stream.
        /// </summary>
        [GodotMethod("put_64")]
        public void Put64(int val)
        {
            NativeCalls.godot_icall_1_0(method_bind_13, Object.GetPtr(this), val);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "put_u64");

        /// <summary>
        /// Put an unsigned 64 bit value into the stream.
        /// </summary>
        [GodotMethod("put_u64")]
        public void PutU64(int val)
        {
            NativeCalls.godot_icall_1_0(method_bind_14, Object.GetPtr(this), val);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "put_float");

        /// <summary>
        /// Put a single-precision float into the stream.
        /// </summary>
        [GodotMethod("put_float")]
        public void PutFloat(float val)
        {
            NativeCalls.godot_icall_1_10(method_bind_15, Object.GetPtr(this), val);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "put_double");

        /// <summary>
        /// Put a double-precision float into the stream.
        /// </summary>
        [GodotMethod("put_double")]
        public void PutDouble(float val)
        {
            NativeCalls.godot_icall_1_10(method_bind_16, Object.GetPtr(this), val);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "put_utf8_string");

        /// <summary>
        /// Put a zero-terminated utf8 string into the stream.
        /// </summary>
        [GodotMethod("put_utf8_string")]
        public void PutUtf8String(string val)
        {
            NativeCalls.godot_icall_1_43(method_bind_17, Object.GetPtr(this), val);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "put_var");

        /// <summary>
        /// Put a Variant into the stream.
        /// </summary>
        [GodotMethod("put_var")]
        public void PutVar(object val)
        {
            NativeCalls.godot_icall_1_463(method_bind_18, Object.GetPtr(this), val);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_8");

        /// <summary>
        /// Get a signed byte from the stream.
        /// </summary>
        [GodotMethod("get_8")]
        public int Get8()
        {
            return NativeCalls.godot_icall_0_1(method_bind_19, Object.GetPtr(this));
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_u8");

        /// <summary>
        /// Get an unsigned byte from the stream.
        /// </summary>
        [GodotMethod("get_u8")]
        public int GetU8()
        {
            return NativeCalls.godot_icall_0_1(method_bind_20, Object.GetPtr(this));
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_16");

        /// <summary>
        /// Get a signed 16 bit value from the stream.
        /// </summary>
        [GodotMethod("get_16")]
        public int Get16()
        {
            return NativeCalls.godot_icall_0_1(method_bind_21, Object.GetPtr(this));
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_u16");

        /// <summary>
        /// Get an unsigned 16 bit value from the stream.
        /// </summary>
        [GodotMethod("get_u16")]
        public int GetU16()
        {
            return NativeCalls.godot_icall_0_1(method_bind_22, Object.GetPtr(this));
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_32");

        /// <summary>
        /// Get a signed 32 bit value from the stream.
        /// </summary>
        [GodotMethod("get_32")]
        public int Get32()
        {
            return NativeCalls.godot_icall_0_1(method_bind_23, Object.GetPtr(this));
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_u32");

        /// <summary>
        /// Get an unsigned 32 bit value from the stream.
        /// </summary>
        [GodotMethod("get_u32")]
        public int GetU32()
        {
            return NativeCalls.godot_icall_0_1(method_bind_24, Object.GetPtr(this));
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_64");

        /// <summary>
        /// Get a signed 64 bit value from the stream.
        /// </summary>
        [GodotMethod("get_64")]
        public int Get64()
        {
            return NativeCalls.godot_icall_0_1(method_bind_25, Object.GetPtr(this));
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_u64");

        /// <summary>
        /// Get an unsigned 64 bit value from the stream.
        /// </summary>
        [GodotMethod("get_u64")]
        public int GetU64()
        {
            return NativeCalls.godot_icall_0_1(method_bind_26, Object.GetPtr(this));
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_float");

        /// <summary>
        /// Get a single-precision float from the stream.
        /// </summary>
        [GodotMethod("get_float")]
        public float GetFloat()
        {
            return NativeCalls.godot_icall_0_9(method_bind_27, Object.GetPtr(this));
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_double");

        /// <summary>
        /// Get a double-precision float from the stream.
        /// </summary>
        [GodotMethod("get_double")]
        public float GetDouble()
        {
            return NativeCalls.godot_icall_0_9(method_bind_28, Object.GetPtr(this));
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_string");

        /// <summary>
        /// Get a string with byte-length "bytes" from the stream.
        /// </summary>
        [GodotMethod("get_string")]
        public string GetString(int bytes)
        {
            return NativeCalls.godot_icall_1_108(method_bind_29, Object.GetPtr(this), bytes);
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_utf8_string");

        /// <summary>
        /// Get a utf8 string with byte-length "bytes" from the stream (this decodes the string sent as utf8).
        /// </summary>
        [GodotMethod("get_utf8_string")]
        public string GetUtf8String(int bytes)
        {
            return NativeCalls.godot_icall_1_108(method_bind_30, Object.GetPtr(this), bytes);
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_var");

        /// <summary>
        /// Get a Variant from the stream.
        /// </summary>
        [GodotMethod("get_var")]
        public object GetVar()
        {
            return NativeCalls.godot_icall_0_461(method_bind_31, Object.GetPtr(this));
        }
    }
}
