using System;

namespace Godot
{
    /// <summary>
    /// SSL Stream peer. This object can be used to connect to SSL servers.
    /// </summary>
    public class StreamPeerSSL : StreamPeer
    {
        public enum Status
        {
            Disconnected = 0,
            Connected = 1,
            ErrorNoCertificate = 2,
            ErrorHostnameMismatch = 3
        }


        private const string nativeName = "StreamPeerSSL";

        public StreamPeerSSL() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_StreamPeerSSL_Ctor(this);
        }

        internal StreamPeerSSL(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "accept_stream");

        /// <summary>
        /// </summary>
        [GodotMethod("accept_stream")]
        public Error AcceptStream(StreamPeer stream)
        {
            return NativeCalls.godot_icall_1_505(method_bind_0, Object.GetPtr(this), Object.GetPtr(stream));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "connect_to_stream");

        /// <summary>
        /// Connect to a peer using an underlying [StreamPeer] "stream", when "validate_certs" is true, [code]StreamPeerSSL[/code] will validate that the certificate presented by the peer matches the "for_hostname".
        /// </summary>
        [GodotMethod("connect_to_stream")]
        public Error ConnectToStream(StreamPeer stream, bool validateCerts = false, string forHostname = "")
        {
            return NativeCalls.godot_icall_3_814(method_bind_1, Object.GetPtr(this), Object.GetPtr(stream), validateCerts, forHostname);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_status");

        /// <summary>
        /// Return the status of the connection, one of STATUS_* enum.
        /// </summary>
        [GodotMethod("get_status")]
        public StreamPeerSSL.Status GetStatus()
        {
            return NativeCalls.godot_icall_0_815(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "disconnect_from_stream");

        /// <summary>
        /// Disconnect from host.
        /// </summary>
        [GodotMethod("disconnect_from_stream")]
        public void DisconnectFromStream()
        {
            NativeCalls.godot_icall_0_12(method_bind_3, Object.GetPtr(this));
        }
    }
}
