using System;

namespace Godot
{
    /// <summary>
    /// TCP Stream peer. This object can be used to connect to TCP servers, or also is returned by a tcp server.
    /// </summary>
    public class StreamPeerTCP : StreamPeer
    {
        public enum Status
        {
            None = 0,
            Connecting = 1,
            Connected = 2,
            Error = 3
        }


        private const string nativeName = "StreamPeerTCP";

        public StreamPeerTCP() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_StreamPeerTCP_Ctor(this);
        }

        internal StreamPeerTCP(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "connect_to_host");

        /// <summary>
        /// Connect to the specified host:port pair. A hostname will be resolved if valid. Returns [OK] on success or [FAILED] on failure.
        /// </summary>
        [GodotMethod("connect_to_host")]
        public Error ConnectToHost(string host, int port)
        {
            return NativeCalls.godot_icall_2_575(method_bind_0, Object.GetPtr(this), host, port);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_connected_to_host");

        /// <summary>
        /// </summary>
        [GodotMethod("is_connected_to_host")]
        public bool IsConnectedToHost()
        {
            return NativeCalls.godot_icall_0_3(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_status");

        /// <summary>
        /// Return the status of the connection, one of STATUS_* enum.
        /// </summary>
        [GodotMethod("get_status")]
        public StreamPeerTCP.Status GetStatus()
        {
            return NativeCalls.godot_icall_0_816(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_connected_host");

        /// <summary>
        /// Return the IP of this peer.
        /// </summary>
        [GodotMethod("get_connected_host")]
        public string GetConnectedHost()
        {
            return NativeCalls.godot_icall_0_2(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_connected_port");

        /// <summary>
        /// Return the port of this peer.
        /// </summary>
        [GodotMethod("get_connected_port")]
        public int GetConnectedPort()
        {
            return NativeCalls.godot_icall_0_1(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "disconnect_from_host");

        /// <summary>
        /// Disconnect from host.
        /// </summary>
        [GodotMethod("disconnect_from_host")]
        public void DisconnectFromHost()
        {
            NativeCalls.godot_icall_0_12(method_bind_5, Object.GetPtr(this));
        }
    }
}
