using System;

namespace Godot
{
    /// <summary>
    /// A texture that is loaded from a .stex file.
    /// </summary>
    public class StreamTexture : Texture
    {
        /// <summary>
        /// The StreamTexture's filepath to a .stex file.
        /// </summary>
        public string LoadPath
        {
            get
            {
                return GetLoadPath();
            }
            set
            {
                Load(value);
            }
        }


        private const string nativeName = "StreamTexture";

        public StreamTexture() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_StreamTexture_Ctor(this);
        }

        internal StreamTexture(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "load");

        [GodotMethod("load")]
        public Error Load(string path)
        {
            return NativeCalls.godot_icall_1_170(method_bind_0, Object.GetPtr(this), path);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_load_path");

        [GodotMethod("get_load_path")]
        public string GetLoadPath()
        {
            return NativeCalls.godot_icall_0_2(method_bind_1, Object.GetPtr(this));
        }
    }
}
