using System;

namespace Godot
{
    /// <summary>
    /// StyleBox is [Resource] that provides an abstract base class for drawing stylized boxes for the UI. StyleBoxes are used for drawing the styles of buttons, line edit backgrounds, tree backgrounds, etc. and also for testing a transparency mask for pointer signals. If mask test fails on a StyleBox assigned as mask to a control, clicks and motion signals will go through it to the one below.
    /// </summary>
    public class StyleBox : Resource
    {
        /// <summary>
        /// </summary>
        public float ContentMarginLeft
        {
            get
            {
                return GetDefaultMargin((Margin)0);
            }
            set
            {
                SetDefaultMargin((Margin)0, value);
            }
        }

        /// <summary>
        /// </summary>
        public float ContentMarginRight
        {
            get
            {
                return GetDefaultMargin((Margin)2);
            }
            set
            {
                SetDefaultMargin((Margin)2, value);
            }
        }

        /// <summary>
        /// </summary>
        public float ContentMarginTop
        {
            get
            {
                return GetDefaultMargin((Margin)1);
            }
            set
            {
                SetDefaultMargin((Margin)1, value);
            }
        }

        /// <summary>
        /// </summary>
        public float ContentMarginBottom
        {
            get
            {
                return GetDefaultMargin((Margin)3);
            }
            set
            {
                SetDefaultMargin((Margin)3, value);
            }
        }


        private const string nativeName = "StyleBox";

        internal StyleBox() {}

        internal StyleBox(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "test_mask");

        /// <summary>
        /// Test a position in a rectangle, return whether it passes the mask test.
        /// </summary>
        [GodotMethod("test_mask")]
        public bool TestMask(Vector2 point, Rect2 rect)
        {
            return NativeCalls.godot_icall_2_817(method_bind_0, Object.GetPtr(this), ref point, ref rect);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_default_margin");

        [GodotMethod("set_default_margin")]
        public void SetDefaultMargin(Margin margin, float offset)
        {
            NativeCalls.godot_icall_2_198(method_bind_1, Object.GetPtr(this), margin, offset);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_default_margin");

        [GodotMethod("get_default_margin")]
        public float GetDefaultMargin(Margin margin)
        {
            return NativeCalls.godot_icall_1_199(method_bind_2, Object.GetPtr(this), margin);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_margin");

        /// <summary>
        /// Return the offset of margin "margin" (see MARGIN_* enum).
        /// </summary>
        [GodotMethod("get_margin")]
        public float GetMargin(Margin margin)
        {
            return NativeCalls.godot_icall_1_199(method_bind_3, Object.GetPtr(this), margin);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_minimum_size");

        /// <summary>
        /// Return the minimum size that this stylebox can be shrunk to.
        /// </summary>
        [GodotMethod("get_minimum_size")]
        public Vector2 GetMinimumSize()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_4, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_center_size");

        /// <summary>
        /// </summary>
        [GodotMethod("get_center_size")]
        public Vector2 GetCenterSize()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_5, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_offset");

        /// <summary>
        /// Return the "offset" of a stylebox, this is a helper function, like writing [code]Vector2(style.get_margin(MARGIN_LEFT), style.get_margin(MARGIN_TOP))[/code].
        /// </summary>
        [GodotMethod("get_offset")]
        public Vector2 GetOffset()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_6, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "draw");

        /// <summary>
        /// </summary>
        [GodotMethod("draw")]
        public void Draw(RID canvasItem, Rect2 rect)
        {
            NativeCalls.godot_icall_2_818(method_bind_7, Object.GetPtr(this), RID.GetPtr(canvasItem), ref rect);
        }
    }
}
