using System;

namespace Godot
{
    /// <summary>
    /// Texture Based 3x3 scale style. This stylebox performs a 3x3 scaling of a texture, where only the center cell is fully stretched. This allows for the easy creation of bordered styles.
    /// </summary>
    public class StyleBoxTexture : StyleBox
    {
        public enum AxisStretchMode
        {
            Stretch = 0,
            Tile = 1,
            TileFit = 2
        }


        /// <summary>
        /// </summary>
        public Resource Texture
        {
            get
            {
                return GetTexture();
            }
            set
            {
                SetTexture(value);
            }
        }

        /// <summary>
        /// </summary>
        public Resource NormalMap
        {
            get
            {
                return GetNormalMap();
            }
            set
            {
                SetNormalMap(value);
            }
        }

        /// <summary>
        /// </summary>
        public Rect2 RegionRect
        {
            get
            {
                return GetRegionRect();
            }
            set
            {
                SetRegionRect(value);
            }
        }

        /// <summary>
        /// </summary>
        public float MarginLeft
        {
            get
            {
                return GetMarginSize((Margin)0);
            }
            set
            {
                SetMarginSize((Margin)0, value);
            }
        }

        /// <summary>
        /// </summary>
        public float MarginRight
        {
            get
            {
                return GetMarginSize((Margin)2);
            }
            set
            {
                SetMarginSize((Margin)2, value);
            }
        }

        /// <summary>
        /// </summary>
        public float MarginTop
        {
            get
            {
                return GetMarginSize((Margin)1);
            }
            set
            {
                SetMarginSize((Margin)1, value);
            }
        }

        /// <summary>
        /// </summary>
        public float MarginBottom
        {
            get
            {
                return GetMarginSize((Margin)3);
            }
            set
            {
                SetMarginSize((Margin)3, value);
            }
        }

        /// <summary>
        /// </summary>
        public float ExpandMarginLeft
        {
            get
            {
                return GetExpandMarginSize((Margin)0);
            }
            set
            {
                SetExpandMarginSize((Margin)0, value);
            }
        }

        /// <summary>
        /// </summary>
        public float ExpandMarginRight
        {
            get
            {
                return GetExpandMarginSize((Margin)2);
            }
            set
            {
                SetExpandMarginSize((Margin)2, value);
            }
        }

        /// <summary>
        /// </summary>
        public float ExpandMarginTop
        {
            get
            {
                return GetExpandMarginSize((Margin)1);
            }
            set
            {
                SetExpandMarginSize((Margin)1, value);
            }
        }

        /// <summary>
        /// </summary>
        public float ExpandMarginBottom
        {
            get
            {
                return GetExpandMarginSize((Margin)3);
            }
            set
            {
                SetExpandMarginSize((Margin)3, value);
            }
        }

        /// <summary>
        /// </summary>
        public StyleBoxTexture.AxisStretchMode AxisStretchHorizontal
        {
            get
            {
                return GetHAxisStretchMode();
            }
            set
            {
                SetHAxisStretchMode(value);
            }
        }

        /// <summary>
        /// </summary>
        public StyleBoxTexture.AxisStretchMode AxisStretchVertical
        {
            get
            {
                return GetVAxisStretchMode();
            }
            set
            {
                SetVAxisStretchMode(value);
            }
        }

        /// <summary>
        /// </summary>
        public Color ModulateColor
        {
            get
            {
                return GetModulate();
            }
            set
            {
                SetModulate(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool DrawCenter
        {
            get
            {
                return IsDrawCenterEnabled();
            }
            set
            {
                SetDrawCenter(value);
            }
        }


        private const string nativeName = "StyleBoxTexture";

        public StyleBoxTexture() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_StyleBoxTexture_Ctor(this);
        }

        internal StyleBoxTexture(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_texture");

        [GodotMethod("set_texture")]
        public void SetTexture(Resource texture)
        {
            NativeCalls.godot_icall_1_24(method_bind_0, Object.GetPtr(this), Object.GetPtr(texture));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_texture");

        [GodotMethod("get_texture")]
        public Resource GetTexture()
        {
            return NativeCalls.godot_icall_0_731(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_normal_map");

        [GodotMethod("set_normal_map")]
        public void SetNormalMap(Resource normalMap)
        {
            NativeCalls.godot_icall_1_24(method_bind_2, Object.GetPtr(this), Object.GetPtr(normalMap));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_normal_map");

        [GodotMethod("get_normal_map")]
        public Resource GetNormalMap()
        {
            return NativeCalls.godot_icall_0_731(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_margin_size");

        [GodotMethod("set_margin_size")]
        public void SetMarginSize(Margin margin, float size)
        {
            NativeCalls.godot_icall_2_198(method_bind_4, Object.GetPtr(this), margin, size);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_margin_size");

        [GodotMethod("get_margin_size")]
        public float GetMarginSize(Margin margin)
        {
            return NativeCalls.godot_icall_1_199(method_bind_5, Object.GetPtr(this), margin);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_expand_margin_size");

        [GodotMethod("set_expand_margin_size")]
        public void SetExpandMarginSize(Margin margin, float size)
        {
            NativeCalls.godot_icall_2_198(method_bind_6, Object.GetPtr(this), margin, size);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_expand_margin_all");

        /// <summary>
        /// </summary>
        [GodotMethod("set_expand_margin_all")]
        public void SetExpandMarginAll(float size)
        {
            NativeCalls.godot_icall_1_10(method_bind_7, Object.GetPtr(this), size);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_expand_margin_individual");

        /// <summary>
        /// </summary>
        [GodotMethod("set_expand_margin_individual")]
        public void SetExpandMarginIndividual(float sizeLeft, float sizeTop, float sizeRight, float sizeBottom)
        {
            NativeCalls.godot_icall_4_822(method_bind_8, Object.GetPtr(this), sizeLeft, sizeTop, sizeRight, sizeBottom);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_expand_margin_size");

        [GodotMethod("get_expand_margin_size")]
        public float GetExpandMarginSize(Margin margin)
        {
            return NativeCalls.godot_icall_1_199(method_bind_9, Object.GetPtr(this), margin);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_region_rect");

        [GodotMethod("set_region_rect")]
        public void SetRegionRect(Rect2 region)
        {
            NativeCalls.godot_icall_1_120(method_bind_10, Object.GetPtr(this), ref region);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_region_rect");

        [GodotMethod("get_region_rect")]
        public Rect2 GetRegionRect()
        {
            object ret = NativeCalls.godot_icall_0_121(method_bind_11, Object.GetPtr(this));
            return (Rect2)ret;
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_draw_center");

        [GodotMethod("set_draw_center")]
        public void SetDrawCenter(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_12, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_draw_center_enabled");

        [GodotMethod("is_draw_center_enabled")]
        public bool IsDrawCenterEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_modulate");

        [GodotMethod("set_modulate")]
        public void SetModulate(Color color)
        {
            NativeCalls.godot_icall_1_202(method_bind_14, Object.GetPtr(this), ref color);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_modulate");

        [GodotMethod("get_modulate")]
        public Color GetModulate()
        {
            object ret = NativeCalls.godot_icall_0_203(method_bind_15, Object.GetPtr(this));
            return (Color)ret;
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_h_axis_stretch_mode");

        [GodotMethod("set_h_axis_stretch_mode")]
        public void SetHAxisStretchMode(StyleBoxTexture.AxisStretchMode mode)
        {
            NativeCalls.godot_icall_1_823(method_bind_16, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_h_axis_stretch_mode");

        [GodotMethod("get_h_axis_stretch_mode")]
        public StyleBoxTexture.AxisStretchMode GetHAxisStretchMode()
        {
            return NativeCalls.godot_icall_0_824(method_bind_17, Object.GetPtr(this));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_v_axis_stretch_mode");

        [GodotMethod("set_v_axis_stretch_mode")]
        public void SetVAxisStretchMode(StyleBoxTexture.AxisStretchMode mode)
        {
            NativeCalls.godot_icall_1_823(method_bind_18, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_v_axis_stretch_mode");

        [GodotMethod("get_v_axis_stretch_mode")]
        public StyleBoxTexture.AxisStretchMode GetVAxisStretchMode()
        {
            return NativeCalls.godot_icall_0_824(method_bind_19, Object.GetPtr(this));
        }
    }
}
