using System;

namespace Godot
{
    /// <summary>
    /// Simple tabs control, similar to [TabContainer] but is only in charge of drawing tabs, not interact with children.
    /// </summary>
    public class Tabs : Control
    {
        public enum CloseButtonDisplayPolicy
        {
            ShowNever = 0,
            ShowActiveOnly = 1,
            ShowAlways = 2,
            Max = 3
        }

        public enum TabAlignEnum
        {
            Left = 0,
            Center = 1,
            Right = 2,
            Max = 3
        }


        /// <summary>
        /// </summary>
        public int CurrentTab
        {
            get
            {
                return GetCurrentTab();
            }
            set
            {
                SetCurrentTab(value);
            }
        }

        /// <summary>
        /// </summary>
        public Tabs.TabAlignEnum TabAlign
        {
            get
            {
                return GetTabAlign();
            }
            set
            {
                SetTabAlign(value);
            }
        }

        /// <summary>
        /// </summary>
        public Tabs.CloseButtonDisplayPolicy TabCloseDisplayPolicy
        {
            get
            {
                return GetTabCloseDisplayPolicy();
            }
            set
            {
                SetTabCloseDisplayPolicy(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool ScrollingEnabled
        {
            get
            {
                return GetScrollingEnabled();
            }
            set
            {
                SetScrollingEnabled(value);
            }
        }


        private const string nativeName = "Tabs";

        public Tabs() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Tabs_Ctor(this);
        }

        internal Tabs(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tab_count");

        /// <summary>
        /// </summary>
        [GodotMethod("get_tab_count")]
        public int GetTabCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_current_tab");

        [GodotMethod("set_current_tab")]
        public void SetCurrentTab(int tabIdx)
        {
            NativeCalls.godot_icall_1_0(method_bind_1, Object.GetPtr(this), tabIdx);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_current_tab");

        [GodotMethod("get_current_tab")]
        public int GetCurrentTab()
        {
            return NativeCalls.godot_icall_0_1(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tab_title");

        /// <summary>
        /// </summary>
        [GodotMethod("set_tab_title")]
        public void SetTabTitle(int tabIdx, string title)
        {
            NativeCalls.godot_icall_2_115(method_bind_3, Object.GetPtr(this), tabIdx, title);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tab_title");

        /// <summary>
        /// </summary>
        [GodotMethod("get_tab_title")]
        public string GetTabTitle(int tabIdx)
        {
            return NativeCalls.godot_icall_1_108(method_bind_4, Object.GetPtr(this), tabIdx);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tab_icon");

        /// <summary>
        /// </summary>
        [GodotMethod("set_tab_icon")]
        public void SetTabIcon(int tabIdx, Texture icon)
        {
            NativeCalls.godot_icall_2_67(method_bind_5, Object.GetPtr(this), tabIdx, Object.GetPtr(icon));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tab_icon");

        /// <summary>
        /// </summary>
        [GodotMethod("get_tab_icon")]
        public Texture GetTabIcon(int tabIdx)
        {
            return NativeCalls.godot_icall_1_157(method_bind_6, Object.GetPtr(this), tabIdx);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tab_disabled");

        /// <summary>
        /// </summary>
        [GodotMethod("set_tab_disabled")]
        public void SetTabDisabled(int tabIdx, bool disabled)
        {
            NativeCalls.godot_icall_2_51(method_bind_7, Object.GetPtr(this), tabIdx, disabled);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tab_disabled");

        /// <summary>
        /// </summary>
        [GodotMethod("get_tab_disabled")]
        public bool GetTabDisabled(int tabIdx)
        {
            return NativeCalls.godot_icall_1_29(method_bind_8, Object.GetPtr(this), tabIdx);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_tab");

        /// <summary>
        /// </summary>
        [GodotMethod("remove_tab")]
        public void RemoveTab(int tabIdx)
        {
            NativeCalls.godot_icall_1_0(method_bind_9, Object.GetPtr(this), tabIdx);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_tab");

        /// <summary>
        /// </summary>
        [GodotMethod("add_tab")]
        public void AddTab(string title = "", Texture icon = null)
        {
            NativeCalls.godot_icall_2_89(method_bind_10, Object.GetPtr(this), title, Object.GetPtr(icon));
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tab_align");

        [GodotMethod("set_tab_align")]
        public void SetTabAlign(Tabs.TabAlignEnum align)
        {
            NativeCalls.godot_icall_1_835(method_bind_11, Object.GetPtr(this), align);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tab_align");

        [GodotMethod("get_tab_align")]
        public Tabs.TabAlignEnum GetTabAlign()
        {
            return NativeCalls.godot_icall_0_836(method_bind_12, Object.GetPtr(this));
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tab_offset");

        /// <summary>
        /// </summary>
        [GodotMethod("get_tab_offset")]
        public int GetTabOffset()
        {
            return NativeCalls.godot_icall_0_1(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_offset_buttons_visible");

        /// <summary>
        /// </summary>
        [GodotMethod("get_offset_buttons_visible")]
        public bool GetOffsetButtonsVisible()
        {
            return NativeCalls.godot_icall_0_3(method_bind_14, Object.GetPtr(this));
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "ensure_tab_visible");

        /// <summary>
        /// </summary>
        [GodotMethod("ensure_tab_visible")]
        public void EnsureTabVisible(int idx)
        {
            NativeCalls.godot_icall_1_0(method_bind_15, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tab_rect");

        /// <summary>
        /// Returns tab [Rect2] with local position and size.
        /// </summary>
        [GodotMethod("get_tab_rect")]
        public Rect2 GetTabRect(int tabIdx)
        {
            object ret = NativeCalls.godot_icall_1_452(method_bind_16, Object.GetPtr(this), tabIdx);
            return (Rect2)ret;
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "move_tab");

        /// <summary>
        /// Rearrange tab.
        /// </summary>
        [GodotMethod("move_tab")]
        public void MoveTab(int from, int to)
        {
            NativeCalls.godot_icall_2_32(method_bind_17, Object.GetPtr(this), from, to);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tab_close_display_policy");

        [GodotMethod("set_tab_close_display_policy")]
        public void SetTabCloseDisplayPolicy(Tabs.CloseButtonDisplayPolicy policy)
        {
            NativeCalls.godot_icall_1_837(method_bind_18, Object.GetPtr(this), policy);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tab_close_display_policy");

        [GodotMethod("get_tab_close_display_policy")]
        public Tabs.CloseButtonDisplayPolicy GetTabCloseDisplayPolicy()
        {
            return NativeCalls.godot_icall_0_838(method_bind_19, Object.GetPtr(this));
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_scrolling_enabled");

        [GodotMethod("set_scrolling_enabled")]
        public void SetScrollingEnabled(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_20, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_scrolling_enabled");

        [GodotMethod("get_scrolling_enabled")]
        public bool GetScrollingEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_21, Object.GetPtr(this));
        }
    }
}
