using System;

namespace Godot
{
    /// <summary>
    /// A texture works by registering an image in the video hardware, which then can be used in 3D models or 2D [Sprite] or GUI [Control].
    /// </summary>
    public class Texture : Resource
    {
        public enum FlagsEnum
        {
            sDefault = 7,
            Mipmaps = 1,
            Repeat = 2,
            Filter = 4,
            AnisotropicFilter = 8,
            ConvertToLinear = 16,
            MirroredRepeat = 32,
            VideoSurface = 4096
        }


        /// <summary>
        /// The texture's flags.
        /// </summary>
        public int Flags
        {
            get
            {
                return GetFlags();
            }
            set
            {
                SetFlags(value);
            }
        }


        private const string nativeName = "Texture";

        internal Texture() {}

        internal Texture(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_width");

        /// <summary>
        /// Return the texture width.
        /// </summary>
        [GodotMethod("get_width")]
        public int GetWidth()
        {
            return NativeCalls.godot_icall_0_1(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_height");

        /// <summary>
        /// Return the texture height.
        /// </summary>
        [GodotMethod("get_height")]
        public int GetHeight()
        {
            return NativeCalls.godot_icall_0_1(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_size");

        /// <summary>
        /// Return the texture size.
        /// </summary>
        [GodotMethod("get_size")]
        public Vector2 GetSize()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_2, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_alpha");

        /// <summary>
        /// </summary>
        [GodotMethod("has_alpha")]
        public bool HasAlpha()
        {
            return NativeCalls.godot_icall_0_3(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_flags");

        [GodotMethod("set_flags")]
        public void SetFlags(int flags)
        {
            NativeCalls.godot_icall_1_0(method_bind_4, Object.GetPtr(this), flags);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_flags");

        [GodotMethod("get_flags")]
        public int GetFlags()
        {
            return NativeCalls.godot_icall_0_1(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "draw");

        /// <summary>
        /// &lt;param name="modulate"&gt;If the param is null, then the default value is new Color(1, 1, 1, 1)&lt;/param&gt;
        /// </summary>
        [GodotMethod("draw")]
        public void Draw(RID canvasItem, Vector2 position, Nullable<Color> modulate = null, bool transpose = false, Texture normalMap = null)
        {
            Color modulate_in = modulate.HasValue ? modulate.Value : new Color(1, 1, 1, 1);
            NativeCalls.godot_icall_5_842(method_bind_6, Object.GetPtr(this), RID.GetPtr(canvasItem), ref position, ref modulate_in, transpose, Object.GetPtr(normalMap));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "draw_rect");

        /// <summary>
        /// &lt;param name="modulate"&gt;If the param is null, then the default value is new Color(1, 1, 1, 1)&lt;/param&gt;
        /// </summary>
        [GodotMethod("draw_rect")]
        public void DrawRect(RID canvasItem, Rect2 rect, bool tile, Nullable<Color> modulate = null, bool transpose = false, Texture normalMap = null)
        {
            Color modulate_in = modulate.HasValue ? modulate.Value : new Color(1, 1, 1, 1);
            NativeCalls.godot_icall_6_843(method_bind_7, Object.GetPtr(this), RID.GetPtr(canvasItem), ref rect, tile, ref modulate_in, transpose, Object.GetPtr(normalMap));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "draw_rect_region");

        /// <summary>
        /// &lt;param name="modulate"&gt;If the param is null, then the default value is new Color(1, 1, 1, 1)&lt;/param&gt;
        /// </summary>
        [GodotMethod("draw_rect_region")]
        public void DrawRectRegion(RID canvasItem, Rect2 rect, Rect2 srcRect, Nullable<Color> modulate = null, bool transpose = false, Texture normalMap = null, bool clipUv = true)
        {
            Color modulate_in = modulate.HasValue ? modulate.Value : new Color(1, 1, 1, 1);
            NativeCalls.godot_icall_7_844(method_bind_8, Object.GetPtr(this), RID.GetPtr(canvasItem), ref rect, ref srcRect, ref modulate_in, transpose, Object.GetPtr(normalMap), clipUv);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_data");

        /// <summary>
        /// </summary>
        [GodotMethod("get_data")]
        public Image GetData()
        {
            return NativeCalls.godot_icall_0_845(method_bind_9, Object.GetPtr(this));
        }
    }
}
