using System;

namespace Godot
{
    /// <summary>
    /// A unit of execution in a process. Can run methods on [Object]s simultaneously. The use of synchronization via [Mutex], [Semaphore] is advised if working with shared objects.
    /// </summary>
    public class Thread : Reference
    {
        public enum Priority
        {
            Low = 0,
            Normal = 1,
            High = 2
        }


        private const string nativeName = "_Thread";

        public Thread() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Thread_Ctor(this);
        }

        internal Thread(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "start");

        /// <summary>
        /// Starts a new [code]Thread[/code] that runs "method" on object "instance" with "userdata" passed as an argument. The "priority" of the [code]Thread[/code] can be changed by passing a PRIORITY_* enum.
        /// Returns OK on success, or ERR_CANT_CREATE on failure.
        /// </summary>
        [GodotMethod("start")]
        public Error Start(Object instance, string method, object userdata = null, int priority = 1)
        {
            return NativeCalls.godot_icall_4_1075(method_bind_0, Object.GetPtr(this), Object.GetPtr(instance), method, userdata, priority);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_id");

        /// <summary>
        /// Returns the current [code]Thread[/code]s id, uniquely identifying it among all threads.
        /// </summary>
        [GodotMethod("get_id")]
        public string GetId()
        {
            return NativeCalls.godot_icall_0_2(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_active");

        /// <summary>
        /// Returns true if this [code]Thread[/code] is currently active. An active [code]Thread[/code] cannot start work on a new method but can be joined with [method wait_to_finish].
        /// </summary>
        [GodotMethod("is_active")]
        public bool IsActive()
        {
            return NativeCalls.godot_icall_0_3(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "wait_to_finish");

        /// <summary>
        /// Joins the [code]Thread[/code] and waits for it to finish. Returns what the method called returned.
        /// </summary>
        [GodotMethod("wait_to_finish")]
        public object WaitToFinish()
        {
            return NativeCalls.godot_icall_0_461(method_bind_3, Object.GetPtr(this));
        }
    }
}
