using System;

namespace Godot
{
    /// <summary>
    /// A TileSet is a library of tiles for a [TileMap]. It contains a list of tiles, each consisting of a sprite and optional collision shapes.
    /// Tiles are referenced by a unique integer ID.
    /// </summary>
    public class TileSet : Resource
    {
        public enum AutotileBindings
        {
            Topleft = 1,
            Top = 2,
            Topright = 4,
            Left = 8,
            Right = 32,
            Bottomleft = 64,
            Bottom = 128,
            Bottomright = 256
        }

        public enum BitmaskMode
        {
            Bitmask2x2 = 0,
            Bitmask3x3 = 1
        }


        private const string nativeName = "TileSet";

        public TileSet() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_TileSet_Ctor(this);
        }

        internal TileSet(bool memoryOwn) : base(memoryOwn) {}

        /// <summary>
        /// </summary>
        [GodotMethod("_forward_subtile_selection")]
        public virtual Vector2 _ForwardSubtileSelection(int autotileId, int bitmask, Object tilemap, Vector2 tileLocation)
        {
            return default(Vector2);
        }

        /// <summary>
        /// </summary>
        [GodotMethod("_is_tile_bound")]
        public virtual bool _IsTileBound(int drawnId, int neighborId)
        {
            return default(bool);
        }

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "create_tile");

        /// <summary>
        /// Create a new tile which will be referenced by the given ID.
        /// </summary>
        [GodotMethod("create_tile")]
        public void CreateTile(int id)
        {
            NativeCalls.godot_icall_1_0(method_bind_0, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "autotile_set_bitmask_mode");

        /// <summary>
        /// </summary>
        [GodotMethod("autotile_set_bitmask_mode")]
        public void AutotileSetBitmaskMode(int id, TileSet.BitmaskMode mode)
        {
            NativeCalls.godot_icall_2_865(method_bind_1, Object.GetPtr(this), id, mode);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "autotile_get_bitmask_mode");

        /// <summary>
        /// </summary>
        [GodotMethod("autotile_get_bitmask_mode")]
        public TileSet.BitmaskMode AutotileGetBitmaskMode(int id)
        {
            return NativeCalls.godot_icall_1_866(method_bind_2, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_set_name");

        /// <summary>
        /// Set the name of the tile, for descriptive purposes.
        /// </summary>
        [GodotMethod("tile_set_name")]
        public void TileSetName(int id, string name)
        {
            NativeCalls.godot_icall_2_115(method_bind_3, Object.GetPtr(this), id, name);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_get_name");

        /// <summary>
        /// Return the name of the tile.
        /// </summary>
        [GodotMethod("tile_get_name")]
        public string TileGetName(int id)
        {
            return NativeCalls.godot_icall_1_108(method_bind_4, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_set_texture");

        /// <summary>
        /// Set the texture of the tile.
        /// </summary>
        [GodotMethod("tile_set_texture")]
        public void TileSetTexture(int id, Texture texture)
        {
            NativeCalls.godot_icall_2_67(method_bind_5, Object.GetPtr(this), id, Object.GetPtr(texture));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_get_texture");

        /// <summary>
        /// Return the texture of the tile.
        /// </summary>
        [GodotMethod("tile_get_texture")]
        public Texture TileGetTexture(int id)
        {
            return NativeCalls.godot_icall_1_157(method_bind_6, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_set_normal_map");

        /// <summary>
        /// </summary>
        [GodotMethod("tile_set_normal_map")]
        public void TileSetNormalMap(int id, Texture normalMap)
        {
            NativeCalls.godot_icall_2_67(method_bind_7, Object.GetPtr(this), id, Object.GetPtr(normalMap));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_get_normal_map");

        /// <summary>
        /// </summary>
        [GodotMethod("tile_get_normal_map")]
        public Texture TileGetNormalMap(int id)
        {
            return NativeCalls.godot_icall_1_157(method_bind_8, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_set_material");

        /// <summary>
        /// Set the material of the tile.
        /// </summary>
        [GodotMethod("tile_set_material")]
        public void TileSetMaterial(int id, ShaderMaterial material)
        {
            NativeCalls.godot_icall_2_67(method_bind_9, Object.GetPtr(this), id, Object.GetPtr(material));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_get_material");

        /// <summary>
        /// Return the material of the tile.
        /// </summary>
        [GodotMethod("tile_get_material")]
        public ShaderMaterial TileGetMaterial(int id)
        {
            return NativeCalls.godot_icall_1_867(method_bind_10, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_set_texture_offset");

        /// <summary>
        /// Set the texture offset of the tile.
        /// </summary>
        [GodotMethod("tile_set_texture_offset")]
        public void TileSetTextureOffset(int id, Vector2 textureOffset)
        {
            NativeCalls.godot_icall_2_288(method_bind_11, Object.GetPtr(this), id, ref textureOffset);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_get_texture_offset");

        /// <summary>
        /// Return the texture offset of the tile.
        /// </summary>
        [GodotMethod("tile_get_texture_offset")]
        public Vector2 TileGetTextureOffset(int id)
        {
            object ret = NativeCalls.godot_icall_1_282(method_bind_12, Object.GetPtr(this), id);
            return (Vector2)ret;
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_set_region");

        /// <summary>
        /// Set the tile sub-region in the texture. This is common in texture atlases.
        /// </summary>
        [GodotMethod("tile_set_region")]
        public void TileSetRegion(int id, Rect2 region)
        {
            NativeCalls.godot_icall_2_451(method_bind_13, Object.GetPtr(this), id, ref region);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_get_region");

        /// <summary>
        /// Return the tile sub-region in the texture.
        /// </summary>
        [GodotMethod("tile_get_region")]
        public Rect2 TileGetRegion(int id)
        {
            object ret = NativeCalls.godot_icall_1_452(method_bind_14, Object.GetPtr(this), id);
            return (Rect2)ret;
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_set_shape");

        /// <summary>
        /// </summary>
        [GodotMethod("tile_set_shape")]
        public void TileSetShape(int id, int shapeId, Shape2D shape)
        {
            NativeCalls.godot_icall_3_868(method_bind_15, Object.GetPtr(this), id, shapeId, Object.GetPtr(shape));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_get_shape");

        /// <summary>
        /// </summary>
        [GodotMethod("tile_get_shape")]
        public Shape2D TileGetShape(int id, int shapeId)
        {
            return NativeCalls.godot_icall_2_236(method_bind_16, Object.GetPtr(this), id, shapeId);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_set_shape_transform");

        /// <summary>
        /// </summary>
        [GodotMethod("tile_set_shape_transform")]
        public void TileSetShapeTransform(int id, int shapeId, Transform2D shapeTransform)
        {
            NativeCalls.godot_icall_3_869(method_bind_17, Object.GetPtr(this), id, shapeId, ref shapeTransform);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_get_shape_transform");

        /// <summary>
        /// </summary>
        [GodotMethod("tile_get_shape_transform")]
        public Transform2D TileGetShapeTransform(int id, int shapeId)
        {
            object ret = NativeCalls.godot_icall_2_870(method_bind_18, Object.GetPtr(this), id, shapeId);
            return (Transform2D)ret;
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_set_shape_one_way");

        /// <summary>
        /// </summary>
        [GodotMethod("tile_set_shape_one_way")]
        public void TileSetShapeOneWay(int id, int shapeId, bool oneWay)
        {
            NativeCalls.godot_icall_3_31(method_bind_19, Object.GetPtr(this), id, shapeId, oneWay);
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_get_shape_one_way");

        /// <summary>
        /// </summary>
        [GodotMethod("tile_get_shape_one_way")]
        public bool TileGetShapeOneWay(int id, int shapeId)
        {
            return NativeCalls.godot_icall_2_33(method_bind_20, Object.GetPtr(this), id, shapeId);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_add_shape");

        /// <summary>
        /// &lt;param name="autotileCoord"&gt;If the param is null, then the default value is new Vector2(0, 0)&lt;/param&gt;
        /// </summary>
        [GodotMethod("tile_add_shape")]
        public void TileAddShape(int id, Shape2D shape, Transform2D shapeTransform, bool oneWay = false, Nullable<Vector2> autotileCoord = null)
        {
            Vector2 autotileCoord_in = autotileCoord.HasValue ? autotileCoord.Value : new Vector2(0, 0);
            NativeCalls.godot_icall_5_871(method_bind_21, Object.GetPtr(this), id, Object.GetPtr(shape), ref shapeTransform, oneWay, ref autotileCoord_in);
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_get_shape_count");

        /// <summary>
        /// </summary>
        [GodotMethod("tile_get_shape_count")]
        public int TileGetShapeCount(int id)
        {
            return NativeCalls.godot_icall_1_6(method_bind_22, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_set_shapes");

        /// <summary>
        /// Set an array of shapes for the tile, enabling physics to collide with it.
        /// </summary>
        [GodotMethod("tile_set_shapes")]
        public void TileSetShapes(int id, object[] shapes)
        {
            NativeCalls.godot_icall_2_512(method_bind_23, Object.GetPtr(this), id, shapes);
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_get_shapes");

        /// <summary>
        /// Return the array of shapes of the tile.
        /// </summary>
        [GodotMethod("tile_get_shapes")]
        public object[] TileGetShapes(int id)
        {
            return NativeCalls.godot_icall_1_116(method_bind_24, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_set_navigation_polygon");

        /// <summary>
        /// Set a navigation polygon for the tile.
        /// </summary>
        [GodotMethod("tile_set_navigation_polygon")]
        public void TileSetNavigationPolygon(int id, NavigationPolygon navigationPolygon)
        {
            NativeCalls.godot_icall_2_67(method_bind_25, Object.GetPtr(this), id, Object.GetPtr(navigationPolygon));
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_get_navigation_polygon");

        /// <summary>
        /// Return the navigation polygon of the tile.
        /// </summary>
        [GodotMethod("tile_get_navigation_polygon")]
        public NavigationPolygon TileGetNavigationPolygon(int id)
        {
            return NativeCalls.godot_icall_1_872(method_bind_26, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_set_navigation_polygon_offset");

        /// <summary>
        /// Set an offset for the tile's navigation polygon.
        /// </summary>
        [GodotMethod("tile_set_navigation_polygon_offset")]
        public void TileSetNavigationPolygonOffset(int id, Vector2 navigationPolygonOffset)
        {
            NativeCalls.godot_icall_2_288(method_bind_27, Object.GetPtr(this), id, ref navigationPolygonOffset);
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_get_navigation_polygon_offset");

        /// <summary>
        /// Return the offset of the tile's navigation polygon.
        /// </summary>
        [GodotMethod("tile_get_navigation_polygon_offset")]
        public Vector2 TileGetNavigationPolygonOffset(int id)
        {
            object ret = NativeCalls.godot_icall_1_282(method_bind_28, Object.GetPtr(this), id);
            return (Vector2)ret;
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_set_light_occluder");

        /// <summary>
        /// Set a light occluder for the tile.
        /// </summary>
        [GodotMethod("tile_set_light_occluder")]
        public void TileSetLightOccluder(int id, OccluderPolygon2D lightOccluder)
        {
            NativeCalls.godot_icall_2_67(method_bind_29, Object.GetPtr(this), id, Object.GetPtr(lightOccluder));
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_get_light_occluder");

        /// <summary>
        /// Return the light occluder of the tile.
        /// </summary>
        [GodotMethod("tile_get_light_occluder")]
        public OccluderPolygon2D TileGetLightOccluder(int id)
        {
            return NativeCalls.godot_icall_1_873(method_bind_30, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_set_occluder_offset");

        /// <summary>
        /// Set an offset for the tile's light occluder.
        /// </summary>
        [GodotMethod("tile_set_occluder_offset")]
        public void TileSetOccluderOffset(int id, Vector2 occluderOffset)
        {
            NativeCalls.godot_icall_2_288(method_bind_31, Object.GetPtr(this), id, ref occluderOffset);
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tile_get_occluder_offset");

        /// <summary>
        /// Return the offset of the tile's light occluder.
        /// </summary>
        [GodotMethod("tile_get_occluder_offset")]
        public Vector2 TileGetOccluderOffset(int id)
        {
            object ret = NativeCalls.godot_icall_1_282(method_bind_32, Object.GetPtr(this), id);
            return (Vector2)ret;
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_tile");

        /// <summary>
        /// Remove the tile referenced by the given ID.
        /// </summary>
        [GodotMethod("remove_tile")]
        public void RemoveTile(int id)
        {
            NativeCalls.godot_icall_1_0(method_bind_33, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear");

        /// <summary>
        /// Clear all tiles.
        /// </summary>
        [GodotMethod("clear")]
        public void Clear()
        {
            NativeCalls.godot_icall_0_12(method_bind_34, Object.GetPtr(this));
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_last_unused_tile_id");

        /// <summary>
        /// Return the ID following the last currently used ID, useful when creating a new tile.
        /// </summary>
        [GodotMethod("get_last_unused_tile_id")]
        public int GetLastUnusedTileId()
        {
            return NativeCalls.godot_icall_0_1(method_bind_35, Object.GetPtr(this));
        }

        private IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "find_tile_by_name");

        /// <summary>
        /// Find the first tile matching the given name.
        /// </summary>
        [GodotMethod("find_tile_by_name")]
        public int FindTileByName(string name)
        {
            return NativeCalls.godot_icall_1_87(method_bind_36, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tiles_ids");

        /// <summary>
        /// Return an array of all currently used tile IDs.
        /// </summary>
        [GodotMethod("get_tiles_ids")]
        public object[] GetTilesIds()
        {
            return NativeCalls.godot_icall_0_21(method_bind_37, Object.GetPtr(this));
        }
    }
}
