using System;

namespace Godot
{
    /// <summary>
    /// Button for touch screen devices. You can set it to be visible on all screens, or only on touch devices.
    /// </summary>
    public class TouchScreenButton : Node2D
    {
        public enum VisibilityModeEnum
        {
            Always = 0,
            TouchscreenOnly = 1
        }


        /// <summary>
        /// The button's texture for the normal state.
        /// </summary>
        public Texture Normal
        {
            get
            {
                return GetTexture();
            }
            set
            {
                SetTexture(value);
            }
        }

        /// <summary>
        /// The button's texture for the pressed state.
        /// </summary>
        public Texture Pressed
        {
            get
            {
                return GetTexturePressed();
            }
            set
            {
                SetTexturePressed(value);
            }
        }

        /// <summary>
        /// The button's bitmask.
        /// </summary>
        public BitMap Bitmask
        {
            get
            {
                return GetBitmask();
            }
            set
            {
                SetBitmask(value);
            }
        }

        /// <summary>
        /// The button's shape.
        /// </summary>
        public Shape2D Shape
        {
            get
            {
                return GetShape();
            }
            set
            {
                SetShape(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the button's shape is centered.
        /// </summary>
        public bool ShapeCentered
        {
            get
            {
                return IsShapeCentered();
            }
            set
            {
                SetShapeCentered(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the button's shape is visible.
        /// </summary>
        public bool ShapeVisible
        {
            get
            {
                return IsShapeVisible();
            }
            set
            {
                SetShapeVisible(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] passby presses are enabled.
        /// </summary>
        public bool PassbyPress
        {
            get
            {
                return IsPassbyPressEnabled();
            }
            set
            {
                SetPassbyPress(value);
            }
        }

        /// <summary>
        /// The button's action. Actions can be handled with [InputEventAction].
        /// </summary>
        public string Action
        {
            get
            {
                return GetAction();
            }
            set
            {
                SetAction(value);
            }
        }

        /// <summary>
        /// The button's visibility mode. See [code]VISIBILITY_*[/code] constants.
        /// </summary>
        public TouchScreenButton.VisibilityModeEnum VisibilityMode
        {
            get
            {
                return GetVisibilityMode();
            }
            set
            {
                SetVisibilityMode(value);
            }
        }


        private const string nativeName = "TouchScreenButton";

        public TouchScreenButton() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_TouchScreenButton_Ctor(this);
        }

        internal TouchScreenButton(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_texture");

        [GodotMethod("set_texture")]
        public void SetTexture(Texture texture)
        {
            NativeCalls.godot_icall_1_24(method_bind_0, Object.GetPtr(this), Object.GetPtr(texture));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_texture");

        [GodotMethod("get_texture")]
        public Texture GetTexture()
        {
            return NativeCalls.godot_icall_0_119(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_texture_pressed");

        [GodotMethod("set_texture_pressed")]
        public void SetTexturePressed(Texture texturePressed)
        {
            NativeCalls.godot_icall_1_24(method_bind_2, Object.GetPtr(this), Object.GetPtr(texturePressed));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_texture_pressed");

        [GodotMethod("get_texture_pressed")]
        public Texture GetTexturePressed()
        {
            return NativeCalls.godot_icall_0_119(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bitmask");

        [GodotMethod("set_bitmask")]
        public void SetBitmask(BitMap bitmask)
        {
            NativeCalls.godot_icall_1_24(method_bind_4, Object.GetPtr(this), Object.GetPtr(bitmask));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bitmask");

        [GodotMethod("get_bitmask")]
        public BitMap GetBitmask()
        {
            return NativeCalls.godot_icall_0_847(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_shape");

        [GodotMethod("set_shape")]
        public void SetShape(Shape2D shape)
        {
            NativeCalls.godot_icall_1_24(method_bind_6, Object.GetPtr(this), Object.GetPtr(shape));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_shape");

        [GodotMethod("get_shape")]
        public Shape2D GetShape()
        {
            return NativeCalls.godot_icall_0_242(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_shape_centered");

        [GodotMethod("set_shape_centered")]
        public void SetShapeCentered(bool @bool)
        {
            NativeCalls.godot_icall_1_11(method_bind_8, Object.GetPtr(this), @bool);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_shape_centered");

        [GodotMethod("is_shape_centered")]
        public bool IsShapeCentered()
        {
            return NativeCalls.godot_icall_0_3(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_shape_visible");

        [GodotMethod("set_shape_visible")]
        public void SetShapeVisible(bool @bool)
        {
            NativeCalls.godot_icall_1_11(method_bind_10, Object.GetPtr(this), @bool);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_shape_visible");

        [GodotMethod("is_shape_visible")]
        public bool IsShapeVisible()
        {
            return NativeCalls.godot_icall_0_3(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_action");

        [GodotMethod("set_action")]
        public void SetAction(string action)
        {
            NativeCalls.godot_icall_1_43(method_bind_12, Object.GetPtr(this), action);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_action");

        [GodotMethod("get_action")]
        public string GetAction()
        {
            return NativeCalls.godot_icall_0_2(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_visibility_mode");

        [GodotMethod("set_visibility_mode")]
        public void SetVisibilityMode(TouchScreenButton.VisibilityModeEnum mode)
        {
            NativeCalls.godot_icall_1_876(method_bind_14, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_visibility_mode");

        [GodotMethod("get_visibility_mode")]
        public TouchScreenButton.VisibilityModeEnum GetVisibilityMode()
        {
            return NativeCalls.godot_icall_0_877(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_passby_press");

        [GodotMethod("set_passby_press")]
        public void SetPassbyPress(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_16, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_passby_press_enabled");

        [GodotMethod("is_passby_press_enabled")]
        public bool IsPassbyPressEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_17, Object.GetPtr(this));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_pressed");

        /// <summary>
        /// Returns [code]true[/code] if this button is currently pressed.
        /// </summary>
        [GodotMethod("is_pressed")]
        public bool IsPressed()
        {
            return NativeCalls.godot_icall_0_3(method_bind_18, Object.GetPtr(this));
        }
    }
}
