using System;

namespace Godot
{
    /// <summary>
    /// Translations are resources that can be loaded/unloaded on demand. They map a string to another string.
    /// </summary>
    public class Translation : Resource
    {
        public string[] Messages
        {
            get
            {
                return _GetMessages();
            }
            set
            {
                _SetMessages(value);
            }
        }

        /// <summary>
        /// </summary>
        public string Locale
        {
            get
            {
                return GetLocale();
            }
            set
            {
                SetLocale(value);
            }
        }


        private const string nativeName = "Translation";

        public Translation() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Translation_Ctor(this);
        }

        internal Translation(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_locale");

        [GodotMethod("set_locale")]
        public void SetLocale(string locale)
        {
            NativeCalls.godot_icall_1_43(method_bind_0, Object.GetPtr(this), locale);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_locale");

        [GodotMethod("get_locale")]
        public string GetLocale()
        {
            return NativeCalls.godot_icall_0_2(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_message");

        /// <summary>
        /// Add a message for translation.
        /// </summary>
        [GodotMethod("add_message")]
        public void AddMessage(string srcMessage, string xlatedMessage)
        {
            NativeCalls.godot_icall_2_69(method_bind_2, Object.GetPtr(this), srcMessage, xlatedMessage);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_message");

        /// <summary>
        /// Return a message for translation.
        /// </summary>
        [GodotMethod("get_message")]
        public string GetMessage(string srcMessage)
        {
            return NativeCalls.godot_icall_1_73(method_bind_3, Object.GetPtr(this), srcMessage);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "erase_message");

        /// <summary>
        /// Erase a message.
        /// </summary>
        [GodotMethod("erase_message")]
        public void EraseMessage(string srcMessage)
        {
            NativeCalls.godot_icall_1_43(method_bind_4, Object.GetPtr(this), srcMessage);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_message_list");

        /// <summary>
        /// Return all the messages (keys).
        /// </summary>
        [GodotMethod("get_message_list")]
        public string[] GetMessageList()
        {
            return NativeCalls.godot_icall_0_72(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_message_count");

        /// <summary>
        /// </summary>
        [GodotMethod("get_message_count")]
        public int GetMessageCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_6, Object.GetPtr(this));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_messages");

        internal void _SetMessages(string[] arg0)
        {
            NativeCalls.godot_icall_1_340(method_bind_7, Object.GetPtr(this), arg0);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_messages");

        internal string[] _GetMessages()
        {
            return NativeCalls.godot_icall_0_72(method_bind_8, Object.GetPtr(this));
        }
    }
}
