using System;
using System.Collections.Generic;

namespace Godot
{
    /// <summary>
    /// Control for a single item inside a [Tree]. May have child [code]TreeItem[/code]s and be styled as well as contain buttons.
    /// </summary>
    public class TreeItem : Object
    {
        public enum TreeCellMode
        {
            String = 0,
            Check = 1,
            Range = 2,
            RangeExpression = 3,
            Icon = 4,
            Custom = 5
        }

        public enum TextAlign
        {
            Left = 0,
            Center = 1,
            Right = 2
        }


        /// <summary>
        /// If [code]true[/code] the TreeItem is collapsed.
        /// </summary>
        public bool Collapsed
        {
            get
            {
                return IsCollapsed();
            }
            set
            {
                SetCollapsed(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] folding is disabled for this TreeItem.
        /// </summary>
        public bool DisableFolding
        {
            get
            {
                return IsFoldingDisabled();
            }
            set
            {
                SetDisableFolding(value);
            }
        }

        /// <summary>
        /// The custom minimum height.
        /// </summary>
        public int CustomMinimumHeight
        {
            get
            {
                return GetCustomMinimumHeight();
            }
            set
            {
                SetCustomMinimumHeight(value);
            }
        }


        private const string nativeName = "TreeItem";

        internal TreeItem() {}

        internal TreeItem(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_cell_mode");

        /// <summary>
        /// Sets the given column's cell mode to [code]mode[/code]. See [code]CELL_MODE_*[/code] constants.
        /// </summary>
        [GodotMethod("set_cell_mode")]
        public void SetCellMode(int column, TreeItem.TreeCellMode mode)
        {
            NativeCalls.godot_icall_2_885(method_bind_0, Object.GetPtr(this), column, mode);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_cell_mode");

        /// <summary>
        /// Returns the column's cell mode. See [code]CELL_MODE_*[/code] constants.
        /// </summary>
        [GodotMethod("get_cell_mode")]
        public TreeItem.TreeCellMode GetCellMode(int column)
        {
            return NativeCalls.godot_icall_1_886(method_bind_1, Object.GetPtr(this), column);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_checked");

        /// <summary>
        /// If [code]true[/code] the column [code]column[/code] is checked.
        /// </summary>
        [GodotMethod("set_checked")]
        public void SetChecked(int column, bool @checked)
        {
            NativeCalls.godot_icall_2_51(method_bind_2, Object.GetPtr(this), column, @checked);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_checked");

        /// <summary>
        /// Returns [code]true[/code] if the given column is checked.
        /// </summary>
        [GodotMethod("is_checked")]
        public bool IsChecked(int column)
        {
            return NativeCalls.godot_icall_1_29(method_bind_3, Object.GetPtr(this), column);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_text");

        /// <summary>
        /// </summary>
        [GodotMethod("set_text")]
        public void SetText(int column, string text)
        {
            NativeCalls.godot_icall_2_115(method_bind_4, Object.GetPtr(this), column, text);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_text");

        /// <summary>
        /// Returns the given column's text.
        /// </summary>
        [GodotMethod("get_text")]
        public string GetText(int column)
        {
            return NativeCalls.godot_icall_1_108(method_bind_5, Object.GetPtr(this), column);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_icon");

        /// <summary>
        /// Sets the given column's icon [Texture].
        /// </summary>
        [GodotMethod("set_icon")]
        public void SetIcon(int column, Texture texture)
        {
            NativeCalls.godot_icall_2_67(method_bind_6, Object.GetPtr(this), column, Object.GetPtr(texture));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_icon");

        /// <summary>
        /// Returns the given column's icon [Texture]. Error if no icon is set.
        /// </summary>
        [GodotMethod("get_icon")]
        public Texture GetIcon(int column)
        {
            return NativeCalls.godot_icall_1_157(method_bind_7, Object.GetPtr(this), column);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_icon_region");

        /// <summary>
        /// Sets the given column's icon's texture region.
        /// </summary>
        [GodotMethod("set_icon_region")]
        public void SetIconRegion(int column, Rect2 region)
        {
            NativeCalls.godot_icall_2_451(method_bind_8, Object.GetPtr(this), column, ref region);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_icon_region");

        /// <summary>
        /// Returns the icon [Texture] region as [Rect2].
        /// </summary>
        [GodotMethod("get_icon_region")]
        public Rect2 GetIconRegion(int column)
        {
            object ret = NativeCalls.godot_icall_1_452(method_bind_9, Object.GetPtr(this), column);
            return (Rect2)ret;
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_icon_max_width");

        /// <summary>
        /// Sets the given column's icon's maximum width.
        /// </summary>
        [GodotMethod("set_icon_max_width")]
        public void SetIconMaxWidth(int column, int width)
        {
            NativeCalls.godot_icall_2_32(method_bind_10, Object.GetPtr(this), column, width);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_icon_max_width");

        /// <summary>
        /// Returns the column's icon's maximum width.
        /// </summary>
        [GodotMethod("get_icon_max_width")]
        public int GetIconMaxWidth(int column)
        {
            return NativeCalls.godot_icall_1_6(method_bind_11, Object.GetPtr(this), column);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_range");

        /// <summary>
        /// </summary>
        [GodotMethod("set_range")]
        public void SetRange(int column, float value)
        {
            NativeCalls.godot_icall_2_28(method_bind_12, Object.GetPtr(this), column, value);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_range");

        /// <summary>
        /// </summary>
        [GodotMethod("get_range")]
        public float GetRange(int column)
        {
            return NativeCalls.godot_icall_1_7(method_bind_13, Object.GetPtr(this), column);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_range_config");

        /// <summary>
        /// </summary>
        [GodotMethod("set_range_config")]
        public void SetRangeConfig(int column, float min, float max, float step, bool expr = false)
        {
            NativeCalls.godot_icall_5_887(method_bind_14, Object.GetPtr(this), column, min, max, step, expr);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_range_config");

        /// <summary>
        /// </summary>
        [GodotMethod("get_range_config")]
        public Dictionary<object, object> GetRangeConfig(int column)
        {
            return NativeCalls.godot_icall_1_888(method_bind_15, Object.GetPtr(this), column);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_metadata");

        /// <summary>
        /// </summary>
        [GodotMethod("set_metadata")]
        public void SetMetadata(int column, object meta)
        {
            NativeCalls.godot_icall_2_453(method_bind_16, Object.GetPtr(this), column, meta);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_metadata");

        /// <summary>
        /// </summary>
        [GodotMethod("get_metadata")]
        public object GetMetadata(int column)
        {
            return NativeCalls.godot_icall_1_454(method_bind_17, Object.GetPtr(this), column);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_custom_draw");

        /// <summary>
        /// Sets the given column's custom draw callback to [code]callback[/code] method on [code]object[/code].
        /// </summary>
        [GodotMethod("set_custom_draw")]
        public void SetCustomDraw(int column, Object @object, string callback)
        {
            NativeCalls.godot_icall_3_889(method_bind_18, Object.GetPtr(this), column, Object.GetPtr(@object), callback);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_collapsed");

        [GodotMethod("set_collapsed")]
        public void SetCollapsed(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_19, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_collapsed");

        [GodotMethod("is_collapsed")]
        public bool IsCollapsed()
        {
            return NativeCalls.godot_icall_0_3(method_bind_20, Object.GetPtr(this));
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_custom_minimum_height");

        [GodotMethod("set_custom_minimum_height")]
        public void SetCustomMinimumHeight(int height)
        {
            NativeCalls.godot_icall_1_0(method_bind_21, Object.GetPtr(this), height);
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_custom_minimum_height");

        [GodotMethod("get_custom_minimum_height")]
        public int GetCustomMinimumHeight()
        {
            return NativeCalls.godot_icall_0_1(method_bind_22, Object.GetPtr(this));
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_next");

        /// <summary>
        /// Returns the next TreeItem in the tree.
        /// </summary>
        [GodotMethod("get_next")]
        public TreeItem GetNext()
        {
            return NativeCalls.godot_icall_0_879(method_bind_23, Object.GetPtr(this));
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_prev");

        /// <summary>
        /// Returns the previous TreeItem in the tree.
        /// </summary>
        [GodotMethod("get_prev")]
        public TreeItem GetPrev()
        {
            return NativeCalls.godot_icall_0_879(method_bind_24, Object.GetPtr(this));
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_parent");

        /// <summary>
        /// Returns the parent TreeItem.
        /// </summary>
        [GodotMethod("get_parent")]
        public TreeItem GetParent()
        {
            return NativeCalls.godot_icall_0_879(method_bind_25, Object.GetPtr(this));
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_children");

        /// <summary>
        /// Returns the TreeItem's child items.
        /// </summary>
        [GodotMethod("get_children")]
        public TreeItem GetChildren()
        {
            return NativeCalls.godot_icall_0_879(method_bind_26, Object.GetPtr(this));
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_next_visible");

        /// <summary>
        /// Returns the next visible TreeItem in the tree.
        /// </summary>
        [GodotMethod("get_next_visible")]
        public TreeItem GetNextVisible()
        {
            return NativeCalls.godot_icall_0_879(method_bind_27, Object.GetPtr(this));
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_prev_visible");

        /// <summary>
        /// Returns the previous visible TreeItem in the tree.
        /// </summary>
        [GodotMethod("get_prev_visible")]
        public TreeItem GetPrevVisible()
        {
            return NativeCalls.godot_icall_0_879(method_bind_28, Object.GetPtr(this));
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_child");

        /// <summary>
        /// Removes the child TreeItem at index [code]index[/code].
        /// </summary>
        [GodotMethod("remove_child")]
        public void RemoveChild(Object child)
        {
            NativeCalls.godot_icall_1_42(method_bind_29, Object.GetPtr(this), Object.GetPtr(child));
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_selectable");

        /// <summary>
        /// If [code]true[/code] the given column is selectable.
        /// </summary>
        [GodotMethod("set_selectable")]
        public void SetSelectable(int column, bool selectable)
        {
            NativeCalls.godot_icall_2_51(method_bind_30, Object.GetPtr(this), column, selectable);
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_selectable");

        /// <summary>
        /// Returns [code]true[/code] if column [code]column[/code] is selectable.
        /// </summary>
        [GodotMethod("is_selectable")]
        public bool IsSelectable(int column)
        {
            return NativeCalls.godot_icall_1_29(method_bind_31, Object.GetPtr(this), column);
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_selected");

        /// <summary>
        /// Returns [code]true[/code] if column [code]column[/code] is selected.
        /// </summary>
        [GodotMethod("is_selected")]
        public bool IsSelected(int column)
        {
            return NativeCalls.godot_icall_1_29(method_bind_32, Object.GetPtr(this), column);
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "select");

        /// <summary>
        /// Selects the column [code]column[/code].
        /// </summary>
        [GodotMethod("select")]
        public void Select(int column)
        {
            NativeCalls.godot_icall_1_0(method_bind_33, Object.GetPtr(this), column);
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "deselect");

        /// <summary>
        /// Deselects the given column.
        /// </summary>
        [GodotMethod("deselect")]
        public void Deselect(int column)
        {
            NativeCalls.godot_icall_1_0(method_bind_34, Object.GetPtr(this), column);
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_editable");

        /// <summary>
        /// If [code]true[/code] column [code]column[/code] is editable.
        /// </summary>
        [GodotMethod("set_editable")]
        public void SetEditable(int column, bool enabled)
        {
            NativeCalls.godot_icall_2_51(method_bind_35, Object.GetPtr(this), column, enabled);
        }

        private IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_editable");

        /// <summary>
        /// Returns [code]true[/code] if column [code]column[/code] is editable.
        /// </summary>
        [GodotMethod("is_editable")]
        public bool IsEditable(int column)
        {
            return NativeCalls.godot_icall_1_29(method_bind_36, Object.GetPtr(this), column);
        }

        private IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_custom_color");

        /// <summary>
        /// Sets the given column's custom color.
        /// </summary>
        [GodotMethod("set_custom_color")]
        public void SetCustomColor(int column, Color color)
        {
            NativeCalls.godot_icall_2_384(method_bind_37, Object.GetPtr(this), column, ref color);
        }

        private IntPtr method_bind_38 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear_custom_color");

        /// <summary>
        /// Resets the color for the given column to default.
        /// </summary>
        [GodotMethod("clear_custom_color")]
        public void ClearCustomColor(int column)
        {
            NativeCalls.godot_icall_1_0(method_bind_38, Object.GetPtr(this), column);
        }

        private IntPtr method_bind_39 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_custom_bg_color");

        /// <summary>
        /// Sets the given column's custom background color and whether to just use it as an outline.
        /// </summary>
        [GodotMethod("set_custom_bg_color")]
        public void SetCustomBgColor(int column, Color color, bool justOutline = false)
        {
            NativeCalls.godot_icall_3_890(method_bind_39, Object.GetPtr(this), column, ref color, justOutline);
        }

        private IntPtr method_bind_40 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear_custom_bg_color");

        /// <summary>
        /// Resets the background color for the given column to default.
        /// </summary>
        [GodotMethod("clear_custom_bg_color")]
        public void ClearCustomBgColor(int column)
        {
            NativeCalls.godot_icall_1_0(method_bind_40, Object.GetPtr(this), column);
        }

        private IntPtr method_bind_41 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_custom_bg_color");

        /// <summary>
        /// Returns the custom background color of column [code]column[/code].
        /// </summary>
        [GodotMethod("get_custom_bg_color")]
        public Color GetCustomBgColor(int column)
        {
            object ret = NativeCalls.godot_icall_1_385(method_bind_41, Object.GetPtr(this), column);
            return (Color)ret;
        }

        private IntPtr method_bind_42 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_custom_as_button");

        /// <summary>
        /// </summary>
        [GodotMethod("set_custom_as_button")]
        public void SetCustomAsButton(int column, bool enable)
        {
            NativeCalls.godot_icall_2_51(method_bind_42, Object.GetPtr(this), column, enable);
        }

        private IntPtr method_bind_43 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_custom_set_as_button");

        /// <summary>
        /// </summary>
        [GodotMethod("is_custom_set_as_button")]
        public bool IsCustomSetAsButton(int column)
        {
            return NativeCalls.godot_icall_1_29(method_bind_43, Object.GetPtr(this), column);
        }

        private IntPtr method_bind_44 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_button");

        /// <summary>
        /// Adds a button with [Texture] [code]button[/code] at column [code]column[/code]. The [code]button_idx[/code] index is used to identify the button when calling other methods. If not specified, the next available index is used, which may be retrieved by calling [code]get_buton_count()[/code] immediately after this method. Optionally, the button can be [code]disabled[/code] and have a [code]tooltip[/code].
        /// </summary>
        [GodotMethod("add_button")]
        public void AddButton(int column, Texture button, int buttonIdx = -1, bool disabled = false, string tooltip = "")
        {
            NativeCalls.godot_icall_5_891(method_bind_44, Object.GetPtr(this), column, Object.GetPtr(button), buttonIdx, disabled, tooltip);
        }

        private IntPtr method_bind_45 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_button_count");

        /// <summary>
        /// Returns the number of buttons in column [code]column[/code]. May be used to get the most recently added button's index, if no index was specified.
        /// </summary>
        [GodotMethod("get_button_count")]
        public int GetButtonCount(int column)
        {
            return NativeCalls.godot_icall_1_6(method_bind_45, Object.GetPtr(this), column);
        }

        private IntPtr method_bind_46 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_button");

        /// <summary>
        /// Returns the [Texture] of the button at index [code]button_idx[/code] in column [code]column[/code].
        /// </summary>
        [GodotMethod("get_button")]
        public Texture GetButton(int column, int buttonIdx)
        {
            return NativeCalls.godot_icall_2_892(method_bind_46, Object.GetPtr(this), column, buttonIdx);
        }

        private IntPtr method_bind_47 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_button");

        /// <summary>
        /// Sets the given column's button [Texture] at index [code]button_idx[/code] to [code]button[/code].
        /// </summary>
        [GodotMethod("set_button")]
        public void SetButton(int column, int buttonIdx, Texture button)
        {
            NativeCalls.godot_icall_3_868(method_bind_47, Object.GetPtr(this), column, buttonIdx, Object.GetPtr(button));
        }

        private IntPtr method_bind_48 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "erase_button");

        /// <summary>
        /// Removes the button at index [code]button_idx[/code] in column [code]column[/code].
        /// </summary>
        [GodotMethod("erase_button")]
        public void EraseButton(int column, int buttonIdx)
        {
            NativeCalls.godot_icall_2_32(method_bind_48, Object.GetPtr(this), column, buttonIdx);
        }

        private IntPtr method_bind_49 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_button_disabled");

        /// <summary>
        /// Returns [code]true[/code] if the button at index [code]button_idx[/code] for the given column is disabled.
        /// </summary>
        [GodotMethod("is_button_disabled")]
        public bool IsButtonDisabled(int column, int buttonIdx)
        {
            return NativeCalls.godot_icall_2_33(method_bind_49, Object.GetPtr(this), column, buttonIdx);
        }

        private IntPtr method_bind_50 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_expand_right");

        /// <summary>
        /// If [code]true[/code] column [code]column[/code] is expanded to the right.
        /// </summary>
        [GodotMethod("set_expand_right")]
        public void SetExpandRight(int column, bool enable)
        {
            NativeCalls.godot_icall_2_51(method_bind_50, Object.GetPtr(this), column, enable);
        }

        private IntPtr method_bind_51 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_expand_right");

        /// <summary>
        /// Returns [code]true[/code] if [code]expand_right[/code] is set.
        /// </summary>
        [GodotMethod("get_expand_right")]
        public bool GetExpandRight(int column)
        {
            return NativeCalls.godot_icall_1_29(method_bind_51, Object.GetPtr(this), column);
        }

        private IntPtr method_bind_52 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tooltip");

        /// <summary>
        /// Sets the given column's tooltip text.
        /// </summary>
        [GodotMethod("set_tooltip")]
        public void SetTooltip(int column, string tooltip)
        {
            NativeCalls.godot_icall_2_115(method_bind_52, Object.GetPtr(this), column, tooltip);
        }

        private IntPtr method_bind_53 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tooltip");

        /// <summary>
        /// Returns the given column's tooltip.
        /// </summary>
        [GodotMethod("get_tooltip")]
        public string GetTooltip(int column)
        {
            return NativeCalls.godot_icall_1_108(method_bind_53, Object.GetPtr(this), column);
        }

        private IntPtr method_bind_54 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_text_align");

        /// <summary>
        /// Sets the given column's text alignment. See [code]ALIGN_*[/code] constants.
        /// </summary>
        [GodotMethod("set_text_align")]
        public void SetTextAlign(int column, TreeItem.TextAlign textAlign)
        {
            NativeCalls.godot_icall_2_893(method_bind_54, Object.GetPtr(this), column, textAlign);
        }

        private IntPtr method_bind_55 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_text_align");

        /// <summary>
        /// Returns the given column's text alignment.
        /// </summary>
        [GodotMethod("get_text_align")]
        public TreeItem.TextAlign GetTextAlign(int column)
        {
            return NativeCalls.godot_icall_1_894(method_bind_55, Object.GetPtr(this), column);
        }

        private IntPtr method_bind_56 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "move_to_top");

        /// <summary>
        /// Moves this TreeItem to the top in the [Tree] hierarchy.
        /// </summary>
        [GodotMethod("move_to_top")]
        public void MoveToTop()
        {
            NativeCalls.godot_icall_0_12(method_bind_56, Object.GetPtr(this));
        }

        private IntPtr method_bind_57 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "move_to_bottom");

        /// <summary>
        /// Moves this TreeItem to the bottom in the [Tree] hierarchy.
        /// </summary>
        [GodotMethod("move_to_bottom")]
        public void MoveToBottom()
        {
            NativeCalls.godot_icall_0_12(method_bind_57, Object.GetPtr(this));
        }

        private IntPtr method_bind_58 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_disable_folding");

        [GodotMethod("set_disable_folding")]
        public void SetDisableFolding(bool disable)
        {
            NativeCalls.godot_icall_1_11(method_bind_58, Object.GetPtr(this), disable);
        }

        private IntPtr method_bind_59 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_folding_disabled");

        [GodotMethod("is_folding_disabled")]
        public bool IsFoldingDisabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_59, Object.GetPtr(this));
        }
    }
}
