using System;

namespace Godot
{
    /// <summary>
    /// This control has the ability to play video streams. The only format accepted is the OGV Theora, so any other format must be converted before using in a project.
    /// </summary>
    public class VideoPlayer : Control
    {
        /// <summary>
        /// </summary>
        public int AudioTrack
        {
            get
            {
                return GetAudioTrack();
            }
            set
            {
                SetAudioTrack(value);
            }
        }

        /// <summary>
        /// </summary>
        public VideoStream Stream
        {
            get
            {
                return GetStream();
            }
            set
            {
                SetStream(value);
            }
        }

        /// <summary>
        /// </summary>
        public float VolumeDb
        {
            get
            {
                return GetVolumeDb();
            }
            set
            {
                SetVolumeDb(value);
            }
        }

        /// <summary>
        /// The volume of the audio track as a linear value.
        /// </summary>
        public float Volume
        {
            get
            {
                return GetVolume();
            }
            set
            {
                SetVolume(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool Autoplay
        {
            get
            {
                return HasAutoplay();
            }
            set
            {
                SetAutoplay(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool Paused
        {
            get
            {
                return IsPaused();
            }
            set
            {
                SetPaused(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool Expand
        {
            get
            {
                return HasExpand();
            }
            set
            {
                SetExpand(value);
            }
        }

        /// <summary>
        /// The amount of milliseconds to store in buffer while playing.
        /// </summary>
        public int BufferingMsec
        {
            get
            {
                return GetBufferingMsec();
            }
            set
            {
                SetBufferingMsec(value);
            }
        }

        /// <summary>
        /// The current position of the stream, in seconds.
        /// </summary>
        public float StreamPosition
        {
            get
            {
                return GetStreamPosition();
            }
            set
            {
                SetStreamPosition(value);
            }
        }

        /// <summary>
        /// </summary>
        public string Bus
        {
            get
            {
                return GetBus();
            }
            set
            {
                SetBus(value);
            }
        }


        private const string nativeName = "VideoPlayer";

        public VideoPlayer() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_VideoPlayer_Ctor(this);
        }

        internal VideoPlayer(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_stream");

        [GodotMethod("set_stream")]
        public void SetStream(VideoStream stream)
        {
            NativeCalls.godot_icall_1_24(method_bind_0, Object.GetPtr(this), Object.GetPtr(stream));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_stream");

        [GodotMethod("get_stream")]
        public VideoStream GetStream()
        {
            return NativeCalls.godot_icall_0_908(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "play");

        /// <summary>
        /// Start the video playback.
        /// </summary>
        [GodotMethod("play")]
        public void Play()
        {
            NativeCalls.godot_icall_0_12(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "stop");

        /// <summary>
        /// Stop the video playback.
        /// </summary>
        [GodotMethod("stop")]
        public void Stop()
        {
            NativeCalls.godot_icall_0_12(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_playing");

        /// <summary>
        /// Get whether or not the video is playing.
        /// </summary>
        [GodotMethod("is_playing")]
        public bool IsPlaying()
        {
            return NativeCalls.godot_icall_0_3(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_paused");

        [GodotMethod("set_paused")]
        public void SetPaused(bool paused)
        {
            NativeCalls.godot_icall_1_11(method_bind_5, Object.GetPtr(this), paused);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_paused");

        [GodotMethod("is_paused")]
        public bool IsPaused()
        {
            return NativeCalls.godot_icall_0_3(method_bind_6, Object.GetPtr(this));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_volume");

        [GodotMethod("set_volume")]
        public void SetVolume(float volume)
        {
            NativeCalls.godot_icall_1_10(method_bind_7, Object.GetPtr(this), volume);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_volume");

        [GodotMethod("get_volume")]
        public float GetVolume()
        {
            return NativeCalls.godot_icall_0_9(method_bind_8, Object.GetPtr(this));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_volume_db");

        [GodotMethod("set_volume_db")]
        public void SetVolumeDb(float db)
        {
            NativeCalls.godot_icall_1_10(method_bind_9, Object.GetPtr(this), db);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_volume_db");

        [GodotMethod("get_volume_db")]
        public float GetVolumeDb()
        {
            return NativeCalls.godot_icall_0_9(method_bind_10, Object.GetPtr(this));
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_audio_track");

        [GodotMethod("set_audio_track")]
        public void SetAudioTrack(int track)
        {
            NativeCalls.godot_icall_1_0(method_bind_11, Object.GetPtr(this), track);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_audio_track");

        [GodotMethod("get_audio_track")]
        public int GetAudioTrack()
        {
            return NativeCalls.godot_icall_0_1(method_bind_12, Object.GetPtr(this));
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_stream_name");

        /// <summary>
        /// Get the name of the video stream.
        /// </summary>
        [GodotMethod("get_stream_name")]
        public string GetStreamName()
        {
            return NativeCalls.godot_icall_0_2(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_stream_position");

        [GodotMethod("set_stream_position")]
        public void SetStreamPosition(float position)
        {
            NativeCalls.godot_icall_1_10(method_bind_14, Object.GetPtr(this), position);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_stream_position");

        [GodotMethod("get_stream_position")]
        public float GetStreamPosition()
        {
            return NativeCalls.godot_icall_0_9(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_autoplay");

        [GodotMethod("set_autoplay")]
        public void SetAutoplay(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_16, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_autoplay");

        [GodotMethod("has_autoplay")]
        public bool HasAutoplay()
        {
            return NativeCalls.godot_icall_0_3(method_bind_17, Object.GetPtr(this));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_expand");

        [GodotMethod("set_expand")]
        public void SetExpand(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_18, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_expand");

        [GodotMethod("has_expand")]
        public bool HasExpand()
        {
            return NativeCalls.godot_icall_0_3(method_bind_19, Object.GetPtr(this));
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_buffering_msec");

        [GodotMethod("set_buffering_msec")]
        public void SetBufferingMsec(int msec)
        {
            NativeCalls.godot_icall_1_0(method_bind_20, Object.GetPtr(this), msec);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_buffering_msec");

        [GodotMethod("get_buffering_msec")]
        public int GetBufferingMsec()
        {
            return NativeCalls.godot_icall_0_1(method_bind_21, Object.GetPtr(this));
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bus");

        [GodotMethod("set_bus")]
        public void SetBus(string bus)
        {
            NativeCalls.godot_icall_1_43(method_bind_22, Object.GetPtr(this), bus);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bus");

        [GodotMethod("get_bus")]
        public string GetBus()
        {
            return NativeCalls.godot_icall_0_2(method_bind_23, Object.GetPtr(this));
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_video_texture");

        /// <summary>
        /// Get the current frame of the video as a [Texture].
        /// </summary>
        [GodotMethod("get_video_texture")]
        public Texture GetVideoTexture()
        {
            return NativeCalls.godot_icall_0_119(method_bind_24, Object.GetPtr(this));
        }
    }
}
