using System;

namespace Godot
{
    /// <summary>
    /// The VisibilityEnabler will disable [RigidBody] and [AnimationPlayer] nodes when they are not visible. It will only affect other nodes within the same scene as the VisibilityEnabler itself.
    /// </summary>
    public class VisibilityEnabler : VisibilityNotifier
    {
        public enum Enabler
        {
            PauseAnimations = 0,
            FreezeBodies = 1,
            Max = 2
        }


        /// <summary>
        /// </summary>
        public bool PauseAnimations
        {
            get
            {
                return IsEnablerEnabled((VisibilityEnabler.Enabler)0);
            }
            set
            {
                SetEnabler((VisibilityEnabler.Enabler)0, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool FreezeBodies
        {
            get
            {
                return IsEnablerEnabled((VisibilityEnabler.Enabler)1);
            }
            set
            {
                SetEnabler((VisibilityEnabler.Enabler)1, value);
            }
        }


        private const string nativeName = "VisibilityEnabler";

        public VisibilityEnabler() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_VisibilityEnabler_Ctor(this);
        }

        internal VisibilityEnabler(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_enabler");

        [GodotMethod("set_enabler")]
        public void SetEnabler(VisibilityEnabler.Enabler enabler, bool enabled)
        {
            NativeCalls.godot_icall_2_925(method_bind_0, Object.GetPtr(this), enabler, enabled);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_enabler_enabled");

        [GodotMethod("is_enabler_enabled")]
        public bool IsEnablerEnabled(VisibilityEnabler.Enabler enabler)
        {
            return NativeCalls.godot_icall_1_926(method_bind_1, Object.GetPtr(this), enabler);
        }
    }
}
