using System;

namespace Godot
{
    /// <summary>
    /// The VisibilityEnabler2D will disable [RigidBody2D], [AnimationPlayer], and other nodes when they are not visible. It will only affect other nodes within the same scene as the VisibilityEnabler2D itself.
    /// </summary>
    public class VisibilityEnabler2D : VisibilityNotifier2D
    {
        public enum Enabler
        {
            PauseAnimations = 0,
            FreezeBodies = 1,
            PauseParticles = 2,
            ParentProcess = 3,
            ParentPhysicsProcess = 4,
            PauseAnimatedSprites = 5,
            Max = 6
        }


        /// <summary>
        /// </summary>
        public bool PauseAnimations
        {
            get
            {
                return IsEnablerEnabled((VisibilityEnabler2D.Enabler)0);
            }
            set
            {
                SetEnabler((VisibilityEnabler2D.Enabler)0, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool FreezeBodies
        {
            get
            {
                return IsEnablerEnabled((VisibilityEnabler2D.Enabler)1);
            }
            set
            {
                SetEnabler((VisibilityEnabler2D.Enabler)1, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool PauseParticles
        {
            get
            {
                return IsEnablerEnabled((VisibilityEnabler2D.Enabler)2);
            }
            set
            {
                SetEnabler((VisibilityEnabler2D.Enabler)2, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool PauseAnimatedSprites
        {
            get
            {
                return IsEnablerEnabled((VisibilityEnabler2D.Enabler)5);
            }
            set
            {
                SetEnabler((VisibilityEnabler2D.Enabler)5, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool ProcessParent
        {
            get
            {
                return IsEnablerEnabled((VisibilityEnabler2D.Enabler)3);
            }
            set
            {
                SetEnabler((VisibilityEnabler2D.Enabler)3, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool PhysicsProcessParent
        {
            get
            {
                return IsEnablerEnabled((VisibilityEnabler2D.Enabler)4);
            }
            set
            {
                SetEnabler((VisibilityEnabler2D.Enabler)4, value);
            }
        }


        private const string nativeName = "VisibilityEnabler2D";

        public VisibilityEnabler2D() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_VisibilityEnabler2D_Ctor(this);
        }

        internal VisibilityEnabler2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_enabler");

        [GodotMethod("set_enabler")]
        public void SetEnabler(VisibilityEnabler2D.Enabler enabler, bool enabled)
        {
            NativeCalls.godot_icall_2_927(method_bind_0, Object.GetPtr(this), enabler, enabled);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_enabler_enabled");

        [GodotMethod("is_enabler_enabled")]
        public bool IsEnablerEnabled(VisibilityEnabler2D.Enabler enabler)
        {
            return NativeCalls.godot_icall_1_928(method_bind_1, Object.GetPtr(this), enabler);
        }
    }
}
