using System;

namespace Godot
{
    /// <summary>
    /// The VisibilityNotifier detects when it is visible on the screen. It also notifies when its bounding rectangle enters or exits the screen or a [Camera]'s view.
    /// </summary>
    public class VisibilityNotifier : Spatial
    {
        /// <summary>
        /// The VisibilityNotifier's bounding box.
        /// </summary>
        public AABB Aabb
        {
            get
            {
                return GetAabb();
            }
            set
            {
                SetAabb(value);
            }
        }


        private const string nativeName = "VisibilityNotifier";

        public VisibilityNotifier() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_VisibilityNotifier_Ctor(this);
        }

        internal VisibilityNotifier(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_aabb");

        [GodotMethod("set_aabb")]
        public void SetAabb(AABB rect)
        {
            NativeCalls.godot_icall_1_117(method_bind_0, Object.GetPtr(this), ref rect);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_aabb");

        [GodotMethod("get_aabb")]
        public AABB GetAabb()
        {
            object ret = NativeCalls.godot_icall_0_118(method_bind_1, Object.GetPtr(this));
            return (AABB)ret;
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_on_screen");

        /// <summary>
        /// If [code]true[/code] the bounding box is on the screen.
        /// </summary>
        [GodotMethod("is_on_screen")]
        public bool IsOnScreen()
        {
            return NativeCalls.godot_icall_0_3(method_bind_2, Object.GetPtr(this));
        }
    }
}
