using System;

namespace Godot
{
    /// <summary>
    /// The VisibilityNotifier2D detects when it is visible on the screen. It also notifies when its bounding rectangle enters or exits the screen or a viewport.
    /// </summary>
    public class VisibilityNotifier2D : Node2D
    {
        /// <summary>
        /// The VisibilityNotifier2D's bounding rectangle.
        /// </summary>
        public Rect2 Rect
        {
            get
            {
                return GetRect();
            }
            set
            {
                SetRect(value);
            }
        }


        private const string nativeName = "VisibilityNotifier2D";

        public VisibilityNotifier2D() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_VisibilityNotifier2D_Ctor(this);
        }

        internal VisibilityNotifier2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_rect");

        [GodotMethod("set_rect")]
        public void SetRect(Rect2 rect)
        {
            NativeCalls.godot_icall_1_120(method_bind_0, Object.GetPtr(this), ref rect);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_rect");

        [GodotMethod("get_rect")]
        public Rect2 GetRect()
        {
            object ret = NativeCalls.godot_icall_0_121(method_bind_1, Object.GetPtr(this));
            return (Rect2)ret;
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_on_screen");

        /// <summary>
        /// If [code]true[/code] the bounding rectangle is on the screen.
        /// </summary>
        [GodotMethod("is_on_screen")]
        public bool IsOnScreen()
        {
            return NativeCalls.godot_icall_0_3(method_bind_2, Object.GetPtr(this));
        }
    }
}
