using System;

namespace Godot
{
    public class VisualInstance : Spatial
    {
        /// <summary>
        /// The render layer(s) this VisualInstance is drawn on.
        /// This object will only be visible for [Camera]s whose cull mask includes the render object this VisualInstance is set to.
        /// </summary>
        public int Layers
        {
            get
            {
                return GetLayerMask();
            }
            set
            {
                SetLayerMask(value);
            }
        }


        private const string nativeName = "VisualInstance";

        internal VisualInstance() {}

        internal VisualInstance(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_base");

        /// <summary>
        /// Sets the base of the VisualInstance, which changes how the engine handles the VisualInstance under the hood.
        /// It is recommended to only use set_base if you know what you're doing.
        /// </summary>
        [GodotMethod("set_base")]
        public void SetBase(RID @base)
        {
            NativeCalls.godot_icall_1_627(method_bind_0, Object.GetPtr(this), RID.GetPtr(@base));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_layer_mask");

        [GodotMethod("set_layer_mask")]
        public void SetLayerMask(int mask)
        {
            NativeCalls.godot_icall_1_0(method_bind_1, Object.GetPtr(this), mask);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_layer_mask");

        [GodotMethod("get_layer_mask")]
        public int GetLayerMask()
        {
            return NativeCalls.godot_icall_0_1(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_transformed_aabb");

        /// <summary>
        /// Returns the transformed [AABB] (also known as the bounding box) for this VisualInstance.
        /// Transformed in this case means the [AABB] plus the position, rotation, and scale of the [Spatial]s [Transform]
        /// </summary>
        [GodotMethod("get_transformed_aabb")]
        public AABB GetTransformedAabb()
        {
            object ret = NativeCalls.godot_icall_0_118(method_bind_3, Object.GetPtr(this));
            return (AABB)ret;
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_aabb");

        /// <summary>
        /// Returns the [AABB] (also known as the bounding box) for this VisualInstance.
        /// </summary>
        [GodotMethod("get_aabb")]
        public AABB GetAabb()
        {
            object ret = NativeCalls.godot_icall_0_118(method_bind_4, Object.GetPtr(this));
            return (AABB)ret;
        }
    }
}
