using System;

namespace Godot
{
    /// <summary>
    /// A built-in function used inside a [VisualScript]. It is usually a math function or an utility function.
    /// See also [@GDScript], for the same functions in the GDScript language.
    /// </summary>
    public class VisualScriptBuiltinFunc : VisualScriptNode
    {
        public enum BuiltinFunc
        {
            MathSin = 0,
            MathCos = 1,
            MathTan = 2,
            MathSinh = 3,
            MathCosh = 4,
            MathTanh = 5,
            MathAsin = 6,
            MathAcos = 7,
            MathAtan = 8,
            MathAtan2 = 9,
            MathSqrt = 10,
            MathFmod = 11,
            MathFposmod = 12,
            MathFloor = 13,
            MathCeil = 14,
            MathRound = 15,
            MathAbs = 16,
            MathSign = 17,
            MathPow = 18,
            MathLog = 19,
            MathExp = 20,
            MathIsnan = 21,
            MathIsinf = 22,
            MathEase = 23,
            MathDecimals = 24,
            MathStepify = 25,
            MathLerp = 26,
            MathInverseLerp = 27,
            MathRangeLerp = 28,
            MathDectime = 29,
            MathRandomize = 30,
            MathRand = 31,
            MathRandf = 32,
            MathRandom = 33,
            MathSeed = 34,
            MathRandseed = 35,
            MathDeg2rad = 36,
            MathRad2deg = 37,
            MathLinear2db = 38,
            MathDb2linear = 39,
            MathPolar2cartesian = 40,
            MathCartesian2polar = 41,
            MathWrap = 42,
            MathWrapf = 43,
            LogicMax = 44,
            LogicMin = 45,
            LogicClamp = 46,
            LogicNearestPo2 = 47,
            ObjWeakref = 48,
            FuncFuncref = 49,
            TypeConvert = 50,
            TypeOf = 51,
            TypeExists = 52,
            TextChar = 53,
            TextStr = 54,
            TextPrint = 55,
            TextPrinterr = 56,
            TextPrintraw = 57,
            VarToStr = 58,
            StrToVar = 59,
            VarToBytes = 60,
            BytesToVar = 61,
            Colorn = 62,
            FuncMax = 63
        }


        /// <summary>
        /// The function to be executed.
        /// </summary>
        public VisualScriptBuiltinFunc.BuiltinFunc Function
        {
            get
            {
                return GetFunc();
            }
            set
            {
                SetFunc(value);
            }
        }


        private const string nativeName = "VisualScriptBuiltinFunc";

        public VisualScriptBuiltinFunc() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_VisualScriptBuiltinFunc_Ctor(this);
        }

        internal VisualScriptBuiltinFunc(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_func");

        [GodotMethod("set_func")]
        public void SetFunc(VisualScriptBuiltinFunc.BuiltinFunc which)
        {
            NativeCalls.godot_icall_1_945(method_bind_0, Object.GetPtr(this), which);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_func");

        [GodotMethod("get_func")]
        public VisualScriptBuiltinFunc.BuiltinFunc GetFunc()
        {
            return NativeCalls.godot_icall_0_946(method_bind_1, Object.GetPtr(this));
        }
    }
}
