using System;

namespace Godot
{
    /// <summary>
    /// This node returns a constant from a given class, such as [@GlobalScope.TYPE_INT]. See the given class' documentation for available constants.
    /// [b]Input Ports:[/b]
    /// none
    /// [b]Output Ports:[/b]
    /// - Data (variant): [code]value[/code]
    /// </summary>
    public class VisualScriptClassConstant : VisualScriptNode
    {
        /// <summary>
        /// The constant's parent class.
        /// </summary>
        public string BaseType
        {
            get
            {
                return GetBaseType();
            }
            set
            {
                SetBaseType(value);
            }
        }

        /// <summary>
        /// The constant to return. See the given class for its available constants.
        /// </summary>
        public string Constant
        {
            get
            {
                return GetClassConstant();
            }
            set
            {
                SetClassConstant(value);
            }
        }


        private const string nativeName = "VisualScriptClassConstant";

        public VisualScriptClassConstant() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_VisualScriptClassConstant_Ctor(this);
        }

        internal VisualScriptClassConstant(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_class_constant");

        [GodotMethod("set_class_constant")]
        public void SetClassConstant(string name)
        {
            NativeCalls.godot_icall_1_43(method_bind_0, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_class_constant");

        [GodotMethod("get_class_constant")]
        public string GetClassConstant()
        {
            return NativeCalls.godot_icall_0_2(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_base_type");

        [GodotMethod("set_base_type")]
        public void SetBaseType(string name)
        {
            NativeCalls.godot_icall_1_43(method_bind_2, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_base_type");

        [GodotMethod("get_base_type")]
        public string GetBaseType()
        {
            return NativeCalls.godot_icall_0_2(method_bind_3, Object.GetPtr(this));
        }
    }
}
