using System;

namespace Godot
{
    /// <summary>
    /// A Visual Script node used to display annotations in the script, so that code may be documented.
    /// Comment nodes can be resized so they encompass a group of nodes.
    /// </summary>
    public class VisualScriptComment : VisualScriptNode
    {
        /// <summary>
        /// The comment node's title.
        /// </summary>
        public string Title
        {
            get
            {
                return GetTitle();
            }
            set
            {
                SetTitle(value);
            }
        }

        /// <summary>
        /// The text inside the comment node.
        /// </summary>
        public string Description
        {
            get
            {
                return GetDescription();
            }
            set
            {
                SetDescription(value);
            }
        }

        /// <summary>
        /// The comment node's size (in pixels).
        /// </summary>
        public Vector2 Size
        {
            get
            {
                return GetSize();
            }
            set
            {
                SetSize(value);
            }
        }


        private const string nativeName = "VisualScriptComment";

        public VisualScriptComment() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_VisualScriptComment_Ctor(this);
        }

        internal VisualScriptComment(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_title");

        [GodotMethod("set_title")]
        public void SetTitle(string title)
        {
            NativeCalls.godot_icall_1_43(method_bind_0, Object.GetPtr(this), title);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_title");

        [GodotMethod("get_title")]
        public string GetTitle()
        {
            return NativeCalls.godot_icall_0_2(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_description");

        [GodotMethod("set_description")]
        public void SetDescription(string description)
        {
            NativeCalls.godot_icall_1_43(method_bind_2, Object.GetPtr(this), description);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_description");

        [GodotMethod("get_description")]
        public string GetDescription()
        {
            return NativeCalls.godot_icall_0_2(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_size");

        [GodotMethod("set_size")]
        public void SetSize(Vector2 size)
        {
            NativeCalls.godot_icall_1_45(method_bind_4, Object.GetPtr(this), ref size);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_size");

        [GodotMethod("get_size")]
        public Vector2 GetSize()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_5, Object.GetPtr(this));
            return (Vector2)ret;
        }
    }
}
