using System;
using System.Collections.Generic;

namespace Godot
{
    /// <summary>
    /// A Visual Script node which calls a base type constructor. It can be used for type conversion as well.
    /// </summary>
    public class VisualScriptConstructor : VisualScriptNode
    {
        public Variant.Type Type
        {
            get
            {
                return GetConstructorType();
            }
            set
            {
                SetConstructorType(value);
            }
        }

        public Dictionary<object, object> Constructor
        {
            get
            {
                return GetConstructor();
            }
            set
            {
                SetConstructor(value);
            }
        }

        private const string nativeName = "VisualScriptConstructor";

        public VisualScriptConstructor() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_VisualScriptConstructor_Ctor(this);
        }

        internal VisualScriptConstructor(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_constructor_type");

        /// <summary>
        /// </summary>
        [GodotMethod("set_constructor_type")]
        public void SetConstructorType(Variant.Type type)
        {
            NativeCalls.godot_icall_1_943(method_bind_0, Object.GetPtr(this), type);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_constructor_type");

        /// <summary>
        /// </summary>
        [GodotMethod("get_constructor_type")]
        public Variant.Type GetConstructorType()
        {
            return NativeCalls.godot_icall_0_944(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_constructor");

        /// <summary>
        /// </summary>
        [GodotMethod("set_constructor")]
        public void SetConstructor(Dictionary<object, object> constructor)
        {
            NativeCalls.godot_icall_1_168(method_bind_2, Object.GetPtr(this), constructor);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_constructor");

        /// <summary>
        /// </summary>
        [GodotMethod("get_constructor")]
        public Dictionary<object, object> GetConstructor()
        {
            return NativeCalls.godot_icall_0_169(method_bind_3, Object.GetPtr(this));
        }
    }
}
