using System;

namespace Godot
{
    /// <summary>
    /// A custom Visual Script node which can be scripted in powerful ways.
    /// </summary>
    public class VisualScriptCustomNode : VisualScriptNode
    {
        public const int StepPushStackBit = 16777216;
        public const int StepGoBackBit = 33554432;
        public const int StepNoAdvanceBit = 67108864;
        public const int StepExitFunctionBit = 134217728;
        public const int StepYieldBit = 268435456;

        public enum StartMode
        {
            BeginSequence = 0,
            ContinueSequence = 1,
            ResumeYield = 2
        }


        private const string nativeName = "VisualScriptCustomNode";

        public VisualScriptCustomNode() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_VisualScriptCustomNode_Ctor(this);
        }

        internal VisualScriptCustomNode(bool memoryOwn) : base(memoryOwn) {}

        /// <summary>
        /// Return the node's title.
        /// </summary>
        [GodotMethod("_get_caption")]
        public virtual string _GetCaption()
        {
            return default(string);
        }

        /// <summary>
        /// Return the node's category.
        /// </summary>
        [GodotMethod("_get_category")]
        public virtual string _GetCategory()
        {
            return default(string);
        }

        /// <summary>
        /// Return the count of input value ports.
        /// </summary>
        [GodotMethod("_get_input_value_port_count")]
        public virtual int _GetInputValuePortCount()
        {
            return default(int);
        }

        /// <summary>
        /// Return the specified input port's name.
        /// </summary>
        [GodotMethod("_get_input_value_port_name")]
        public virtual string _GetInputValuePortName(int idx)
        {
            return default(string);
        }

        /// <summary>
        /// Return the specified input port's type. See the TYPE_* enum in [@GlobalScope].
        /// </summary>
        [GodotMethod("_get_input_value_port_type")]
        public virtual int _GetInputValuePortType(int idx)
        {
            return default(int);
        }

        /// <summary>
        /// Return the amount of output [b]sequence[/b] ports.
        /// </summary>
        [GodotMethod("_get_output_sequence_port_count")]
        public virtual int _GetOutputSequencePortCount()
        {
            return default(int);
        }

        /// <summary>
        /// Return the specified [b]sequence[/b] output's name.
        /// </summary>
        [GodotMethod("_get_output_sequence_port_text")]
        public virtual string _GetOutputSequencePortText(int idx)
        {
            return default(string);
        }

        /// <summary>
        /// Return the amount of output value ports.
        /// </summary>
        [GodotMethod("_get_output_value_port_count")]
        public virtual int _GetOutputValuePortCount()
        {
            return default(int);
        }

        /// <summary>
        /// Return the specified output's name.
        /// </summary>
        [GodotMethod("_get_output_value_port_name")]
        public virtual string _GetOutputValuePortName(int idx)
        {
            return default(string);
        }

        /// <summary>
        /// Return the specified output's type. See the TYPE_* enum in [@GlobalScope].
        /// </summary>
        [GodotMethod("_get_output_value_port_type")]
        public virtual int _GetOutputValuePortType(int idx)
        {
            return default(int);
        }

        /// <summary>
        /// Return the custom node's text, which is shown right next to the input [b]sequence[/b] port (if there is none, on the place that is usually taken by it).
        /// </summary>
        [GodotMethod("_get_text")]
        public virtual string _GetText()
        {
            return default(string);
        }

        /// <summary>
        /// Return the size of the custom node's working memory. See [method _step] for more details.
        /// </summary>
        [GodotMethod("_get_working_memory_size")]
        public virtual int _GetWorkingMemorySize()
        {
            return default(int);
        }

        /// <summary>
        /// Return whether the custom node has an input [b]sequence[/b] port.
        /// </summary>
        [GodotMethod("_has_input_sequence_port")]
        public virtual bool _HasInputSequencePort()
        {
            return default(bool);
        }

        /// <summary>
        /// Execute the custom node's logic, returning the index of the output sequence port to use or a [String] when there is an error.
        /// The [code]inputs[/code] array contains the values of the input ports.
        /// [code]outputs[/code] is an array whose indices should be set to the respective outputs.
        /// The [code]start_mode[/code] is usually [code]START_MODE_BEGIN_SEQUENCE[/code], unless you have used the STEP_* constants.
        /// [code]working_mem[/code] is an array which can be used to persist information between runs of the custom node.
        /// When returning, you can mask the returned value with one of the STEP_* constants.
        /// </summary>
        [GodotMethod("_step")]
        public virtual object _Step(object[] inputs, object[] outputs, int startMode, object[] workingMem)
        {
            return default(object);
        }
    }
}
