using System;

namespace Godot
{
    /// <summary>
    /// A Visual Script node which deconstructs a base type instance into its parts.
    /// </summary>
    public class VisualScriptDeconstruct : VisualScriptNode
    {
        /// <summary>
        /// The type to deconstruct.
        /// </summary>
        public Variant.Type Type
        {
            get
            {
                return GetDeconstructType();
            }
            set
            {
                SetDeconstructType(value);
            }
        }

        public object[] ElemCache
        {
            get
            {
                return _GetElemCache();
            }
            set
            {
                _SetElemCache(value);
            }
        }


        private const string nativeName = "VisualScriptDeconstruct";

        public VisualScriptDeconstruct() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_VisualScriptDeconstruct_Ctor(this);
        }

        internal VisualScriptDeconstruct(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_deconstruct_type");

        [GodotMethod("set_deconstruct_type")]
        public void SetDeconstructType(Variant.Type type)
        {
            NativeCalls.godot_icall_1_943(method_bind_0, Object.GetPtr(this), type);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_deconstruct_type");

        [GodotMethod("get_deconstruct_type")]
        public Variant.Type GetDeconstructType()
        {
            return NativeCalls.godot_icall_0_944(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_elem_cache");

        internal void _SetElemCache(object[] _Cache)
        {
            NativeCalls.godot_icall_1_154(method_bind_2, Object.GetPtr(this), _Cache);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_elem_cache");

        internal object[] _GetElemCache()
        {
            return NativeCalls.godot_icall_0_21(method_bind_3, Object.GetPtr(this));
        }
    }
}
