using System;

namespace Godot
{
    /// <summary>
    /// Emits a specified signal when it is executed.
    /// [b]Input Ports:[/b]
    /// - Sequence: [code]emit[/code]
    /// [b]Output Ports:[/b]
    /// - Sequence
    /// </summary>
    public class VisualScriptEmitSignal : VisualScriptNode
    {
        /// <summary>
        /// The signal to emit.
        /// </summary>
        public string Signal
        {
            get
            {
                return GetSignal();
            }
            set
            {
                SetSignal(value);
            }
        }


        private const string nativeName = "VisualScriptEmitSignal";

        public VisualScriptEmitSignal() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_VisualScriptEmitSignal_Ctor(this);
        }

        internal VisualScriptEmitSignal(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_signal");

        [GodotMethod("set_signal")]
        public void SetSignal(string name)
        {
            NativeCalls.godot_icall_1_43(method_bind_0, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_signal");

        [GodotMethod("get_signal")]
        public string GetSignal()
        {
            return NativeCalls.godot_icall_0_2(method_bind_1, Object.GetPtr(this));
        }
    }
}
