using System;

namespace Godot
{
    /// <summary>
    /// Returns a local variable's value. "Var Name" must be supplied, with an optional type.
    /// [b]Input Ports:[/b]
    /// none
    /// [b]Output Ports:[/b]
    /// - Data (variant): [code]get[/code]
    /// </summary>
    public class VisualScriptLocalVar : VisualScriptNode
    {
        /// <summary>
        /// The local variable's name.
        /// </summary>
        public string VarName
        {
            get
            {
                return GetVarName();
            }
            set
            {
                SetVarName(value);
            }
        }

        /// <summary>
        /// The local variable's type.
        /// </summary>
        public Variant.Type Type
        {
            get
            {
                return GetVarType();
            }
            set
            {
                SetVarType(value);
            }
        }


        private const string nativeName = "VisualScriptLocalVar";

        public VisualScriptLocalVar() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_VisualScriptLocalVar_Ctor(this);
        }

        internal VisualScriptLocalVar(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_var_name");

        [GodotMethod("set_var_name")]
        public void SetVarName(string name)
        {
            NativeCalls.godot_icall_1_43(method_bind_0, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_var_name");

        [GodotMethod("get_var_name")]
        public string GetVarName()
        {
            return NativeCalls.godot_icall_0_2(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_var_type");

        [GodotMethod("set_var_type")]
        public void SetVarType(Variant.Type type)
        {
            NativeCalls.godot_icall_1_943(method_bind_2, Object.GetPtr(this), type);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_var_type");

        [GodotMethod("get_var_type")]
        public Variant.Type GetVarType()
        {
            return NativeCalls.godot_icall_0_944(method_bind_3, Object.GetPtr(this));
        }
    }
}
