using System;

namespace Godot
{
    /// <summary>
    /// Changes a local variable's value to the given input. The new value is also provided on an output Data port.
    /// [b]Input Ports:[/b]
    /// - Sequence
    /// - Data (variant): [code]set[/code]
    /// [b]Output Ports:[/b]
    /// - Sequence
    /// - Data (variant): [code]get[/code]
    /// </summary>
    public class VisualScriptLocalVarSet : VisualScriptNode
    {
        /// <summary>
        /// The local variable's name.
        /// </summary>
        public string VarName
        {
            get
            {
                return GetVarName();
            }
            set
            {
                SetVarName(value);
            }
        }

        /// <summary>
        /// The local variable's type.
        /// </summary>
        public Variant.Type Type
        {
            get
            {
                return GetVarType();
            }
            set
            {
                SetVarType(value);
            }
        }


        private const string nativeName = "VisualScriptLocalVarSet";

        public VisualScriptLocalVarSet() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_VisualScriptLocalVarSet_Ctor(this);
        }

        internal VisualScriptLocalVarSet(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_var_name");

        [GodotMethod("set_var_name")]
        public void SetVarName(string name)
        {
            NativeCalls.godot_icall_1_43(method_bind_0, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_var_name");

        [GodotMethod("get_var_name")]
        public string GetVarName()
        {
            return NativeCalls.godot_icall_0_2(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_var_type");

        [GodotMethod("set_var_type")]
        public void SetVarType(Variant.Type type)
        {
            NativeCalls.godot_icall_1_943(method_bind_2, Object.GetPtr(this), type);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_var_type");

        [GodotMethod("get_var_type")]
        public Variant.Type GetVarType()
        {
            return NativeCalls.godot_icall_0_944(method_bind_3, Object.GetPtr(this));
        }
    }
}
