using System;

namespace Godot
{
    /// <summary>
    /// A node which is part of a [VisualScript]. Not to be confused with [Node], which is a part of a [SceneTree].
    /// </summary>
    public class VisualScriptNode : Resource
    {
        public object[] _DefaultInputValues
        {
            get
            {
                return _GetDefaultInputValues();
            }
            set
            {
                _SetDefaultInputValues(value);
            }
        }

        private const string nativeName = "VisualScriptNode";

        internal VisualScriptNode() {}

        internal VisualScriptNode(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_visual_script");

        /// <summary>
        /// Returns the [VisualScript] instance the node is bound to.
        /// </summary>
        [GodotMethod("get_visual_script")]
        public VisualScript GetVisualScript()
        {
            return NativeCalls.godot_icall_0_957(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_default_input_value");

        /// <summary>
        /// Change the default value of a given port.
        /// </summary>
        [GodotMethod("set_default_input_value")]
        public void SetDefaultInputValue(int portIdx, object value)
        {
            NativeCalls.godot_icall_2_453(method_bind_1, Object.GetPtr(this), portIdx, value);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_default_input_value");

        /// <summary>
        /// Returns the default value of a given port. The default value is used when nothing is connected to the port.
        /// </summary>
        [GodotMethod("get_default_input_value")]
        public object GetDefaultInputValue(int portIdx)
        {
            return NativeCalls.godot_icall_1_454(method_bind_2, Object.GetPtr(this), portIdx);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "ports_changed_notify");

        /// <summary>
        /// Notify that the node's ports have changed. Usually used in conjunction with [VisualScriptCustomNode] .
        /// </summary>
        [GodotMethod("ports_changed_notify")]
        public void PortsChangedNotify()
        {
            NativeCalls.godot_icall_0_12(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_default_input_values");

        internal void _SetDefaultInputValues(object[] values)
        {
            NativeCalls.godot_icall_1_154(method_bind_4, Object.GetPtr(this), values);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_default_input_values");

        internal object[] _GetDefaultInputValues()
        {
            return NativeCalls.godot_icall_0_21(method_bind_5, Object.GetPtr(this));
        }
    }
}
