using System;

namespace Godot
{
    /// <summary>
    /// Ends the execution of a function and returns control to the calling function. Optionally, it can return a [Variant] value.
    /// [b]Input Ports:[/b]
    /// - Sequence
    /// - Data (variant): [code]result[/code] (optional)
    /// [b]Output Ports:[/b]
    /// none
    /// </summary>
    public class VisualScriptReturn : VisualScriptNode
    {
        /// <summary>
        /// If [code]true[/code] the [code]return[/code] input port is available.
        /// </summary>
        public bool ReturnEnabled
        {
            get
            {
                return IsReturnValueEnabled();
            }
            set
            {
                SetEnableReturnValue(value);
            }
        }

        /// <summary>
        /// The return value's data type.
        /// </summary>
        public Variant.Type ReturnType
        {
            get
            {
                return GetReturnType();
            }
            set
            {
                SetReturnType(value);
            }
        }


        private const string nativeName = "VisualScriptReturn";

        public VisualScriptReturn() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_VisualScriptReturn_Ctor(this);
        }

        internal VisualScriptReturn(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_return_type");

        [GodotMethod("set_return_type")]
        public void SetReturnType(Variant.Type type)
        {
            NativeCalls.godot_icall_1_943(method_bind_0, Object.GetPtr(this), type);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_return_type");

        [GodotMethod("get_return_type")]
        public Variant.Type GetReturnType()
        {
            return NativeCalls.godot_icall_0_944(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_enable_return_value");

        [GodotMethod("set_enable_return_value")]
        public void SetEnableReturnValue(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_2, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_return_value_enabled");

        [GodotMethod("is_return_value_enabled")]
        public bool IsReturnValueEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_3, Object.GetPtr(this));
        }
    }
}
