using System;

namespace Godot
{
    /// <summary>
    /// Steps through a series of one or more output Sequence ports. The [code]current[/code] data port outputs the currently executing item.
    /// [b]Input Ports:[/b]
    /// - Sequence: [code]in order[/code]
    /// [b]Output Ports:[/b]
    /// - Sequence: [code]1[/code]
    /// - Sequence: [code]2 - n[/code] (optional)
    /// - Data (int): [code]current[/code]
    /// </summary>
    public class VisualScriptSequence : VisualScriptNode
    {
        /// <summary>
        /// The number of steps in the sequence.
        /// </summary>
        public int Steps
        {
            get
            {
                return GetSteps();
            }
            set
            {
                SetSteps(value);
            }
        }


        private const string nativeName = "VisualScriptSequence";

        public VisualScriptSequence() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_VisualScriptSequence_Ctor(this);
        }

        internal VisualScriptSequence(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_steps");

        [GodotMethod("set_steps")]
        public void SetSteps(int steps)
        {
            NativeCalls.godot_icall_1_0(method_bind_0, Object.GetPtr(this), steps);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_steps");

        [GodotMethod("get_steps")]
        public int GetSteps()
        {
            return NativeCalls.godot_icall_0_1(method_bind_1, Object.GetPtr(this));
        }
    }
}
