using System;

namespace Godot
{
    /// <summary>
    /// Server for anything visible. The visual server is the API backend for everything visible. The whole scene system mounts on it to display.
    /// The visual server is completely opaque, the internals are entirely implementation specific and cannot be accessed.
    /// </summary>
    public static class VisualServer
    {
        public const int NoIndexArray = -1;
        public const int ArrayWeightsSize = 4;
        public const int CanvasItemZMin = -4096;
        public const int CanvasItemZMax = 4096;
        public const int MaxGlowLevels = 7;
        public const int MaxCursors = 8;
        public const int MaterialRenderPriorityMin = -128;
        public const int MaterialRenderPriorityMax = 127;

        public enum BlendShapeMode
        {
            Normalized = 0,
            Relative = 1
        }

        public enum PrimitiveType
        {
            Points = 0,
            Lines = 1,
            LineStrip = 2,
            LineLoop = 3,
            Triangles = 4,
            TriangleStrip = 5,
            TriangleFan = 6,
            Max = 7
        }

        public enum RenderInfo
        {
            ObjectsInFrame = 0,
            VerticesInFrame = 1,
            MaterialChangesInFrame = 2,
            ShaderChangesInFrame = 3,
            SurfaceChangesInFrame = 4,
            DrawCallsInFrame = 5,
            UsageVideoMemTotal = 6,
            VideoMemUsed = 7,
            TextureMemUsed = 8,
            VertexMemUsed = 9
        }

        public enum NinePatchAxisMode
        {
            Stretch = 0,
            Tile = 1,
            TileFit = 2
        }

        public enum ViewportRenderInfo
        {
            ObjectsInFrame = 0,
            VerticesInFrame = 1,
            MaterialChangesInFrame = 2,
            ShaderChangesInFrame = 3,
            SurfaceChangesInFrame = 4,
            DrawCallsInFrame = 5,
            Max = 6
        }

        public enum ViewportClearMode
        {
            Always = 0,
            Never = 1,
            OnlyNextFrame = 2
        }

        public enum ShaderMode
        {
            Spatial = 0,
            CanvasItem = 1,
            Particles = 2,
            Max = 3
        }

        public enum ViewportDebugDraw
        {
            Disabled = 0,
            Unshaded = 1,
            Overdraw = 2,
            Wireframe = 3
        }

        public enum ViewportUsage
        {
            Usage2d = 0,
            Usage2dNoSampling = 1,
            Usage3d = 2,
            Usage3dNoEffects = 3
        }

        public enum TextureFlags
        {
            Mipmaps = 1,
            Repeat = 2,
            Filter = 4,
            AnisotropicFilter = 8,
            ConvertToLinear = 16,
            MirroredRepeat = 32,
            Cubemap = 2048,
            UsedForStreaming = 4096,
            sDefault = 7
        }

        public enum Features
        {
            Shaders = 0,
            Multithreaded = 1
        }

        public enum InstanceType
        {
            None = 0,
            Mesh = 1,
            Multimesh = 2,
            Immediate = 3,
            Particles = 4,
            Light = 5,
            ReflectionProbe = 6,
            GiProbe = 7,
            LightmapCapture = 8,
            Max = 9,
            GeometryMask = 30
        }

        public enum CanvasLightShadowFilter
        {
            None = 0,
            Pcf3 = 1,
            Pcf5 = 2,
            Pcf7 = 3,
            Pcf9 = 4,
            Pcf13 = 5
        }

        public enum ScenarioDebugMode
        {
            Disabled = 0,
            Wireframe = 1,
            Overdraw = 2,
            Shadeless = 3
        }

        public enum ViewportUpdateMode
        {
            Disabled = 0,
            Once = 1,
            WhenVisible = 2,
            Always = 3
        }

        public enum ArrayFormat
        {
            FormatVertex = 1,
            FormatNormal = 2,
            FormatTangent = 4,
            FormatColor = 8,
            FormatTexUv = 16,
            FormatTexUv2 = 32,
            FormatBones = 64,
            FormatWeights = 128,
            FormatIndex = 256,
            CompressVertex = 512,
            CompressNormal = 1024,
            CompressTangent = 2048,
            CompressColor = 4096,
            CompressTexUv = 8192,
            CompressTexUv2 = 16384,
            CompressBones = 32768,
            CompressWeights = 65536,
            CompressIndex = 131072,
            FlagUse2dVertices = 262144,
            FlagUse16BitBones = 524288,
            CompressDefault = 97792
        }

        public enum CanvasLightMode
        {
            Add = 0,
            Sub = 1,
            Mix = 2,
            Mask = 3
        }

        public enum LightParam
        {
            Energy = 0,
            Specular = 2,
            Range = 3,
            Attenuation = 4,
            SpotAngle = 5,
            SpotAttenuation = 6,
            ContactShadowSize = 7,
            ShadowMaxDistance = 8,
            ShadowSplit1Offset = 9,
            ShadowSplit2Offset = 10,
            ShadowSplit3Offset = 11,
            ShadowNormalBias = 12,
            ShadowBias = 13,
            ShadowBiasSplitScale = 14,
            Max = 15
        }

        public enum ArrayType
        {
            Vertex = 0,
            Normal = 1,
            Tangent = 2,
            Color = 3,
            TexUv = 4,
            TexUv2 = 5,
            Bones = 6,
            Weights = 7,
            Index = 8,
            Max = 9
        }

        public enum CanvasOccluderPolygonCullMode
        {
            Disabled = 0,
            Clockwise = 1,
            CounterClockwise = 2
        }

        public enum ViewportMSAA
        {
            Disabled = 0,
            Msaa2x = 1,
            Msaa4x = 2,
            Msaa8x = 3,
            Msaa16x = 4
        }

        public enum LightType
        {
            Directional = 0,
            Omni = 1,
            Spot = 2
        }

        public enum CubeMapSide
        {
            Left = 0,
            Right = 1,
            Bottom = 2,
            Top = 3,
            Front = 4,
            Back = 5
        }


        private const string nativeName = "VisualServer";
        internal static IntPtr ptr = NativeCalls.godot_icall_VisualServer_get_singleton();

        private static IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "force_sync");

        /// <summary>
        /// Syncronizes threads.
        /// </summary>
        [GodotMethod("force_sync")]
        public static void ForceSync()
        {
            NativeCalls.godot_icall_0_12(method_bind_0, ptr);
        }

        private static IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "force_draw");

        /// <summary>
        /// Draws a frame. Same as [method draw].
        /// </summary>
        [GodotMethod("force_draw")]
        public static void ForceDraw(bool swapBuffers = true)
        {
            NativeCalls.godot_icall_1_11(method_bind_1, ptr, swapBuffers);
        }

        private static IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "texture_create");

        /// <summary>
        /// Creates an empty texture.
        /// </summary>
        [GodotMethod("texture_create")]
        public static RID TextureCreate()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_2, ptr);
            return new RID(ret);
        }

        private static IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "texture_create_from_image");

        /// <summary>
        /// Creates a texture, allocates the space for an image, and fills in the image.
        /// </summary>
        [GodotMethod("texture_create_from_image")]
        public static RID TextureCreateFromImage(Image image, int flags = 7)
        {
            IntPtr ret = NativeCalls.godot_icall_2_970(method_bind_3, ptr, Object.GetPtr(image), flags);
            return new RID(ret);
        }

        private static IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "texture_allocate");

        /// <summary>
        /// Allocates space for a texture's image or video.
        /// </summary>
        [GodotMethod("texture_allocate")]
        public static void TextureAllocate(RID texture, int width, int height, Image.Format format, int flags = 7)
        {
            NativeCalls.godot_icall_5_971(method_bind_4, ptr, RID.GetPtr(texture), width, height, format, flags);
        }

        private static IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "texture_set_data");

        /// <summary>
        /// Sets the texture's image data. If it's a CubeMap, it sets the image data at a cube side.
        /// </summary>
        [GodotMethod("texture_set_data")]
        public static void TextureSetData(RID texture, Image image, VisualServer.CubeMapSide cubeSide = (VisualServer.CubeMapSide)0)
        {
            NativeCalls.godot_icall_3_972(method_bind_5, ptr, RID.GetPtr(texture), Object.GetPtr(image), cubeSide);
        }

        private static IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "texture_get_data");

        /// <summary>
        /// Returns a copy of a texture's image unless it's a CubeMap, in wich case it returns the [RID] of the image at one of the cubes sides.
        /// </summary>
        [GodotMethod("texture_get_data")]
        public static Image TextureGetData(RID texture, VisualServer.CubeMapSide cubeSide = (VisualServer.CubeMapSide)0)
        {
            return NativeCalls.godot_icall_2_973(method_bind_6, ptr, RID.GetPtr(texture), cubeSide);
        }

        private static IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "texture_set_flags");

        /// <summary>
        /// Sets the texture's flags. See [enum TextureFlags] for options
        /// </summary>
        [GodotMethod("texture_set_flags")]
        public static void TextureSetFlags(RID texture, int flags)
        {
            NativeCalls.godot_icall_2_626(method_bind_7, ptr, RID.GetPtr(texture), flags);
        }

        private static IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "texture_get_flags");

        /// <summary>
        /// Returns the flags of a texture.
        /// </summary>
        [GodotMethod("texture_get_flags")]
        public static int TextureGetFlags(RID texture)
        {
            return NativeCalls.godot_icall_1_623(method_bind_8, ptr, RID.GetPtr(texture));
        }

        private static IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "texture_get_format");

        /// <summary>
        /// Returns the format of the texture's image.
        /// </summary>
        [GodotMethod("texture_get_format")]
        public static Image.Format TextureGetFormat(RID texture)
        {
            return NativeCalls.godot_icall_1_974(method_bind_9, ptr, RID.GetPtr(texture));
        }

        private static IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "texture_get_texid");

        /// <summary>
        /// Returns the opengl id of the texture's image.
        /// </summary>
        [GodotMethod("texture_get_texid")]
        public static int TextureGetTexid(RID texture)
        {
            return NativeCalls.godot_icall_1_623(method_bind_10, ptr, RID.GetPtr(texture));
        }

        private static IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "texture_get_width");

        /// <summary>
        /// Returns the texture's width.
        /// </summary>
        [GodotMethod("texture_get_width")]
        public static int TextureGetWidth(RID texture)
        {
            return NativeCalls.godot_icall_1_623(method_bind_11, ptr, RID.GetPtr(texture));
        }

        private static IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "texture_get_height");

        /// <summary>
        /// Returns the texture's height.
        /// </summary>
        [GodotMethod("texture_get_height")]
        public static int TextureGetHeight(RID texture)
        {
            return NativeCalls.godot_icall_1_623(method_bind_12, ptr, RID.GetPtr(texture));
        }

        private static IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "texture_set_size_override");

        /// <summary>
        /// Overwrites the texture's width and height.
        /// </summary>
        [GodotMethod("texture_set_size_override")]
        public static void TextureSetSizeOverride(RID texture, int width, int height)
        {
            NativeCalls.godot_icall_3_975(method_bind_13, ptr, RID.GetPtr(texture), width, height);
        }

        private static IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "texture_set_path");

        /// <summary>
        /// Sets the texture's path.
        /// </summary>
        [GodotMethod("texture_set_path")]
        public static void TextureSetPath(RID texture, string path)
        {
            NativeCalls.godot_icall_2_976(method_bind_14, ptr, RID.GetPtr(texture), path);
        }

        private static IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "texture_get_path");

        /// <summary>
        /// Returns the texture's path.
        /// </summary>
        [GodotMethod("texture_get_path")]
        public static string TextureGetPath(RID texture)
        {
            return NativeCalls.godot_icall_1_977(method_bind_15, ptr, RID.GetPtr(texture));
        }

        private static IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "texture_set_shrink_all_x2_on_set_data");

        /// <summary>
        /// If [code]true[/code], sets internal processes to shrink all image data to half the size.
        /// </summary>
        [GodotMethod("texture_set_shrink_all_x2_on_set_data")]
        public static void TextureSetShrinkAllX2OnSetData(bool shrink)
        {
            NativeCalls.godot_icall_1_11(method_bind_16, ptr, shrink);
        }

        private static IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "texture_debug_usage");

        /// <summary>
        /// Returns a list of all the textures and their information.
        /// </summary>
        [GodotMethod("texture_debug_usage")]
        public static object[] TextureDebugUsage()
        {
            return NativeCalls.godot_icall_0_21(method_bind_17, ptr);
        }

        private static IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "textures_keep_original");

        /// <summary>
        /// If [code]true[/code], the image will be stored in the texture's images array if overwritten.
        /// </summary>
        [GodotMethod("textures_keep_original")]
        public static void TexturesKeepOriginal(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_18, ptr, enable);
        }

        private static IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "sky_create");

        /// <summary>
        /// Creates an empty sky.
        /// </summary>
        [GodotMethod("sky_create")]
        public static RID SkyCreate()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_19, ptr);
            return new RID(ret);
        }

        private static IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "sky_set_texture");

        /// <summary>
        /// Sets a sky's texture.
        /// </summary>
        [GodotMethod("sky_set_texture")]
        public static void SkySetTexture(RID sky, RID cubeMap, int radianceSize)
        {
            NativeCalls.godot_icall_3_978(method_bind_20, ptr, RID.GetPtr(sky), RID.GetPtr(cubeMap), radianceSize);
        }

        private static IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "shader_create");

        /// <summary>
        /// Creates an empty shader.
        /// </summary>
        [GodotMethod("shader_create")]
        public static RID ShaderCreate()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_21, ptr);
            return new RID(ret);
        }

        private static IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "shader_set_code");

        /// <summary>
        /// Sets a shader's code.
        /// </summary>
        [GodotMethod("shader_set_code")]
        public static void ShaderSetCode(RID shader, string code)
        {
            NativeCalls.godot_icall_2_976(method_bind_22, ptr, RID.GetPtr(shader), code);
        }

        private static IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "shader_get_code");

        /// <summary>
        /// Returns a shader's code.
        /// </summary>
        [GodotMethod("shader_get_code")]
        public static string ShaderGetCode(RID shader)
        {
            return NativeCalls.godot_icall_1_977(method_bind_23, ptr, RID.GetPtr(shader));
        }

        private static IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "shader_get_param_list");

        /// <summary>
        /// Returns the parameters of a shader.
        /// </summary>
        [GodotMethod("shader_get_param_list")]
        public static object[] ShaderGetParamList(RID shader)
        {
            return NativeCalls.godot_icall_1_979(method_bind_24, ptr, RID.GetPtr(shader));
        }

        private static IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "shader_set_default_texture_param");

        /// <summary>
        /// Sets a shader's default texture. Overwrites the texture given by name.
        /// </summary>
        [GodotMethod("shader_set_default_texture_param")]
        public static void ShaderSetDefaultTextureParam(RID shader, string name, RID texture)
        {
            NativeCalls.godot_icall_3_980(method_bind_25, ptr, RID.GetPtr(shader), name, RID.GetPtr(texture));
        }

        private static IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "shader_get_default_texture_param");

        /// <summary>
        /// Returns a default texture from a shader searched by name.
        /// </summary>
        [GodotMethod("shader_get_default_texture_param")]
        public static RID ShaderGetDefaultTextureParam(RID shader, string name)
        {
            IntPtr ret = NativeCalls.godot_icall_2_981(method_bind_26, ptr, RID.GetPtr(shader), name);
            return new RID(ret);
        }

        private static IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "material_create");

        /// <summary>
        /// Returns an empty material.
        /// </summary>
        [GodotMethod("material_create")]
        public static RID MaterialCreate()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_27, ptr);
            return new RID(ret);
        }

        private static IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "material_set_shader");

        /// <summary>
        /// Sets a shader material's shader.
        /// </summary>
        [GodotMethod("material_set_shader")]
        public static void MaterialSetShader(RID shaderMaterial, RID shader)
        {
            NativeCalls.godot_icall_2_615(method_bind_28, ptr, RID.GetPtr(shaderMaterial), RID.GetPtr(shader));
        }

        private static IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "material_get_shader");

        /// <summary>
        /// Returns the shader of a certain material's shader. Returns an empty RID if the material doesn't have a shader.
        /// </summary>
        [GodotMethod("material_get_shader")]
        public static RID MaterialGetShader(RID shaderMaterial)
        {
            IntPtr ret = NativeCalls.godot_icall_1_616(method_bind_29, ptr, RID.GetPtr(shaderMaterial));
            return new RID(ret);
        }

        private static IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "material_set_param");

        /// <summary>
        /// Sets a materials parameter.
        /// </summary>
        [GodotMethod("material_set_param")]
        public static void MaterialSetParam(RID material, string parameter, object value)
        {
            NativeCalls.godot_icall_3_982(method_bind_30, ptr, RID.GetPtr(material), parameter, value);
        }

        private static IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "material_get_param");

        /// <summary>
        /// Returns the value of a certain material's parameter.
        /// </summary>
        [GodotMethod("material_get_param")]
        public static object MaterialGetParam(RID material, string parameter)
        {
            return NativeCalls.godot_icall_2_983(method_bind_31, ptr, RID.GetPtr(material), parameter);
        }

        private static IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "material_set_render_priority");

        /// <summary>
        /// Sets a material's render priority.
        /// </summary>
        [GodotMethod("material_set_render_priority")]
        public static void MaterialSetRenderPriority(RID material, int priority)
        {
            NativeCalls.godot_icall_2_626(method_bind_32, ptr, RID.GetPtr(material), priority);
        }

        private static IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "material_set_line_width");

        /// <summary>
        /// Sets a materials line width.
        /// </summary>
        [GodotMethod("material_set_line_width")]
        public static void MaterialSetLineWidth(RID material, float width)
        {
            NativeCalls.godot_icall_2_680(method_bind_33, ptr, RID.GetPtr(material), width);
        }

        private static IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "material_set_next_pass");

        /// <summary>
        /// Sets an objects next material.
        /// </summary>
        [GodotMethod("material_set_next_pass")]
        public static void MaterialSetNextPass(RID material, RID nextMaterial)
        {
            NativeCalls.godot_icall_2_615(method_bind_34, ptr, RID.GetPtr(material), RID.GetPtr(nextMaterial));
        }

        private static IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "mesh_create");

        /// <summary>
        /// Creates a new mesh.
        /// </summary>
        [GodotMethod("mesh_create")]
        public static RID MeshCreate()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_35, ptr);
            return new RID(ret);
        }

        private static IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "mesh_add_surface_from_arrays");

        /// <summary>
        /// Adds a surface generated from the Arrays to a mesh. See PRIMITIVE_TYPE_* constants for types.
        /// &lt;param name="blendShapes"&gt;If the param is null, then the default value is new object[] {}&lt;/param&gt;
        /// </summary>
        [GodotMethod("mesh_add_surface_from_arrays")]
        public static void MeshAddSurfaceFromArrays(RID mesh, VisualServer.PrimitiveType primtive, object[] arrays, object[] blendShapes = null, int compressFormat = 97792)
        {
            object[] blendShapes_in = blendShapes != null ? blendShapes : new object[] {};
            NativeCalls.godot_icall_5_984(method_bind_36, ptr, RID.GetPtr(mesh), primtive, arrays, blendShapes_in, compressFormat);
        }

        private static IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "mesh_set_blend_shape_count");

        /// <summary>
        /// Sets a mesh's blend shape count.
        /// </summary>
        [GodotMethod("mesh_set_blend_shape_count")]
        public static void MeshSetBlendShapeCount(RID mesh, int amount)
        {
            NativeCalls.godot_icall_2_626(method_bind_37, ptr, RID.GetPtr(mesh), amount);
        }

        private static IntPtr method_bind_38 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "mesh_get_blend_shape_count");

        /// <summary>
        /// Returns a mesh's blend shape count.
        /// </summary>
        [GodotMethod("mesh_get_blend_shape_count")]
        public static int MeshGetBlendShapeCount(RID mesh)
        {
            return NativeCalls.godot_icall_1_623(method_bind_38, ptr, RID.GetPtr(mesh));
        }

        private static IntPtr method_bind_39 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "mesh_set_blend_shape_mode");

        /// <summary>
        /// Sets a mesh's blend shape mode.
        /// </summary>
        [GodotMethod("mesh_set_blend_shape_mode")]
        public static void MeshSetBlendShapeMode(RID mesh, VisualServer.BlendShapeMode mode)
        {
            NativeCalls.godot_icall_2_985(method_bind_39, ptr, RID.GetPtr(mesh), mode);
        }

        private static IntPtr method_bind_40 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "mesh_get_blend_shape_mode");

        /// <summary>
        /// Returns a mesh's blend shape mode.
        /// </summary>
        [GodotMethod("mesh_get_blend_shape_mode")]
        public static VisualServer.BlendShapeMode MeshGetBlendShapeMode(RID mesh)
        {
            return NativeCalls.godot_icall_1_986(method_bind_40, ptr, RID.GetPtr(mesh));
        }

        private static IntPtr method_bind_41 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "mesh_surface_set_material");

        /// <summary>
        /// Sets a mesh's surface's material.
        /// </summary>
        [GodotMethod("mesh_surface_set_material")]
        public static void MeshSurfaceSetMaterial(RID mesh, int surface, RID material)
        {
            NativeCalls.godot_icall_3_620(method_bind_41, ptr, RID.GetPtr(mesh), surface, RID.GetPtr(material));
        }

        private static IntPtr method_bind_42 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "mesh_surface_get_material");

        /// <summary>
        /// Returns a mesh's surface's material.
        /// </summary>
        [GodotMethod("mesh_surface_get_material")]
        public static RID MeshSurfaceGetMaterial(RID mesh, int surface)
        {
            IntPtr ret = NativeCalls.godot_icall_2_624(method_bind_42, ptr, RID.GetPtr(mesh), surface);
            return new RID(ret);
        }

        private static IntPtr method_bind_43 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "mesh_surface_get_array_len");

        /// <summary>
        /// Returns a mesh's surface's amount of vertices.
        /// </summary>
        [GodotMethod("mesh_surface_get_array_len")]
        public static int MeshSurfaceGetArrayLen(RID mesh, int surface)
        {
            return NativeCalls.godot_icall_2_987(method_bind_43, ptr, RID.GetPtr(mesh), surface);
        }

        private static IntPtr method_bind_44 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "mesh_surface_get_array_index_len");

        /// <summary>
        /// Returns a mesh's surface's amount of indices.
        /// </summary>
        [GodotMethod("mesh_surface_get_array_index_len")]
        public static int MeshSurfaceGetArrayIndexLen(RID mesh, int surface)
        {
            return NativeCalls.godot_icall_2_987(method_bind_44, ptr, RID.GetPtr(mesh), surface);
        }

        private static IntPtr method_bind_45 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "mesh_surface_get_array");

        /// <summary>
        /// Returns a mesh's surface's vertex buffer.
        /// </summary>
        [GodotMethod("mesh_surface_get_array")]
        public static byte[] MeshSurfaceGetArray(RID mesh, int surface)
        {
            return NativeCalls.godot_icall_2_988(method_bind_45, ptr, RID.GetPtr(mesh), surface);
        }

        private static IntPtr method_bind_46 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "mesh_surface_get_index_array");

        /// <summary>
        /// Returns a mesh's surface's index buffer.
        /// </summary>
        [GodotMethod("mesh_surface_get_index_array")]
        public static byte[] MeshSurfaceGetIndexArray(RID mesh, int surface)
        {
            return NativeCalls.godot_icall_2_988(method_bind_46, ptr, RID.GetPtr(mesh), surface);
        }

        private static IntPtr method_bind_47 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "mesh_surface_get_arrays");

        /// <summary>
        /// Returns a mesh's surface's buffer arrays.
        /// </summary>
        [GodotMethod("mesh_surface_get_arrays")]
        public static object[] MeshSurfaceGetArrays(RID mesh, int surface)
        {
            return NativeCalls.godot_icall_2_989(method_bind_47, ptr, RID.GetPtr(mesh), surface);
        }

        private static IntPtr method_bind_48 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "mesh_surface_get_blend_shape_arrays");

        /// <summary>
        /// Returns a mesh's surface's arrays for blend shapes
        /// </summary>
        [GodotMethod("mesh_surface_get_blend_shape_arrays")]
        public static object[] MeshSurfaceGetBlendShapeArrays(RID mesh, int surface)
        {
            return NativeCalls.godot_icall_2_989(method_bind_48, ptr, RID.GetPtr(mesh), surface);
        }

        private static IntPtr method_bind_49 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "mesh_surface_get_format");

        /// <summary>
        /// Returns the format of a mesh's surface.
        /// </summary>
        [GodotMethod("mesh_surface_get_format")]
        public static int MeshSurfaceGetFormat(RID mesh, int surface)
        {
            return NativeCalls.godot_icall_2_987(method_bind_49, ptr, RID.GetPtr(mesh), surface);
        }

        private static IntPtr method_bind_50 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "mesh_surface_get_primitive_type");

        /// <summary>
        /// Returns the primitive type of a mesh's surface.
        /// </summary>
        [GodotMethod("mesh_surface_get_primitive_type")]
        public static VisualServer.PrimitiveType MeshSurfaceGetPrimitiveType(RID mesh, int surface)
        {
            return NativeCalls.godot_icall_2_990(method_bind_50, ptr, RID.GetPtr(mesh), surface);
        }

        private static IntPtr method_bind_51 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "mesh_surface_get_aabb");

        /// <summary>
        /// Returns a mesh's surface's aabb.
        /// </summary>
        [GodotMethod("mesh_surface_get_aabb")]
        public static AABB MeshSurfaceGetAabb(RID mesh, int surface)
        {
            object ret = NativeCalls.godot_icall_2_991(method_bind_51, ptr, RID.GetPtr(mesh), surface);
            return (AABB)ret;
        }

        private static IntPtr method_bind_52 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "mesh_surface_get_skeleton_aabb");

        /// <summary>
        /// Returns the aabb of a mesh's surface's skeleton.
        /// </summary>
        [GodotMethod("mesh_surface_get_skeleton_aabb")]
        public static object[] MeshSurfaceGetSkeletonAabb(RID mesh, int surface)
        {
            return NativeCalls.godot_icall_2_989(method_bind_52, ptr, RID.GetPtr(mesh), surface);
        }

        private static IntPtr method_bind_53 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "mesh_remove_surface");

        /// <summary>
        /// Removes a mesh's surface.
        /// </summary>
        [GodotMethod("mesh_remove_surface")]
        public static void MeshRemoveSurface(RID mesh, int index)
        {
            NativeCalls.godot_icall_2_626(method_bind_53, ptr, RID.GetPtr(mesh), index);
        }

        private static IntPtr method_bind_54 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "mesh_get_surface_count");

        /// <summary>
        /// Returns a mesh's number of surfaces.
        /// </summary>
        [GodotMethod("mesh_get_surface_count")]
        public static int MeshGetSurfaceCount(RID mesh)
        {
            return NativeCalls.godot_icall_1_623(method_bind_54, ptr, RID.GetPtr(mesh));
        }

        private static IntPtr method_bind_55 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "mesh_set_custom_aabb");

        /// <summary>
        /// Sets a mesh's custom aabb.
        /// </summary>
        [GodotMethod("mesh_set_custom_aabb")]
        public static void MeshSetCustomAabb(RID mesh, AABB aabb)
        {
            NativeCalls.godot_icall_2_992(method_bind_55, ptr, RID.GetPtr(mesh), ref aabb);
        }

        private static IntPtr method_bind_56 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "mesh_get_custom_aabb");

        /// <summary>
        /// Returns a mesh's custom aabb.
        /// </summary>
        [GodotMethod("mesh_get_custom_aabb")]
        public static AABB MeshGetCustomAabb(RID mesh)
        {
            object ret = NativeCalls.godot_icall_1_993(method_bind_56, ptr, RID.GetPtr(mesh));
            return (AABB)ret;
        }

        private static IntPtr method_bind_57 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "mesh_clear");

        /// <summary>
        /// Removes all surfaces from a mesh.
        /// </summary>
        [GodotMethod("mesh_clear")]
        public static void MeshClear(RID mesh)
        {
            NativeCalls.godot_icall_1_627(method_bind_57, ptr, RID.GetPtr(mesh));
        }

        private static IntPtr method_bind_58 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_create");

        /// <summary>
        /// Creates an empty viewport.
        /// </summary>
        [GodotMethod("viewport_create")]
        public static RID ViewportCreate()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_58, ptr);
            return new RID(ret);
        }

        private static IntPtr method_bind_59 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_set_use_arvr");

        /// <summary>
        /// If [code]true[/code] the viewport should use augmented or virtual reality technologies. See [ARVRInterface].
        /// </summary>
        [GodotMethod("viewport_set_use_arvr")]
        public static void ViewportSetUseArvr(RID viewport, bool useArvr)
        {
            NativeCalls.godot_icall_2_610(method_bind_59, ptr, RID.GetPtr(viewport), useArvr);
        }

        private static IntPtr method_bind_60 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_set_size");

        /// <summary>
        /// Sets the viewport's width and height.
        /// </summary>
        [GodotMethod("viewport_set_size")]
        public static void ViewportSetSize(RID viewport, int width, int height)
        {
            NativeCalls.godot_icall_3_975(method_bind_60, ptr, RID.GetPtr(viewport), width, height);
        }

        private static IntPtr method_bind_61 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_set_active");

        /// <summary>
        /// If [code]true[/code], sets the viewport active, else sets it inactive.
        /// </summary>
        [GodotMethod("viewport_set_active")]
        public static void ViewportSetActive(RID viewport, bool active)
        {
            NativeCalls.godot_icall_2_610(method_bind_61, ptr, RID.GetPtr(viewport), active);
        }

        private static IntPtr method_bind_62 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_set_parent_viewport");

        /// <summary>
        /// Sets the viewport's parent to another viewport.
        /// </summary>
        [GodotMethod("viewport_set_parent_viewport")]
        public static void ViewportSetParentViewport(RID viewport, RID parentViewport)
        {
            NativeCalls.godot_icall_2_615(method_bind_62, ptr, RID.GetPtr(viewport), RID.GetPtr(parentViewport));
        }

        private static IntPtr method_bind_63 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_attach_to_screen");

        /// <summary>
        /// Attaches a viewport to a screen.
        /// &lt;param name="rect"&gt;If the param is null, then the default value is new Rect2(0, 0, 0, 0)&lt;/param&gt;
        /// </summary>
        [GodotMethod("viewport_attach_to_screen")]
        public static void ViewportAttachToScreen(RID viewport, Nullable<Rect2> rect = null, int screen = 0)
        {
            Rect2 rect_in = rect.HasValue ? rect.Value : new Rect2(0, 0, 0, 0);
            NativeCalls.godot_icall_3_994(method_bind_63, ptr, RID.GetPtr(viewport), ref rect_in, screen);
        }

        private static IntPtr method_bind_64 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_detach");

        /// <summary>
        /// Detaches the viewport from the screen.
        /// </summary>
        [GodotMethod("viewport_detach")]
        public static void ViewportDetach(RID viewport)
        {
            NativeCalls.godot_icall_1_627(method_bind_64, ptr, RID.GetPtr(viewport));
        }

        private static IntPtr method_bind_65 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_set_update_mode");

        /// <summary>
        /// Sets when the viewport should be updated. See VIEWPORT_UPDATE_MODE_* constants for options.
        /// </summary>
        [GodotMethod("viewport_set_update_mode")]
        public static void ViewportSetUpdateMode(RID viewport, VisualServer.ViewportUpdateMode updateMode)
        {
            NativeCalls.godot_icall_2_995(method_bind_65, ptr, RID.GetPtr(viewport), updateMode);
        }

        private static IntPtr method_bind_66 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_set_vflip");

        /// <summary>
        /// If [code]true[/code] the viewport's rendering should be flipped vertically.
        /// </summary>
        [GodotMethod("viewport_set_vflip")]
        public static void ViewportSetVflip(RID viewport, bool enabled)
        {
            NativeCalls.godot_icall_2_610(method_bind_66, ptr, RID.GetPtr(viewport), enabled);
        }

        private static IntPtr method_bind_67 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_set_clear_mode");

        /// <summary>
        /// Sets the clear mode of a viewport. See VIEWPORT_CLEAR_MODE_* constants for options.
        /// </summary>
        [GodotMethod("viewport_set_clear_mode")]
        public static void ViewportSetClearMode(RID viewport, VisualServer.ViewportClearMode clearMode)
        {
            NativeCalls.godot_icall_2_996(method_bind_67, ptr, RID.GetPtr(viewport), clearMode);
        }

        private static IntPtr method_bind_68 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_get_texture");

        /// <summary>
        /// Returns the viewport's last rendered frame.
        /// </summary>
        [GodotMethod("viewport_get_texture")]
        public static RID ViewportGetTexture(RID viewport)
        {
            IntPtr ret = NativeCalls.godot_icall_1_616(method_bind_68, ptr, RID.GetPtr(viewport));
            return new RID(ret);
        }

        private static IntPtr method_bind_69 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_set_hide_scenario");

        /// <summary>
        /// </summary>
        [GodotMethod("viewport_set_hide_scenario")]
        public static void ViewportSetHideScenario(RID viewport, bool hidden)
        {
            NativeCalls.godot_icall_2_610(method_bind_69, ptr, RID.GetPtr(viewport), hidden);
        }

        private static IntPtr method_bind_70 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_set_hide_canvas");

        /// <summary>
        /// If [code]true[/code] the viewport's canvas should not be rendered.
        /// </summary>
        [GodotMethod("viewport_set_hide_canvas")]
        public static void ViewportSetHideCanvas(RID viewport, bool hidden)
        {
            NativeCalls.godot_icall_2_610(method_bind_70, ptr, RID.GetPtr(viewport), hidden);
        }

        private static IntPtr method_bind_71 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_set_disable_environment");

        /// <summary>
        /// If [code]true[/code] rendering of a viewport's environment should be disabled.
        /// </summary>
        [GodotMethod("viewport_set_disable_environment")]
        public static void ViewportSetDisableEnvironment(RID viewport, bool disabled)
        {
            NativeCalls.godot_icall_2_610(method_bind_71, ptr, RID.GetPtr(viewport), disabled);
        }

        private static IntPtr method_bind_72 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_set_disable_3d");

        /// <summary>
        /// If [code]true[/code] a viewport's 3D rendering should be disabled.
        /// </summary>
        [GodotMethod("viewport_set_disable_3d")]
        public static void ViewportSetDisable3d(RID viewport, bool disabled)
        {
            NativeCalls.godot_icall_2_610(method_bind_72, ptr, RID.GetPtr(viewport), disabled);
        }

        private static IntPtr method_bind_73 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_attach_camera");

        /// <summary>
        /// Sets a viewport's camera.
        /// </summary>
        [GodotMethod("viewport_attach_camera")]
        public static void ViewportAttachCamera(RID viewport, RID camera)
        {
            NativeCalls.godot_icall_2_615(method_bind_73, ptr, RID.GetPtr(viewport), RID.GetPtr(camera));
        }

        private static IntPtr method_bind_74 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_set_scenario");

        /// <summary>
        /// Sets a viewport's scenario.
        /// The scenario contains information about the [enum ScenarioDebugMode], environment information, reflection atlas etc.
        /// </summary>
        [GodotMethod("viewport_set_scenario")]
        public static void ViewportSetScenario(RID viewport, RID scenario)
        {
            NativeCalls.godot_icall_2_615(method_bind_74, ptr, RID.GetPtr(viewport), RID.GetPtr(scenario));
        }

        private static IntPtr method_bind_75 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_attach_canvas");

        /// <summary>
        /// Sets a viewport's canvas.
        /// </summary>
        [GodotMethod("viewport_attach_canvas")]
        public static void ViewportAttachCanvas(RID viewport, RID canvas)
        {
            NativeCalls.godot_icall_2_615(method_bind_75, ptr, RID.GetPtr(viewport), RID.GetPtr(canvas));
        }

        private static IntPtr method_bind_76 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_remove_canvas");

        /// <summary>
        /// Detaches a viewport from a canvas and vice versa.
        /// </summary>
        [GodotMethod("viewport_remove_canvas")]
        public static void ViewportRemoveCanvas(RID viewport, RID canvas)
        {
            NativeCalls.godot_icall_2_615(method_bind_76, ptr, RID.GetPtr(viewport), RID.GetPtr(canvas));
        }

        private static IntPtr method_bind_77 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_set_canvas_transform");

        /// <summary>
        /// Sets the transformation of a viewport's canvas.
        /// </summary>
        [GodotMethod("viewport_set_canvas_transform")]
        public static void ViewportSetCanvasTransform(RID viewport, RID canvas, Transform2D offset)
        {
            NativeCalls.godot_icall_3_619(method_bind_77, ptr, RID.GetPtr(viewport), RID.GetPtr(canvas), ref offset);
        }

        private static IntPtr method_bind_78 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_set_transparent_background");

        /// <summary>
        /// If [code]true[/code] the viewport should render its background as transparent.
        /// </summary>
        [GodotMethod("viewport_set_transparent_background")]
        public static void ViewportSetTransparentBackground(RID viewport, bool enabled)
        {
            NativeCalls.godot_icall_2_610(method_bind_78, ptr, RID.GetPtr(viewport), enabled);
        }

        private static IntPtr method_bind_79 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_set_global_canvas_transform");

        /// <summary>
        /// Sets the viewport's global transformation matrix.
        /// </summary>
        [GodotMethod("viewport_set_global_canvas_transform")]
        public static void ViewportSetGlobalCanvasTransform(RID viewport, Transform2D transform)
        {
            NativeCalls.godot_icall_2_629(method_bind_79, ptr, RID.GetPtr(viewport), ref transform);
        }

        private static IntPtr method_bind_80 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_set_canvas_layer");

        /// <summary>
        /// Sets the renderlayer for a viewport's canvas.
        /// </summary>
        [GodotMethod("viewport_set_canvas_layer")]
        public static void ViewportSetCanvasLayer(RID viewport, RID canvas, int layer)
        {
            NativeCalls.godot_icall_3_978(method_bind_80, ptr, RID.GetPtr(viewport), RID.GetPtr(canvas), layer);
        }

        private static IntPtr method_bind_81 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_set_shadow_atlas_size");

        /// <summary>
        /// Sets the size of the shadow atlas's images.
        /// </summary>
        [GodotMethod("viewport_set_shadow_atlas_size")]
        public static void ViewportSetShadowAtlasSize(RID viewport, int size)
        {
            NativeCalls.godot_icall_2_626(method_bind_81, ptr, RID.GetPtr(viewport), size);
        }

        private static IntPtr method_bind_82 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_set_shadow_atlas_quadrant_subdivision");

        /// <summary>
        /// Sets the shadow atlas quadrant's subdivision.
        /// </summary>
        [GodotMethod("viewport_set_shadow_atlas_quadrant_subdivision")]
        public static void ViewportSetShadowAtlasQuadrantSubdivision(RID viewport, int quadrant, int subdivision)
        {
            NativeCalls.godot_icall_3_975(method_bind_82, ptr, RID.GetPtr(viewport), quadrant, subdivision);
        }

        private static IntPtr method_bind_83 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_set_msaa");

        /// <summary>
        /// Sets the anti-aliasing mode. see [enum ViewportMSAA] for options.
        /// </summary>
        [GodotMethod("viewport_set_msaa")]
        public static void ViewportSetMsaa(RID viewport, VisualServer.ViewportMSAA msaa)
        {
            NativeCalls.godot_icall_2_997(method_bind_83, ptr, RID.GetPtr(viewport), msaa);
        }

        private static IntPtr method_bind_84 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_set_hdr");

        /// <summary>
        /// If [code]true[/code] the viewport should render to hdr.
        /// </summary>
        [GodotMethod("viewport_set_hdr")]
        public static void ViewportSetHdr(RID viewport, bool enabled)
        {
            NativeCalls.godot_icall_2_610(method_bind_84, ptr, RID.GetPtr(viewport), enabled);
        }

        private static IntPtr method_bind_85 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_set_usage");

        /// <summary>
        /// Sets what should be rendered in the viewport. See VIEWPORT_USAGE_* constants for options.
        /// </summary>
        [GodotMethod("viewport_set_usage")]
        public static void ViewportSetUsage(RID viewport, VisualServer.ViewportUsage usage)
        {
            NativeCalls.godot_icall_2_998(method_bind_85, ptr, RID.GetPtr(viewport), usage);
        }

        private static IntPtr method_bind_86 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_get_render_info");

        /// <summary>
        /// Returns a viewport's render info. for options see VIEWPORT_RENDER_INFO* constants.
        /// </summary>
        [GodotMethod("viewport_get_render_info")]
        public static int ViewportGetRenderInfo(RID viewport, VisualServer.ViewportRenderInfo info)
        {
            return NativeCalls.godot_icall_2_999(method_bind_86, ptr, RID.GetPtr(viewport), info);
        }

        private static IntPtr method_bind_87 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "viewport_set_debug_draw");

        /// <summary>
        /// Sets the debug draw mode of a viewport. See VIEWPORT_DEBUG_DRAW_* constants for options.
        /// </summary>
        [GodotMethod("viewport_set_debug_draw")]
        public static void ViewportSetDebugDraw(RID viewport, VisualServer.ViewportDebugDraw draw)
        {
            NativeCalls.godot_icall_2_1000(method_bind_87, ptr, RID.GetPtr(viewport), draw);
        }

        private static IntPtr method_bind_88 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_create");

        /// <summary>
        /// Creates a canvas and returns the assigned [RID].
        /// </summary>
        [GodotMethod("canvas_create")]
        public static RID CanvasCreate()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_88, ptr);
            return new RID(ret);
        }

        private static IntPtr method_bind_89 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_set_item_mirroring");

        /// <summary>
        /// A copy of the canvas item will be drawn with a local offset of the mirroring [Vector2].
        /// </summary>
        [GodotMethod("canvas_set_item_mirroring")]
        public static void CanvasSetItemMirroring(RID canvas, RID item, Vector2 mirroring)
        {
            NativeCalls.godot_icall_3_1001(method_bind_89, ptr, RID.GetPtr(canvas), RID.GetPtr(item), ref mirroring);
        }

        private static IntPtr method_bind_90 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_set_modulate");

        /// <summary>
        /// Modulates all colors in the given canvas.
        /// </summary>
        [GodotMethod("canvas_set_modulate")]
        public static void CanvasSetModulate(RID canvas, Color color)
        {
            NativeCalls.godot_icall_2_1002(method_bind_90, ptr, RID.GetPtr(canvas), ref color);
        }

        private static IntPtr method_bind_91 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_create");

        /// <summary>
        /// Creates a new [CanvasItem] and returns its [RID].
        /// </summary>
        [GodotMethod("canvas_item_create")]
        public static RID CanvasItemCreate()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_91, ptr);
            return new RID(ret);
        }

        private static IntPtr method_bind_92 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_set_parent");

        /// <summary>
        /// Sets the parent for the [CanvasItem].
        /// </summary>
        [GodotMethod("canvas_item_set_parent")]
        public static void CanvasItemSetParent(RID item, RID parent)
        {
            NativeCalls.godot_icall_2_615(method_bind_92, ptr, RID.GetPtr(item), RID.GetPtr(parent));
        }

        private static IntPtr method_bind_93 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_set_visible");

        /// <summary>
        /// Sets if the canvas item (including its children) is visible.
        /// </summary>
        [GodotMethod("canvas_item_set_visible")]
        public static void CanvasItemSetVisible(RID item, bool visible)
        {
            NativeCalls.godot_icall_2_610(method_bind_93, ptr, RID.GetPtr(item), visible);
        }

        private static IntPtr method_bind_94 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_set_light_mask");

        /// <summary>
        /// The light mask. See [LightOccluder2D] for more information on light masks.
        /// </summary>
        [GodotMethod("canvas_item_set_light_mask")]
        public static void CanvasItemSetLightMask(RID item, int mask)
        {
            NativeCalls.godot_icall_2_626(method_bind_94, ptr, RID.GetPtr(item), mask);
        }

        private static IntPtr method_bind_95 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_set_transform");

        /// <summary>
        /// Sets the [CanvasItem]'s [Transform2D].
        /// </summary>
        [GodotMethod("canvas_item_set_transform")]
        public static void CanvasItemSetTransform(RID item, Transform2D transform)
        {
            NativeCalls.godot_icall_2_629(method_bind_95, ptr, RID.GetPtr(item), ref transform);
        }

        private static IntPtr method_bind_96 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_set_clip");

        /// <summary>
        /// Sets clipping for the [CanvasItem].
        /// </summary>
        [GodotMethod("canvas_item_set_clip")]
        public static void CanvasItemSetClip(RID item, bool clip)
        {
            NativeCalls.godot_icall_2_610(method_bind_96, ptr, RID.GetPtr(item), clip);
        }

        private static IntPtr method_bind_97 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_set_distance_field_mode");

        /// <summary>
        /// </summary>
        [GodotMethod("canvas_item_set_distance_field_mode")]
        public static void CanvasItemSetDistanceFieldMode(RID item, bool enabled)
        {
            NativeCalls.godot_icall_2_610(method_bind_97, ptr, RID.GetPtr(item), enabled);
        }

        private static IntPtr method_bind_98 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_set_custom_rect");

        /// <summary>
        /// Defines a custom drawing rectangle for the [CanvasItem].
        /// &lt;param name="rect"&gt;If the param is null, then the default value is new Rect2(0, 0, 0, 0)&lt;/param&gt;
        /// </summary>
        [GodotMethod("canvas_item_set_custom_rect")]
        public static void CanvasItemSetCustomRect(RID item, bool useCustomRect, Nullable<Rect2> rect = null)
        {
            Rect2 rect_in = rect.HasValue ? rect.Value : new Rect2(0, 0, 0, 0);
            NativeCalls.godot_icall_3_1003(method_bind_98, ptr, RID.GetPtr(item), useCustomRect, ref rect_in);
        }

        private static IntPtr method_bind_99 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_set_modulate");

        /// <summary>
        /// Sets the color that modulates the [CanvasItem] and its children.
        /// </summary>
        [GodotMethod("canvas_item_set_modulate")]
        public static void CanvasItemSetModulate(RID item, Color color)
        {
            NativeCalls.godot_icall_2_1002(method_bind_99, ptr, RID.GetPtr(item), ref color);
        }

        private static IntPtr method_bind_100 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_set_self_modulate");

        /// <summary>
        /// Sets the color that modulates the [CanvasItem] without children.
        /// </summary>
        [GodotMethod("canvas_item_set_self_modulate")]
        public static void CanvasItemSetSelfModulate(RID item, Color color)
        {
            NativeCalls.godot_icall_2_1002(method_bind_100, ptr, RID.GetPtr(item), ref color);
        }

        private static IntPtr method_bind_101 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_set_draw_behind_parent");

        /// <summary>
        /// Sets [CanvasItem] to be drawn behind its parent.
        /// </summary>
        [GodotMethod("canvas_item_set_draw_behind_parent")]
        public static void CanvasItemSetDrawBehindParent(RID item, bool enabled)
        {
            NativeCalls.godot_icall_2_610(method_bind_101, ptr, RID.GetPtr(item), enabled);
        }

        private static IntPtr method_bind_102 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_add_line");

        /// <summary>
        /// Adds a line command to the [CanvasItem]'s draw commands.
        /// </summary>
        [GodotMethod("canvas_item_add_line")]
        public static void CanvasItemAddLine(RID item, Vector2 from, Vector2 to, Color color, float width = 1f, bool antialiased = false)
        {
            NativeCalls.godot_icall_6_1004(method_bind_102, ptr, RID.GetPtr(item), ref from, ref to, ref color, width, antialiased);
        }

        private static IntPtr method_bind_103 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_add_polyline");

        /// <summary>
        /// Adds a polyline, which is a line from mutliple points with a width, to the [CanvasItem]'s draw commands.
        /// </summary>
        [GodotMethod("canvas_item_add_polyline")]
        public static void CanvasItemAddPolyline(RID item, Vector2[] points, Color[] colors, float width = 1f, bool antialiased = false)
        {
            NativeCalls.godot_icall_5_1005(method_bind_103, ptr, RID.GetPtr(item), points, colors, width, antialiased);
        }

        private static IntPtr method_bind_104 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_add_rect");

        /// <summary>
        /// Adds a rectangle to the [CanvasItem]'s draw commands.
        /// </summary>
        [GodotMethod("canvas_item_add_rect")]
        public static void CanvasItemAddRect(RID item, Rect2 rect, Color color)
        {
            NativeCalls.godot_icall_3_1006(method_bind_104, ptr, RID.GetPtr(item), ref rect, ref color);
        }

        private static IntPtr method_bind_105 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_add_circle");

        /// <summary>
        /// Adds a circle command to the [CanvasItem]'s draw commands.
        /// </summary>
        [GodotMethod("canvas_item_add_circle")]
        public static void CanvasItemAddCircle(RID item, Vector2 pos, float radius, Color color)
        {
            NativeCalls.godot_icall_4_1007(method_bind_105, ptr, RID.GetPtr(item), ref pos, radius, ref color);
        }

        private static IntPtr method_bind_106 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_add_texture_rect");

        /// <summary>
        /// Adds a textured rect to the [CanvasItem]'s draw commands.
        /// &lt;param name="modulate"&gt;If the param is null, then the default value is new Color(1, 1, 1, 1)&lt;/param&gt;
        /// &lt;param name="normalMap"&gt;If the param is null, then the default value is new RID()&lt;/param&gt;
        /// </summary>
        [GodotMethod("canvas_item_add_texture_rect")]
        public static void CanvasItemAddTextureRect(RID item, Rect2 rect, RID texture, bool tile = false, Nullable<Color> modulate = null, bool transpose = false, RID normalMap = null)
        {
            Color modulate_in = modulate.HasValue ? modulate.Value : new Color(1, 1, 1, 1);
            RID normalMap_in = normalMap != null ? normalMap : new RID();
            NativeCalls.godot_icall_7_1008(method_bind_106, ptr, RID.GetPtr(item), ref rect, RID.GetPtr(texture), tile, ref modulate_in, transpose, RID.GetPtr(normalMap_in));
        }

        private static IntPtr method_bind_107 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_add_texture_rect_region");

        /// <summary>
        /// Adds a texture rect with region setting to the [CanvasItem]'s draw commands.
        /// &lt;param name="modulate"&gt;If the param is null, then the default value is new Color(1, 1, 1, 1)&lt;/param&gt;
        /// &lt;param name="normalMap"&gt;If the param is null, then the default value is new RID()&lt;/param&gt;
        /// </summary>
        [GodotMethod("canvas_item_add_texture_rect_region")]
        public static void CanvasItemAddTextureRectRegion(RID item, Rect2 rect, RID texture, Rect2 srcRect, Nullable<Color> modulate = null, bool transpose = false, RID normalMap = null, bool clipUv = true)
        {
            Color modulate_in = modulate.HasValue ? modulate.Value : new Color(1, 1, 1, 1);
            RID normalMap_in = normalMap != null ? normalMap : new RID();
            NativeCalls.godot_icall_8_1009(method_bind_107, ptr, RID.GetPtr(item), ref rect, RID.GetPtr(texture), ref srcRect, ref modulate_in, transpose, RID.GetPtr(normalMap_in), clipUv);
        }

        private static IntPtr method_bind_108 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_add_nine_patch");

        /// <summary>
        /// Adds a nine patch image to the [CanvasItem]'s draw commands.
        /// See [NinePatchRect] for more explanation.
        /// &lt;param name="modulate"&gt;If the param is null, then the default value is new Color(1, 1, 1, 1)&lt;/param&gt;
        /// &lt;param name="normalMap"&gt;If the param is null, then the default value is new RID()&lt;/param&gt;
        /// </summary>
        [GodotMethod("canvas_item_add_nine_patch")]
        public static void CanvasItemAddNinePatch(RID item, Rect2 rect, Rect2 source, RID texture, Vector2 topleft, Vector2 bottomright, VisualServer.NinePatchAxisMode xAxisMode = (VisualServer.NinePatchAxisMode)0, VisualServer.NinePatchAxisMode yAxisMode = (VisualServer.NinePatchAxisMode)0, bool drawCenter = true, Nullable<Color> modulate = null, RID normalMap = null)
        {
            Color modulate_in = modulate.HasValue ? modulate.Value : new Color(1, 1, 1, 1);
            RID normalMap_in = normalMap != null ? normalMap : new RID();
            NativeCalls.godot_icall_11_1010(method_bind_108, ptr, RID.GetPtr(item), ref rect, ref source, RID.GetPtr(texture), ref topleft, ref bottomright, xAxisMode, yAxisMode, drawCenter, ref modulate_in, RID.GetPtr(normalMap_in));
        }

        private static IntPtr method_bind_109 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_add_primitive");

        /// <summary>
        /// Adds a primitive to the [CanvasItem]'s draw commands.
        /// &lt;param name="normalMap"&gt;If the param is null, then the default value is new RID()&lt;/param&gt;
        /// </summary>
        [GodotMethod("canvas_item_add_primitive")]
        public static void CanvasItemAddPrimitive(RID item, Vector2[] points, Color[] colors, Vector2[] uvs, RID texture, float width = 1f, RID normalMap = null)
        {
            RID normalMap_in = normalMap != null ? normalMap : new RID();
            NativeCalls.godot_icall_7_1011(method_bind_109, ptr, RID.GetPtr(item), points, colors, uvs, RID.GetPtr(texture), width, RID.GetPtr(normalMap_in));
        }

        private static IntPtr method_bind_110 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_add_polygon");

        /// <summary>
        /// Adds a polygon to the [CanvasItem]'s draw commands.
        /// &lt;param name="uvs"&gt;If the param is null, then the default value is new Vector2[] {}&lt;/param&gt;
        /// &lt;param name="texture"&gt;If the param is null, then the default value is new RID()&lt;/param&gt;
        /// &lt;param name="normalMap"&gt;If the param is null, then the default value is new RID()&lt;/param&gt;
        /// </summary>
        [GodotMethod("canvas_item_add_polygon")]
        public static void CanvasItemAddPolygon(RID item, Vector2[] points, Color[] colors, Vector2[] uvs = null, RID texture = null, RID normalMap = null, bool antialiased = false)
        {
            Vector2[] uvs_in = uvs != null ? uvs : new Vector2[] {};
            RID texture_in = texture != null ? texture : new RID();
            RID normalMap_in = normalMap != null ? normalMap : new RID();
            NativeCalls.godot_icall_7_1012(method_bind_110, ptr, RID.GetPtr(item), points, colors, uvs_in, RID.GetPtr(texture_in), RID.GetPtr(normalMap_in), antialiased);
        }

        private static IntPtr method_bind_111 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_add_triangle_array");

        /// <summary>
        /// Adds a triangle array to the [CanvasItem]'s draw commands.
        /// &lt;param name="uvs"&gt;If the param is null, then the default value is new Vector2[] {}&lt;/param&gt;
        /// &lt;param name="texture"&gt;If the param is null, then the default value is new RID()&lt;/param&gt;
        /// &lt;param name="normalMap"&gt;If the param is null, then the default value is new RID()&lt;/param&gt;
        /// </summary>
        [GodotMethod("canvas_item_add_triangle_array")]
        public static void CanvasItemAddTriangleArray(RID item, int[] indices, Vector2[] points, Color[] colors, Vector2[] uvs = null, RID texture = null, int count = -1, RID normalMap = null)
        {
            Vector2[] uvs_in = uvs != null ? uvs : new Vector2[] {};
            RID texture_in = texture != null ? texture : new RID();
            RID normalMap_in = normalMap != null ? normalMap : new RID();
            NativeCalls.godot_icall_8_1013(method_bind_111, ptr, RID.GetPtr(item), indices, points, colors, uvs_in, RID.GetPtr(texture_in), count, RID.GetPtr(normalMap_in));
        }

        private static IntPtr method_bind_112 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_add_mesh");

        /// <summary>
        /// Adds a [Mesh] to the [CanvasItem]'s draw commands. Only affects its aabb at the moment.
        /// &lt;param name="skeleton"&gt;If the param is null, then the default value is new RID()&lt;/param&gt;
        /// </summary>
        [GodotMethod("canvas_item_add_mesh")]
        public static void CanvasItemAddMesh(RID item, RID mesh, RID skeleton = null)
        {
            RID skeleton_in = skeleton != null ? skeleton : new RID();
            NativeCalls.godot_icall_3_1014(method_bind_112, ptr, RID.GetPtr(item), RID.GetPtr(mesh), RID.GetPtr(skeleton_in));
        }

        private static IntPtr method_bind_113 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_add_multimesh");

        /// <summary>
        /// Adds a [MultiMesh] to the [CanvasItem]'s draw commands. Only affects its aabb at the moment.
        /// &lt;param name="skeleton"&gt;If the param is null, then the default value is new RID()&lt;/param&gt;
        /// </summary>
        [GodotMethod("canvas_item_add_multimesh")]
        public static void CanvasItemAddMultimesh(RID item, RID mesh, RID skeleton = null)
        {
            RID skeleton_in = skeleton != null ? skeleton : new RID();
            NativeCalls.godot_icall_3_1014(method_bind_113, ptr, RID.GetPtr(item), RID.GetPtr(mesh), RID.GetPtr(skeleton_in));
        }

        private static IntPtr method_bind_114 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_add_particles");

        /// <summary>
        /// Adds a particles system to the [CanvasItem]'s draw commands.
        /// </summary>
        [GodotMethod("canvas_item_add_particles")]
        public static void CanvasItemAddParticles(RID item, RID particles, RID texture, RID normalMap, int hFrames, int vFrames)
        {
            NativeCalls.godot_icall_6_1015(method_bind_114, ptr, RID.GetPtr(item), RID.GetPtr(particles), RID.GetPtr(texture), RID.GetPtr(normalMap), hFrames, vFrames);
        }

        private static IntPtr method_bind_115 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_add_set_transform");

        /// <summary>
        /// Adds a [Transform2D] command to the [CanvasItem]'s draw commands.
        /// This sets the extra_matrix uniform when executed. This affects the later command's of the canvas item.
        /// </summary>
        [GodotMethod("canvas_item_add_set_transform")]
        public static void CanvasItemAddSetTransform(RID item, Transform2D transform)
        {
            NativeCalls.godot_icall_2_629(method_bind_115, ptr, RID.GetPtr(item), ref transform);
        }

        private static IntPtr method_bind_116 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_add_clip_ignore");

        /// <summary>
        /// If ignore is [code]true[/code], the VisualServer does not perform clipping.
        /// </summary>
        [GodotMethod("canvas_item_add_clip_ignore")]
        public static void CanvasItemAddClipIgnore(RID item, bool ignore)
        {
            NativeCalls.godot_icall_2_610(method_bind_116, ptr, RID.GetPtr(item), ignore);
        }

        private static IntPtr method_bind_117 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_set_sort_children_by_y");

        /// <summary>
        /// Sets if [CanvasItem]'s children should be sorted by y-position.
        /// </summary>
        [GodotMethod("canvas_item_set_sort_children_by_y")]
        public static void CanvasItemSetSortChildrenByY(RID item, bool enabled)
        {
            NativeCalls.godot_icall_2_610(method_bind_117, ptr, RID.GetPtr(item), enabled);
        }

        private static IntPtr method_bind_118 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_set_z_index");

        /// <summary>
        /// Sets the [CanvasItem]'s z-index, i.e. its draw order (lower indexes are drawn first).
        /// </summary>
        [GodotMethod("canvas_item_set_z_index")]
        public static void CanvasItemSetZIndex(RID item, int zIndex)
        {
            NativeCalls.godot_icall_2_626(method_bind_118, ptr, RID.GetPtr(item), zIndex);
        }

        private static IntPtr method_bind_119 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_set_z_as_relative_to_parent");

        /// <summary>
        /// If this is enabled, the z-index of the parent will be added to the children's z-index.
        /// </summary>
        [GodotMethod("canvas_item_set_z_as_relative_to_parent")]
        public static void CanvasItemSetZAsRelativeToParent(RID item, bool enabled)
        {
            NativeCalls.godot_icall_2_610(method_bind_119, ptr, RID.GetPtr(item), enabled);
        }

        private static IntPtr method_bind_120 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_set_copy_to_backbuffer");

        /// <summary>
        /// Sets the [CanvasItem] to copy a rect to the backbuffer.
        /// </summary>
        [GodotMethod("canvas_item_set_copy_to_backbuffer")]
        public static void CanvasItemSetCopyToBackbuffer(RID item, bool enabled, Rect2 rect)
        {
            NativeCalls.godot_icall_3_1003(method_bind_120, ptr, RID.GetPtr(item), enabled, ref rect);
        }

        private static IntPtr method_bind_121 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_clear");

        /// <summary>
        /// Clears the [CanvasItem] and removes all commands in it.
        /// </summary>
        [GodotMethod("canvas_item_clear")]
        public static void CanvasItemClear(RID item)
        {
            NativeCalls.godot_icall_1_627(method_bind_121, ptr, RID.GetPtr(item));
        }

        private static IntPtr method_bind_122 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_set_draw_index");

        /// <summary>
        /// Sets the index for the [CanvasItem].
        /// </summary>
        [GodotMethod("canvas_item_set_draw_index")]
        public static void CanvasItemSetDrawIndex(RID item, int index)
        {
            NativeCalls.godot_icall_2_626(method_bind_122, ptr, RID.GetPtr(item), index);
        }

        private static IntPtr method_bind_123 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_set_material");

        /// <summary>
        /// Sets a new material to the [CanvasItem].
        /// </summary>
        [GodotMethod("canvas_item_set_material")]
        public static void CanvasItemSetMaterial(RID item, RID material)
        {
            NativeCalls.godot_icall_2_615(method_bind_123, ptr, RID.GetPtr(item), RID.GetPtr(material));
        }

        private static IntPtr method_bind_124 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_item_set_use_parent_material");

        /// <summary>
        /// Sets if the [CanvasItem] uses its parent's material.
        /// </summary>
        [GodotMethod("canvas_item_set_use_parent_material")]
        public static void CanvasItemSetUseParentMaterial(RID item, bool enabled)
        {
            NativeCalls.godot_icall_2_610(method_bind_124, ptr, RID.GetPtr(item), enabled);
        }

        private static IntPtr method_bind_125 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_light_create");

        /// <summary>
        /// Creates a canvas light.
        /// </summary>
        [GodotMethod("canvas_light_create")]
        public static RID CanvasLightCreate()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_125, ptr);
            return new RID(ret);
        }

        private static IntPtr method_bind_126 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_light_attach_to_canvas");

        /// <summary>
        /// Attaches the canvas light to the canvas. Removes it from its previous canvas.
        /// </summary>
        [GodotMethod("canvas_light_attach_to_canvas")]
        public static void CanvasLightAttachToCanvas(RID light, RID canvas)
        {
            NativeCalls.godot_icall_2_615(method_bind_126, ptr, RID.GetPtr(light), RID.GetPtr(canvas));
        }

        private static IntPtr method_bind_127 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_light_set_enabled");

        /// <summary>
        /// Enables or disables a canvas light.
        /// </summary>
        [GodotMethod("canvas_light_set_enabled")]
        public static void CanvasLightSetEnabled(RID light, bool enabled)
        {
            NativeCalls.godot_icall_2_610(method_bind_127, ptr, RID.GetPtr(light), enabled);
        }

        private static IntPtr method_bind_128 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_light_set_scale");

        /// <summary>
        /// </summary>
        [GodotMethod("canvas_light_set_scale")]
        public static void CanvasLightSetScale(RID light, float scale)
        {
            NativeCalls.godot_icall_2_680(method_bind_128, ptr, RID.GetPtr(light), scale);
        }

        private static IntPtr method_bind_129 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_light_set_transform");

        /// <summary>
        /// Sets the canvas light's [Transform2D].
        /// </summary>
        [GodotMethod("canvas_light_set_transform")]
        public static void CanvasLightSetTransform(RID light, Transform2D transform)
        {
            NativeCalls.godot_icall_2_629(method_bind_129, ptr, RID.GetPtr(light), ref transform);
        }

        private static IntPtr method_bind_130 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_light_set_texture");

        /// <summary>
        /// </summary>
        [GodotMethod("canvas_light_set_texture")]
        public static void CanvasLightSetTexture(RID light, RID texture)
        {
            NativeCalls.godot_icall_2_615(method_bind_130, ptr, RID.GetPtr(light), RID.GetPtr(texture));
        }

        private static IntPtr method_bind_131 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_light_set_texture_offset");

        /// <summary>
        /// </summary>
        [GodotMethod("canvas_light_set_texture_offset")]
        public static void CanvasLightSetTextureOffset(RID light, Vector2 offset)
        {
            NativeCalls.godot_icall_2_644(method_bind_131, ptr, RID.GetPtr(light), ref offset);
        }

        private static IntPtr method_bind_132 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_light_set_color");

        /// <summary>
        /// Sets the color for a light.
        /// </summary>
        [GodotMethod("canvas_light_set_color")]
        public static void CanvasLightSetColor(RID light, Color color)
        {
            NativeCalls.godot_icall_2_1002(method_bind_132, ptr, RID.GetPtr(light), ref color);
        }

        private static IntPtr method_bind_133 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_light_set_height");

        /// <summary>
        /// Sets a canvas light's height.
        /// </summary>
        [GodotMethod("canvas_light_set_height")]
        public static void CanvasLightSetHeight(RID light, float height)
        {
            NativeCalls.godot_icall_2_680(method_bind_133, ptr, RID.GetPtr(light), height);
        }

        private static IntPtr method_bind_134 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_light_set_energy");

        /// <summary>
        /// Sets a canvas light's energy.
        /// </summary>
        [GodotMethod("canvas_light_set_energy")]
        public static void CanvasLightSetEnergy(RID light, float energy)
        {
            NativeCalls.godot_icall_2_680(method_bind_134, ptr, RID.GetPtr(light), energy);
        }

        private static IntPtr method_bind_135 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_light_set_z_range");

        /// <summary>
        /// </summary>
        [GodotMethod("canvas_light_set_z_range")]
        public static void CanvasLightSetZRange(RID light, int minZ, int maxZ)
        {
            NativeCalls.godot_icall_3_975(method_bind_135, ptr, RID.GetPtr(light), minZ, maxZ);
        }

        private static IntPtr method_bind_136 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_light_set_layer_range");

        /// <summary>
        /// The layer range that gets rendered with this light.
        /// </summary>
        [GodotMethod("canvas_light_set_layer_range")]
        public static void CanvasLightSetLayerRange(RID light, int minLayer, int maxLayer)
        {
            NativeCalls.godot_icall_3_975(method_bind_136, ptr, RID.GetPtr(light), minLayer, maxLayer);
        }

        private static IntPtr method_bind_137 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_light_set_item_cull_mask");

        /// <summary>
        /// The light mask. See [LightOccluder2D] for more information on light masks
        /// </summary>
        [GodotMethod("canvas_light_set_item_cull_mask")]
        public static void CanvasLightSetItemCullMask(RID light, int mask)
        {
            NativeCalls.godot_icall_2_626(method_bind_137, ptr, RID.GetPtr(light), mask);
        }

        private static IntPtr method_bind_138 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_light_set_item_shadow_cull_mask");

        /// <summary>
        /// The shadow mask. binary about wich layers this canvas light affects wich canvas item's shadows. See [LightOccluder2D] for more information on light masks.
        /// </summary>
        [GodotMethod("canvas_light_set_item_shadow_cull_mask")]
        public static void CanvasLightSetItemShadowCullMask(RID light, int mask)
        {
            NativeCalls.godot_icall_2_626(method_bind_138, ptr, RID.GetPtr(light), mask);
        }

        private static IntPtr method_bind_139 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_light_set_mode");

        /// <summary>
        /// The mode of the light, see CANVAS_LIGHT_MODE_* constants.
        /// </summary>
        [GodotMethod("canvas_light_set_mode")]
        public static void CanvasLightSetMode(RID light, VisualServer.CanvasLightMode mode)
        {
            NativeCalls.godot_icall_2_1016(method_bind_139, ptr, RID.GetPtr(light), mode);
        }

        private static IntPtr method_bind_140 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_light_set_shadow_enabled");

        /// <summary>
        /// Enables or disables the canvas light's shadow.
        /// </summary>
        [GodotMethod("canvas_light_set_shadow_enabled")]
        public static void CanvasLightSetShadowEnabled(RID light, bool enabled)
        {
            NativeCalls.godot_icall_2_610(method_bind_140, ptr, RID.GetPtr(light), enabled);
        }

        private static IntPtr method_bind_141 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_light_set_shadow_buffer_size");

        /// <summary>
        /// Sets the width of the shadow buffer, size gets scaled to the next power of two for this.
        /// </summary>
        [GodotMethod("canvas_light_set_shadow_buffer_size")]
        public static void CanvasLightSetShadowBufferSize(RID light, int size)
        {
            NativeCalls.godot_icall_2_626(method_bind_141, ptr, RID.GetPtr(light), size);
        }

        private static IntPtr method_bind_142 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_light_set_shadow_gradient_length");

        /// <summary>
        /// Sets the length of the shadow's gradient.
        /// </summary>
        [GodotMethod("canvas_light_set_shadow_gradient_length")]
        public static void CanvasLightSetShadowGradientLength(RID light, float length)
        {
            NativeCalls.godot_icall_2_680(method_bind_142, ptr, RID.GetPtr(light), length);
        }

        private static IntPtr method_bind_143 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_light_set_shadow_filter");

        /// <summary>
        /// Sets the canvas light's shadow's filter, see CANVAS_LIGHT_SHADOW_FILTER_* constants.
        /// </summary>
        [GodotMethod("canvas_light_set_shadow_filter")]
        public static void CanvasLightSetShadowFilter(RID light, VisualServer.CanvasLightShadowFilter filter)
        {
            NativeCalls.godot_icall_2_1017(method_bind_143, ptr, RID.GetPtr(light), filter);
        }

        private static IntPtr method_bind_144 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_light_set_shadow_color");

        /// <summary>
        /// Sets the color of the canvas light's shadow.
        /// </summary>
        [GodotMethod("canvas_light_set_shadow_color")]
        public static void CanvasLightSetShadowColor(RID light, Color color)
        {
            NativeCalls.godot_icall_2_1002(method_bind_144, ptr, RID.GetPtr(light), ref color);
        }

        private static IntPtr method_bind_145 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_light_set_shadow_smooth");

        /// <summary>
        /// Smoothens the shadow. The lower, the more smooth.
        /// </summary>
        [GodotMethod("canvas_light_set_shadow_smooth")]
        public static void CanvasLightSetShadowSmooth(RID light, float smooth)
        {
            NativeCalls.godot_icall_2_680(method_bind_145, ptr, RID.GetPtr(light), smooth);
        }

        private static IntPtr method_bind_146 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_light_occluder_create");

        /// <summary>
        /// Creates a light occluder.
        /// </summary>
        [GodotMethod("canvas_light_occluder_create")]
        public static RID CanvasLightOccluderCreate()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_146, ptr);
            return new RID(ret);
        }

        private static IntPtr method_bind_147 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_light_occluder_attach_to_canvas");

        /// <summary>
        /// Attaches a light occluder to the canvas. Removes it from its previous canvas.
        /// </summary>
        [GodotMethod("canvas_light_occluder_attach_to_canvas")]
        public static void CanvasLightOccluderAttachToCanvas(RID occluder, RID canvas)
        {
            NativeCalls.godot_icall_2_615(method_bind_147, ptr, RID.GetPtr(occluder), RID.GetPtr(canvas));
        }

        private static IntPtr method_bind_148 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_light_occluder_set_enabled");

        /// <summary>
        /// Enables or disables light occluder.
        /// </summary>
        [GodotMethod("canvas_light_occluder_set_enabled")]
        public static void CanvasLightOccluderSetEnabled(RID occluder, bool enabled)
        {
            NativeCalls.godot_icall_2_610(method_bind_148, ptr, RID.GetPtr(occluder), enabled);
        }

        private static IntPtr method_bind_149 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_light_occluder_set_polygon");

        /// <summary>
        /// Sets a light occluder's polygon.
        /// </summary>
        [GodotMethod("canvas_light_occluder_set_polygon")]
        public static void CanvasLightOccluderSetPolygon(RID occluder, RID polygon)
        {
            NativeCalls.godot_icall_2_615(method_bind_149, ptr, RID.GetPtr(occluder), RID.GetPtr(polygon));
        }

        private static IntPtr method_bind_150 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_light_occluder_set_transform");

        /// <summary>
        /// Sets a light occluder's [Transform2D].
        /// </summary>
        [GodotMethod("canvas_light_occluder_set_transform")]
        public static void CanvasLightOccluderSetTransform(RID occluder, Transform2D transform)
        {
            NativeCalls.godot_icall_2_629(method_bind_150, ptr, RID.GetPtr(occluder), ref transform);
        }

        private static IntPtr method_bind_151 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_light_occluder_set_light_mask");

        /// <summary>
        /// The light mask. See [LightOccluder2D] for more information on light masks
        /// </summary>
        [GodotMethod("canvas_light_occluder_set_light_mask")]
        public static void CanvasLightOccluderSetLightMask(RID occluder, int mask)
        {
            NativeCalls.godot_icall_2_626(method_bind_151, ptr, RID.GetPtr(occluder), mask);
        }

        private static IntPtr method_bind_152 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_occluder_polygon_create");

        /// <summary>
        /// Creates a new light occluder polygon.
        /// </summary>
        [GodotMethod("canvas_occluder_polygon_create")]
        public static RID CanvasOccluderPolygonCreate()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_152, ptr);
            return new RID(ret);
        }

        private static IntPtr method_bind_153 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_occluder_polygon_set_shape");

        /// <summary>
        /// Sets the shape of the occluder polygon.
        /// </summary>
        [GodotMethod("canvas_occluder_polygon_set_shape")]
        public static void CanvasOccluderPolygonSetShape(RID occluderPolygon, Vector2[] shape, bool closed)
        {
            NativeCalls.godot_icall_3_1018(method_bind_153, ptr, RID.GetPtr(occluderPolygon), shape, closed);
        }

        private static IntPtr method_bind_154 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_occluder_polygon_set_shape_as_lines");

        /// <summary>
        /// Sets the shape of the occluder polygon as lines.
        /// </summary>
        [GodotMethod("canvas_occluder_polygon_set_shape_as_lines")]
        public static void CanvasOccluderPolygonSetShapeAsLines(RID occluderPolygon, Vector2[] shape)
        {
            NativeCalls.godot_icall_2_1019(method_bind_154, ptr, RID.GetPtr(occluderPolygon), shape);
        }

        private static IntPtr method_bind_155 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "canvas_occluder_polygon_set_cull_mode");

        /// <summary>
        /// Sets an occluder polygons cull mode. See CANVAS_OCCLUDER_POLYGON_CULL_MODE_* constants.
        /// </summary>
        [GodotMethod("canvas_occluder_polygon_set_cull_mode")]
        public static void CanvasOccluderPolygonSetCullMode(RID occluderPolygon, VisualServer.CanvasOccluderPolygonCullMode mode)
        {
            NativeCalls.godot_icall_2_1020(method_bind_155, ptr, RID.GetPtr(occluderPolygon), mode);
        }

        private static IntPtr method_bind_156 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "black_bars_set_margins");

        /// <summary>
        /// Sets margin size, where black bars (or images, if [method black_bars_set_images] was used) are rendered.
        /// </summary>
        [GodotMethod("black_bars_set_margins")]
        public static void BlackBarsSetMargins(int left, int top, int right, int bottom)
        {
            NativeCalls.godot_icall_4_819(method_bind_156, ptr, left, top, right, bottom);
        }

        private static IntPtr method_bind_157 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "black_bars_set_images");

        /// <summary>
        /// Sets images to be rendered in the window margin.
        /// </summary>
        [GodotMethod("black_bars_set_images")]
        public static void BlackBarsSetImages(RID left, RID top, RID right, RID bottom)
        {
            NativeCalls.godot_icall_4_1021(method_bind_157, ptr, RID.GetPtr(left), RID.GetPtr(top), RID.GetPtr(right), RID.GetPtr(bottom));
        }

        private static IntPtr method_bind_158 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "free");

        /// <summary>
        /// Tries to free an object in the VisualServer.
        /// </summary>
        [GodotMethod("free")]
        public static void Free(RID rid)
        {
            NativeCalls.godot_icall_1_627(method_bind_158, ptr, RID.GetPtr(rid));
        }

        private static IntPtr method_bind_159 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "request_frame_drawn_callback");

        /// <summary>
        /// Schedules a callback to the corresponding named 'method' on 'where' after a frame has been drawn.
        /// The callback method must use only 1 argument which will be called with 'userdata'.
        /// </summary>
        [GodotMethod("request_frame_drawn_callback")]
        public static void RequestFrameDrawnCallback(Object where, string method, object userdata)
        {
            NativeCalls.godot_icall_3_907(method_bind_159, ptr, Object.GetPtr(where), method, userdata);
        }

        private static IntPtr method_bind_160 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "draw");

        /// <summary>
        /// Draws a frame.
        /// </summary>
        [GodotMethod("draw")]
        public static void Draw(bool swapBuffers = true)
        {
            NativeCalls.godot_icall_1_11(method_bind_160, ptr, swapBuffers);
        }

        private static IntPtr method_bind_161 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "sync");

        /// <summary>
        /// </summary>
        [GodotMethod("sync")]
        public static void Sync()
        {
            NativeCalls.godot_icall_0_12(method_bind_161, ptr);
        }

        private static IntPtr method_bind_162 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_changed");

        /// <summary>
        /// Returns [code]true[/code] if changes have been made to the VisualServer's data. [method draw] is usually called if this happens.
        /// </summary>
        [GodotMethod("has_changed")]
        public static bool HasChanged()
        {
            return NativeCalls.godot_icall_0_3(method_bind_162, ptr);
        }

        private static IntPtr method_bind_163 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "init");

        /// <summary>
        /// Initializes the visual server.
        /// </summary>
        [GodotMethod("init")]
        public static void Init()
        {
            NativeCalls.godot_icall_0_12(method_bind_163, ptr);
        }

        private static IntPtr method_bind_164 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "finish");

        /// <summary>
        /// Removes buffers and clears testcubes.
        /// </summary>
        [GodotMethod("finish")]
        public static void Finish()
        {
            NativeCalls.godot_icall_0_12(method_bind_164, ptr);
        }

        private static IntPtr method_bind_165 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_render_info");

        /// <summary>
        /// Returns a certain information, see RENDER_INFO_* for options.
        /// </summary>
        [GodotMethod("get_render_info")]
        public static int GetRenderInfo(VisualServer.RenderInfo info)
        {
            return NativeCalls.godot_icall_1_1022(method_bind_165, ptr, info);
        }

        private static IntPtr method_bind_166 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_test_cube");

        /// <summary>
        /// Returns the id of the test cube. Creates one if none exists.
        /// </summary>
        [GodotMethod("get_test_cube")]
        public static RID GetTestCube()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_166, ptr);
            return new RID(ret);
        }

        private static IntPtr method_bind_167 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_test_texture");

        /// <summary>
        /// Returns the id of the test texture. Creates one if none exists.
        /// </summary>
        [GodotMethod("get_test_texture")]
        public static RID GetTestTexture()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_167, ptr);
            return new RID(ret);
        }

        private static IntPtr method_bind_168 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_white_texture");

        /// <summary>
        /// Returns the id of a white texture. Creates one if none exists.
        /// </summary>
        [GodotMethod("get_white_texture")]
        public static RID GetWhiteTexture()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_168, ptr);
            return new RID(ret);
        }

        private static IntPtr method_bind_169 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "make_sphere_mesh");

        /// <summary>
        /// Returns a mesh of a sphere with the given amount of horizontal and vertical subdivisions.
        /// </summary>
        [GodotMethod("make_sphere_mesh")]
        public static RID MakeSphereMesh(int latitudes, int longitudes, float radius)
        {
            IntPtr ret = NativeCalls.godot_icall_3_1023(method_bind_169, ptr, latitudes, longitudes, radius);
            return new RID(ret);
        }

        private static IntPtr method_bind_170 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_boot_image");

        /// <summary>
        /// Sets a boot image. The color defines the background color and if scale is [code]true[/code], the image will be scaled to fit the screen size.
        /// </summary>
        [GodotMethod("set_boot_image")]
        public static void SetBootImage(Image image, Color color, bool scale)
        {
            NativeCalls.godot_icall_3_1024(method_bind_170, ptr, Object.GetPtr(image), ref color, scale);
        }

        private static IntPtr method_bind_171 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_default_clear_color");

        /// <summary>
        /// </summary>
        [GodotMethod("set_default_clear_color")]
        public static void SetDefaultClearColor(Color color)
        {
            NativeCalls.godot_icall_1_202(method_bind_171, ptr, ref color);
        }

        private static IntPtr method_bind_172 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_feature");

        /// <summary>
        /// </summary>
        [GodotMethod("has_feature")]
        public static bool HasFeature(VisualServer.Features feature)
        {
            return NativeCalls.godot_icall_1_1025(method_bind_172, ptr, feature);
        }

        private static IntPtr method_bind_173 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_os_feature");

        /// <summary>
        /// Returns true, if the OS supports a certain feature. Features might be s3tc, etc, etc2 and pvrtc,
        /// </summary>
        [GodotMethod("has_os_feature")]
        public static bool HasOsFeature(string feature)
        {
            return NativeCalls.godot_icall_1_70(method_bind_173, ptr, feature);
        }

        private static IntPtr method_bind_174 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_debug_generate_wireframes");

        /// <summary>
        /// </summary>
        [GodotMethod("set_debug_generate_wireframes")]
        public static void SetDebugGenerateWireframes(bool generate)
        {
            NativeCalls.godot_icall_1_11(method_bind_174, ptr, generate);
        }
    }
}
