using System;

namespace Godot
{
    /// <summary>
    /// Windowdialog is the base class for all window-based dialogs. It's a by-default toplevel [Control] that draws a window decoration and allows motion and resizing.
    /// </summary>
    public class WindowDialog : Popup
    {
        /// <summary>
        /// The text displayed in the window's title bar. Default value: "Save a File".
        /// </summary>
        public string WindowTitle
        {
            get
            {
                return GetTitle();
            }
            set
            {
                SetTitle(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the user can resize the window. Default value: [code]false[/code].
        /// </summary>
        public bool Resizable
        {
            get
            {
                return GetResizable();
            }
            set
            {
                SetResizable(value);
            }
        }


        private const string nativeName = "WindowDialog";

        public WindowDialog() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_WindowDialog_Ctor(this);
        }

        internal WindowDialog(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_title");

        [GodotMethod("set_title")]
        public void SetTitle(string title)
        {
            NativeCalls.godot_icall_1_43(method_bind_0, Object.GetPtr(this), title);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_title");

        [GodotMethod("get_title")]
        public string GetTitle()
        {
            return NativeCalls.godot_icall_0_2(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_resizable");

        [GodotMethod("set_resizable")]
        public void SetResizable(bool resizable)
        {
            NativeCalls.godot_icall_1_11(method_bind_2, Object.GetPtr(this), resizable);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_resizable");

        [GodotMethod("get_resizable")]
        public bool GetResizable()
        {
            return NativeCalls.godot_icall_0_3(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_close_button");

        /// <summary>
        /// Return the close [TextureButton].
        /// </summary>
        [GodotMethod("get_close_button")]
        public TextureButton GetCloseButton()
        {
            return NativeCalls.godot_icall_0_1026(method_bind_4, Object.GetPtr(this));
        }
    }
}
