using System;

namespace Godot
{
    /// <summary>
    /// Class that has everything pertaining to a 2D world. A physics space, a visual scenario and a sound space. 2D nodes register their resources into the current 2D world.
    /// </summary>
    public class World2D : Resource
    {
        /// <summary>
        /// The [RID] of this world's canvas resource. Used by the [VisualServer] for 2D drawing.
        /// </summary>
        public RID Canvas
        {
            get
            {
                return GetCanvas();
            }
        }

        /// <summary>
        /// The [RID] of this world's physics space resource. Used by the [Physics2DServer] for 2D physics, treating it as both a space and an area.
        /// </summary>
        public RID Space
        {
            get
            {
                return GetSpace();
            }
        }

        /// <summary>
        /// The state of this world's physics space. This allows arbitrary querying for collision.
        /// </summary>
        public Physics2DDirectSpaceState DirectSpaceState
        {
            get
            {
                return GetDirectSpaceState();
            }
        }


        private const string nativeName = "World2D";

        public World2D() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_World2D_Ctor(this);
        }

        internal World2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_canvas");

        [GodotMethod("get_canvas")]
        public RID GetCanvas()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_0, Object.GetPtr(this));
            return new RID(ret);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_space");

        [GodotMethod("get_space")]
        public RID GetSpace()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_1, Object.GetPtr(this));
            return new RID(ret);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_direct_space_state");

        [GodotMethod("get_direct_space_state")]
        public Physics2DDirectSpaceState GetDirectSpaceState()
        {
            return NativeCalls.godot_icall_0_601(method_bind_2, Object.GetPtr(this));
        }
    }
}
