using System;

namespace Godot
{
    /// <summary>
    /// The [code]WorldEnvironment[/code] node can be added to a scene in order to set default [Environment] variables for the scene. The [code]WorldEnvironment[/code] can be overridden by an [Environment] node set on the current [Camera]. Additionally, only one [code]WorldEnvironment[/code] may be instanced in a given scene at a time. The [code]WorldEnvironment[/code] allows the user to specify default lighting parameters (e.g. ambient lighting), various post-processing effects (e.g. SSAO, DOF, Tonemapping), and how to draw the background (e.g. solid color, skybox).
    /// </summary>
    public class WorldEnvironment : Node
    {
        /// <summary>
        /// </summary>
        public Environment Environment
        {
            get
            {
                return GetEnvironment();
            }
            set
            {
                SetEnvironment(value);
            }
        }


        private const string nativeName = "WorldEnvironment";

        public WorldEnvironment() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_WorldEnvironment_Ctor(this);
        }

        internal WorldEnvironment(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_environment");

        [GodotMethod("set_environment")]
        public void SetEnvironment(Environment env)
        {
            NativeCalls.godot_icall_1_24(method_bind_0, Object.GetPtr(this), Object.GetPtr(env));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_environment");

        [GodotMethod("get_environment")]
        public Environment GetEnvironment()
        {
            return NativeCalls.godot_icall_0_189(method_bind_1, Object.GetPtr(this));
        }
    }
}
