using System;

namespace Godot
{
    /// <summary>
    /// This class can serve as base to make custom XML parsers. Since XML is a very flexible standard, this interface is low level so it can be applied to any possible schema.
    /// </summary>
    public class XMLParser : Reference
    {
        public enum NodeType
        {
            None = 0,
            Element = 1,
            ElementEnd = 2,
            Text = 3,
            Comment = 4,
            Cdata = 5,
            Unknown = 6
        }


        private const string nativeName = "XMLParser";

        public XMLParser() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_XMLParser_Ctor(this);
        }

        internal XMLParser(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "read");

        /// <summary>
        /// Read the next node of the file. This returns an error code.
        /// </summary>
        [GodotMethod("read")]
        public Error Read()
        {
            return NativeCalls.godot_icall_0_325(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_node_type");

        /// <summary>
        /// Get the type of the current node. Compare with [code]NODE_*[/code] constants.
        /// </summary>
        [GodotMethod("get_node_type")]
        public XMLParser.NodeType GetNodeType()
        {
            return NativeCalls.godot_icall_0_1027(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_node_name");

        /// <summary>
        /// Get the name of the current element node. This will raise an error if the current node type is not [code]NODE_ELEMENT[/code] nor [code]NODE_ELEMENT_END[/code]
        /// </summary>
        [GodotMethod("get_node_name")]
        public string GetNodeName()
        {
            return NativeCalls.godot_icall_0_2(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_node_data");

        /// <summary>
        /// Get the contents of a text node. This will raise an error in any other type of node.
        /// </summary>
        [GodotMethod("get_node_data")]
        public string GetNodeData()
        {
            return NativeCalls.godot_icall_0_2(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_node_offset");

        /// <summary>
        /// Get the byte offset of the current node since the beginning of the file or buffer.
        /// </summary>
        [GodotMethod("get_node_offset")]
        public int GetNodeOffset()
        {
            return NativeCalls.godot_icall_0_1(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_attribute_count");

        /// <summary>
        /// Get the amount of attributes in the current element.
        /// </summary>
        [GodotMethod("get_attribute_count")]
        public int GetAttributeCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_attribute_name");

        /// <summary>
        /// Get the name of the attribute specified by the index in [code]idx[/code] argument.
        /// </summary>
        [GodotMethod("get_attribute_name")]
        public string GetAttributeName(int idx)
        {
            return NativeCalls.godot_icall_1_108(method_bind_6, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_attribute_value");

        /// <summary>
        /// Get the value of the attribute specified by the index in [code]idx[/code] argument.
        /// </summary>
        [GodotMethod("get_attribute_value")]
        public string GetAttributeValue(int idx)
        {
            return NativeCalls.godot_icall_1_108(method_bind_7, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_attribute");

        /// <summary>
        /// Check whether or not the current element has a certain attribute.
        /// </summary>
        [GodotMethod("has_attribute")]
        public bool HasAttribute(string name)
        {
            return NativeCalls.godot_icall_1_70(method_bind_8, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_named_attribute_value");

        /// <summary>
        /// Get the value of a certain attribute of the current element by name. This will raise an error if the element has no such attribute.
        /// </summary>
        [GodotMethod("get_named_attribute_value")]
        public string GetNamedAttributeValue(string name)
        {
            return NativeCalls.godot_icall_1_73(method_bind_9, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_named_attribute_value_safe");

        /// <summary>
        /// Get the value of a certain attribute of the current element by name. This will return an empty [String] if the attribute is not found.
        /// </summary>
        [GodotMethod("get_named_attribute_value_safe")]
        public string GetNamedAttributeValueSafe(string name)
        {
            return NativeCalls.godot_icall_1_73(method_bind_10, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_empty");

        /// <summary>
        /// Check whether the current element is empty (this only works for completely empty tags, e.g. &lt;element \&gt;).
        /// </summary>
        [GodotMethod("is_empty")]
        public bool IsEmpty()
        {
            return NativeCalls.godot_icall_0_3(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_current_line");

        /// <summary>
        /// Get the current line in the parsed file (currently not implemented).
        /// </summary>
        [GodotMethod("get_current_line")]
        public int GetCurrentLine()
        {
            return NativeCalls.godot_icall_0_1(method_bind_12, Object.GetPtr(this));
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "skip_section");

        /// <summary>
        /// Skips the current section. If the node contains other elements, they will be ignored and the cursor will go to the closing of the current element.
        /// </summary>
        [GodotMethod("skip_section")]
        public void SkipSection()
        {
            NativeCalls.godot_icall_0_12(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "seek");

        /// <summary>
        /// Move the buffer cursor to a certain offset (since the beginning) and read the next node there. This returns an error code.
        /// </summary>
        [GodotMethod("seek")]
        public Error Seek(int position)
        {
            return NativeCalls.godot_icall_1_1028(method_bind_14, Object.GetPtr(this), position);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "open");

        /// <summary>
        /// Open a XML file for parsing. This returns an error code.
        /// </summary>
        [GodotMethod("open")]
        public Error Open(string file)
        {
            return NativeCalls.godot_icall_1_170(method_bind_15, Object.GetPtr(this), file);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "open_buffer");

        /// <summary>
        /// Open a XML raw buffer for parsing. This returns an error code.
        /// </summary>
        [GodotMethod("open_buffer")]
        public Error OpenBuffer(byte[] buffer)
        {
            return NativeCalls.godot_icall_1_430(method_bind_16, Object.GetPtr(this), buffer);
        }
    }
}
