using System;

namespace Godot
{
    /// <summary>
    /// Sort all child nodes based on their Y positions. The child node must inherit from [CanvasItem] for it to be sorted. Nodes that have a higher Y position will be drawn later, so they will appear on top of nodes that have a lower Y position.
    /// </summary>
    public class YSort : Node2D
    {
        /// <summary>
        /// </summary>
        public bool SortEnabled
        {
            get
            {
                return IsSortEnabled();
            }
            set
            {
                SetSortEnabled(value);
            }
        }


        private const string nativeName = "YSort";

        public YSort() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_YSort_Ctor(this);
        }

        internal YSort(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_sort_enabled");

        [GodotMethod("set_sort_enabled")]
        public void SetSortEnabled(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_0, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_sort_enabled");

        [GodotMethod("is_sort_enabled")]
        public bool IsSortEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_1, Object.GetPtr(this));
        }
    }
}
